#------------------------------------------------------------------------------
# Makefile Rule Definitions
#------------------------------------------------------------------------------
# $Author: agibert $
# $Date: 2008/11/12 02:25:22 $
# $Revision: 1.1 $
# $Label:  $
#------------------------------------------------------------------------------
# Makefile Target:		clean all dep install uninstall
# Default Makefile Target:	all
# Makefile Support Switches:	ORACLE_SUPPORT TUX_SUPPORT MQS_SUPPORT
# Makefile Switches:		VERBOSE WARNING DEBUG OPTIMIZE STATIC DYNAMIC
# Makefile Switches:		PROFILE STRIP DOTI DOLINT EFENCE DMALLOC PURIFY
# Default Makefile Switches:	STATIC=1 DEBUG=1
# Makefile Variables:		CC SQLCHECK
# Default Makefile Variables:	SQLCHECK=FULL
# Tuxedo Server Tag:		TXSERVICE_NAME TXSERVICE_TYPE
# Tuxedo Service Type:		NONE TX_SERVER_OXA TX_CLIENT_ORA
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
# IMR - Interlinking Message Router                                            
# Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)                
# mailto:arnaud@rx3.net                                                        
# http://www.rx3.org/dvp/?dvp=imr                                              
#------------------------------------------------------------------------------
# This file is part of IMR                                                     
#                                                                              
# This program is free software; you can redistribute it and/or                
# modify it under the terms of the GNU General Public License                  
# as published by the Free Software Foundation; either version 2               
# of the License, or (at your option) any later version.                       
#                                                                              
# This program is distributed in the hope that it will be useful,              
# but WITHOUT ANY WARRANTY; without even the implied warranty of               
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                
# GNU General Public License for more details.                                 
#                                                                              
# You should have received a copy of the GNU General Public License            
# along with this program; if not, write to the Free Software Foundation,      
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.          
#------------------------------------------------------------------------------





#------------------------------------------------------------------------------
# Variables
#------------------------------------------------------------------------------

TARGET_FILES=		$(TARGET)
INSTALL_FILES=		$(TARGET)
UNINSTALL_FILES=	$(TARGET)

ifeq ($(suffix $(TARGET)),$(SHLIB_SUFFIX))
        INSTALL_DIR=		$(GLOBAL_LIB_DIR)
        TARGET_FILES+=		$(TARGET:$(SHLIB_SUFFIX)=.a)
        INSTALL_FILES+=		$(TARGET:$(SHLIB_SUFFIX)=.a)
        UNINSTALL_FILES+=	$(TARGET:$(SHLIB_SUFFIX)=.a) .pure $(TARGET:$(SHLIB_SUFFIX)=_pure_*$(SHLIB_SUFFIX)) $(TARGET:$(SHLIB_SUFFIX)=_pure_*.a)

        ifdef DOLINT
                LINT_TARGET=		llib-l$(subst lib,,$(basename $(TARGET))).ln
                LINT_LIB=		$(LINT_TARGET)
                TARGET_FILES+=		$(LINT_TARGET)
                INSTALL_FILES+=		$(LINT_TARGET)
                UNINSTALL_FILES+=	$(LINT_TARGET)
        endif
else
        ifeq ($(suffix $(TARGET)),.xml)
                INSTALL_DIR=		$(GLOBAL_LIB_DIR)/imr
        else
                INSTALL_DIR=		$(GLOBAL_BIN_DIR)

                ifdef DOLINT
                        LINT_FILE=	$(basename $(TARGET)).lint
                        LINT_LIB=	
                        TARGET_FILES+=	$(LINT_FILE)
                endif
        endif
endif

ifdef DOTI
        TARGET_FILES+=	$(OBJ:.o=.i)
endif





#------------------------------------------------------------------------------
# External targets
#------------------------------------------------------------------------------

all: dep
	$(QUIET)$(MAKE) $(SILENT) $(MAKEOVERRIDES) build

clean:
	@echo -e '$(ECHO_HEAD)Cleaning up...'
	$(QUIET)$(RM) -f						\
		$(shell for file in $(OBJ:.o=);				\
			do						\
				if [ -f $$file.pc ];			\
				then					\
					echo $$file.c $$file.lis;	\
				fi;					\
			done)						\
		$(OBJ:.o=.dep)						\
		$(OBJ:.o=.i)						\
		$(OBJ:.o=.ln)						\
		$(OBJ:.o=.sublint)					\
		$(OBJ:.o=_pure_*)					\
		$(OBJ)							\
		$(TARGET_FILES)						\
		llib-l$(subst lib,,$(basename $(TARGET))).ln		\
		llib-l$(subst lib,,$(basename $(TARGET))).sublint	\
		BS-*.c BS-*.o						\
		BS-*_pure_*						\
		BC-*.c BC-*.o						\
		BC-*_pure_*						\
		logfile*						\
		.pure

install: $(patsubst %,$(INSTALL_DIR)/%,$(INSTALL_FILES))

uninstall:
	$(QUIET)for file in $(UNINSTALL_FILES);				\
	do								\
		echo -e "$(ECHO_HEAD)Uninstalling $$file from $(INSTALL_DIR)..."; \
		$(RM) -f $(INSTALL_DIR)/$$file;				\
	done

dep:
	$(QUIET)$(MAKE) $(SILENT) $(MAKEOVERRIDES) depend





#------------------------------------------------------------------------------
# Internal targets
#------------------------------------------------------------------------------

depend:	$(OBJ:.o=.dep)

build: $(TARGET_FILES)





#------------------------------------------------------------------------------
# Rules
#------------------------------------------------------------------------------

%.dep	: %.c
	@echo -e '$(ECHO_HEAD)Dependencing $<...'
	$(QUIET)$(MAKEDEP) $(MAKEDEP_FLAGS) -o.o $<  >$@
	$(QUIET)$(MAKEDEP) $(MAKEDEP_FLAGS) -o.i $< >>$@

%.c	: %.pc
	@echo -e '$(ECHO_HEAD)Procing $<...'
	$(QUIET)$(PROC) $(DEVNULL) $(PROC_FLAGS) oname=$@ iname=$< $(PROC_FILTER)

%.i	: %.c
	@echo -e '$(ECHO_HEAD)Precompilling $<...'
	$(QUIET)$(CPP) $(CPPFLAGS) $(INCLUDE) -o $@ $<

%.ln	: %.c
	@echo -e '$(ECHO_HEAD)Linting $<...'
	$(QUIET)$(LINT) $(LINTFLAGS) $(INCLUDE) -c > $(<:.c=.sublint) $<

%.o	: %.c
	@echo -e '$(ECHO_HEAD)Compilling $<...'
	$(QUIET)$(CC) $(CFLAGS) $(INCLUDE)   -o $@ $<

$(LINT_LIB): $(OBJ:.o=.ln)
	@echo -e '$(ECHO_HEAD)Lib Linting $@...'
	$(QUIET)$(LINT) $(LINTFLAGS) -o $(subst lib,,$(basename $(TARGET))) > $(@:.ln=.sublint) $(OBJ:.o=.ln)

%$(SHLIB_SUFFIX): %.o
	@echo -e '$(ECHO_HEAD)Linking $@...'
	$(QUIET)$(LD) $(LDFLAGS)           -o $@ $(OBJ)

%.a	: %.o
	@echo -e '$(ECHO_HEAD)Linking $@...'
	$(QUIET)$(AR) $(ARFLAGS)              $@ $(OBJ)

%.lint	: %.ln
	@echo -e '$(ECHO_HEAD)Link Linting $@...'
	$(QUIET)$(LINT) $(LINTFLAGS) $(LINTLIBS) $< >$@

%	: %.o
ifeq ($(TXSERVICE_TYPE),TX_SERVER_OXA)
	@echo -e '$(ECHO_HEAD)Building Tuxedo server $@... '
	$(QUIET)$(PRELINK) $(TXBLDSRV) $(TXBLD_FLAGS)  -o $@ -f "$(OBJ)" $(TXBLD_LIB) -s "$(TXSERVICE_NAME)" $(PRELINK_FILTER)
else
        ifeq ($(TXSERVICE_TYPE),TX_CLIENT_ORA)
		@echo -e '$(ECHO_HEAD)Building Tuxedo client $@...'
		$(QUIET)$(PRELINK) $(TXBLDCLN) $(TXBLD_FLAGS)  -o $@ -f "$(OBJ)" $(TXBLD_LIB) $(PRELINK_FILTER)
                ifndef DEBUG
			$(QUIET)$(RM) BC-*.c
                endif
        else
		@echo -e '$(ECHO_HEAD)Linking $@...'
                ifdef PURIFY
			@echo -e '$(ECHO_HEAD)Purifing $@... '
			$(PURIFIER) $(PURIFIER_FLAGS) $(CC) $(LINK_FLAGS) -o $@ $(OBJ) $(LIB) $(PRELINK_FILTER)
                else
			$(QUIET)$(CC) $(LINK_FLAGS) -o $@ $(OBJ) $(LIB)
                endif
        endif
endif
ifdef DEBUG
        ifeq ($(OS),HP-UX)
	        @echo -e '$(ECHO_HEAD)Pxdbing $@...'
	        $(QUIET)$(PXDB) $(PXDBFLAGS) $@
        endif
else
        ifdef STRIP
		@echo -e '$(ECHO_HEAD)Striping $@...'
		$(QUIET)$(STRIPPER) $@
        endif
endif

%.xml	: %.xml.m4
	@echo -e '$(ECHO_HEAD)M4ing $@... '
	$(QUIET)$(M4) $(M4_FLAGS) > $@ $<

$(INSTALL_DIR)/%: $(INSTALL_DIR) all
	@echo -e '$(ECHO_HEAD)Installing $(subst $(INSTALL_DIR)/,,$@) into $(INSTALL_DIR)...'
	$(QUIET)$(CP) $(subst $(INSTALL_DIR)/,,$@) $(INSTALL_DIR)

$(INSTALL_DIR):
	@echo -e '$(ECHO_HEAD)Creatinging $(INSTALL_DIR)...'
	$(QUIET)$(MKDIR) $(INSTALL_DIR)





#------------------------------------------------------------------------------
# Suffixes
#------------------------------------------------------------------------------

.SECONDARY:	$(OBJ:.o=.c) $(OBJ:.o=.dep) $(OBJ:.o=.ln) $(OBJ:.o=.sublint)
.SUFFIXES:	.pc .c .h .dep .i .ln .sublint .o $(SHLIB_SUFFIX) .a .lint
.PHONY:		clean all dep install uninstall build depend
.DELETE_ON_ERROR:





#------------------------------------------------------------------------------
# Dependencies
#------------------------------------------------------------------------------

$(OBJ): Makefile $(MAKEFILE_GLOB) $(MAKEFILE_PATH)
$(TARGET_FILES): $(OBJ) $(OBJ_RAW) Makefile $(MAKEFILE_GLOB) $(MAKEFILE_PATH)

ifneq ($(suffix $(TARGET)),$(SHLIB_SUFFIX))
        ifneq ($(suffix $(TARGET)),.xml)
                $(TARGET): $(patsubst %,$(GLOBAL_LIB_DIR)/lib%$(SHLIB_SUFFIX),$(LIB_LIST))
        endif
endif

ifneq (,$(findstring clean,$(MAKECMDGOALS)))
dep: clean
endif

ifneq (,$(findstring uninstall,$(MAKECMDGOALS)))
dep: uninstall
endif

ifeq ($(MAKECMDGOALS),build)
        ifneq (,$(OBJ))
                -include $(OBJ:.o=.dep)
        endif
endif
