#------------------------------------------------------------------------------
# Makefile Var Definitions
#------------------------------------------------------------------------------
# $Author: agibert $
# $Date: 2008/11/12 02:25:22 $
# $Revision: 1.1 $
# $Label:  $
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
# IMR - Interlinking Message Router                                            
# Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)                
# mailto:arnaud@rx3.net                                                        
# http://www.rx3.org/dvp/?dvp=imr                                              
#------------------------------------------------------------------------------
# This file is part of IMR                                                     
#                                                                              
# This program is free software; you can redistribute it and/or                
# modify it under the terms of the GNU General Public License                  
# as published by the Free Software Foundation; either version 2               
# of the License, or (at your option) any later version.                       
#                                                                              
# This program is distributed in the hope that it will be useful,              
# but WITHOUT ANY WARRANTY; without even the implied warranty of               
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                
# GNU General Public License for more details.                                 
#                                                                              
# You should have received a copy of the GNU General Public License            
# along with this program; if not, write to the Free Software Foundation,      
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.          
#------------------------------------------------------------------------------





#------------------------------------------------------------------------------
# Curent Module Dir
#------------------------------------------------------------------------------

LOCAL_INC_DIR=		../include



#------------------------------------------------------------------------------
# Global Dir
#------------------------------------------------------------------------------

GLOBAL_INC_DIR=		../../include
GLOBAL_BIN_DIR=		../../../bin
GLOBAL_LIB_DIR=		../../../lib

LIB_LIST=		msg_router mapping_mng io_mng static_data_mng log_mng xml node z



#------------------------------------------------------------------------------
# Constants
#------------------------------------------------------------------------------

MAKE		= make
MAKEDEP		= makedepend
SHELL		= bash
RM      	= rm
MKDIR		= mkdir -p
CP		= cp
CAT		= cat
M4              = m4
PROC		= proc
CC		=
#CC		= cc
#CC		= gcc
CPP		= $(CC)
LD		=
#LD      	= ld
#LD      	= $(CC)
AR      	= ar
TXBLDSRV	= buildserver
TXBLDCLN	= buildclient
TXMKFML		= mkfldhdr
TXMKVIEW	= viewc
PXDB		= pxdb
LINT		= lint
STRIPPER	= strip
PURIFIER	= purify

COMMA		= ,





#------------------------------------------------------------------------------
# Variables
#------------------------------------------------------------------------------

OS		= $(shell uname)

ifndef CC
        ifeq ($(OS),HP-UX)
                CC=	cc
                LD=	ld
        else
                CC=	gcc
                LD=	gcc
        endif
endif

ifdef VERBOSE
	QUIET=
	SILENT=
	PROC_FILTER=
	PRELINK_FILTER=
	TXBLDSRV+= -v
	TXBLDCLN+= -v
else
	QUIET=@
	SILENT=-s
	PROC_FILTER=| grep -v -e '^$$' -e '^Pro' -e '^(c)'		\
			   -e '^Copyright' -e '^System default option';	\
		    [[ $${PIPESTATUS[0]} = 0 ]]
	PRELINK_FILTER=2>&1 | grep -v -e '^Purify'			\
			   -e '^Instrumenting: ';			\
		    [[ $${PIPESTATUS[0]} = 0 ]]
endif

ifdef DYNAMIC
	STATIC=
else
	STATIC=1
endif

ifdef OPTIMIZE
	DEBUG=
else
	DEBUG=1
endif

ifdef DEBUG
	TXBLDSRV+= -k
endif

ifndef SQLCHECK
	SQLCHECK=FULL
endif

ifndef ORACLE_SUPPORT
        ifndef ORACLE_HOME
                ORACLE_SUPPORT	= 0
        else
                ORACLE_SUPPORT	= 1
        endif
endif

ifndef TUX_SUPPORT
        ifndef TUXDIR
                TUX_SUPPORT	= 0
        else
                TUX_SUPPORT	= 1
        endif
endif

ifndef MQS_SUPPORT
        ifndef MQM_HOME
                MQS_SUPPORT	= 0
        else
                MQS_SUPPORT	= 1
        endif
endif



# Flags Init
INCLUDE=
LIB_cc=
LIB_gcc=
LIB=
M4_FLAGS=
MAKEDEP_FLAGS=
PROC_FLAGS=
CPPFLAGS=
CPPFLAGS_cc=
CPPFLAGS_gcc=
CFLAGS=
CFLAGS_cc=
CFLAGS_gcc=
LDFLAGS=
TXBLD_FLAGS=
TXBLD_FLAGS_cc=
TXBLD_FLAGS_gcc=
LDFLAGS=
LDFLAGS_cc=
LDFLAGS_gcc=
LINK_FLAGS=
LINK_FLAGS_cc=
LINK_FLAGS_gcc=
ARFLAGS=




PURIFIER_FLAGS=	-user-path=$(shell echo "$(patsubst %,lib%/src,$(LIB_LIST)) imrd/src" | sed 's/ /\:/g')

ifdef PURIFY
	PRELINK=echo -e '$(ECHO_HEAD)Purifing $@... ';	\
		export CC="$(PURIFIER) $(PURIFIER_FLAGS) $(CC)";
endif



ifeq ($(ORACLE_SUPPORT),1)
        INCLUDE+=	-I$(ORACLE_HOME)/precomp/public
        LIB+=		-L$(ORACLE_HOME)/lib -lclntsh
        M4_FLAGS+=	-DORACLE_SUPPORT
        MAKEDEP_FLAGS+=	-DORACLE_SUPPORT
        PROC_FLAGS+=	DEFINE=ORACLE_SUPPORT
        CPPFLAGS+=	-DORACLE_SUPPORT
        CFLAGS+=	-DORACLE_SUPPORT
endif

ifeq ($(TUX_SUPPORT),1)
        INCLUDE+=	-I$(TUXDIR)/include
        LIB+=		-L$(TUXDIR)/lib -lfml -lfml32 -ltux
        M4_FLAGS+=	-DTUX_SUPPORT
        MAKEDEP_FLAGS+=	-DTUX_SUPPORT
        PROC_FLAGS+=	DEFINE=TUX_SUPPORT
        CPPFLAGS+=	-DTUX_SUPPORT
        CFLAGS+=	-DTUX_SUPPORT
endif

ifeq ($(MQS_SUPPORT),1)
        INCLUDE+=	-I$(MQM_HOME)/inc
        LIB+=		-L$(MQM_HOME)/lib64 -lmqic
        M4_FLAGS+=	-DMQS_SUPPORT
        MAKEDEP_FLAGS+=	-DMQS_SUPPORT
        PROC_FLAGS+=	DEFINE=MQS_SUPPORT
        CPPFLAGS+=	-DMQS_SUPPORT
        CFLAGS+=	-DMQS_SUPPORT
endif



# OS Dependant Flags

ifeq ($(OS),HP-UX)
        SHLIB_SUFFIX=	.sl
        M4_FLAGS:=	-B100000 -S100000 $(M4_FLAGS)
        MAKEDEP_FLAGS+=	-z -D _HPUX_SOURCE -D __hpux
        PROC_FLAGS+=	DEFINE=_PA_RISC2_0
        CPPFLAGS+=	-D _HPUX_SOURCE -D __hpux
        CFLAGS+=	-D _HPUX_SOURCE -D __hpux
        LDFLAGS+=	-b
        LDFLAGS+=	

        ifeq ($(ARCH),32BITS)
                MAKEDEP_FLAGS+=	
                CPPFLAGS_cc+=	+DD32
                CPPFLAGS_gcc+= 	
                CFLAGS_cc+=	+DD32
                CFLAGS_gcc+= 	
                TXBLD_FLAGS_cc+= +DD32
                TXBLD_FLAGS_gcc+=
                LINK_FLAGS_cc+= +DD32
                LINK_FLAGS_gcc+=
                LDFLAGS_cc+=
                LDFLAGS_gcc+=
        endif

        ifeq ($(ARCH),64BITS)
                MAKEDEP_FLAGS+=	-D__LP64__
                CPPFLAGS_cc+=	+DD64
                CPPFLAGS_gcc+= 	
                CFLAGS_cc+=	+DD64
                CFLAGS_gcc+= 	
                TXBLD_FLAGS_cc+= +DD64
                TXBLD_FLAGS_gcc+=
                LINK_FLAGS_cc+= +DD64
                LINK_FLAGS_gcc+=
                LDFLAGS_cc+=
                LDFLAGS_gcc+=
        endif
else
        SHLIB_SUFFIX=	.so
        M4_FLAGS+=
        MAKEDEP_FLAGS+=	
        PROC_FLAGS+=
        CPPFLAGS+=
        CFLAGS+=
        LDFLAGS+=	-shared
endif



# Compiler Dependant Flags

INCLUDE+=	-I.							\
		$(patsubst %,-I../../lib%/include,$(LIB_LIST))		\
		-I$(GLOBAL_INC_DIR) 					\
		$(shell for inc in $(EXTRA_INC);			\
			do						\
				echo "-I$$inc ";			\
			done)



ifeq ($(SQLCHECK),FULL)
PROC_FLAGS+=	USERID=$(USR_IMRMGR)/$(PWD_IMRMGR)
endif

PROC_FLAGS+=	SQLCHECK=$(SQLCHECK)                                    \
		RELEASE_CURSOR=NO					\
		HOLD_CURSOR=YES						\
		MAXOPENCURSOR=100					\
		DBMS=V7							\
		MODE=ORACLE						\
		CODE=ANSI_C						\
		$(patsubst -I%,INCLUDE=%,$(INCLUDE))




CPPFLAGS_cc+=	-P -C -Ae -w
CPPFLAGS_gcc+=	-E -C

CFLAGS_cc+=	-c -Ae +z
CFLAGS_gcc+=	-c -fpic  -pipe

TXBLD_FLAGS_cc+=	-Wl,+s,-Bimmediate
TXBLD_FLAGS_gcc+=	

LDFLAGS_cc+=
LDFLAGS_gcc+=

LINK_FLAGS_cc+=
LINK_FLAGS_gcc+=



ifdef DEBUG
        CFLAGS_cc+=	-g
        CFLAGS_gcc+= 	-g
        TXBLD_FLAGS_cc+= -g
        TXBLD_FLAGS_gcc+= -g
        LDFLAGS_cc+=	
        LDFLAGS_gcc+=	
        LINK_FLAGS_cc+=	-g
        LINK_FLAGS_gcc+= -g
endif

ifdef OPTIMIZE
        CFLAGS_cc+=	-O
        CFLAGS_gcc+= 	-O3
        TXBLD_FLAGS_cc+= -O
        TXBLD_FLAGS_gcc+= -O3
        LDFLAGS_cc+=	
        LDFLAGS_gcc+=	-O3
        LINK_FLAGS_cc+=	-O
        LINK_FLAGS_gcc+= -Wl,-O3
endif

ifdef PROFILE
        CFLAGS_cc+=	-pg
        CFLAGS_gcc+= 	-pg
        TXBLD_FLAGS_cc+= -pg
        TXBLD_FLAGS_gcc+= -pg
endif



ifdef WARNING
        CFLAGS_cc+=+w1 +M2
        CFLAGS_gcc+=-Wformat -Wnonnull -Wimplicit -Wmain -Wmissing-braces \
		-Wparentheses -Wsequence-point -Wreturn-type -Wswitch	\
		-Wswitch-default -Wswitch-enum -Wtrigraphs -Wunused 	\
		-Wuninitialized -Wunknown-pragmas -Wstrict-aliasing -W	\
		-Wfloat-equal -Wundef -Wendif-labels -Wshadow		\
		-Wpointer-arith -Wbad-function-cast -Wcast-qual		\
		-Wcast-align -Wwrite-strings -Wconversion		\
		-Wsign-compare -Waggregate-return -Wstrict-prototypes	\
		-Wmissing-prototypes -Wmissing-declarations		\
		-Wmissing-noreturn -Wmissing-format-attribute -Wpacked	\
		-Wredundant-decls -Wnested-externs -Wunreachable-code 	\
		-Winline -Wlong-long -Wdisabled-optimization
        TXBLD_FLAGS_cc+=+w1 +M2
        TXBLD_FLAGS_gcc+=-Wformat -Wnonnull -Wimplicit -Wmain -Wmissing-braces \
		-Wparentheses -Wsequence-point -Wreturn-type -Wswitch	\
		-Wswitch-default -Wswitch-enum -Wtrigraphs -Wunused 	\
		-Wuninitialized -Wunknown-pragmas -Wstrict-aliasing -W	\
		-Wfloat-equal -Wundef -Wendif-labels -Wshadow		\
		-Wpointer-arith -Wbad-function-cast -Wcast-qual		\
		-Wcast-align -Wwrite-strings -Wconversion		\
		-Wsign-compare -Waggregate-return -Wstrict-prototypes	\
		-Wmissing-prototypes -Wmissing-declarations		\
		-Wmissing-noreturn -Wmissing-format-attribute -Wpacked	\
		-Wredundant-decls -Wnested-externs -Wunreachable-code 	\
		-Winline -Wlong-long -Wdisabled-optimization
endif



ifdef EFENCE
else
        ifdef DMALLOC
                CFLAGS+= -DDMALLOC
        endif
endif



ifdef STATIC
        ifeq ($(CC),cc)
                LIB+=	-Wl,-a,archive_shared

                ifeq ($(ARCH),64BITS)
                        LIB+=	-Wl,+forceload
                else
                        LIB+=	-Wl,-E
                endif
        else
                LIB+=	-Wl,-export-dynamic,-whole-archive,-Bstatic
        endif
endif

LIB+=		-L$(GLOBAL_LIB_DIR)					\
		$(patsubst %,-l%,$(LIB_LIST))

ifdef STATIC
        ifeq ($(CC),cc)
                ifeq ($(ARCH),64BITS)
	                LIB+=	-Wl,+noforceload
                endif

                LIB+=	-Wl,-a,shared_archive
        else
                LIB+=	-Wl,-no-whole-archive,-Bdynamic
        endif
endif

ifdef EFENCE
        LIB+=	-lefence -lrt
else
        ifdef DMALLOC
                LIB+= -ldmalloc
        endif
endif

LIB_cc+=	-lnsl -lm -lpthread

LIB_gcc+=	-lpthread -lm -ldl



TXBLD_LIB= -f "$(LIB) -ltmib"



TXSERVICE_NAME=	$(shell if [ -f $(TARGET).c ];				\
			then						\
				grep TXSERVICE_NAME $(TARGET).c |	\
					sed "s/.*: //";			\
			 fi)
TXSERVICE_TYPE=	$(shell if [ -f $(TARGET).c ];				\
			then						\
				grep TXSERVICE_TYPE $(TARGET).c |	\
					sed "s/.*: //";			\
			 fi)



PXDBFLAGS=	-s on $(EXTRA_PXDBFLAGS)



LINTFLAGS=	$(EXTRA_LINTFLAGS)

LINTLIBS=	$(GLOBAL_LIB_DIR)/*.ln



# Flags Merging

MAKEDEP_FLAGS+=	$(INCLUDE) -f- $(EXTRA_MAKEDEP_FLAGS)
PROC_FLAGS+=	$(EXTRA_PROC_FLAGS)
CPPFLAGS+=	$(CPPFLAGS_$(CPP)) $(EXTRA_CPPFLAGS)
CFLAGS+=	$(CFLAGS_$(CC)) $(EXTRA_CFLAGS)
LINK_FLAGS+=	$(LINK_FLAGS_$(CC)) $(EXTRA_LINK_FLAGS)
LDFLAGS+=	$(LDFLAGS_$(CC)) $(EXTRA_LDFLAGS)
LIB+=           $(LIB_$(CC)) $(EXTRA_LIB)
ARFLAGS+=	-cr $(EXTRA_ARFLAGS)
TXBLD_FLAGS+=	-f "$(TXBLD_FLAGS_$(CC))" -r $(ORACLE_RM) $(EXTRA_TXBLD_FLAGS)



ifeq ($(CC),gcc)
        TXBLDSRV:=CC=cc.gcc; $(TXBLDSRV)
        TXBLDCLN:=CC=cc.gcc; $(TXBLDCLN)
endif
