/*----------------------------------------------------------------------------*/
/* $Workfile:   imrd.c  $                                                     */                                            
/*----------------------------------------------------------------------------*/
/* $Author: agibert $                                                      */
/* $Date: 2008/11/12 02:25:22 $                                           */
/* $Revision: 1.1 $                                                       */
/* $Label:  $                                                         */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#define _IMRD_C_

#include "imrd.h" 







/*----------------------------------------------------------------------------*/
/* Config_Init         			                 	  	      */
/*----------------------------------------------------------------------------*/
/* Config_Ptr:   Config structure to initialize                               */
/* ArgC:         Argument number                                              */
/* ArgV:         Argument tab                                                 */
/*----------------------------------------------------------------------------*/

IMRT_Status  Config_Init( IMRT_Config  *Config_Ptr, int  ArgC, char  **ArgV)
{
    struct stat     stat_buf;
    IMRT_Module_Id  module_id;


    /* LM Init */

    strcpy( Config_Ptr->LM.Log_File, "-");

    Config_Ptr->LM.Module_Nb = IMRD_MODULE_NB;

    for( module_id = 0; module_id < IMRD_MODULE_NB; module_id++)
    {
	if( module_id == IMRD_MODULE_ID_UNKNOWN)
	{
	    Config_Ptr->LM.Level_Tab[ IMRD_LOG_TYPE_ID_TRACE][ module_id]   = IMRD_LOG_LEVEL_UNSET;

	    Config_Ptr->LM.Level_Tab[ IMRD_LOG_TYPE_ID_TRACE][ module_id]   = IMRD_LOG_LEVEL_UNSET;
	    Config_Ptr->LM.Level_Tab[ IMRD_LOG_TYPE_ID_INFO][ module_id]    = IMRD_LOG_LEVEL_DEFAULT;
	    Config_Ptr->LM.Level_Tab[ IMRD_LOG_TYPE_ID_WARNING][ module_id] = IMRD_LOG_LEVEL_DEFAULT;
	    Config_Ptr->LM.Level_Tab[ IMRD_LOG_TYPE_ID_ERROR][ module_id]   = IMRD_LOG_LEVEL_DEFAULT;
	}
	else
	{
	    Config_Ptr->LM.Level_Tab[ IMRD_LOG_TYPE_ID_TRACE][   module_id] = IMRD_LOG_LEVEL_UNSET;
	    Config_Ptr->LM.Level_Tab[ IMRD_LOG_TYPE_ID_INFO][    module_id] = IMRD_LOG_LEVEL_UNSET;
	    Config_Ptr->LM.Level_Tab[ IMRD_LOG_TYPE_ID_WARNING][ module_id] = IMRD_LOG_LEVEL_UNSET;
	    Config_Ptr->LM.Level_Tab[ IMRD_LOG_TYPE_ID_ERROR][   module_id] = IMRD_LOG_LEVEL_UNSET;
	}
    }



    /* SDM Init */

    Config_Ptr->SDM.Flag = IMRD_SDM_FLAG_XML;

    strncpy( Config_Ptr->SDM.Lib_Path, dirname( ArgV[0]), IMRD_PATH_LEN);
    strncat( Config_Ptr->SDM.Lib_Path, "/../lib/imr", IMRD_PATH_LEN);

    if( ( stat( Config_Ptr->SDM.Lib_Path, &stat_buf) == -1) || ( ! S_ISDIR( stat_buf.st_mode)))
    {
	strcpy( Config_Ptr->SDM.Lib_Path, ".");
    }



    /* MM Init */

    Config_Ptr->MM.XML_Format = IMRD_FALSE;



    /* MR Init */

    Config_Ptr->MR.Mapping_Forced                      = IMRD_FALSE;
    Config_Ptr->MR.Continue_Forced                     = IMRD_FALSE;
    Config_Ptr->MR.Read_Mode                           = IMRD_READ_MODE_WAIT;
    Config_Ptr->MR.Log_Id_Struct_Input.Bit_Nb          = 0;
    Config_Ptr->MR.Log_Id_Struct_Input.Log_Size        = 0;
    Config_Ptr->MR.Log_Id_Struct_Input.Log_Id_Bit_Ptr  = NULL;
    Config_Ptr->MR.Log_Id_Struct_Output.Bit_Nb         = 0;
    Config_Ptr->MR.Log_Id_Struct_Output.Log_Size       = 0;
    Config_Ptr->MR.Log_Id_Struct_Output.Log_Id_Bit_Ptr = NULL;



    /* IMRD Init */

    strcpy( Config_Ptr->IMRD.Medium_URL_Input,  "file://\\nMSG_BREAK\\n@in.txt");
    strcpy( Config_Ptr->IMRD.Appl_Name_Input,   "bo_raw");
     Config_Ptr->IMRD.Appl_Id_Input = IMRD_APPL_ID_UNDEFINEDED;

    strcpy( Config_Ptr->IMRD.Medium_URL_Output, "file://\\nMSG_BREAK\\n@out.txt");
    strcpy( Config_Ptr->IMRD.Appl_Name_Output,  "cristal_raw");
    Config_Ptr->IMRD.Appl_Id_Output = IMRD_APPL_ID_UNDEFINEDED;

    strcpy( Config_Ptr->IMRD.Medium_URL_Trash,  "file://\\nMSG_BREAK\\n@trash.txt");

    Config_Ptr->IMRD.Version_Mode = IMRD_FALSE;
    Config_Ptr->IMRD.Help_Mode    = IMRD_FALSE;
    Config_Ptr->IMRD.Dump_Mode    = IMRD_SDM_FLAG_UNKNOWN;
    Config_Ptr->IMRD.Core_Dump    = IMRD_FALSE;



    return( IMRS_OK);
}





/*----------------------------------------------------------------------------*/
/* Config_Deinit      			                 	  	      */
/*----------------------------------------------------------------------------*/
/* Config_Ptr:   Config structure to deinitialize                             */
/*----------------------------------------------------------------------------*/

IMRT_Status  Config_Deinit( IMRT_Config  *Config_Ptr)
{
    if( Config_Ptr->MR.Log_Id_Struct_Input.Log_Id_Bit_Ptr != NULL)
    {
	free( Config_Ptr->MR.Log_Id_Struct_Input.Log_Id_Bit_Ptr);
    }

    if( Config_Ptr->MR.Log_Id_Struct_Output.Log_Id_Bit_Ptr)
    {
	free( Config_Ptr->MR.Log_Id_Struct_Output.Log_Id_Bit_Ptr);
    }

    return( IMRS_OK);
}





/*----------------------------------------------------------------------------*/
/* Args_Parse         			                 	  	      */
/*----------------------------------------------------------------------------*/
/* Config_Ptr:   Configuration structure to update                            */
/* ArgC:         Argument number                                              */
/* ArgV:         Argument tab                                                 */
/*----------------------------------------------------------------------------*/

IMRT_Status  Args_Parse( IMRT_Config  *Config_Ptr, int  ArgC, char  **ArgV)
{
    int             arg_cur, i, j;
    char            value_tmp[ IMRD_ARG_NAME_SIZE];
    IMRT_Module_Id  module_id;


    for( arg_cur = 1; arg_cur < ArgC; arg_cur++)
    {
	for( i = 0;
	     ( i < IMRD_ARG_NB) && ( strcmp( IMRG_Arg_Tab[i].Name_Short, ArgV[arg_cur]) != 0) && ( strcmp( IMRG_Arg_Tab[i].Name_Long, ArgV[arg_cur]) != 0);
	     i++);

	

	if( ( i > 0) && ( i < IMRD_ARG_NB))
	{

	    switch( i)
	    {
		case IMRD_ARG_VERSION:
		{
		    Config_Ptr->IMRD.Version_Mode = IMRD_TRUE;

		    break;
		}

		case IMRD_ARG_HELP:
		{
		    Config_Ptr->IMRD.Help_Mode = IMRD_TRUE;

		    break;
		}

		case IMRD_ARG_VERBOSE_LEVEL_DEFAULT:
		{
		    if( ( arg_cur + 1) < ArgC)
		    {
			strncpy( value_tmp, ArgV[ ++arg_cur], IMRD_ARG_NAME_LEN);

			Config_Ptr->LM.Level_Tab[ IMRD_LOG_TYPE_ID_TRACE][   IMRD_MODULE_ID_UNKNOWN] = atoi( value_tmp);
		    }
		    else
		    {
			IMRD_LM_LOG_ERROR_1( "Missing argument value [%s] !", IMRG_Arg_Tab[ i].Name_Short);

			return( IMRS_KO);
		    }

		    break;
		}

		case IMRD_ARG_VERBOSE_LEVEL_MODULE:
		{
		    if( ( arg_cur + 2) < ArgC )
		    {
			strncpy( value_tmp, ArgV[ ++arg_cur], IMRD_ARG_NAME_LEN);
			j = atoi( value_tmp);

			strncpy( value_tmp, ArgV[ ++arg_cur], IMRD_ARG_NAME_LEN);

			for( module_id = 1;
			     ( module_id < IMRD_MODULE_NB) && ( strcmp( value_tmp, IMRG_IMRD_Base.Module_Entry_Tab[ module_id].Module_Ptr->Name) != 0);
			     module_id++);

			if( module_id == IMRD_MODULE_NB)
			{
			    IMRD_LM_LOG_ERROR_1( "Module not found [%s] !", value_tmp);

			    return( IMRS_KO);
			}

			Config_Ptr->LM.Level_Tab[ IMRD_LOG_TYPE_ID_TRACE][ module_id] = j;
		    }
		    else
		    {
			IMRD_LM_LOG_ERROR_1( "Missing argument value [%s] !", IMRG_Arg_Tab[ i].Name_Short);

			return( IMRS_KO);
		    }

		    break;
		}

		case IMRD_ARG_LOG_FILE:
		{
		    if( ( arg_cur + 1) < ArgC)
		    {
			strncpy( ( char *) Config_Ptr->LM.Log_File, ArgV[ ++arg_cur], IMRD_PATH_LEN);
		    }
		    else
		    {
			IMRD_LM_LOG_ERROR_1( "Missing argument value [%s] !", IMRG_Arg_Tab[ i].Name_Short);

			return( IMRS_KO);
		    }

		    break;
		}

		case IMRD_ARG_SD_READ_XML:
		{
		    Config_Ptr->SDM.Flag = IMRD_SDM_FLAG_XML;

		    break;
		}

		case IMRD_ARG_SD_READ_SQL:
		{
		    Config_Ptr->SDM.Flag = IMRD_SDM_FLAG_SQL;

		    break;
		}

		case IMRD_ARG_SD_WRITE_XML:
		{
		    Config_Ptr->IMRD.Dump_Mode = IMRD_SDM_FLAG_XML;

		    break;
		}

		case IMRD_ARG_SD_WRITE_SQL:
		{
		    Config_Ptr->IMRD.Dump_Mode = IMRD_SDM_FLAG_SQL;

		    break;
		}

		case IMRD_ARG_LIBRARY_DIR:
		{
		    if( ( arg_cur + 1) < ArgC)
		    {
			strncpy( ( char *) Config_Ptr->SDM.Lib_Path, ArgV[ ++arg_cur], IMRD_PATH_LEN);
		    }
		    else
		    {
			IMRD_LM_LOG_ERROR_1( "Missing argument value [%s] !", IMRG_Arg_Tab[ i].Name_Short);

			return( IMRS_KO);
		    }

		    break;
		}

		case IMRD_ARG_MEDIUM_URL_INPUT:
		{
		    if( ( arg_cur + 1) < ArgC)
		    {
			strncpy( ( char *) Config_Ptr->IMRD.Medium_URL_Input, ArgV[ ++arg_cur], IMRD_MEDIUM_URL_LEN);
		    }
		    else
		    {
			IMRD_LM_LOG_ERROR_1( "Missing argument value [%s] !", IMRG_Arg_Tab[ i].Name_Short);

			return( IMRS_KO);
		    }

		    break;
		}

		case IMRD_ARG_APPL_NAME_INPUT:
		{
		    if( ( arg_cur + 1) < ArgC)
		    {
			strncpy( ( char *) Config_Ptr->IMRD.Appl_Name_Input, ArgV[ ++arg_cur], IMRD_APPL_NAME_LEN);
		    }
		    else
		    {
			IMRD_LM_LOG_ERROR_1( "Missing argument value [%s] !", IMRG_Arg_Tab[i].Name_Short);

			return( IMRS_KO);
		    }

		    break;
		}

		case IMRD_ARG_APPL_ID_INPUT:
		{
		    if( ( arg_cur + 1) < ArgC)
		    {
			strncpy( value_tmp, ArgV[++arg_cur], IMRD_ARG_NAME_LEN);
			
			Config_Ptr->IMRD.Appl_Id_Input = atoi( value_tmp);
			Config_Ptr->IMRD.Appl_Name_Input[0] = '\0';
		    }
		    else
		    {
			IMRD_LM_LOG_ERROR_1( "Missing argument value [%s] !", IMRG_Arg_Tab[i].Name_Short);

			return( IMRS_KO);
		    }

		    break;
		}

		case IMRD_ARG_MEDIUM_URL_OUTPUT:
		{
		    if( ( arg_cur + 1) < ArgC)
		    {
			strncpy( ( char *) Config_Ptr->IMRD.Medium_URL_Output, ArgV[ ++arg_cur], IMRD_MEDIUM_URL_LEN);
		    }
		    else
		    {
			IMRD_LM_LOG_ERROR_1( "Missing argument value [%s] !", IMRG_Arg_Tab[ i].Name_Short);

			return( IMRS_KO);
		    }

		    break;
		}

		case IMRD_ARG_APPL_NAME_OUTPUT:
		{
		    if( ( arg_cur + 1) < ArgC)
		    {
			strncpy( ( char *) Config_Ptr->IMRD.Appl_Name_Output, ArgV[ ++arg_cur], IMRD_APPL_NAME_LEN);
		    }
		    else
		    {
			IMRD_LM_LOG_ERROR_1( "Missing argument value [%s] !", IMRG_Arg_Tab[i].Name_Short);

			return( IMRS_KO);
		    }

		    break;
		}

		case IMRD_ARG_APPL_ID_OUTPUT:
		{
		    if( ( arg_cur + 1) < ArgC)
		    {
			strncpy( value_tmp, ArgV[ ++arg_cur], IMRD_ARG_NAME_LEN);
			
			Config_Ptr->IMRD.Appl_Id_Output = atoi( value_tmp);
			Config_Ptr->IMRD.Appl_Name_Output[0] = '\0';
		    }
		    else
		    {
			IMRD_LM_LOG_ERROR_1( "Missing argument value [%s] !", IMRG_Arg_Tab[i].Name_Short);

			return( IMRS_KO);
		    }

		    break;
		}

		case IMRD_ARG_MEDIUM_URL_TRASH:
		{
		    if( ( arg_cur + 1) < ArgC)
		    {
			strncpy( ( char *) Config_Ptr->IMRD.Medium_URL_Trash, ArgV[ ++arg_cur], IMRD_MEDIUM_URL_LEN);
		    }
		    else
		    {
			IMRD_LM_LOG_ERROR_1( "Missing argument value [%s] !", IMRG_Arg_Tab[ i].Name_Short);

			return( IMRS_KO);
		    }

		    break;
		}

		case IMRD_ARG_MAPPING_FORCED:
		{
		    Config_Ptr->MR.Mapping_Forced = IMRD_TRUE;

		    break;
		}

		case IMRD_ARG_CONTINUE_FORCED:
		{
		    Config_Ptr->MR.Continue_Forced = IMRD_TRUE;

		    break;
		}

		case IMRD_ARG_NO_WAIT:
		{
		    Config_Ptr->MR.Read_Mode = IMRD_READ_MODE_NO_WAIT;

		    break;
		}

		case IMRD_ARG_XML_FORMAT:
		{
		    Config_Ptr->MM.XML_Format = IMRD_TRUE;

		    break;
		}

		case IMRD_ARG_CORE_DUMP:
		{
		    Config_Ptr->IMRD.Core_Dump = IMRD_TRUE;

		    break;
		}
		
		case IMRD_ARG_LOG_ID_INPUT_MSG :
		{
		    if( ( arg_cur + 2) < ArgC)
		    {
			if( atoi( ArgV[ arg_cur + 1]) < 0)
			{
			    IMRD_LM_LOG_ERROR_1( "Invalid Log_Id_Offset: [%s] !", ArgV[ arg_cur + 1]);

			    return( IMRS_KO);
			}

			if( atoi( ArgV[ arg_cur + 2]) <= 0)
			{
			    IMRD_LM_LOG_ERROR_1( "Invalid Log_Id_Size: [%d] !", ArgV[ arg_cur + 2]);

			    return( IMRS_KO);
			}

			if( ( Config_Ptr->MR.Log_Id_Struct_Input.Log_Id_Bit_Ptr =
			      realloc( Config_Ptr->MR.Log_Id_Struct_Input.Log_Id_Bit_Ptr,   
				       ( ( Config_Ptr->MR.Log_Id_Struct_Input.Bit_Nb + 1)) * sizeof( IMRT_Log_Id_Bit))) == NULL)
			{
			    IMRD_LM_LOG_ERROR_1( "Can't allocate Input Log_Id_Bit_Ptr (%d) !", errno);

			    return( IMRS_KO);
			}

			Config_Ptr->MR.Log_Id_Struct_Input.Log_Id_Bit_Ptr[ Config_Ptr->MR.Log_Id_Struct_Input.Bit_Nb].Offset  = atoi( ArgV[ ++arg_cur]);
			Config_Ptr->MR.Log_Id_Struct_Input.Log_Id_Bit_Ptr[ Config_Ptr->MR.Log_Id_Struct_Input.Bit_Nb].Size    = atoi( ArgV[ ++arg_cur]);
			Config_Ptr->MR.Log_Id_Struct_Input.Log_Size                                                          += atoi( ArgV[   arg_cur]);
			Config_Ptr->MR.Log_Id_Struct_Input.Bit_Nb++;
		    }
		    else
		    {
			IMRD_LM_LOG_ERROR_1( "Missing argument value [%s] !", IMRG_Arg_Tab[ i].Name_Short);
			return( IMRS_KO);
		    }

		    break;
		}

		case IMRD_ARG_LOG_ID_OUTPUT_MSG :
		{
		    if( ( arg_cur + 2) < ArgC)
		    {
			if( atoi( ArgV[ arg_cur + 1]) < 0)
			{
			    IMRD_LM_LOG_ERROR_1( "Invalid Log_Id_Offset: [%s] !", ArgV[ arg_cur + 1]);

			    return( IMRS_KO);
			}

			if( atoi( ArgV[ arg_cur + 2]) <= 0)
			{
			    IMRD_LM_LOG_ERROR_1( "Invalid Log_Id_Size: [%d] !", ArgV[ arg_cur + 2]);

			    return( IMRS_KO);
			}

			if( ( Config_Ptr->MR.Log_Id_Struct_Output.Log_Id_Bit_Ptr =
			      realloc( Config_Ptr->MR.Log_Id_Struct_Output.Log_Id_Bit_Ptr,   
				       ( ( Config_Ptr->MR.Log_Id_Struct_Output.Bit_Nb + 1)) * sizeof( IMRT_Log_Id_Bit))) == NULL)
			{
			    IMRD_LM_LOG_ERROR_1( "Can't allocate Output Log_Id_Bit_Ptr (%d) !", errno);

			    return( IMRS_KO);
			}

			Config_Ptr->MR.Log_Id_Struct_Output.Log_Id_Bit_Ptr[ Config_Ptr->MR.Log_Id_Struct_Output.Bit_Nb].Offset  = atoi( ArgV[ ++arg_cur]);
			Config_Ptr->MR.Log_Id_Struct_Output.Log_Id_Bit_Ptr[ Config_Ptr->MR.Log_Id_Struct_Output.Bit_Nb].Size    = atoi( ArgV[ ++arg_cur]);
			Config_Ptr->MR.Log_Id_Struct_Output.Log_Size                                                           += atoi( ArgV[   arg_cur]);
			Config_Ptr->MR.Log_Id_Struct_Output.Bit_Nb++;
		    }
		    else
		    {
			IMRD_LM_LOG_ERROR_1( "Missing argument value [%s] !", IMRG_Arg_Tab[ i].Name_Short);
			return( IMRS_KO);
		    }
		    
		    break;
		}


		default:
		{
		    IMRD_LM_LOG_ERROR_1( "Internal parse error [%d] !", i);
		    
		    return( IMRS_KO);
		}
	    }
	}
	else
	{
	    IMRD_LM_LOG_ERROR_1( "Unreconized argument: (%s) !", ArgV[arg_cur]);

	    return( IMRS_KO);
	}
    }

    return( IMRS_OK);
}





/*----------------------------------------------------------------------------*/
/* Args_Log      			                 	  	      */
/*----------------------------------------------------------------------------*/
/* Config_Ptr:   Configuration structure to dump                              */
/*----------------------------------------------------------------------------*/

IMRT_Status  Args_Log( IMRT_Config  *Config_Ptr)
{
    IMRT_Module_Id  module_id;
    int             idx;
   

    IMRD_LM_LOG_INFO_3( "Lib_Dir: [%s]   Static_Data_Read: [%s]   Static_Data_Write: [%s]",
			Config_Ptr->SDM.Lib_Path,
			( ( Config_Ptr->SDM.Flag == IMRD_SDM_FLAG_XML) ? "XML" : ( ( Config_Ptr->SDM.Flag == IMRD_SDM_FLAG_SQL) ? "SQL" : "Unknown")),
			( ( Config_Ptr->IMRD.Dump_Mode == IMRD_SDM_FLAG_XML) ? "XML" : ( ( Config_Ptr->IMRD.Dump_Mode == IMRD_SDM_FLAG_SQL) ? "SQL" : "Unknown")));
    IMRD_LM_LOG_INFO_3( "Mapping_Forced: [%s]   Continue_Forced: [%s]   Read_Mode: [%s]",
			( ( Config_Ptr->MR.Mapping_Forced == IMRD_TRUE) ? "True" : "False"),
			( ( Config_Ptr->MR.Continue_Forced == IMRD_TRUE) ? "True" : "False"),
			( ( Config_Ptr->MR.Read_Mode == IMRD_READ_MODE_WAIT) ? "Wait" : ( ( Config_Ptr->MR.Read_Mode == IMRD_READ_MODE_NO_WAIT) ? "No Wait" : "Unknown")));
    IMRD_LM_LOG_INFO_2( "XML_Format: [%s]   Core_Dump: [%s]", ( ( Config_Ptr->MM.XML_Format == IMRD_TRUE) ? "True" : "False"), ( ( Config_Ptr->IMRD.Core_Dump == IMRD_TRUE) ? "True" : "False"));
    IMRD_LM_LOG_INFO_1( "Medium_URL_Input:  [%s]", Config_Ptr->IMRD.Medium_URL_Input);
    IMRD_LM_LOG_INFO_1( "Medium_URL_Output: [%s]", Config_Ptr->IMRD.Medium_URL_Output);
    IMRD_LM_LOG_INFO_1( "Medium_URL_Trash:  [%s]", Config_Ptr->IMRD.Medium_URL_Trash);
    IMRD_LM_LOG_INFO_0( "Log Level Tab: Module Id                               Trace   Info   Warning  Error");
    
    for( module_id = 0; module_id < IMRD_MODULE_NB; module_id++)
    {
	IMRD_LM_LOG_INFO_5( "Log Level tab: [%-34s]:   (%3d)   (%3d)   (%3d)   (%3d)",
			    ( ( IMRG_IMRD_Base.Module_Entry_Tab[ module_id].Module_Ptr == NULL) ? "Default" : IMRG_IMRD_Base.Module_Entry_Tab[ module_id].Module_Ptr->Name_Long),
			    Config_Ptr->LM.Level_Tab[ IMRD_LOG_TYPE_ID_TRACE][   module_id],
			    Config_Ptr->LM.Level_Tab[ IMRD_LOG_TYPE_ID_INFO][    module_id],
			    Config_Ptr->LM.Level_Tab[ IMRD_LOG_TYPE_ID_WARNING][ module_id],
			    Config_Ptr->LM.Level_Tab[ IMRD_LOG_TYPE_ID_ERROR][   module_id]);
    }

    for( idx = 0 ; idx < Config_Ptr->MR.Log_Id_Struct_Input.Bit_Nb; idx++)
    {
	IMRD_LM_LOG_INFO_2( "Log_Id_Bit_Input:    Offset: (%5d)   Size: (%5d)",
			    Config_Ptr->MR.Log_Id_Struct_Input.Log_Id_Bit_Ptr[ idx].Offset,
			    Config_Ptr->MR.Log_Id_Struct_Input.Log_Id_Bit_Ptr[ idx].Size);
    }

    for( idx = 0 ; idx < Config_Ptr->MR.Log_Id_Struct_Output.Bit_Nb; idx++)
    {
	IMRD_LM_LOG_INFO_2( "Log_Id_Bit_Output:   Offset: (%5d)   Size: (%5d)",
			    Config_Ptr->MR.Log_Id_Struct_Output.Log_Id_Bit_Ptr[ idx].Offset,
			    Config_Ptr->MR.Log_Id_Struct_Output.Log_Id_Bit_Ptr[ idx].Size);
    }

    return( IMRS_OK);
}





/*----------------------------------------------------------------------------*/
/* Version_Log      			                 	  	      */
/*----------------------------------------------------------------------------*/

IMRT_Status  Version_Log()
{
    IMRT_Module_Id   module_id;


    for( module_id = 0; module_id < IMRD_MODULE_NB; module_id++)
    {
	if( IMRG_IMRD_Base.Module_Entry_Tab[ module_id].Module_Ptr != NULL)
	{
	    IMRD_LM_LOG_INFO_2( "[%-4s]: [%s]", IMRG_IMRD_Base.Module_Entry_Tab[ module_id].Module_Ptr->Name, IMRG_IMRD_Base.Module_Entry_Tab[ module_id].Module_Ptr->Version);
	}
    }

    return( IMRS_OK);
}





/*----------------------------------------------------------------------------*/
/* Version_Print     			                 	  	      */
/*----------------------------------------------------------------------------*/

IMRT_Status  Version_Print()
{
    IMRT_Module_Id   module_id;


    for( module_id = 0; module_id < IMRD_MODULE_NB; module_id++)
    {
	if( IMRG_IMRD_Base.Module_Entry_Tab[ module_id].Module_Ptr != NULL)
	{
	    fprintf( stderr, "[%-4s]: [%s]\n", IMRG_IMRD_Base.Module_Entry_Tab[ module_id].Module_Ptr->Name, IMRG_IMRD_Base.Module_Entry_Tab[ module_id].Module_Ptr->Version);
	}
    }

    return( IMRS_OK);
}





/*----------------------------------------------------------------------------*/
/* Help_Print    			                 	  	      */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

IMRT_Status  Help_Print( void)
{
    int  i;


    for( i = 0; i < IMRD_ARG_NB; i++)
    {
	if( IMRG_Arg_Tab[i].Name_Short[0] != '\0')
	    fprintf( stderr, "%-6s%-25s%s\n", IMRG_Arg_Tab[i].Name_Short, IMRG_Arg_Tab[i].Name_Long, IMRG_Arg_Tab[i].Help);
    }

    return( IMRS_OK);
}






/*----------------------------------------------------------------------------*/
/* Lib_Open                                                                   */
/*----------------------------------------------------------------------------*/
/* Module_Entry_Ptr:   Module to open                                         */
/*----------------------------------------------------------------------------*/

IMRT_Status  Lib_Open( IMRT_Module_Entry  *Module_Entry_Ptr)
{
    IMRT_Status  status;


    if( Module_Entry_Ptr->Module_Ptr->Lib.Open_Ptr != NULL)
    {
	IMRD_LM_LOG_TRACE_1( IMRD_LOG_LEVEL_VERBOSE_0, "Opening %s library...", Module_Entry_Ptr->Module_Ptr->Name_Long);

	if( ( status = Module_Entry_Ptr->Module_Ptr->Lib.Open_Ptr( Module_Entry_Ptr->Config_Ptr)) != IMRS_OK)
	{
	    Module_Entry_Ptr->Status = IMRD_LIB_STATUS_ERROR;
	
	    IMRD_LM_LOG_ERROR_1( "Can't open %s library !", Module_Entry_Ptr->Module_Ptr->Name_Long);
	}
	else
	{
	    Module_Entry_Ptr->Status = IMRD_LIB_STATUS_OPENED;
	
	    IMRD_LM_LOG_TRACE_1( IMRD_LOG_LEVEL_VERBOSE_0, "%s library opened !", Module_Entry_Ptr->Module_Ptr->Name_Long);
	}
    }

    return( status);
}





/*----------------------------------------------------------------------------*/
/* Libs_Open    			                 	  	      */
/*----------------------------------------------------------------------------*/
/* Module_Entry_Tab:   Module tab                                             */
/* Module_Nb:          Number of module                                       */
/*----------------------------------------------------------------------------*/

IMRT_Status  Libs_Open( IMRT_Module_Entry  *Module_Entry_Tab, IMRT_Module_Id  Module_Nb)
{
    IMRT_Status     status;
    IMRT_Module_Id  i;


    for( i = 0, status = IMRS_OK; ( i < Module_Nb) && ( status == IMRS_OK); i++)
    {
	if( ( Module_Entry_Tab[i].Module_Ptr != NULL)
	    && ( Module_Entry_Tab[i].Module_Ptr->Type == IMRD_MODULE_TYPE_LIBRARY) 
	    && ( Module_Entry_Tab[i].Module_Ptr->Lib.Open_Ptr != NULL)
	    && ( Module_Entry_Tab[i].Status == IMRD_LIB_STATUS_CLOSED))
	{
	    status = Lib_Open( &( Module_Entry_Tab[i]));
	}
    }

    return( status);
}





/*----------------------------------------------------------------------------*/
/* Lib_Close                                                                  */
/*----------------------------------------------------------------------------*/
/* Module_Entry_Ptr:   Module to close                                        */
/*----------------------------------------------------------------------------*/

IMRT_Status  Lib_Close( IMRT_Module_Entry  *Module_Entry_Ptr)
{
    IMRT_Status  status;


    IMRD_LM_LOG_TRACE_1( IMRD_LOG_LEVEL_VERBOSE_0, "Closinging %s library...", Module_Entry_Ptr->Module_Ptr->Name_Long);

    if( ( status = Module_Entry_Ptr->Module_Ptr->Lib.Close_Ptr()) != IMRS_OK)
    {
	Module_Entry_Ptr->Status = IMRD_LIB_STATUS_ERROR;
	
	IMRD_LM_LOG_ERROR_1( "Can't close %s library !", Module_Entry_Ptr->Module_Ptr->Name_Long);
    }
    else
    {
	Module_Entry_Ptr->Status = IMRD_LIB_STATUS_CLOSED;

	IMRD_LM_LOG_TRACE_1( IMRD_LOG_LEVEL_VERBOSE_0, "%s library closed !", Module_Entry_Ptr->Module_Ptr->Name_Long);
    }

    return( status);
}





/*----------------------------------------------------------------------------*/
/* Libs_Close    			                 	  	      */
/*----------------------------------------------------------------------------*/
/* Module_Entry_Tab:   Module tab                                             */
/* Module_Nb:          Number of module                                       */
/*----------------------------------------------------------------------------*/

IMRT_Status  Libs_Close( IMRT_Module_Entry  *Module_Entry_Tab, IMRT_Module_Id  Module_Nb)
{
    IMRT_Status     status;
    IMRT_Module_Id  i;


    for( i = ( Module_Nb - 1), status = IMRS_OK; ( i > -1) && ( status == IMRS_OK); i--)
    {
	if( ( Module_Entry_Tab[i].Module_Ptr != NULL) && ( Module_Entry_Tab[i].Module_Ptr->Lib.Open_Ptr != NULL))
	{
	    if( Module_Entry_Tab[i].Status == IMRD_LIB_STATUS_OPENED)
	    {
		status = Lib_Close( &( Module_Entry_Tab[i]));
	    }
	    else
	    {
		IMRD_LM_LOG_TRACE_1( IMRD_LOG_LEVEL_VERBOSE_0, "%s library not opened !", Module_Entry_Tab[i].Module_Ptr->Name_Long);
	    }
	}
    }

    return( status);
}





/*------------------------------------------------------------------------------*/
/* Signal_Trap                                                                  */
/*------------------------------------------------------------------------------*/

void  Signal_Trap( int  Sig_Num)
{
	
    fprintf( stderr, "Signal: (%d) !\n", Sig_Num);
    
    switch( Sig_Num)
    {
	case SIGBUS:
	case SIGSEGV:
	{
	    if( ( IMRG_Base.Sig_Trapped == SIGBUS) || ( IMRG_Base.Sig_Trapped == SIGSEGV))
	    {
		fprintf( stderr, "Imediate exit !!!\n");
		exit( -1);
	    }
	    else
	    {
		fprintf( stderr, "Long Jumping...\n");
		IMRG_Base.Sig_Trapped = Sig_Num;
		
		siglongjmp( IMRG_Base.SigLongJmp_Env, 1);
	    }
	    
	    break;
	}
	
	case SIGINT:
	case SIGQUIT:
	case SIGTERM:
	default:
	{
	    if( IMRG_Base.Sig_Trapped != 0)
	    {
		fprintf( stderr, "Signal rejected: (%d) !\n", Sig_Num);
	    }
	    else
	    {
		IMRG_Base.Sig_Trapped = Sig_Num;
	    }
	    
	    break;
	}
    }
}





/*----------------------------------------------------------------------------*/
/* Signal_Init    			                 	  	      */
/*----------------------------------------------------------------------------*/

IMRT_Status  Signal_Init()
{
    int               rc;
    struct sigaction  act = {0};

    act.sa_handler = Signal_Trap;


    if( ( rc = sigaction( SIGINT, &act, &( IMRG_IMRD_Base.OAct_Int))) != 0)
    {
	IMRD_LM_LOG_ERROR_2( "Can't trap signal (%d)   errno: (%d) !", SIGINT, errno);

	return( IMRS_KO);
    }

    if( ( rc = sigaction( SIGQUIT, &act, &( IMRG_IMRD_Base.OAct_Quit))) != 0)
    {
	IMRD_LM_LOG_ERROR_2( "Can't trap signal (%d)   errno: (%d) !", SIGQUIT, errno);

	return( IMRS_KO);
    }

    if( ( rc = sigaction( SIGTERM, &act, &( IMRG_IMRD_Base.OAct_Int))) != 0)
    {
	IMRD_LM_LOG_ERROR_2( "Can't trap signal (%d)   errno: (%d) !", SIGTERM, errno);

	return( IMRS_KO);
    }

    if( IMRG_IMRD_Base.Config.IMRD.Core_Dump == IMRD_FALSE)
    {
	if( ( rc = sigaction( SIGBUS, &act, &( IMRG_IMRD_Base.OAct_Bus))) != 0)
	{
	    IMRD_LM_LOG_ERROR_2( "Can't trap signal (%d)   errno: (%d) !", SIGBUS, errno);
	    
	    return( IMRS_KO);
	}

	if( ( rc = sigaction( SIGSEGV, &act, &( IMRG_IMRD_Base.OAct_SegV))) != 0)
	{
	    IMRD_LM_LOG_ERROR_2( "Can't trap signal (%d)   errno: (%d) !", SIGSEGV, errno);
	    
	    return( IMRS_KO);
	}
    }

    return( IMRS_OK);
}





/*----------------------------------------------------------------------------*/
/* main         			                 	  	      */
/*----------------------------------------------------------------------------*/
/* ArgC:  Argument number                                                     */
/* ArgV:  Argument tab                                                        */
/*----------------------------------------------------------------------------*/

int  main( int  ArgC, char  **ArgV)
{
    IMRT_Status      status;
    NDT_Status       nd_status;
    int              exit_status;
    IMRT_Config     *config_ptr = &( IMRG_IMRD_Base.Config);
    IMRT_Module_Id   module_id;


    if( sigsetjmp( IMRG_Base.SigLongJmp_Env, 1) != 0 )
    {
	IMRD_LM_LOG_TRACE_1( IMRD_LOG_LEVEL_VERBOSE_0, "Exiting on signal (%d)...", IMRG_Base.Sig_Trapped);

	ND_Library_StdErr_Set( NULL);
	exit_status = 0;
    }
    else
    {
	if( ( status = Config_Init( config_ptr, ArgC, ArgV)) != IMRS_OK)
	{
	    IMRD_LM_LOG_ERROR_0( "Can't init config !");

	    exit( -1);
	}

	if( ( status = Args_Parse( config_ptr, ArgC, ArgV)) != IMRS_OK)
	{
	    IMRD_LM_LOG_ERROR_0( "Can't parse args !");

	    exit( -1);
	}

	if( ( status = Signal_Init()) != IMRS_OK)
	{
	    IMRD_LM_LOG_ERROR_0( "Can't init signals !");

	    exit( -1);
	}

	if( config_ptr->IMRD.Version_Mode == IMRD_TRUE)
	{
	    Version_Print();
	    exit( 0);
	}

	if( config_ptr->IMRD.Help_Mode == IMRD_TRUE)
	{
	    Help_Print();
	    exit( 0);
	}

	if( ( status = Lib_Open( &( IMRG_IMRD_Base.Module_Entry_Tab[1]))) != IMRS_OK)
	{
	    exit( -1);
	}

	if( ( status = Version_Log()) != IMRS_OK)
	{
	    IMRD_LM_LOG_ERROR_0( "Can't log versions !");

	    exit( -1);
	}

	if( ( nd_status = ND_Library_Open( NDD_TRUE)) != NDS_OK)
	{
	    IMRD_LM_LOG_ERROR_1( "Can't open node library (%d) !", nd_status);

	    exit( -1);
	}

	IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Node library opened !");
    
	if( ( status = Libs_Open( IMRG_IMRD_Base.Module_Entry_Tab, IMRD_MODULE_NB)) != IMRS_OK)
	{
	    exit_status = -1;
	}
	else
	{
	    if( ( status = Args_Log( config_ptr)) != IMRS_OK)
	    {
		exit_status = -1;
	    }
	    else
	    {
		switch( config_ptr->IMRD.Dump_Mode)
		{
		    case IMRD_SDM_FLAG_XML:
		    {
			if( IMR_SDM_XML_Dump() != IMRS_OK)
			{
			    exit_status = -1;
			}
			else
			{
			    exit_status = 0;
			}

			break;
		    }

		    case IMRD_SDM_FLAG_SQL:
		    {
			if( IMR_SDM_SQL_Dump() != IMRS_OK)
			{
			    exit_status = -1;
			}
			else
			{
			    exit_status = 0;
			}

			break;
		    }

		    default:
		    {
			if( ( config_ptr->IMRD.Appl_Id_Input == IMRD_APPL_ID_UNDEFINEDED) && ( IMR_SDM_Appl_Id_Get( &( config_ptr->IMRD.Appl_Id_Input), config_ptr->IMRD.Appl_Name_Input) != IMRS_OK))
			{
			    IMRD_LM_LOG_ERROR_1( "Can't get Appl_Id from [%s] !", config_ptr->IMRD.Appl_Name_Input);
			    
			    exit_status = -1;
			}
			else
			{
			    if( ( config_ptr->IMRD.Appl_Id_Output == IMRD_APPL_ID_UNDEFINEDED) && ( IMR_SDM_Appl_Id_Get( &( config_ptr->IMRD.Appl_Id_Output), config_ptr->IMRD.Appl_Name_Output) != IMRS_OK))
			    {
				IMRD_LM_LOG_ERROR_1( "Can't get Appl_Id from [%s] !", config_ptr->IMRD.Appl_Name_Output);

				exit_status = -1;
			    }
			    else
			    {
				IMRD_LM_LOG_INFO_2( "Appl_Name_Input:  [%s]   Appl_Id_Input:  (%d)", config_ptr->IMRD.Appl_Name_Input, config_ptr->IMRD.Appl_Id_Input);
				IMRD_LM_LOG_INFO_2( "Appl_Name_Output: [%s]   Appl_Id_Output: (%d)", config_ptr->IMRD.Appl_Name_Output, config_ptr->IMRD.Appl_Id_Output);

				if( ( status = IMR_MR_Proceed( config_ptr->IMRD.Medium_URL_Output, config_ptr->IMRD.Appl_Id_Output, config_ptr->IMRD.Medium_URL_Trash, config_ptr->IMRD.Medium_URL_Input, config_ptr->IMRD.Appl_Id_Input)) != IMRS_OK)
				{
				    exit_status = -1;
				}
				else
				{
				    exit_status = 0;
				}
			    }
			}

			break;
		    }
		}
	    }
	}
    }
    
    if( ( status = Libs_Close( IMRG_IMRD_Base.Module_Entry_Tab, IMRD_MODULE_NB)) != IMRS_OK)
    {
	exit_status = -1;
    }

    if( ( nd_status = ND_Library_Close()) != NDS_OK)
    {
	IMRD_LM_LOG_ERROR_1( "Can't close node library (%d)", nd_status);
	    
	exit_status = -1;
    }
    else
    {
	IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Node library closed !");
    }

    if( ( status = Config_Deinit( config_ptr)) != IMRS_OK)
    {
	IMRD_LM_LOG_ERROR_0( "Can't deinit config !");
	
	exit_status = -1;
    }

    exit( exit_status);
}
