/*----------------------------------------------------------------------------*/
/* File: imrd.h 				         		      */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#ifndef _IMRD_H_
#define _IMRD_H_



#include <imr.h>







/*----------------------------------------------------------------------------*/
/* Local Return Status                                                        */
/*----------------------------------------------------------------------------*/





/*----------------------------------------------------------------------------*/
/* Module Definitions                                                         */
/*----------------------------------------------------------------------------*/

#define IMRD_IMRD_MODULE_ID                ( IMRT_Module_Id) 6
#define IMRD_IMRD_MODULE_NAME            ( IMRT_Module_Name) "imrd"
#define IMRD_IMRD_MODULE_NAME_LONG  ( IMRT_Module_Name_Long) "Interlinking Message Router Daemon"
#define IMRD_IMRD_MODULE_VERSION      ( IMRT_Module_Version) "@(#) VERSIONIMR: $Label:  $   $Date: 2008/11/12 02:25:22 $   $Workfile:   imrd.h  $"

#define IMRD_MODULE_PTR                                      &IMRG_IMRD_Module





/*----------------------------------------------------------------------------*/
/* Local Definitions                                                          */
/*----------------------------------------------------------------------------*/

typedef struct IMRT_Module_Entry
{
	IMRT_Module      *Module_Ptr;
	IMRT_Lib_Status   Status;
	IMRT_Lib_Config  *Config_Ptr;
}  IMRT_Module_Entry;



typedef struct IMRT_IMRD_Config
{
	IMRT_Medium_URL  Medium_URL_Input, Medium_URL_Output, Medium_URL_Trash;
	IMRT_Appl_Name   Appl_Name_Input,  Appl_Name_Output;
	IMRT_Appl_Id     Appl_Id_Input,    Appl_Id_Output;
	IMRT_Boolean     Version_Mode,     Help_Mode;
	IMRT_SDM_Flag    Dump_Mode;
	IMRT_Boolean     Core_Dump;
}  IMRT_IMRD_Config;



typedef struct IMRT_Config
{
	IMRT_LM_Config    LM;
	IMRT_SDM_Config   SDM;
	IMRT_IOM_Config   IOM;
	IMRT_MM_Config    MM;
	IMRT_MR_Config    MR;
	IMRT_IMRD_Config  IMRD;
}  IMRT_Config;



#define IMRD_MODULE_NB  ( IMRT_Module_Id) 7



typedef struct IMRT_IMRD_Base
{
//	IMRT_Module_Nb     Module_Nb;
	IMRT_Module_Entry  Module_Entry_Tab[ IMRD_MODULE_NB];
	IMRT_Config        Config;
	int                Sig_Trapped;
	sigjmp_buf         SigLongJmp_Env;
	struct sigaction   OAct_Int, OAct_Quit, OAct_Term, OAct_Bus, OAct_SegV;

}  IMRT_IMRD_Base;





/*----------------------------------------------------------------------------*/
/* Global Variables                                                           */
/*----------------------------------------------------------------------------*/

IMRT_Module  IMRG_IMRD_Module = 
{ 
    IMRD_IMRD_MODULE_ID,
    IMRD_IMRD_MODULE_NAME,
    IMRD_IMRD_MODULE_NAME_LONG,
    IMRD_IMRD_MODULE_VERSION,
    IMRD_MODULE_TYPE_OBJECT,
    { 
	IMRD_LIB_STATUS_UNKNOWN,
	NULL,
	NULL
    }
};



IMRT_IMRD_Base  IMRG_IMRD_Base = 
{
//    IMRD_LIB_NB,
    {
	{ NULL,              IMRD_LIB_STATUS_UNKNOWN,                      NULL},
	{ &IMRG_LM_Module,   IMRD_LIB_STATUS_CLOSED,  (IMRT_Lib_Config  *) &( IMRG_IMRD_Base.Config.LM)},
	{ &IMRG_SDM_Module,  IMRD_LIB_STATUS_CLOSED,  (IMRT_Lib_Config  *) &( IMRG_IMRD_Base.Config.SDM)},
	{ &IMRG_IOM_Module,  IMRD_LIB_STATUS_CLOSED,  (IMRT_Lib_Config  *) &( IMRG_IMRD_Base.Config.IOM)},
	{ &IMRG_MM_Module,   IMRD_LIB_STATUS_CLOSED,  (IMRT_Lib_Config  *) &( IMRG_IMRD_Base.Config.MM)},
	{ &IMRG_MR_Module,   IMRD_LIB_STATUS_CLOSED,  (IMRT_Lib_Config  *) &( IMRG_IMRD_Base.Config.MR)},
	{ &IMRG_IMRD_Module, IMRD_LIB_STATUS_UNKNOWN, (IMRT_Lib_Config  *) &( IMRG_IMRD_Base.Config.IMRD)}
    },
    {
	{ "", 0, { 0 } },
	{ IMRD_SDM_FLAG_UNKNOWN, ""},
	{ NULL},
	{ IMRD_FALSE},
	{ IMRD_FALSE, IMRD_READ_MODE_WAIT},
	{ "", "", "", "", "", 0, 0, IMRD_FALSE, IMRD_FALSE, IMRD_SDM_FLAG_UNKNOWN, IMRD_FALSE}
    }
};



IMRT_Base  IMRG_Base = 
{
    0,
    {0}
};



#define IMRD_ARG_NAME_LEN   ( short) 63
#define IMRD_ARG_NAME_SIZE  ( short) (IMRD_ARG_NAME_LEN + 1)

typedef char  IMRT_Arg_Name[ IMRD_ARG_NAME_SIZE];



#define IMRD_ARG_HELP_LEN   ( short) 128
#define IMRD_ARG_HELP_SIZE  ( short) 128

typedef char  IMRT_Arg_Help[ IMRD_ARG_HELP_SIZE];



typedef short  IMRT_ARG_ID;

#define IMRD_ARG_NB  ( IMRT_ARG_ID) 25

#define IMRD_ARG_UNKNOWN                ( IMRT_ARG_ID)  0
#define IMRD_ARG_VERSION                ( IMRT_ARG_ID)  1
#define IMRD_ARG_HELP                   ( IMRT_ARG_ID)  2
#define IMRD_ARG_VERBOSE_LEVEL_DEFAULT  ( IMRT_ARG_ID)  3
#define IMRD_ARG_VERBOSE_LEVEL_MODULE   ( IMRT_ARG_ID)  4
#define IMRD_ARG_LOG_FILE               ( IMRT_ARG_ID)  5
#define IMRD_ARG_SD_READ_XML            ( IMRT_ARG_ID)  6
#define IMRD_ARG_SD_READ_SQL            ( IMRT_ARG_ID)  7
#define IMRD_ARG_SD_WRITE_XML           ( IMRT_ARG_ID)  8
#define IMRD_ARG_SD_WRITE_SQL           ( IMRT_ARG_ID)  9
#define IMRD_ARG_LIBRARY_DIR            ( IMRT_ARG_ID) 10
#define IMRD_ARG_MEDIUM_URL_INPUT       ( IMRT_ARG_ID) 11
#define IMRD_ARG_MEDIUM_URL_OUTPUT      ( IMRT_ARG_ID) 12
#define IMRD_ARG_MEDIUM_URL_TRASH       ( IMRT_ARG_ID) 13
#define IMRD_ARG_APPL_NAME_INPUT        ( IMRT_ARG_ID) 14
#define IMRD_ARG_APPL_NAME_OUTPUT       ( IMRT_ARG_ID) 15
#define IMRD_ARG_APPL_ID_INPUT          ( IMRT_ARG_ID) 16
#define IMRD_ARG_APPL_ID_OUTPUT         ( IMRT_ARG_ID) 17
#define IMRD_ARG_MAPPING_FORCED         ( IMRT_ARG_ID) 18
#define IMRD_ARG_CONTINUE_FORCED        ( IMRT_ARG_ID) 19
#define IMRD_ARG_NO_WAIT                ( IMRT_ARG_ID) 20
#define IMRD_ARG_XML_FORMAT             ( IMRT_ARG_ID) 21
#define IMRD_ARG_CORE_DUMP              ( IMRT_ARG_ID) 22
#define IMRD_ARG_LOG_ID_INPUT_MSG       ( IMRT_ARG_ID) 23
#define IMRD_ARG_LOG_ID_OUTPUT_MSG      ( IMRT_ARG_ID) 24


typedef struct IMRT_Arg
{
	IMRT_Arg_Name   Name_Short;
	IMRT_Arg_Name   Name_Long;
	IMRT_Arg_Help   Help;
}  IMRT_Arg;



IMRT_Arg  IMRG_Arg_Tab[ IMRD_ARG_NB] =
{
    { "",      "",                        "Unknown"},
    { "-V",    "--version",               "Print version"},
    { "-h",    "--help",                  "Print this help"},
    { "-vld",  "--verbose_level_default", "Set the genral verbose level"},
    { "-vlm",  "--verbose_level_module",  "Set the module verbose level"},
    { "-lf",   "--log_file",              "Set log file"},
#ifdef ORACLE_SUPPORT
    { "-sdrx", "--static_data_read_xml",  "Read  static data from xml repository"},
    { "-sdrs", "--static_data_read_sql",  "Read  static data from sql repository"},
    { "-sdwx", "--static_data_write_xml", "Write static data to   xml repository"},
    { "-sdws", "--static_data_write_sql", "Write static data to   sql repository"},
#else
    { "",      "",                        "None"},
    { "",      "",                        "None"},
    { "",      "",                        "None"},
    { "",      "",                        "None"},
#endif /* ORACLE_SUPPORT */
    { "-ld",   "--library_dir",           "Set imr library dir"},
    { "-mui",  "--medium_url_input",      "Set input  medium url"},
    { "-muo",  "--medium_url_output",     "Set output medium url"},
    { "-mut",  "--medium_url_trash",      "Set trash  medium url"},
    { "-ani",  "--appl_name_input",       "Set input  appl name"},
    { "-ano",  "--appl_name_output",      "Set output appl name"},
    { "-aii",  "--appl_id_input",         "Set input  appl id"},
    { "-aio",  "--appl_id_output",        "Set output appl id"},
    { "-mf",   "--mapping_forced",        "Output bad mapped buffer"},
    { "-cf",   "--continue_forced",       "Continue on mapping error"},
    { "-nw",   "--no_wait",               "Don't wait if nothing to read"},
    { "-xf",   "--xml_format",            "Format xml output buffer"},
    { "-cd",   "--core_dump",             "Dump core on SIGSEGV and SIGBUS"},
    { "-iim",  "--id_input_message",      "Log Id input  message"},
    { "-iom",  "--id_output_message",     "Log Id output message"}
};





/*----------------------------------------------------------------------------*/
/* Prototypes                                                                 */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* Config_Init         			                 	  	      */
/*----------------------------------------------------------------------------*/
/* Config_Ptr:   Config structure to initialize                               */
/* ArgC:         Argument number                                              */
/* ArgV:         Argument tab                                                 */
/*----------------------------------------------------------------------------*/

IMRT_Status  Config_Init( IMRT_Config  *, int, char  **);



/*----------------------------------------------------------------------------*/
/* Config_Deinit      			                 	  	      */
/*----------------------------------------------------------------------------*/
/* Config_Ptr:   Config structure to deinitialize                             */
/*----------------------------------------------------------------------------*/

IMRT_Status  Config_Deinit( IMRT_Config  *);



/*----------------------------------------------------------------------------*/
/* Args_Parse         			                 	  	      */
/*----------------------------------------------------------------------------*/
/* Config_Ptr:   Configuration structure to update                            */
/* ArgC:         Argument number                                              */
/* ArgV:         Argument tab                                                 */
/*----------------------------------------------------------------------------*/

IMRT_Status  Args_Parse( IMRT_Config  *, int, char  **);



/*----------------------------------------------------------------------------*/
/* Args_Log      			                 	  	      */
/*----------------------------------------------------------------------------*/
/* Config_Ptr:   Configuration structure to dump                              */
/*----------------------------------------------------------------------------*/

IMRT_Status  Args_Log( IMRT_Config  *);



/*----------------------------------------------------------------------------*/
/* Version_Log      			                 	  	      */
/*----------------------------------------------------------------------------*/

IMRT_Status  Version_Log( void);



/*----------------------------------------------------------------------------*/
/* Version_Print      			                 	  	      */
/*----------------------------------------------------------------------------*/

IMRT_Status  Version_Print( void);



/*----------------------------------------------------------------------------*/
/* Help_Print    			                 	  	      */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

IMRT_Status  Help_Print( void);



/*----------------------------------------------------------------------------*/
/* Lib_Open                                                                   */
/*----------------------------------------------------------------------------*/
/* Module_Entry_Ptr:   Module to open                                         */
/*----------------------------------------------------------------------------*/

IMRT_Status  Lib_Open( IMRT_Module_Entry  *);



/*----------------------------------------------------------------------------*/
/* Libs_Open    			                 	  	      */
/*----------------------------------------------------------------------------*/
/* Module_Entry_Tab:   Module tab                                             */
/* Module_Nb:          Number of module                                       */
/*----------------------------------------------------------------------------*/

IMRT_Status  Libs_Open( IMRT_Module_Entry  *, IMRT_Module_Id);



/*----------------------------------------------------------------------------*/
/* Lib_Close                                                                  */
/*----------------------------------------------------------------------------*/
/* Module_Entry_Ptr:   Module to close                                        */
/*----------------------------------------------------------------------------*/

IMRT_Status  Lib_Close( IMRT_Module_Entry  *);



/*----------------------------------------------------------------------------*/
/* Libs_Close    			                 	  	      */
/*----------------------------------------------------------------------------*/
/* Module_Entry_Tab:   Module tab                                             */
/* Module_Nb:          Number of module                                       */
/*----------------------------------------------------------------------------*/

IMRT_Status  Libs_Close( IMRT_Module_Entry  *, IMRT_Module_Id);



/*------------------------------------------------------------------------------*/
/* Signal_Trap                                                                  */
/*------------------------------------------------------------------------------*/

void  Signal_Trap( int);



/*----------------------------------------------------------------------------*/
/* Signal_Init    			                 	  	      */
/*----------------------------------------------------------------------------*/

IMRT_Status  Signal_Init();



/*----------------------------------------------------------------------------*/
/* main                                                                       */
/*----------------------------------------------------------------------------*/
/* ArgC:           argument number                                            */
/* ArgV:           Argument tab                                               */
/*----------------------------------------------------------------------------*/

int main( int, char  **);





/*----------------------------------------------------------------------------*/

#endif
