/*----------------------------------------------------------------------------*/
/* File: imr.h                                                                */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#ifndef _IMR_H_
#define _IMR_H_



#include <errno.h>
#include <stdlib.h>
#include <dlfcn.h>
#include <libgen.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <signal.h>
#include <setjmp.h>

#include <node.h>







#define TRACE 1



/*----------------------------------------------------------------------------*/
/* Global definitions                                                         */
/*----------------------------------------------------------------------------*/

typedef short  IMRT_Boolean;

#define IMRD_FALSE  ( IMRT_Boolean) 0
#define IMRD_TRUE   ( IMRT_Boolean) 1

#define IMRD_NO     'n'
#define IMRD_YES    'y'

#define IMRD_MAX(A,B)  (((A) < (B)) ? (B) : (A))
#define IMRD_MIN(A,B)  (((A) > (B)) ? (B) : (A))





/*----------------------------------------------------------------------------*/
/* Status definition                                                          */
/*----------------------------------------------------------------------------*/

typedef short  IMRT_Status;

#define IMRS_OK          ( IMRT_Status)  0

#define IMRS_KO          ( IMRT_Status)  1
#define IMRS_NO_IDENT    ( IMRT_Status) -2
#define IMRS_BAD_FORMAT	 ( IMRT_Status) -3
#define IMRS_ROLLBACK    ( IMRT_Status) -4

#define IMRS_NO_DATA     ( IMRT_Status)  3
#define IMRS_SIGNAL      ( IMRT_Status)  4



/*----------------------------------------------------------------------------*/
/* Module and Lib definitions                                                 */
/*----------------------------------------------------------------------------*/

typedef short  IMRT_Module_Id;

#define IMRD_MODULE_ID_UNKNOWN  (IMRT_Module_Id) 0

#define IMRD_MODULE_NB_MAX      (IMRT_Module_Id) 10



typedef char  *IMRT_Module_Name;


typedef char  *IMRT_Module_Name_Long;



typedef char  *IMRT_Module_Version;



typedef	short  IMRT_Module_Type;

#define IMRD_MODULE_TYPE_UNKNOWN  ( IMRT_Module_Type) 0
#define IMRD_MODULE_TYPE_OBJECT   ( IMRT_Module_Type) 1
#define IMRD_MODULE_TYPE_LIBRARY  ( IMRT_Module_Type) 2



typedef	short  IMRT_Lib_Status;

#define IMRD_LIB_STATUS_UNKNOWN  ( IMRT_Lib_Status) 0
#define IMRD_LIB_STATUS_CLOSED   ( IMRT_Lib_Status) 1
#define IMRD_LIB_STATUS_OPENED   ( IMRT_Lib_Status) 2
#define IMRD_LIB_STATUS_ERROR    ( IMRT_Lib_Status) 3



typedef struct IMRT_Lib_Config
{
	char  *dummy;
}  IMRT_Lib_Config;



typedef	IMRT_Status  ( IMRT_Lib_Open)(  IMRT_Lib_Config  *);
typedef IMRT_Status  ( IMRT_Lib_Close)( void);



typedef struct IMRT_Lib
{
	IMRT_Lib_Status   Status;
	IMRT_Lib_Open    *Open_Ptr;
	IMRT_Lib_Close   *Close_Ptr;
}  IMRT_Lib;



typedef struct IMRT_Module
{
	IMRT_Module_Id          Id;
	IMRT_Module_Name        Name;
	IMRT_Module_Name_Long   Name_Long;
	IMRT_Module_Version     Version;
	IMRT_Module_Type        Type;
	IMRT_Lib                Lib;
}  IMRT_Module;





/*----------------------------------------------------------------------------*/
/* IMRT_Path definition                                                       */
/*----------------------------------------------------------------------------*/

#define IMRD_PATH_LEN   ( short) 255
#define IMRD_PATH_SIZE  ( short) ( IMRD_PATH_LEN + 1)

typedef  char  IMRT_Path[ IMRD_PATH_SIZE];

typedef short  IMRT_Lib_Flag;
#define IRMD_LIB_FLAG_SDM_XML (IMRT_Lib_Flag)0
#define IRMD_LIB_FLAG_SDM_SQL (IMRT_Lib_Flag)1





/*----------------------------------------------------------------------------*/
/* IMRT_Tag definition                                                        */
/*----------------------------------------------------------------------------*/

#define IMRD_TAG_LEN         ( short) 100
#define IMRD_TAG_SIZE        ( short) ( IMRD_TAG_LEN  + 1)

typedef char  IMRT_Tag[ IMRD_TAG_SIZE];





/*----------------------------------------------------------------------------*/
/* IMRT_Cnv_Id definition                                                     */
/*----------------------------------------------------------------------------*/

typedef short  IMRT_Cnv_Id;

#define IMRD_CNV_ID_UNKNOWN  ( IMRT_Cnv_Id) 0





/*----------------------------------------------------------------------------*/
/* IMRT_Cnv_Name definition                                                   */
/*----------------------------------------------------------------------------*/

#define IMRD_CNV_NAME_LEN   ( short) 65
#define IMRD_CNV_NAME_SIZE  ( short) ( IMRD_CNV_NAME_LEN + 1)

typedef char  IMRT_Cnv_Name[ IMRD_CNV_NAME_SIZE];

#define IMRD_CNV_NAME_UNKNOWN   "Unknown"





/*----------------------------------------------------------------------------*/
/* IMRT_Cnv_Attr definition                                                   */
/*----------------------------------------------------------------------------*/

#define IMRD_CNV_ATTR_LEN   ( short) 20
#define IMRD_CNV_ATTR_SIZE  ( short) ( IMRD_CNV_ATTR_LEN + 1)

typedef char  IMRT_Cnv_Attr[ IMRD_CNV_ATTR_SIZE];

#define IMRD_CNV_ATTR_UNKNOWN   "UNKNOWN"





/*----------------------------------------------------------------------------*/
/* IMRT_Cnv definition                                                        */
/*----------------------------------------------------------------------------*/

typedef struct IMRT_Cnv
{
	IMRT_Cnv_Id    Id;
	IMRT_Cnv_Name  Name;
} IMRT_Cnv;




/*----------------------------------------------------------------------------*/
/* IMRT_Field definition                                                      */
/*----------------------------------------------------------------------------*/

#define IMRD_FIXED_SIZE_UNKNOWN  ( size_t) -1



#define IMRD_NAME_LEN         ( short) 255
#define IMRD_NAME_SIZE        ( short) ( IMRD_NAME_LEN      + 1)

typedef char  IMRT_Field_Name[     IMRD_NAME_SIZE];



#define IMRD_VALUE_ID_LEN     ( short) 20
#define IMRD_VALUE_ID_SIZE    ( short) ( IMRD_VALUE_ID_LEN  + 1)

typedef char  IMRT_Field_Value_Id[ IMRD_VALUE_ID_SIZE];


#define IMRD_EXIST_LEN     ( short) 1
#define IMRD_EXIST_SIZE    ( short) ( IMRD_EXIST_LEN  + 1)

typedef char  IMRT_Field_Exist[ IMRD_EXIST_SIZE];


#define IMRD_VALUE_LEN       ( short) 150
#define IMRD_VALUE_SIZE      ( short) ( IMRD_VALUE_LEN  + 1)

typedef char  IMRT_Value[          IMRD_VALUE_SIZE];



typedef struct IMRT_Field_Fixed
{
	size_t         Offset;
	size_t         Size;
	IMRT_Value     Def_Val;
	IMRT_Cnv       *Cnv_Ptr;
	IMRT_Cnv_Attr  Cnv_Attr;
} IMRT_Field_Fixed;



#define IMRD_FML_TAG_LEN      ( short) 50
#define IMRD_FML_TAG_SIZE     ( short) ( IMRD_FML_TAG_LEN   + 1)

typedef char  IMRT_FML_Tag[  IMRD_FML_TAG_SIZE];


#define IMRD_FML_TAG_UNKNOWN  ""





typedef struct IMRT_Field_FML
{
	IMRT_FML_Tag   Tag;
	IMRT_Value     Def_Val;
	IMRT_Cnv      *Cnv_Ptr;
	IMRT_Cnv_Attr  Cnv_Attr;
} IMRT_Field_FML;



#define IMRD_XML_TAG_LEN      ( short) 150
#define IMRD_XML_TAG_SIZE     ( short) ( IMRD_XML_TAG_LEN   + 1)

typedef char  IMRT_XML_Tag[  IMRD_XML_TAG_SIZE];

#define IMRD_XML_TAG_UNKNOWN  ""



typedef struct IMRT_Field_XML
{
	IMRT_XML_Tag   Tag;
	IMRT_Value     Def_Val;
	IMRT_Cnv      *Cnv_Ptr;
	IMRT_Cnv_Attr  Cnv_Attr;
} IMRT_Field_XML;



#define IMRD_TAGGED_TAG_LEN   ( short) 20
#define IMRD_TAGGED_TAG_SIZE  ( short) ( IMRD_TAGGED_TAG_LEN + 1)

typedef char  IMRT_Tagged_Tag[   IMRD_TAGGED_TAG_SIZE];

#define IMRD_TAGGED_TAG_UNKNOWN  ""




typedef struct IMRT_Field_Tagged
{
	IMRT_Tagged_Tag   Tag;
	IMRT_Value        Def_Val;
	IMRT_Cnv         *Cnv_Ptr;
	IMRT_Cnv_Attr     Cnv_Attr;
} IMRT_Field_Tagged;



#define IMRD_INTERNAL_TAG_LEN     ( short) 20
#define IMRD_INTERNAL_TAG_SIZE    ( short) ( IMRD_INTERNAL_TAG_LEN  + 1)

typedef char  IMRT_Internal_Tag[ IMRD_INTERNAL_TAG_SIZE];

#define IMRD_INTERNAL_TAG_UNKNOWN  ""




typedef struct IMRT_Field_Internal
{
	IMRT_Internal_Tag  Tag;
	IMRT_Value         Def_Val;
	IMRT_Cnv          *Cnv_Ptr;
	IMRT_Cnv_Attr      Cnv_Attr;
} IMRT_Field_Internal;




typedef short  IMRT_Delimited_Id;

#define IMRD_DELIMITED_ID_UNKNOWN  ( IMRT_Delimited_Id) -1



typedef struct IMRT_Field_Delimited
{
	IMRT_Delimited_Id Id;
	IMRT_Value        Def_Val;
	IMRT_Cnv         *Cnv_Ptr;
	IMRT_Cnv_Attr     Cnv_Attr;
} IMRT_Field_Delimited;



/*----------------------------------------------------------------------------*/
/* IMRT_Field_Id definition                                                   */
/*----------------------------------------------------------------------------*/

typedef int  IMRT_Field_Id;

#define IMRD_FIELD_ID_UNKNOWN  ( IMRT_Field_Id) 0



typedef struct IMRT_Field
{
	IMRT_Field_Id        Id;
	IMRT_Field_Name      Name;
	IMRT_Field_Value_Id  Value_Id;
	IMRT_Boolean         Exist;
	IMRT_Field_Fixed     Fixed;
	IMRT_Field_FML       FML;
	IMRT_Field_XML       XML;
	IMRT_Field_Tagged    Tagged;
	IMRT_Field_Internal  Internal;
	IMRT_Field_Delimited Delimited;
}  IMRT_Field;





/*----------------------------------------------------------------------------*/
/* IMRT_FieldMap_Id definition                                                */
/*----------------------------------------------------------------------------*/

typedef int  IMRT_FieldMap_Id;

#define IMRD_FIELDMAP_ID_UNKNOWN  ( IMRT_FieldMap_Id) 0



/*----------------------------------------------------------------------------*/
/* IMRT_FieldMap definition                                                   */
/*----------------------------------------------------------------------------*/

typedef struct IMRT_FieldMap
{
	IMRT_FieldMap_Id    Id;
	IMRT_Field         *Field_Input_Ptr;
	IMRT_Field         *Field_Output_Ptr;
}  IMRT_FieldMap;





/*----------------------------------------------------------------------------*/
/* IMRT_Msg definition                                                        */
/*----------------------------------------------------------------------------*/

typedef short  IMRT_Msg_Id;

#define IMRD_MSG_ID_UNKNOWN  ( IMRT_Msg_Id  ) 0



typedef char  IMRT_Msg_Name[ IMRD_NAME_SIZE];

#define IMRD_MSG_NAME_UNKNOWN   "UNKNOWN"



typedef struct IMRT_Msg
{
	IMRT_Msg_Id     Id;
	IMRT_Msg_Name   Name;
	size_t          Size;
	NDT_Root       *Field_Struct_Ptr;
}  IMRT_Msg;





/*----------------------------------------------------------------------------*/
/* IMRT_Fmt definition                                                        */
/*----------------------------------------------------------------------------*/



/* #define IMRT_LG_MESSAGE_TYPE 60 */



typedef int  IMRT_Fmt_Id;

#define IMRD_FMT_ID_UNKNOWN  ( IMRT_Fmt_Id) 0



#define IMRD_FMT_NAME_LEN   ( short) 20
#define IMRD_FMT_NAME_SIZE  ( short) ( IMRD_FMT_NAME_LEN + 1)

typedef char  IMRT_Fmt_Name[ IMRD_FMT_NAME_SIZE];

#define IMRD_FMT_NAME_UNKNOWN   "Unknown"




typedef short  IMRT_Branche_Id;

#define IMRD_BRANCHE_ID_UNKNOWN  ( IMRT_Branche_Id  ) 0



typedef short  IMRT_FmtAttr_Id;

#define IMRD_FMTATTR_ID_UNKNOWN  ( IMRT_FmtAttr_Id) 0



#define IMRD_FMTATTR_NAME_LEN   ( short) 65
#define IMRD_FMTATTR_NAME_SIZE  ( short) ( IMRD_FMTATTR_NAME_LEN + 1)

typedef char  IMRT_FmtAttr_Name[ IMRD_FMTATTR_NAME_SIZE];

#define IMRD_FMTATTR_NAME_UNKNOWN   "unknown"




#define IMRD_FMTATTR_VALUE_LEN   ( short) 20
#define IMRD_FMTATTR_VALUE_SIZE  ( short) ( IMRD_FMTATTR_VALUE_LEN + 1)

typedef char  IMRT_FmtAttr_Value[ IMRD_FMTATTR_VALUE_SIZE];




/*----------------------------------------------------------------------------*/
/* IMRT_FmtAttr definition                                                    */
/*----------------------------------------------------------------------------*/

typedef struct IMRT_FmtAttr
{
	IMRT_FmtAttr_Id     Id;
	IMRT_FmtAttr_Name   Name;
	IMRT_FmtAttr_Value  Value;
}  IMRT_FmtAttr;




/*----------------------------------------------------------------------------*/
/* IMRT_FmtMsg definition                                                     */
/*----------------------------------------------------------------------------*/

typedef struct IMRT_FmtMsg
{
	IMRT_Msg         *Msg_Ptr;
	IMRT_Branche_Id   Branche_Id;
	IMRT_FmtAttr     *FmtAttr_Ptr;
}  IMRT_FmtMsg;




typedef struct IMRT_Fmt
{
	IMRT_Fmt_Id     Id;
	IMRT_Fmt_Name   Name;
	NDT_Root       *FmtMsg_Struct_Ptr;
}  IMRT_Fmt;





/*----------------------------------------------------------------------------*/
/* IMRT_Appl_Id definition                                                    */
/*----------------------------------------------------------------------------*/

typedef short  IMRT_Appl_Id;

#define IMRD_APPL_ID_UNKNOWN  ( IMRT_Appl_Id) 0

#define IMRD_APPL_ID_UNDEFINEDED  ( IMRT_Appl_Id) -1




/*----------------------------------------------------------------------------*/
/* IMRT_Appl_Name definition                                                  */
/*----------------------------------------------------------------------------*/

#define IMRD_APPL_NAME_LEN   ( short) 30
#define IMRD_APPL_NAME_SIZE  ( short) ( IMRD_APPL_NAME_LEN + 1)

typedef char  IMRT_Appl_Name[ IMRD_APPL_NAME_SIZE];

#define IMRD_APPL_NAME_UNKNOWN   "UNKNOWN"





/*----------------------------------------------------------------------------*/
/* IMRT_Buffer definition                                                     */
/*----------------------------------------------------------------------------*/

typedef short IMRT_Dir_Id;

#define IMRD_DIR_ID_UNKNOWN  ( IMRT_Dir_Id) 0
#define IMRD_DIR_ID_IN       ( IMRT_Dir_Id) 1
#define IMRD_DIR_ID_OUT      ( IMRT_Dir_Id) 2



typedef short IMRT_Buffer_Status;

#define IMRD_BUFFER_STATUS_UNKNOWN    ( IMRT_Buffer_Status) 0
#define IMRD_BUFFER_STATUS_ALLOCATED  ( IMRT_Buffer_Status) 1
#define IMRD_BUFFER_STATUS_FREED      ( IMRT_Buffer_Status) 2



typedef short IMRT_Buffer_Flag;

#define IMRD_BUFFER_FLAG_UNKNOWN  ( IMRT_Buffer_Flag) 0
#define IMRD_BUFFER_FLAG_OPENED   ( IMRT_Buffer_Flag) 1
#define IMRD_BUFFER_FLAG_CLOSED   ( IMRT_Buffer_Flag) 2



typedef char IMRT_Buffer_Data;



typedef struct IMRT_Buffer
{
   IMRT_Msg_Id          Msg_Id;
   IMRT_Appl_Id         Appl_Id_Input;
   IMRT_Appl_Id         Appl_Id_Output;
   IMRT_Dir_Id          Dir_Id;
   IMRT_Fmt_Id          Fmt_Id;
   IMRT_FmtAttr_Value   FmtAttr_Value;
   IMRT_Buffer_Status   Status;
   IMRT_Buffer_Flag     Flag;
   size_t		Data_Buf_Len;
   size_t               Data_Buf_Size;
   IMRT_Buffer_Data    *Data_Buf_Ptr;
   IMRT_Boolean         Data_Tmp_Changed;
   IMRT_Buffer_Data    *Data_Tmp_Ptr;
}  IMRT_Buffer;



typedef short  IMRT_Buffer_Id;

#define IMRT_BUFFER_ID_MAX_NB  ( IMRT_Buffer_Id) 50



/*----------------------------------------------------------------------------*/
/* IMRT_IOM_Read_Mode definition                                              */
/*----------------------------------------------------------------------------*/

typedef short   IMRT_Read_Mode;

#define IMRD_READ_MODE_UNKNOWN  ( IMRT_Read_Mode) 0
#define IMRD_READ_MODE_WAIT     ( IMRT_Read_Mode) 1
#define IMRD_READ_MODE_NO_WAIT  ( IMRT_Read_Mode) 2



/*----------------------------------------------------------------------------*/
/* IMRT_Base                                                                  */
/*----------------------------------------------------------------------------*/

typedef struct IMRT_Base
{
	int          Sig_Trapped;
	sigjmp_buf   SigLongJmp_Env;
}  IMRT_Base;



#   ifdef _IMRD_C_

#   else

extern IMRT_Base  IMRG_Base;

#   endif




/*----------------------------------------------------------------------------*/
/* IMR libraries                                                              */
/*----------------------------------------------------------------------------*/

#include <log_mng.h>
#include <static_data_mng.h>
#include <io_mng.h>
#include <mapping_mng.h>
#include <msg_router.h>

#endif
