/*----------------------------------------------------------------------------*/
/* $Workfile:   io_mng.h  $                                                   */                                            
/*----------------------------------------------------------------------------*/
/* $Author: agibert $                                                      */
/* $Date: 2008/11/12 02:25:22 $                                           */
/* $Revision: 1.1 $                                                       */
/* $Label:  $                                                         */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#ifndef _IO_MNG_H_
#define _IO_MNG_H_



#include <imr.h>

#ifdef TUX_SUPPORT
#   include <atmi.h>
#   include <fml32.h>
#   include <tpadm.h>
#endif

#ifdef MQS_SUPPORT
#   include <cmqc.h>
#endif




/*----------------------------------------------------------------------------*/
/* IMRD_IOM_API definition                                                    */
/*----------------------------------------------------------------------------*/

#   ifdef _LIBIO_MNG_
#       define IMRD_IOM_API  
#   else
#       define IMRD_IOM_API  extern
#   endif





/*----------------------------------------------------------------------------*/
/* IOM_Config definition                                                      */
/*----------------------------------------------------------------------------*/

typedef struct IMRT_IOM_Config
{
	char  *dummy;

}  IMRT_IOM_Config;





/*----------------------------------------------------------------------------*/
/* IMRT_File_Delim definition                                                 */
/*----------------------------------------------------------------------------*/

#define IMRD_FILE_DELIM_LEN   (short) 32
#define IMRD_FILE_DELIM_SIZE  (short) ( IMRD_FILE_DELIM_LEN + 1)

typedef char   IMRT_File_Delim[ IMRD_FILE_DELIM_SIZE];



/*----------------------------------------------------------------------------*/
/* IMRT_File_Buffer definition                                                */
/*----------------------------------------------------------------------------*/

#define IMRD_FILE_BUFFER_LEN   (size_t) 4096
#define IMRD_FILE_BUFFER_SIZE  (size_t) ( IMRD_FILE_BUFFER_LEN + 1)

typedef char  IMRT_File_Buffer;



/*----------------------------------------------------------------------------*/
/* IMRT_Medium_File definition                                                */
/*----------------------------------------------------------------------------*/

typedef struct IMRT_Medium_File
{
	IMRT_File_Delim    Delim;
	size_t             Delim_Len;
	size_t             Msg_Len;
	IMRT_Path          Path;
	FILE              *Handle_Ptr;
	IMRT_File_Buffer  *Buffer_Ptr;
	size_t             Buffer_Len;
	IMRT_File_Buffer  *Cur_Ptr;

}  IMRT_Medium_File;



#ifdef TUX_SUPPORT
/*----------------------------------------------------------------------------*/
/* IMRT_Tux_Queue_Name definition                                             */
/*----------------------------------------------------------------------------*/

#define IMRD_TUX_QUEUE_NAME_LEN   ( short) 255
#define IMRD_TUX_QUEUE_NAME_SIZE  ( short) ( IMRD_TUX_QUEUE_NAME_LEN + 1)

typedef char  IMRT_Tux_Queue_Name[ IMRD_TUX_QUEUE_NAME_SIZE];



/*----------------------------------------------------------------------------*/
/* IMRT_Tux_Queue_Space definition                                            */
/*----------------------------------------------------------------------------*/

#define IMRD_TUX_QUEUE_SPACE_LEN   ( short) 255
#define IMRD_TUX_QUEUE_SPACE_SIZE  ( short) ( IMRD_TUX_QUEUE_SPACE_LEN + 1)

typedef char  IMRT_Tux_Queue_Space[ IMRD_TUX_QUEUE_SPACE_SIZE];



/*----------------------------------------------------------------------------*/
/* IMRT_Tux_Open_Info definition                                              */
/*----------------------------------------------------------------------------*/

#define IMRD_TUX_OPEN_INFO_LEN   ( short) 255
#define IMRD_TUX_OPEN_INFO_SIZE  ( short) ( IMRD_TUX_OPEN_INFO_LEN + 1)

typedef char  IMRT_Tux_Open_Info[ IMRD_TUX_OPEN_INFO_SIZE];



/*----------------------------------------------------------------------------*/
/* IMRT_Tux_Lmid definition                                                   */
/*----------------------------------------------------------------------------*/

#define IMRD_TUX_LMID_LEN   ( short) 255
#define IMRD_TUX_LMID_SIZE  ( short) ( IMRD_TUX_LMID_LEN + 1)

typedef char  IMRT_Tux_Lmid[ IMRD_TUX_LMID_SIZE];



#define IMRD_TUX_BUFFER_SIZE   ( size_t) 10000

typedef FBFR32  IMRT_Tux_Buffer;



/*----------------------------------------------------------------------------*/
/* IMRT_Medium_Tux definition                                                 */
/*----------------------------------------------------------------------------*/

typedef struct IMRT_Medium_Tux
{
	IMRT_Tux_Queue_Space   Queue_Space;
	IMRT_Tux_Queue_Name    Queue_Name;

}  IMRT_Medium_Tux;
#endif /* MQS_SUPPORT */



#ifdef MQS_SUPPORT
/*----------------------------------------------------------------------------*/
/* IMRT_MQS_Queue_Manager definition                                          */
/*----------------------------------------------------------------------------*/

#define IMRD_MQS_QUEUE_MNG_LEN   ( short) 255
#define IMRD_MQS_QUEUE_MNG_SIZE  ( short) ( IMRD_MQS_QUEUE_MNG_LEN + 1)

typedef  char  IMRT_MQS_Queue_Mng[ IMRD_MQS_QUEUE_MNG_SIZE];



/*----------------------------------------------------------------------------*/
/* IMRT_MQS_Queue_Name definition                                             */
/*----------------------------------------------------------------------------*/

#define IMRD_MQS_QUEUE_NAME_LEN   ( short) 255
#define IMRD_MQS_QUEUE_NAME_SIZE  ( short) ( IMRD_MQS_QUEUE_NAME_LEN + 1)

typedef  char  IMRT_MQS_Queue_Name[ IMRD_MQS_QUEUE_NAME_SIZE];




#define IMRD_MQS_BUFFER_LEN   (size_t) 4096
#define IMRD_MQS_BUFFER_SIZE  (size_t) ( IMRD_MQS_BUFFER_LEN + 1)

typedef MQBYTE  IMRT_MQS_Buffer;



/*----------------------------------------------------------------------------*/
/* IMRT_Mediul_MQS definition                                                 */
/*----------------------------------------------------------------------------*/

typedef struct IMRT_Medium_MQS
{
	IMRT_MQS_Queue_Mng   Queue_Mng;
	IMRT_MQS_Queue_Name  Queue_Name;
	MQHCONN              Conn_Hdl;
	MQHOBJ               Obj_Hdl;

}  IMRT_Medium_MQS;
#endif /* MQS_SUPPORT */



/*----------------------------------------------------------------------------*/
/* IMRT_Medium definitions                                                    */
/*----------------------------------------------------------------------------*/

#define IMRD_MEDIUM_NAME_LEN   ( short) 64
#define IMRD_MEDIUM_NAME_SIZE  ( short) ( IMRD_MEDIUM_NAME_LEN + 1)

typedef char   IMRT_Medium_Name[ IMRD_MEDIUM_NAME_SIZE];



typedef short  IMRT_Medium_Type;

#define IMRD_MEDIUM_TYPE_UNKNOWN  ( IMRT_Medium_Type) 0
#define IMRD_MEDIUM_TYPE_FILE     ( IMRT_Medium_Type) 1

#ifdef MQS_SUPPORT
#   define IMRD_MEDIUM_TYPE_TUX      ( IMRT_Medium_Type) 2
#endif /* TUX_SUPPORT */

#ifdef MQS_SUPPORT
#   define IMRD_MEDIUM_TYPE_MQS      ( IMRT_Medium_Type) 3
#endif /* MQS_SUPPORT */



typedef short  IMRT_Medium_Status;

#define IMRD_MEDIUM_STATUS_UNKNOWN  ( IMRT_Medium_Status) 0
#define IMRD_MEDIUM_STATUS_OPENED   ( IMRT_Medium_Status) 1
#define IMRD_MEDIUM_STATUS_CLOSED   ( IMRT_Medium_Status) 2



#define IMRD_MEDIUM_URL_LEN   ( short) 255
#define IMRD_MEDIUM_URL_SIZE  ( short) ( IMRD_MEDIUM_URL_LEN + 1)

typedef char   IMRT_Medium_URL[ IMRD_MEDIUM_URL_SIZE];



#define IMRD_MEDIUM_PATH_LEN   ( short) 255
#define IMRD_MEDIUM_PATH_SIZE  ( short) ( IMRD_MEDIUM_PATH_LEN + 1)

typedef char   IMRT_Medium_Path[ IMRD_MEDIUM_PATH_SIZE];

typedef short IMRT_Medium_Tran_Flag;

#define IMRD_MEDIUM_TRAN_FLAG_UNKNOWN  ( IMRT_Medium_Tran_Flag) 0
#define IMRD_MEDIUM_TRAN_FLAG_IN       ( IMRT_Medium_Tran_Flag) 1
#define IMRD_MEDIUM_TRAN_FLAG_OUT      ( IMRT_Medium_Tran_Flag) 2


typedef struct IMRT_Medium
{
	IMRT_Medium_Name       Name;
	IMRT_Medium_Type       Type;
	IMRT_Medium_Status     Status;
	IMRT_Medium_URL        URL;
	IMRT_Appl_Id           Appl_Id_Input;
	IMRT_Appl_Id           Appl_Id_Output;
	IMRT_Dir_Id            Dir_Id;
	IMRT_Medium_Tran_Flag  Tran_Flag;
	size_t                 Byte_Nb;
	size_t                 Msg_Nb;

	IMRT_Medium_File       File;

#ifdef TUX_SUPPORT
	IMRT_Medium_Tux        Tux;
#endif /* TUX_SUPPORT */

#ifdef MQS_SUPPORT
	IMRT_Medium_MQS        MQS;
#endif /* MQS_SUPPORT */
	
}  IMRT_Medium;



/*----------------------------------------------------------------------------*/
/* IMRT_Tran definitions                                                    */
/*----------------------------------------------------------------------------*/

typedef int  IMRT_Medium_Nb;

typedef struct IMRT_Tran
{
	IMRT_Medium     **Medium_Ptr_Tab;
	IMRT_Medium_Nb    Medium_Nb;

}  IMRT_Tran;






/*----------------------------------------------------------------------------*/
/* Global Variables                                                           */
/*----------------------------------------------------------------------------*/

#   ifdef _LIBIO_MNG_

#   else

extern IMRT_Module  IMRG_IOM_Module;

#   endif







/******************************************************************************/
/*IO Manager API                                                              */
/******************************************************************************/



/*----------------------------------------------------------------------------*/
/* Open IO Manager                                                            */
/*----------------------------------------------------------------------------*/
/* IOM_Config_Ptr:   Setup IOM init mode                                      */
/*----------------------------------------------------------------------------*/

IMRD_IOM_API IMRT_Status  IMR_IOM_Library_Open( IMRT_IOM_Config  *);



/*----------------------------------------------------------------------------*/
/* Close IO Manager                                                           */
/*----------------------------------------------------------------------------*/

IMRD_IOM_API IMRT_Status  IMR_IOM_Library_Close( void);



/*----------------------------------------------------------------------------*/
/* Open Medium                                                                */
/*----------------------------------------------------------------------------*/
/* Medium_Ptr_Ptr:   Reference to the Pointer to the opened medium            */
/* Medium_Name:      Name of the medium                                       */
/* Medium_URL:       Medium URL to open                                       */
/* Appl_Id_Input:    Input application Id link with the medium                */
/* Appl_Id_Output:   Output application Id link with the medium               */
/* Dir_Id:           Medium open direction                                    */
/*----------------------------------------------------------------------------*/

IMRD_IOM_API IMRT_Status  IMR_IOM_Medium_Open( IMRT_Medium  **, IMRT_Medium_Name, IMRT_Medium_URL, IMRT_Appl_Id, IMRT_Appl_Id, IMRT_Dir_Id);



/*----------------------------------------------------------------------------*/
/* Close Medium                                                               */
/*----------------------------------------------------------------------------*/
/* Medium_Ptr:   Pointer to the medium to close                               */
/*----------------------------------------------------------------------------*/

IMRD_IOM_API IMRT_Status  IMR_IOM_Medium_Close( IMRT_Medium  *);



/*----------------------------------------------------------------------------*/
/* Read Medium                                                                */
/*----------------------------------------------------------------------------*/
/* Buffer_Ptr:   Pointer to the buffer to write                               */
/* Medium_Ptr:   Pointer to the medium to read                                */
/* Read_Mode:    Read mode                                                    */
/*----------------------------------------------------------------------------*/

IMRD_IOM_API IMRT_Status  IMR_IOM_Medium_Read( IMRT_Buffer  *, IMRT_Medium  *, IMRT_Read_Mode);



/*----------------------------------------------------------------------------*/
/* Write Medium                                                               */
/*----------------------------------------------------------------------------*/
/* Medium_Ptr:   Pointer to the medium to write                               */
/* Buffer_Ptr:   Pointer to the buffer to read                                */
/*----------------------------------------------------------------------------*/

IMRD_IOM_API IMRT_Status  IMR_IOM_Medium_Write( IMRT_Medium  *, IMRT_Buffer  *);



/*----------------------------------------------------------------------------*/
/* Tran Begin                                                                 */
/*----------------------------------------------------------------------------*/
/* Tran_Ptr_Ptr:   Reference to the pointer of the transaction to begin       */
/* Medium_Count:   Number of medium to enclose into the transaction           */
/* Medium_1_Ptr:   Pointer to the first medium                                */
/* ...                                                                        */
/*----------------------------------------------------------------------------*/

IMRD_IOM_API IMRT_Status  IMR_IOM_Tran_Begin( IMRT_Tran  **, IMRT_Medium_Nb, IMRT_Medium  *, ...);



/*----------------------------------------------------------------------------*/
/* Tran Commit                                                                */
/*----------------------------------------------------------------------------*/
/* Tran_Ptr:   Pointer to the transaction to commit                           */
/*----------------------------------------------------------------------------*/

IMRD_IOM_API IMRT_Status  IMR_IOM_Tran_Commit( IMRT_Tran  *);



/*----------------------------------------------------------------------------*/
/* Tran Rollback                                                              */
/*----------------------------------------------------------------------------*/
/* Tran_Ptr:   Pointer to the transaction to rollback                         */
/*----------------------------------------------------------------------------*/

IMRD_IOM_API IMRT_Status  IMR_IOM_Tran_Rollback( IMRT_Tran  *);





/*----------------------------------------------------------------------------*/

#endif
