/*----------------------------------------------------------------------------*/
/* File: io_hdl_file.c   		              		  	      */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#define _LIBIO_MNG_
#define _IO_HDL_FILE_C_



#include "io_hdl_file.h" 





/*----------------------------------------------------------------------------*/
/* IO_Hdl_File_Open                                                           */
/*----------------------------------------------------------------------------*/
/* Medium_Ptr:    Pointer to the medium to open                               */
/* Medium_Path:   Medium Path to open                                         */
/* Dir_Id:        Medium open direction                                       */
/*----------------------------------------------------------------------------*/

IMRT_Status  IO_Hdl_File_Open( IMRT_Medium  *Medium_Ptr, IMRT_Medium_Path  Medium_Path, IMRT_Dir_Id  Dir_Id)
{
    char  *type_ptr, *path_ptr, *read_ptr, *write_ptr;


    if( Dir_Id == IMRD_DIR_ID_IN)
    {
	type_ptr = "r";
    }
    else
    {
	if( Dir_Id == IMRD_DIR_ID_OUT)
	{
	    type_ptr = "w";
	}
	else
	{
	    IMRD_LM_LOG_ERROR_1( "Invalid Dir_Id (%d) !", Dir_Id);

	    return( IMRS_KO);
	}
    }

    if( ( path_ptr = strstr( Medium_Path, "@")) != NULL)
    {
	if( ( path_ptr == Medium_Path) || ( ( path_ptr - Medium_Path + 1) == strlen( Medium_Path)))
	{
	    IMRD_LM_LOG_ERROR_1( "Invalid file path [%s] !", Medium_Path);

	    return( IMRS_KO);
	}

	for( read_ptr = Medium_Path, write_ptr = Medium_Ptr->File.Delim; read_ptr < path_ptr; read_ptr++, write_ptr++)
	{
	    if( ( *read_ptr == '\\') && ( read_ptr < ( path_ptr - 1)))
	    {
		switch( *( read_ptr + 1))
		{
		    case 'n':
		    {
			*write_ptr = '\n';
			break;
		    }

		    case 't':
		    {
			*write_ptr = '\t';
			break;
		    }

		    case '\\':
		    {
			*write_ptr = '\n';
			break;
		    }

		    default:
		    {
			*write_ptr = '?';
			break;
		    }
		}

		read_ptr++;
	    }
	    else
	    {
		*write_ptr = *read_ptr;
	    }
	} 

	*write_ptr = '\0';

	Medium_Ptr->File.Delim_Len = write_ptr - Medium_Ptr->File.Delim;
	Medium_Ptr->File.Msg_Len = 0;

	strcpy( Medium_Ptr->File.Path, ( path_ptr + 1));
    }
    else
    {
	if( ( path_ptr = strstr( Medium_Path, "#")) != NULL)
	{
	    if( ( path_ptr == Medium_Path) || ( ( path_ptr - Medium_Path + 1) == strlen( Medium_Path)))
	    {
		IMRD_LM_LOG_ERROR_1( "Invalid file path [%s] !", Medium_Path);

		return( IMRS_KO);
	    }

	    Medium_Ptr->File.Delim_Len = 0;
	    Medium_Ptr->File.Delim[ Medium_Ptr->File.Delim_Len] = '\0';

	    Medium_Ptr->File.Msg_Len = atoi( Medium_Path);

	    if( Medium_Ptr->File.Msg_Len <= 0)
	    {
		IMRD_LM_LOG_ERROR_1( "Invalid message len [%s] !", Medium_Path);

		return( IMRS_KO);
	    }

	    strcpy( Medium_Ptr->File.Path, ( path_ptr + 1));
	}
	else
	{
	    Medium_Ptr->File.Delim_Len = 0;
	    Medium_Ptr->File.Delim[ Medium_Ptr->File.Delim_Len] = '\0';

	    Medium_Ptr->File.Msg_Len = 0;

	    strcpy( Medium_Ptr->File.Path, Medium_Path);
	}
    }

    if( ( Medium_Ptr->File.Handle_Ptr = fopen( Medium_Ptr->File.Path, type_ptr)) == NULL)
    {
	IMRD_LM_LOG_ERROR_2( "Can't open file [%s] (%d) !", Medium_Ptr->File.Path, errno);

	return( IMRS_KO);
    }
    
    if( ( Medium_Ptr->File.Buffer_Ptr = malloc( IMRD_FILE_BUFFER_SIZE)) == NULL)
    {
	IMRD_LM_LOG_ERROR_1( "Can't allocate File_Buffer (%d) !", errno);

	return( IMRS_KO);
    }

    Medium_Ptr->File.Buffer_Len = 0;
    Medium_Ptr->File.Cur_Ptr = Medium_Ptr->File.Buffer_Ptr + IMRD_FILE_BUFFER_LEN;

    IMRD_LM_LOG_TRACE_4( IMRD_LOG_LEVEL_VERBOSE_1, "File: [%s]   mode: [%s]   delim: [%s]   len: [%ld]   opened !", Medium_Ptr->File.Path, type_ptr, Medium_Ptr->File.Delim, Medium_Ptr->File.Msg_Len);

    return( IMRS_OK);
}





/*----------------------------------------------------------------------------*/
/* IO_Hdl_File_Close                                                          */
/*----------------------------------------------------------------------------*/
/* Medium_Ptr:    Pointer to the medium to close                              */
/*----------------------------------------------------------------------------*/

IMRT_Status  IO_Hdl_File_Close( IMRT_Medium  *Medium_Ptr)
{
    if( Medium_Ptr->File.Buffer_Ptr != NULL)
    {
	free( Medium_Ptr->File.Buffer_Ptr);
    }

    if( fclose( Medium_Ptr->File.Handle_Ptr) == EOF)
    {
	IMRD_LM_LOG_ERROR_2( "Can't close file [%s] (%d) !", Medium_Ptr->File.Path, errno);

	return( IMRS_KO);
    }

    IMRD_LM_LOG_TRACE_1( IMRD_LOG_LEVEL_VERBOSE_1, "File: [%s] closed !", Medium_Ptr->File.Path);

    return( IMRS_OK);
}



/*----------------------------------------------------------------------------*/
/* IO_Hdl_File_Mem_Str                                                        */
/*----------------------------------------------------------------------------*/
/* Occur_Ptr_Ptr: Pointer to the first occurrence                             */
/* Area_Ptr:      Memory Area                                                 */
/* Area_Size:     Size of the Area_Ptr                                        */
/* Search_Ptr:    Area to search in Area_Ptr                                  */
/* Search_Size:   Size of the Search_Ptr                                      */
/*----------------------------------------------------------------------------*/

IMRT_Status  IO_Hdl_File_Mem_Str( char **Occur_Ptr_Ptr, const char *Area_Ptr, size_t Area_Size, const char *Search_Ptr, size_t Search_Size)
{

	char * Pos = (char *)Area_Ptr;

	*Occur_Ptr_Ptr = NULL;

	while((Pos != NULL) && (*Occur_Ptr_Ptr == NULL))
	{
		Pos = memchr(Pos, Search_Ptr[0], Area_Size - (Pos - Area_Ptr));
		if ((Area_Size - (Pos - Area_Ptr)) < Search_Size)
		{
			Pos = NULL;
		}
		else
		{
			if (Pos != NULL)
			{
				if (memcmp(Pos, Search_Ptr, Search_Size) == 0)
				{
					*Occur_Ptr_Ptr = Pos;
				}
				else
				{
					Pos += 1;
				}
			}
		}
	}

	return( IMRS_OK);
}


/*----------------------------------------------------------------------------*/
/* IO_Hdl_File_Read                                                           */
/*----------------------------------------------------------------------------*/
/* Buffer_Ptr:    Buffer to write data in                                     */
/* Read_Len_Ptr:  Number of byte readen                                       */
/* Medium_Path:   Medium to read                                              */
/* Read_Mode:     Read mode                                                   */
/*----------------------------------------------------------------------------*/

IMRT_Status  IO_Hdl_File_Read( IMRT_Buffer  *Buffer_Ptr, size_t *Read_Len_Ptr, IMRT_Medium  *Medium_Ptr, IMRT_Read_Mode  Read_Mode)
{
    IMRT_Buffer_Data  *next_ptr;
    IMRT_Buffer_Data  *delim_ptr;
    IMRT_Buffer_Data  *buffer_data_ptr  = NULL;
    size_t             buffer_data_size = 1;
    size_t             copy_data_size   = 0;
    size_t             read_buffer_len  = IMRD_FILE_BUFFER_LEN - Medium_Ptr->File.Delim_Len;
	IMRT_Status  status_tmp;

	*Read_Len_Ptr = 0;

    if( Medium_Ptr->File.Msg_Len > 0)                                                                                                 /* Msg_Len split mode */
    {
	do
	{
	    if( Medium_Ptr->File.Cur_Ptr >= ( Medium_Ptr->File.Buffer_Ptr + Medium_Ptr->File.Buffer_Len))                          /* File needs to be read */
	    {
		Medium_Ptr->File.Cur_Ptr = Medium_Ptr->File.Buffer_Ptr;

		Medium_Ptr->File.Buffer_Len = fread( Medium_Ptr->File.Buffer_Ptr, sizeof( char), IMRD_FILE_BUFFER_LEN, Medium_Ptr->File.Handle_Ptr);

		if( ferror( Medium_Ptr->File.Handle_Ptr) != 0)
		{
		    IMRD_LM_LOG_ERROR_2( "Can't read file [%s] (%d) !", Medium_Ptr->File.Path, ferror( Medium_Ptr->File.Handle_Ptr));
		    
		    return( IMRS_KO);
		}
	    }

	    copy_data_size = IMRD_MIN( ( Medium_Ptr->File.Buffer_Len - ( Medium_Ptr->File.Cur_Ptr - Medium_Ptr->File.Buffer_Ptr)),
				       ( Medium_Ptr->File.Msg_Len - buffer_data_size + 1));

	    if( copy_data_size != 0)
	    {
		if( ( buffer_data_ptr = realloc( buffer_data_ptr, ( buffer_data_size + copy_data_size))) == NULL)
		{
		    IMRD_LM_LOG_ERROR_1( "Can't allocate Buffer_Data (%d) !", errno);
		    
		    return( IMRS_KO);
		}
		
		memcpy( ( buffer_data_ptr + buffer_data_size - 1), Medium_Ptr->File.Cur_Ptr, copy_data_size);  /* Copy data from read buffer to copy buffer */
		buffer_data_size += copy_data_size;
		
		Medium_Ptr->File.Cur_Ptr += copy_data_size;
	    }
	}
	while( ( buffer_data_size < Medium_Ptr->File.Msg_Len) && ( Medium_Ptr->File.Buffer_Len > 0));
    }
    else                                                                                                                            /* Delimitor split mode */
    {
	do
	{
	    if( Medium_Ptr->File.Cur_Ptr >= ( Medium_Ptr->File.Buffer_Ptr + read_buffer_len))                                      /* File needs to be read */
	    {
		if( Medium_Ptr->File.Cur_Ptr < ( Medium_Ptr->File.Buffer_Ptr + IMRD_FILE_BUFFER_LEN))                 /* Footer needs to be moved to header */
		{
		    Medium_Ptr->File.Buffer_Len += Medium_Ptr->File.Buffer_Ptr - Medium_Ptr->File.Cur_Ptr;
		    memcpy( Medium_Ptr->File.Buffer_Ptr, Medium_Ptr->File.Cur_Ptr, Medium_Ptr->File.Buffer_Len);
		}
		else
		{
		    Medium_Ptr->File.Buffer_Len = 0;                                                                                     /* Buffer is empty */
		}
		
		Medium_Ptr->File.Cur_Ptr = Medium_Ptr->File.Buffer_Ptr;
		
		Medium_Ptr->File.Buffer_Len += fread( ( Medium_Ptr->File.Buffer_Ptr + Medium_Ptr->File.Buffer_Len),
						      sizeof( char), ( IMRD_FILE_BUFFER_LEN - Medium_Ptr->File.Buffer_Len),
						      Medium_Ptr->File.Handle_Ptr);
		
		if( ferror( Medium_Ptr->File.Handle_Ptr) != 0)
		{
		    IMRD_LM_LOG_ERROR_2( "Can't read file [%s] (%d) !", Medium_Ptr->File.Path, ferror( Medium_Ptr->File.Handle_Ptr));
		    
		    return( IMRS_KO);
		}
	    }
	    
	    status_tmp = IO_Hdl_File_Mem_Str(&delim_ptr, Medium_Ptr->File.Cur_Ptr, Medium_Ptr->File.Buffer_Len - (Medium_Ptr->File.Cur_Ptr - Medium_Ptr->File.Buffer_Ptr), Medium_Ptr->File.Delim, Medium_Ptr->File.Delim_Len);
	    
	    if( ( Medium_Ptr->File.Delim_Len == 0) ||  ( delim_ptr == NULL))  /* Seek delim. */
	    {
		delim_ptr = Medium_Ptr->File.Buffer_Ptr + IMRD_MIN( Medium_Ptr->File.Buffer_Len, read_buffer_len);                          /* No delimitor */
		next_ptr = delim_ptr;
	    }
	    else
	    {
		next_ptr = delim_ptr + Medium_Ptr->File.Delim_Len;                                                                       /* Delimitor found */
		*Read_Len_Ptr = *Read_Len_Ptr + Medium_Ptr->File.Delim_Len;
	    }
	    
	    copy_data_size = delim_ptr - Medium_Ptr->File.Cur_Ptr;
	    
	    if( copy_data_size != 0)
	    {
		if( ( buffer_data_ptr = realloc( buffer_data_ptr, ( buffer_data_size + copy_data_size))) == NULL)
		{
		    IMRD_LM_LOG_ERROR_1( "Can't allocate Buffer_Data (%d) !", errno);
		    
		    return( IMRS_KO);
		}
		
		memcpy( ( buffer_data_ptr + buffer_data_size - 1), Medium_Ptr->File.Cur_Ptr, copy_data_size);  /* Copy data from read buffer to copy buffer */
		buffer_data_size += copy_data_size;
		
		Medium_Ptr->File.Cur_Ptr = next_ptr;
	    }
	}
	while( ( next_ptr == delim_ptr) && ( Medium_Ptr->File.Buffer_Len >= read_buffer_len));
    }

    if( IMR_SDM_Fmt_Id_Get( &( Buffer_Ptr->Fmt_Id), "Fixed") != IMRS_OK)
    {
	IMRD_LM_LOG_ERROR_1( "Can't get FmtId: (%d) !", Buffer_Ptr->Fmt_Id);
	
	return( IMRS_KO);
    }

    if( buffer_data_size <= 1)
    {
	IMRD_LM_LOG_TRACE_1( IMRD_LOG_LEVEL_VERBOSE_1, "No more data to read from file: [%s] !", Medium_Ptr->File.Path);

	return( IMRS_NO_DATA);
    }
    else
    {
	buffer_data_ptr[ ( buffer_data_size - 1)] = '\0';

	Buffer_Ptr->Data_Buf_Ptr  = buffer_data_ptr;
	Buffer_Ptr->Data_Buf_Len  = buffer_data_size - 1;
	Buffer_Ptr->Data_Buf_Size = buffer_data_size;
	Buffer_Ptr->Status =        IMRD_BUFFER_STATUS_ALLOCATED;

	*Read_Len_Ptr = *Read_Len_Ptr + buffer_data_size - 1;

	IMRD_LM_LOG_TRACE_2( IMRD_LOG_LEVEL_VERBOSE_1, "Read (%d) bytes from file: [%s] ", Buffer_Ptr->Data_Buf_Len, Medium_Ptr->File.Path);

	return( IMRS_OK);
    }
}





/*----------------------------------------------------------------------------*/
/* IO_Hdl_File_Write                                                          */
/*----------------------------------------------------------------------------*/
/* Medium_Ptr:      Pointer to the medium to write data to                    */
/* Write_Len_Ptr:   Number of byte writen                                     */
/* Buffer_Ptr:      Buffer to read                                            */
/*----------------------------------------------------------------------------*/

IMRT_Status  IO_Hdl_File_Write( IMRT_Medium  *Medium_Ptr, size_t  *Write_Len_Ptr, IMRT_Buffer  *Buffer_Ptr)
{
    char    *padding_buffer_ptr;
    long     padding_len;


    *Write_Len_Ptr = 0;

    if( ( Medium_Ptr->File.Delim_Len != 0) && ( Medium_Ptr->Msg_Nb > 0))
    {
	*Write_Len_Ptr = fwrite( Medium_Ptr->File.Delim, sizeof( char), Medium_Ptr->File.Delim_Len, Medium_Ptr->File.Handle_Ptr);

	if( ferror( Medium_Ptr->File.Handle_Ptr) != 0)
	{
	    IMRD_LM_LOG_ERROR_2( "Can't write file [%s] (%d) !", Medium_Ptr->URL, ferror( Medium_Ptr->File.Handle_Ptr));

	    return( IMRS_KO);
	}
    }

    padding_len = Medium_Ptr->File.Msg_Len - Buffer_Ptr->Data_Buf_Len;

    if( Medium_Ptr->File.Msg_Len != 0 )
    {
	if( padding_len < 2)
	{
	    IMRD_LM_LOG_ERROR_2( "Buffer size: (%d) too big for message split size: (%d) !", Buffer_Ptr->Data_Buf_Len, Medium_Ptr->File.Msg_Len);

	    return( IMRS_KO);
	}

	if( ( padding_buffer_ptr = malloc( padding_len)) == NULL)
	{
	    IMRD_LM_LOG_ERROR_1( "Can't allocate padding_buffer (%d) !", errno);

	    return( IMRS_KO);
	}
	
	memset( padding_buffer_ptr, ' ', padding_len);

	padding_buffer_ptr[0] = '\0';
	padding_buffer_ptr[1] = '\0';
    }

    *Write_Len_Ptr += fwrite( Buffer_Ptr->Data_Buf_Ptr, sizeof( char), Buffer_Ptr->Data_Buf_Len, Medium_Ptr->File.Handle_Ptr);

    if( ferror( Medium_Ptr->File.Handle_Ptr) != 0)
    {
	IMRD_LM_LOG_ERROR_2( "Can't write data in file [%s] (%d) !", Medium_Ptr->URL, ferror( Medium_Ptr->File.Handle_Ptr));

	return( IMRS_KO);
    }

    if( Medium_Ptr->File.Msg_Len != 0 )
    {
	*Write_Len_Ptr += fwrite( padding_buffer_ptr, sizeof( char), padding_len, Medium_Ptr->File.Handle_Ptr);
	
	free( padding_buffer_ptr);

	if( ferror( Medium_Ptr->File.Handle_Ptr) != 0)
	{
	    IMRD_LM_LOG_ERROR_2( "Can't write padding in file [%s] (%d) !", Medium_Ptr->URL, ferror( Medium_Ptr->File.Handle_Ptr));
	    
	    return( IMRS_KO);
	}
    }

    fflush( Medium_Ptr->File.Handle_Ptr);

    IMRD_LM_LOG_TRACE_2( IMRD_LOG_LEVEL_VERBOSE_1, "Wrote (%d) bytes to file: [%s] ", *Write_Len_Ptr, Medium_Ptr->File.Path);

    return( IMRS_OK);
}





/*----------------------------------------------------------------------------*/
/* IO_Hdl_File_Begin                                                          */
/*----------------------------------------------------------------------------*/
/* IMRT_Medium  *Medium_Ptr                                                   */
/*----------------------------------------------------------------------------*/

IMRT_Status  IO_Hdl_File_Begin( IMRT_Medium  *Medium_Ptr)
{
    return(IMRS_OK);
}





/*----------------------------------------------------------------------------*/
/* IO_Hdl_File_Commit                                                         */
/*----------------------------------------------------------------------------*/
/* IMRT_Medium  *Medium_Ptr                                                   */
/*----------------------------------------------------------------------------*/

IMRT_Status  IO_Hdl_File_Commit( IMRT_Medium  *Medium_Ptr)
{
    return(IMRS_OK);
}





/*----------------------------------------------------------------------------*/
/* IO_Hdl_File_Rollback                                                       */
/*----------------------------------------------------------------------------*/
/* IMRT_Medium  *Medium_Ptr                                                   */
/*----------------------------------------------------------------------------*/

IMRT_Status  IO_Hdl_File_Rollback( IMRT_Medium  *Medium_Ptr)
{
    return(IMRS_OK);
}
