/*----------------------------------------------------------------------------*/
/* File: io_hdl_mqs.c   		              		  	      */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#define _LIBIO_MNG_
#define _IO_HDL_MQS_C_



#include "io_hdl_mqs.h" 





/*----------------------------------------------------------------------------*/
/* IO_Hdl_MQS_Open                                                            */
/*----------------------------------------------------------------------------*/
/* Medium_Ptr:    Pointer to the medium to open                               */
/* Medium_Path:   Medium Path to open                                         */
/* Dir_Id:        Medium open direction                                       */
/*----------------------------------------------------------------------------*/

IMRT_Status  IO_Hdl_MQS_Open( IMRT_Medium  *Medium_Ptr, IMRT_Medium_Path  Medium_Path, IMRT_Dir_Id  Dir_Id)
{
    MQLONG        comp_code;                         /* completion code               */
    MQLONG        reason_code;                       /* reason code                   */
    char         *pos_slash_ptr;
    MQOD          od             = {MQOD_DEFAULT};   /* Object Descriptor             */
    MQLONG        options;                           /* MQOPEN/MQCLOSE options        */


    if( Dir_Id == IMRD_DIR_ID_IN)
    {
	/*positionnenment des option d'ouverture du de la Queue*/
	options = MQOO_INPUT_AS_Q_DEF | MQOO_FAIL_IF_QUIESCING; /* open queue for input  but not if MQM stopping   */
    }
    else
    {
	if( Dir_Id == IMRD_DIR_ID_OUT)
	{
	    /*positionnenment des option d'ouverture du de la Queue*/
	    options = MQOO_OUTPUT | MQOO_FAIL_IF_QUIESCING;     /* open queue for output but not if MQM stopping   */
	}
	else
	{
	    IMRD_LM_LOG_ERROR_1( "Invalid Dir_Id (%d) !", Dir_Id);

	    return( IMRS_KO);
	}
    }

    pos_slash_ptr = strstr( Medium_Path, "/");
    
    if( pos_slash_ptr == NULL)
    {
	IMRD_LM_LOG_ERROR_1( "Invalid mqs path [%s] !", Medium_Path);
	return( IMRS_KO);
    }

    strncpy( Medium_Ptr->MQS.Queue_Mng, Medium_Path, strlen( Medium_Path) - strlen( pos_slash_ptr));
    Medium_Ptr->MQS.Queue_Mng[ strlen( Medium_Path) - strlen( pos_slash_ptr)] = '\0';
	    
    strcpy( Medium_Ptr->MQS.Queue_Name, ( pos_slash_ptr + 1));
		
    /*queue manager connection*/
    MQCONN( Medium_Ptr->MQS.Queue_Mng, &( Medium_Ptr->MQS.Conn_Hdl), &comp_code, &reason_code);

    if( comp_code == MQCC_FAILED)
    {
	IMRD_LM_LOG_ERROR_2( "MQCONN ended with reason code (%d) for this path [%s] !", reason_code, Medium_Path);

	return( IMRS_KO);
    }

    strcpy( od.ObjectName, Medium_Ptr->MQS.Queue_Name);

    /*open the queue*/
    MQOPEN( Medium_Ptr->MQS.Conn_Hdl, &od, options, &( Medium_Ptr->MQS.Obj_Hdl), &comp_code, &reason_code);
		
    /* report reason, if any; stop if failed      */
    if( reason_code != MQRC_NONE)
    {
	IMRD_LM_LOG_ERROR_2( "MQOPEN ended with reason code (%d) for this path [%s] !", reason_code, Medium_Path);

	return( IMRS_KO);
    }
    
    if( comp_code == MQCC_FAILED)
    {
	IMRD_LM_LOG_ERROR_1( "Unable to open queue for this path [%s] !", Medium_Path);

	return( IMRS_KO);
    }
    else
    {
	IMRD_LM_LOG_TRACE_3( IMRD_LOG_LEVEL_VERBOSE_1, "Queue_Mng: [%s]   Queue: [%s]   Dir: [%s] opened !", Medium_Ptr->MQS.Queue_Mng, Medium_Ptr->MQS.Queue_Name, ( ( Dir_Id == IMRD_DIR_ID_IN) ? "in" : ( ( Dir_Id == IMRD_DIR_ID_OUT) ? "out" : "unknown")));

	return( IMRS_OK);
    }
}





/*----------------------------------------------------------------------------*/
/* IO_Hdl_MQS_Close                                                           */
/*----------------------------------------------------------------------------*/
/* Medium_Ptr:    Pointer to the medium to close                              */
/*----------------------------------------------------------------------------*/

IMRT_Status  IO_Hdl_MQS_Close( IMRT_Medium  *Medium_Ptr)
{
    MQLONG        comp_code;                         /* completion code               */
    MQLONG        reason_code;                       /* reason code                   */
    char         *pos_slash_ptr;
    MQOD          od             = {MQOD_DEFAULT};   /* Object Descriptor             */


    MQCLOSE( Medium_Ptr->MQS.Conn_Hdl, &( Medium_Ptr->MQS.Obj_Hdl), MQCO_NONE, &comp_code, &reason_code);
 
    if( reason_code != MQRC_NONE)
    {
	IMRD_LM_LOG_ERROR_2( "MQCLOSE ended with reason code [%d] for queue: [%s] !", reason_code, Medium_Ptr->MQS.Queue_Name);

	return( IMRS_KO);
    }

    MQDISC( &Medium_Ptr->MQS.Conn_Hdl, &comp_code, &reason_code);

    /* report reason, if any     */
    if( reason_code != MQRC_NONE)
    {
	IMRD_LM_LOG_ERROR_2( "MQCLOSE ended with reason code [%d] for queue: [%s] !", reason_code, Medium_Ptr->MQS.Queue_Name);

	return( IMRS_KO);
    }
    else
    {
	IMRD_LM_LOG_TRACE_2( IMRD_LOG_LEVEL_VERBOSE_1, "Queue_Mng: [%s]   Queue: [%s] closed !", Medium_Ptr->MQS.Queue_Mng, Medium_Ptr->MQS.Queue_Name);

	return( IMRS_OK);
    }
}





/*----------------------------------------------------------------------------*/
/* IO_Hdl_MQS_Read                                                            */
/*----------------------------------------------------------------------------*/
/* Buffer_Ptr:    Buffer to write data in                                     */
/* Read_Len_Ptr:  Number of byte writen                                       */
/* Medium_Path:   Medium to read                                              */
/* Read_Mode:     Read mode                                                   */
/*----------------------------------------------------------------------------*/

IMRT_Status  IO_Hdl_MQS_Read( IMRT_Buffer  *Buffer_Ptr, size_t  *Read_Len_Ptr, IMRT_Medium  *Medium_Ptr, IMRT_Read_Mode  Read_Mode)
{
    MQLONG            comp_code;                        /* completion code               */
    MQLONG            reason_code;                      /* reason code                   */
    IMRT_MQS_Buffer  *buffer_ptr;
    MQLONG            buffer_len;
    MQGMO             gmo          = {MQGMO_DEFAULT};   /* get message options           */
    MQMD              md           = {MQMD_DEFAULT};    /* Message Descriptor            */


    

    switch( Read_Mode)
    {
	case IMRD_READ_MODE_WAIT:
	{
	    gmo.Options = MQGMO_WAIT ;
	    break;
	}
	
	case IMRD_READ_MODE_NO_WAIT:
	{
	    gmo.Options = MQGMO_NO_WAIT ;
	    break;
	}

	default:
	{
	    IMRD_LM_LOG_ERROR_1( "Unknown Read_Mode: (%d) !", Read_Mode);

	    return( IMRS_KO);
	}
    }

    gmo.Options      |= MQGMO_CONVERT;

    if( Medium_Ptr->Tran_Flag == IMRD_MEDIUM_TRAN_FLAG_IN)
    {
	gmo.Options |= MQGMO_SYNCPOINT;
    }

//    gmo.WaitInterval =  MQWI_UNLIMITED;      
    gmo.WaitInterval = 10000;

    if( ( buffer_ptr = malloc( IMRD_MQS_BUFFER_SIZE)) == NULL)
    {
	IMRD_LM_LOG_ERROR_1( "Can't allocate MQS Buffer (%d) !", errno);
	
	return( IMRS_KO);
    }

    memcpy( md.MsgId,    MQMI_NONE, sizeof(md.MsgId));
    memcpy( md.CorrelId, MQCI_NONE, sizeof(md.CorrelId));
    md.Encoding       = MQENC_NATIVE;
    md.CodedCharSetId = MQCCSI_Q_MGR;

    do
    {

	MQGET( Medium_Ptr->MQS.Conn_Hdl, Medium_Ptr->MQS.Obj_Hdl, &md, &gmo, IMRD_MQS_BUFFER_SIZE, buffer_ptr, &buffer_len, &comp_code, &reason_code);

    } while( ( Read_Mode == IMRD_READ_MODE_WAIT) && ( reason_code == MQRC_NO_MSG_AVAILABLE) && ( IMRG_Base.Sig_Trapped == 0));

    if( reason_code != MQRC_NONE)
    {
	if( reason_code == MQRC_NO_MSG_AVAILABLE)
	{
	    free( buffer_ptr);

	    if( IMRG_Base.Sig_Trapped == 0)
	    {
		IMRD_LM_LOG_TRACE_2( IMRD_LOG_LEVEL_VERBOSE_1, "No more data to read from Queue_Mng: [%s]   Queue: [%s] !", Medium_Ptr->MQS.Queue_Mng, Medium_Ptr->MQS.Queue_Name);

		return( IMRS_NO_DATA);
	    }
	    else
	    {
		IMRD_LM_LOG_TRACE_2( IMRD_LOG_LEVEL_VERBOSE_1, "Signal received during reading from Queue_Mng: [%s]   Queue: [%s] !", Medium_Ptr->MQS.Queue_Mng, Medium_Ptr->MQS.Queue_Name);

		return( IMRS_SIGNAL);
	    }
	}
	else
	{
	    if( reason_code == MQRC_TRUNCATED_MSG_FAILED)
	    {
		IMRD_LM_LOG_ERROR_2( "MQGET failed: truncated message   rc: [%d]   for queue: [%s] !", reason_code, Medium_Ptr->MQS.Queue_Name);

		return( IMRS_KO);
	    }
	    else
	    {
		IMRD_LM_LOG_ERROR_2( "MQGET failed:    rc: [%d]   for queue: [%s] !", reason_code, Medium_Ptr->MQS.Queue_Name);

		return( IMRS_KO);
	    }
	}
    }
    else
    {
	if( comp_code == MQCC_FAILED)
	{
	    IMRD_LM_LOG_ERROR_2( "MQGET failed: ???   rc: [%d]   for queue: [%s] !", reason_code, Medium_Ptr->MQS.Queue_Name);
	    
	    return( IMRS_KO);
	}
	else
	{
	    
	    buffer_ptr[ buffer_len] = '\0';            /* add terminator */
	    
	    Buffer_Ptr->Data_Buf_Ptr  = (IMRT_Buffer_Data *)buffer_ptr;
	    Buffer_Ptr->Data_Buf_Len  = (size_t)buffer_len;
	    Buffer_Ptr->Data_Buf_Size = IMRD_MQS_BUFFER_SIZE;
	    Buffer_Ptr->Status        = IMRD_BUFFER_STATUS_ALLOCATED;
	    
	    *Read_Len_Ptr =  Buffer_Ptr->Data_Buf_Len;

	    if( IMR_SDM_Fmt_Id_Get( &( Buffer_Ptr->Fmt_Id), "Fixed") != IMRS_OK)
	    {
		IMRD_LM_LOG_ERROR_1( "Can't get FmtId: (%d)  !", Buffer_Ptr->Fmt_Id);
		
		return( IMRS_KO);
	    }
	    else
	    {
		IMRD_LM_LOG_TRACE_3( IMRD_LOG_LEVEL_VERBOSE_1, "Read (%d) bytes from Queue_Mng: [%s]    Queue: [%s] !", Buffer_Ptr->Data_Buf_Len, Medium_Ptr->MQS.Queue_Mng, Medium_Ptr->MQS.Queue_Name);
		
		return( IMRS_OK);
	    }
	}
    }
}




/*----------------------------------------------------------------------------*/
/* IO_Hdl_MQS_Write                                                          */
/*----------------------------------------------------------------------------*/
/* Medium_Ptr:      Pointer to the medium to write data to                    */
/* Write_Len_Ptr:   Number of byte writen                                     */
/* Buffer_Ptr:      Buffer to read                                            */
/*----------------------------------------------------------------------------*/

IMRT_Status  IO_Hdl_MQS_Write( IMRT_Medium  *Medium_Ptr, size_t  *Write_Len_Ptr, IMRT_Buffer  *Buffer_Ptr)
{
    MQLONG       comp_code;                        /* completion code               */
    MQLONG       reason_code;                      /* reason code                   */
    MQPMO        pmo          = {MQPMO_DEFAULT};   /* put message options           */
    MQMD         md           = {MQMD_DEFAULT};    /* Message Descriptor            */


    memcpy( md.Format, MQFMT_STRING, (size_t)MQ_FORMAT_LENGTH);

    pmo.Options |= MQPMO_NEW_MSG_ID | MQPMO_NEW_CORREL_ID;

    if( Medium_Ptr->Tran_Flag == IMRD_MEDIUM_TRAN_FLAG_IN)
    {
	pmo.Options |= MQPMO_SYNCPOINT;
    }

    MQPUT( Medium_Ptr->MQS.Conn_Hdl, Medium_Ptr->MQS.Obj_Hdl, &md, &pmo, (MQLONG)( Buffer_Ptr->Data_Buf_Len), Buffer_Ptr->Data_Buf_Ptr, &comp_code, &reason_code);

    if( reason_code != MQRC_NONE)
    {
	IMRD_LM_LOG_ERROR_2( "MQPUT failed: (%d)   for queue: [%s] !", reason_code, Medium_Ptr->MQS.Queue_Name);

	return( IMRS_KO);
    }
    else
    {
	*Write_Len_Ptr =  Buffer_Ptr->Data_Buf_Len;
	IMRD_LM_LOG_TRACE_3( IMRD_LOG_LEVEL_VERBOSE_1, "Wrote (%d) bytes to Queue_Mng: [%s]    Queue: [%s]!", *Write_Len_Ptr, Medium_Ptr->MQS.Queue_Mng, Medium_Ptr->MQS.Queue_Name);

	return( IMRS_OK);
    }
}



/*----------------------------------------------------------------------------*/
/* IO_Hdl_MQS_Begin                                                           */
/*----------------------------------------------------------------------------*/
/* Medium_Ptr:      Pointer to the medium                                     */
/*----------------------------------------------------------------------------*/

IMRT_Status  IO_Hdl_MQS_Begin( IMRT_Medium  *Medium_Ptr)
{

    return(IMRS_OK);
 
}



/*----------------------------------------------------------------------------*/
/* IO_Hdl_MQS_Commit                                                          */
/*----------------------------------------------------------------------------*/
/* Medium_Ptr:      Pointer to the medium                                     */
/*----------------------------------------------------------------------------*/


IMRT_Status  IO_Hdl_MQS_Commit( IMRT_Medium  *Medium_Ptr)
{
    MQLONG       comp_code;                        /* completion code               */
    MQLONG       reason_code;                      /* reason code                   */
    IMRT_Status  status;

    status = IMRS_OK; 

    MQCMIT(Medium_Ptr->MQS.Conn_Hdl,
	   &comp_code,
	   &reason_code);
    if( reason_code != MQRC_NONE)
    {
	IMRD_LM_LOG_ERROR_2( "MQCMIT failed: (%d)   for queue: [%s] !", reason_code, Medium_Ptr->MQS.Queue_Name);
	status = IMRS_KO;
    }
    return(status);
 
}

/*----------------------------------------------------------------------------*/
/* IO_Hdl_MQS_Rollback                                                        */
/*----------------------------------------------------------------------------*/
/* Medium_Ptr:      Pointer to the medium                                     */
/*----------------------------------------------------------------------------*/

IMRT_Status  IO_Hdl_MQS_Rollback( IMRT_Medium  *Medium_Ptr)
{
    MQLONG       comp_code;                        /* completion code               */
    MQLONG       reason_code;                      /* reason code                   */
    IMRT_Status  status;

    status = IMRS_OK; 
 
    MQBACK(Medium_Ptr->MQS.Conn_Hdl,
	   &comp_code,
	   &reason_code);
    if( reason_code != MQRC_NONE)
    {
	IMRD_LM_LOG_ERROR_2( "MQBACK failed: (%d)   for queue: [%s] !", reason_code, Medium_Ptr->MQS.Queue_Name);
	status = IMRS_KO;
    }
 
    return(status);
 
}
