/*----------------------------------------------------------------------------*/
/* $Workfile:   libio_mng.c  $                                                */                                            
/*----------------------------------------------------------------------------*/
/* $Author: agibert $                                                      */
/* $Date: 2008/11/12 02:25:22 $                                           */
/* $Revision: 1.1 $                                                       */
/* $Label:  $                                                         */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#define _LIBIO_MNG_
#define _LIBIO_MNG_C_

#include "libio_mng.h"







/******************************************************************************/
/* IO Manager API                                                             */
/******************************************************************************/



/*----------------------------------------------------------------------------*/
/* Open IO Manager                                                            */
/*----------------------------------------------------------------------------*/
/* Lib_Flag:   Setup IOM init mode                                            */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_IOM_Library_Open( IMRT_IOM_Config  *Config_Ptr)
{
    if( IMRG_IOM_Module.Lib.Status != IMRD_LIB_STATUS_CLOSED)
    {
	IMRD_LM_LOG_ERROR_0( "Library not closed !");

	return( IMRS_KO);
    }
    else
    {
	IMRG_IOM_Module.Lib.Status = IMRD_LIB_STATUS_OPENED;

	IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_1, "Library opened !");

	return( IMRS_OK);
    }
}





/*----------------------------------------------------------------------------*/
/* Close IO Manager                                                           */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_IOM_Library_Close()
{
    if( IMRG_IOM_Module.Lib.Status != IMRD_LIB_STATUS_OPENED)
    {
	IMRD_LM_LOG_ERROR_0( "Library not opened !");

	return( IMRS_KO);
    }
    else
    {
	IMRG_IOM_Module.Lib.Status = IMRD_LIB_STATUS_CLOSED;

	IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_1, "Library closed !");

	return( IMRS_OK);
    }
}





/*----------------------------------------------------------------------------*/
/* Parse Medium URL                                                           */
/*----------------------------------------------------------------------------*/
/* Hdl_Name:      Handler name parsed                                         */
/* Medium_Path:   Medium path parsed                                          */
/* Medium_URL:    Medium URL to parse                                         */
/*----------------------------------------------------------------------------*/

IMRT_Status  IOM_URL_Parse( IMRT_IO_Hdl_Name Hdl_Name, IMRT_Medium_Path  Medium_Path, IMRT_Medium_URL  Medium_URL)
{
    char  *ptr;


    if( ( ( ptr = strstr( Medium_URL, "://")) == NULL) || ( ptr == Medium_URL) || ( ( ptr - Medium_URL + 3) == strlen( Medium_URL)))
    {
	return( IMRS_KO);
    }

    strncpy( Hdl_Name, Medium_URL, ( ptr - Medium_URL));
    Hdl_Name[ ptr - Medium_URL] = '\0';

    strcpy( Medium_Path, ( ptr + 3));

    return( IMRS_OK);
}




	
/*----------------------------------------------------------------------------*/
/* Get Medium Type from Handler Name                                          */
/*----------------------------------------------------------------------------*/
/* Medium_Type_Ptr:   Medium type to return                                   */
/* Handler_Name:      IO Handler Name                                         */
/*----------------------------------------------------------------------------*/

IMRT_Status  IOM_Medium_Type_Get( IMRT_Medium_Type  *Medium_Type_Ptr,  IMRT_IO_Hdl_Name  Hdl_Name)
{
    IMRT_Medium_Type  i;


    for( i = 0; ( i < IMRD_IO_HDL_NB) && ( strcmp( IMRG_IOM_Base.IO_Hdl_Tab[i].Name, Hdl_Name) != 0) ; i++);

    if( i < IMRD_IO_HDL_NB)
    {
	*Medium_Type_Ptr = i;
	return( IMRS_OK);
    }
    else
    {
	return( IMRS_KO);
    }
}





/*----------------------------------------------------------------------------*/
/* Open Medium                                                                */
/*----------------------------------------------------------------------------*/
/* Medium_Ptr_Ptr:   Reference to the Pointer to the opened medium            */
/* Medium_Name:      Name of the medium                                       */
/* Medium_URL:       Medium URL to open                                       */
/* Appl_Id_Input:    Input application Id link with the medium                */
/* Appl_Id_Output:   Output application Id link with the medium               */
/* Dir_Id:           Medium open direction                                    */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_IOM_Medium_Open( IMRT_Medium  **Medium_Ptr_Ptr, IMRT_Medium_Name  Medium_Name, IMRT_Medium_URL  Medium_URL, IMRT_Appl_Id  Appl_Id_Input, IMRT_Appl_Id  Appl_Id_Output, IMRT_Dir_Id  Dir_Id)
{
    IMRT_Status       status;
    IMRT_IO_Hdl_Name  hdl_name;
    IMRT_Medium_Path  medium_path;
    IMRT_Medium_Type  medium_type;


    if( ( status = IOM_URL_Parse( hdl_name, medium_path, Medium_URL)) != IMRS_OK)
    {
	IMRD_LM_LOG_ERROR_1( "Can't parse URL [%s] !", Medium_URL);

	return( IMRS_KO);
    }
	
    if( ( status = IOM_Medium_Type_Get( &medium_type,  hdl_name)) != IMRS_OK)
    {
	IMRD_LM_LOG_ERROR_1( "Can't get medium type [%s] !", hdl_name);
	
	return( IMRS_KO);
    }

    if( ( *Medium_Ptr_Ptr = malloc( sizeof( IMRT_Medium))) == NULL)
    {
	IMRD_LM_LOG_ERROR_2( "Can't allocate Medium [%s]: (%d) !", Medium_Name, errno);

	return( IMRS_KO);
    }

    strcpy( ( *Medium_Ptr_Ptr)->Name, Medium_Name);
    ( *Medium_Ptr_Ptr)->Type =   medium_type;
    ( *Medium_Ptr_Ptr)->Status = IMRD_MEDIUM_STATUS_CLOSED;
    strcpy( ( *Medium_Ptr_Ptr)->URL, Medium_URL);
    ( *Medium_Ptr_Ptr)->Appl_Id_Input = Appl_Id_Input;
    ( *Medium_Ptr_Ptr)->Appl_Id_Output = Appl_Id_Output;
    ( *Medium_Ptr_Ptr)->Dir_Id = Dir_Id;
    ( *Medium_Ptr_Ptr)->Tran_Flag = IMRD_MEDIUM_TRAN_FLAG_OUT;
    ( *Medium_Ptr_Ptr)->Byte_Nb = 0;
    ( *Medium_Ptr_Ptr)->Msg_Nb = 0;


    if( IMRG_IOM_Base.IO_Hdl_Tab[ medium_type].Open_Ptr( *Medium_Ptr_Ptr, medium_path, Dir_Id) != IMRS_OK)
    {
	IMRD_LM_LOG_ERROR_1( "Can't open Medium [%s] !", Medium_Name);
	free( *Medium_Ptr_Ptr);

	return( IMRS_KO);
    }
    else
    {
	( *Medium_Ptr_Ptr)->Status = IMRD_MEDIUM_STATUS_OPENED;

	return( IMRS_OK);
    }
}    
   




/*----------------------------------------------------------------------------*/
/* Close Medium                                                               */
/*----------------------------------------------------------------------------*/
/* Medium_Ptr:   Pointer to the medium to close                               */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_IOM_Medium_Close( IMRT_Medium  *Medium_Ptr)
{
    if(  Medium_Ptr->Status == IMRD_MEDIUM_STATUS_OPENED)
    {
	if( IMRG_IOM_Base.IO_Hdl_Tab[ Medium_Ptr->Type].Close_Ptr( Medium_Ptr) != IMRS_OK)
	{
	    IMRD_LM_LOG_ERROR_0( "Can't close Medium !");

	    return( IMRS_KO);
	}
    }

    IMRD_LM_LOG_INFO_4( "Medium [%s]: (%d) bytes %s in (%d) messages.", Medium_Ptr->Name, Medium_Ptr->Byte_Nb, ( ( Medium_Ptr->Dir_Id == IMRD_DIR_ID_IN) ? "read" : "writen"), Medium_Ptr->Msg_Nb);

    free( Medium_Ptr);

    return( IMRS_OK);
}





/*----------------------------------------------------------------------------*/
/* Read Medium                                                                */
/*----------------------------------------------------------------------------*/
/* Buffer_Ptr:   Pointer to the buffer to write                               */
/* Medium_Ptr:   Pointer to the medium to read                                */
/* Read_Mode:    Read mode                                                    */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_IOM_Medium_Read( IMRT_Buffer  *Buffer_Ptr, IMRT_Medium  *Medium_Ptr, IMRT_Read_Mode  Read_Mode)
{
    size_t       read_len;
    IMRT_Status  status;


    if(  Medium_Ptr->Status == IMRD_MEDIUM_STATUS_OPENED)
    {
	if( ( status = IMRG_IOM_Base.IO_Hdl_Tab[ Medium_Ptr->Type].Read_Ptr( Buffer_Ptr, &read_len, Medium_Ptr, Read_Mode)) == IMRS_KO)
	{
	    IMRD_LM_LOG_ERROR_1( "Can't read Medium [%s] !", Medium_Ptr->Name);

	    return( IMRS_KO);
	}

	if( status == IMRS_OK)
	{
	    Medium_Ptr->Byte_Nb += read_len;
	    Medium_Ptr->Msg_Nb++;

	    Buffer_Ptr->Appl_Id_Input  = Medium_Ptr->Appl_Id_Input;
	    Buffer_Ptr->Appl_Id_Output = Medium_Ptr->Appl_Id_Output;
	    Buffer_Ptr->Dir_Id         = Medium_Ptr->Dir_Id;
	    Buffer_Ptr->Flag           = IMRD_BUFFER_FLAG_CLOSED;
	}
    }
    else
    {
	IMRD_LM_LOG_ERROR_1( "Can't read closed Medium [%s] !", Medium_Ptr->Name);

	return( IMRS_KO);
    }

    return( status);
}





/*----------------------------------------------------------------------------*/
/* Write Medium                                                               */
/*----------------------------------------------------------------------------*/
/* Medium_Ptr:   Pointer to the medium to write                               */
/* Buffer_Ptr:   Pointer to the buffer to read                                */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_IOM_Medium_Write( IMRT_Medium  *Medium_Ptr, IMRT_Buffer  *Buffer_Ptr)
{
    size_t  write_len;


    if(  Medium_Ptr->Status == IMRD_MEDIUM_STATUS_OPENED)
    {
	if( IMRG_IOM_Base.IO_Hdl_Tab[ Medium_Ptr->Type].Write_Ptr( Medium_Ptr, &write_len, Buffer_Ptr) != IMRS_OK)
	{
	    IMRD_LM_LOG_ERROR_1( "Can't write Medium [%s]!", Medium_Ptr->Name);

	    return( IMRS_KO);
	}

	Medium_Ptr->Byte_Nb += write_len;
	Medium_Ptr->Msg_Nb++;
    }
    else
    {
	IMRD_LM_LOG_ERROR_1( "Can't write closed Medium [%s]!", Medium_Ptr->Name);

	return( IMRS_KO);
    }

    return( IMRS_OK);
}






/*----------------------------------------------------------------------------*/
/* Tran Begin                                                                 */
/*----------------------------------------------------------------------------*/
/* Tran_Ptr_Ptr:   Reference to the pointer of the transaction to begin       */
/* Medium_Count:   Number of medium to enclose into the transaction           */
/* Medium_1_Ptr:   Pointer to the first medium                                */
/* ...                                                                        */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_IOM_Tran_Begin( IMRT_Tran  **Tran_Ptr_Ptr, IMRT_Medium_Nb  Medium_Nb, IMRT_Medium  *Medium_1_Ptr, ...)
{
    int           index;
    va_list       medium_list;
    IMRT_Medium  *medium_ptr;
    IMRT_Status   status;


    status = IMRS_OK;
    medium_ptr = NULL;

    if( Medium_Nb != 0)
    {
	va_start( medium_list, Medium_1_Ptr);

	
	if( ( *Tran_Ptr_Ptr = (IMRT_Tran  *)malloc( sizeof( IMRT_Tran))) == NULL)
	{
	    IMRD_LM_LOG_ERROR_1( "Can't malloc Tran: (%d) !", errno);
	    return( IMRS_KO);
	}

	if( ( (*Tran_Ptr_Ptr)->Medium_Ptr_Tab = (IMRT_Medium  **)malloc( sizeof(IMRT_Medium  **) * Medium_Nb)) == NULL)
	{
	    IMRD_LM_LOG_ERROR_1( "Can't malloc Tran Medium_Ptr_Tab: (%d) !", errno);
	    return( IMRS_KO);
	}

	(*Tran_Ptr_Ptr)->Medium_Nb  = Medium_Nb;
	
	for( index = 0; index < Medium_Nb; index++)
	{
	    if( index == 0)
	    {
		( ( *Tran_Ptr_Ptr)->Medium_Ptr_Tab)[index] = Medium_1_Ptr;

		medium_ptr = Medium_1_Ptr;
	    }
	    else
	    {
		medium_ptr = (IMRT_Medium  *)va_arg( medium_list, IMRT_Medium  *);
		if( medium_ptr != NULL)
		{
		    (*Tran_Ptr_Ptr)->Medium_Ptr_Tab[index] = medium_ptr;
		}
		else
		{
		    IMRD_LM_LOG_ERROR_0( "Can't get medium !");
		    status = IMRS_KO;
		}
	    }

	    if( status == IMRS_OK)
	    {	
		status = IMRG_IOM_Base.IO_Hdl_Tab[medium_ptr->Type].Begin_Ptr( medium_ptr);

		if( status != IMRS_OK)
		{
		    IMRD_LM_LOG_ERROR_1( "Begin failed for Medium [%s] !", medium_ptr->Name);
		}
		else
		{
		    (*Tran_Ptr_Ptr)->Medium_Ptr_Tab[index]->Tran_Flag =  IMRD_MEDIUM_TRAN_FLAG_IN;
		}
	    }	

	}
	va_end( medium_list);
    }
    else
    {
	IMRD_LM_LOG_ERROR_1( "BEGIN : Invalid number of medium (%d) !",Medium_Nb );
	status = IMRS_KO ;
    }

    return( status);
}





/*----------------------------------------------------------------------------*/
/* Tran Commit                                                                */
/*----------------------------------------------------------------------------*/
/* Tran_Ptr:   Pointer to the transaction to commit                           */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_IOM_Tran_Commit( IMRT_Tran  *Tran_Ptr)
{
    IMRT_Status  status;
    int          index;


    status = IMRS_OK;

    IMRD_LM_LOG_TRACE_1( IMRD_LOG_LEVEL_VERBOSE_1, "Medium Nb to commit: (%d)", Tran_Ptr->Medium_Nb );

    for( index = 0; index < Tran_Ptr->Medium_Nb; index++)
    {
	IMRD_LM_LOG_TRACE_2( IMRD_LOG_LEVEL_VERBOSE_1, "Medium [%s] Tran_Flag: (%d)", Tran_Ptr->Medium_Ptr_Tab[index]->Name, Tran_Ptr->Medium_Ptr_Tab[index]->Tran_Flag);

	if( Tran_Ptr->Medium_Ptr_Tab[index]->Tran_Flag == IMRD_MEDIUM_TRAN_FLAG_IN)
	{
	    IMRD_LM_LOG_TRACE_1( IMRD_LOG_LEVEL_VERBOSE_1, "Commiting medium [%s]...",Tran_Ptr->Medium_Ptr_Tab[index]->Name); 

	    status = IMRG_IOM_Base.IO_Hdl_Tab[Tran_Ptr->Medium_Ptr_Tab[index]->Type].Commit_Ptr(Tran_Ptr->Medium_Ptr_Tab[index]);

	    if( status == IMRS_OK)
	    {
		Tran_Ptr->Medium_Ptr_Tab[index]->Tran_Flag = IMRD_MEDIUM_TRAN_FLAG_OUT;
	    }
	    else
	    {
		IMRD_LM_LOG_ERROR_1( "Commit failed for medium [%s] !",  Tran_Ptr->Medium_Ptr_Tab[index]->Name);
	    }
	}    

    }

    if( Tran_Ptr->Medium_Ptr_Tab != NULL)
    {
	free( Tran_Ptr->Medium_Ptr_Tab);
    }

    free( Tran_Ptr);

    return( status);
}





/*----------------------------------------------------------------------------*/
/* Tran Rollback                                                              */
/*----------------------------------------------------------------------------*/
/* Tran_Ptr:   Pointer to the transaction to rollback                         */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_IOM_Tran_Rollback( IMRT_Tran  *Tran_Ptr)
{

    IMRT_Status  status;
    int          index;


    status = IMRS_OK;

    IMRD_LM_LOG_TRACE_1( IMRD_LOG_LEVEL_VERBOSE_1, "Medium Nb to rollback: (%d)", Tran_Ptr->Medium_Nb );

    for( index = 0; index < Tran_Ptr->Medium_Nb; index++)
    {
	IMRD_LM_LOG_TRACE_2( IMRD_LOG_LEVEL_VERBOSE_1, "Medium [%s] Tran_Flag: (%d)", Tran_Ptr->Medium_Ptr_Tab[index]->Name, Tran_Ptr->Medium_Ptr_Tab[index]->Tran_Flag);

	if( Tran_Ptr->Medium_Ptr_Tab[index]->Tran_Flag == IMRD_MEDIUM_TRAN_FLAG_IN)
        {
	    status = IMRG_IOM_Base.IO_Hdl_Tab[Tran_Ptr->Medium_Ptr_Tab[index]->Type].Rollback_Ptr(Tran_Ptr->Medium_Ptr_Tab[index]);

	    if(status == IMRS_OK)
	    {
		Tran_Ptr->Medium_Ptr_Tab[index]->Tran_Flag = IMRD_MEDIUM_TRAN_FLAG_OUT;
	    }
	    else
            {
		IMRD_LM_LOG_ERROR_1( "Rollback failed for medium [%s] !", Tran_Ptr->Medium_Ptr_Tab[index]->Name);
	    }
	}
    }

    if( Tran_Ptr->Medium_Ptr_Tab != NULL)
    {
	free( Tran_Ptr->Medium_Ptr_Tab);
    }

    free( Tran_Ptr);

    return( status);
}





