/*----------------------------------------------------------------------------*/
/* File: libio_mng.h 				         		      */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#ifndef _LIBIO_MNG_H_
#define _LIBIO_MNG_H_



/*----------------------------------------------------------------------------*/
/* Global Include                                                             */
/*----------------------------------------------------------------------------*/

#include <imr.h>





/*----------------------------------------------------------------------------*/
/* Lib Definitions                                                            */
/*----------------------------------------------------------------------------*/

#define IMRD_IOM_MODULE_ID                ( IMRT_Module_Id) 3
#define IMRD_IOM_MODULE_NAME            ( IMRT_Module_Name) "iom"
#define IMRD_IOM_MODULE_NAME_LONG  ( IMRT_Module_Name_Long) "IO Manager"
#define IMRD_IOM_MODULE_VERSION      ( IMRT_Module_Version) "@(#) VERSIONIMR: $Label:  $   $Date: 2008/11/12 02:25:22 $   $Workfile:   libio_mng.h  $"

#define IMRD_MODULE_PTR                                     &IMRG_IOM_Module





/*----------------------------------------------------------------------------*/
/* Local Definitions                                                          */
/*----------------------------------------------------------------------------*/

#define IMRD_IO_HDL_NAME_LEN   (short) 255
#define IMRD_IO_HDL_NAME_SIZE  (short) ( IMRD_IO_HDL_NAME_LEN + 1)

typedef char   IMRT_IO_Hdl_Name[ IMRD_IO_HDL_NAME_SIZE];



typedef IMRT_Status  ( IMRT_IO_Hdl_Open) ( IMRT_Medium  *, IMRT_Medium_Path, IMRT_Dir_Id);
typedef IMRT_Status  ( IMRT_IO_Hdl_Close)( IMRT_Medium  *);
typedef IMRT_Status  ( IMRT_IO_Hdl_Read) ( IMRT_Buffer  *, size_t  *, IMRT_Medium  *, IMRT_Read_Mode);
typedef IMRT_Status  ( IMRT_IO_Hdl_Write)( IMRT_Medium  *, size_t  *, IMRT_Buffer  *);
typedef IMRT_Status  ( IMRT_IO_Hdl_Begin) ( IMRT_Medium  *);
typedef IMRT_Status  ( IMRT_IO_Hdl_Commit) ( IMRT_Medium  *);
typedef IMRT_Status  ( IMRT_IO_Hdl_Rollback)( IMRT_Medium  *);



typedef struct IMRT_IO_Hdl
{
	IMRT_IO_Hdl_Name     Name;
	IMRT_IO_Hdl_Open     *Open_Ptr;
	IMRT_IO_Hdl_Close    *Close_Ptr;
	IMRT_IO_Hdl_Read     *Read_Ptr;
	IMRT_IO_Hdl_Write    *Write_Ptr;
        IMRT_IO_Hdl_Begin    *Begin_Ptr;
	IMRT_IO_Hdl_Commit   *Commit_Ptr;
	IMRT_IO_Hdl_Rollback *Rollback_Ptr;
 
	
}  IMRT_IO_Hdl;



typedef short  IMRT_IO_Hdl_Nb;

#define IMRD_IO_HDL_NB  ( IMRT_IO_Hdl_Nb) 4



typedef struct IMRT_IO_Base
{
	IMRT_IO_Hdl  IO_Hdl_Tab[ IMRD_IO_HDL_NB];
}  IMRT_IO_Base;





/*----------------------------------------------------------------------------*/
/* Prototypes                                                                 */
/*----------------------------------------------------------------------------*/

#   ifdef _LIBIO_MNG_C_

IMRT_Status  IOM_URL_Parse( IMRT_IO_Hdl_Name, IMRT_Medium_Path, IMRT_Medium_URL);
IMRT_Status  IOM_Medium_Type_Get( IMRT_Medium_Type  *,  IMRT_IO_Hdl_Name);

#   endif





/*----------------------------------------------------------------------------*/
/* Local Includes                                                             */
/*----------------------------------------------------------------------------*/

#include "io_hdl_file.h"

#ifdef TUX_SUPPORT
#   include "io_hdl_tux.h"
#endif /* TUX_SUPPORT */

#ifdef MQS_SUPPORT
#   include "io_hdl_mqs.h"
#endif /* MQS_SUPPORT */



/*----------------------------------------------------------------------------*/
/* Global Variables                                                           */
/*----------------------------------------------------------------------------*/

#   ifdef _LIBIO_MNG_C_

IMRT_IO_Base  IMRG_IOM_Base = 
{
    {
	{ "Unknown",         NULL,              NULL,              NULL,             NULL, NULL, NULL, NULL},
	{ "file", IO_Hdl_File_Open, IO_Hdl_File_Close, IO_Hdl_File_Read, IO_Hdl_File_Write,IO_Hdl_File_Begin,IO_Hdl_File_Commit,IO_Hdl_File_Rollback },

#ifdef TUX_SUPPORT
	{ "tux",  IO_Hdl_Tux_Open,  IO_Hdl_Tux_Close,  IO_Hdl_Tux_Read,  IO_Hdl_Tux_Write, IO_Hdl_Tux_Begin, IO_Hdl_Tux_Commit, IO_Hdl_Tux_Rollback},
#endif /* TUX_SUPPORT */

#ifdef MQS_SUPPORT
	{ "mqs",  IO_Hdl_MQS_Open,  IO_Hdl_MQS_Close,  IO_Hdl_MQS_Read,  IO_Hdl_MQS_Write, IO_Hdl_MQS_Begin, IO_Hdl_MQS_Commit, IO_Hdl_MQS_Rollback},
#endif /* MQS_SUPPORT */
    }
};



IMRT_Module  IMRG_IOM_Module = 
{
    IMRD_IOM_MODULE_ID,
    IMRD_IOM_MODULE_NAME,
    IMRD_IOM_MODULE_NAME_LONG,
    IMRD_IOM_MODULE_VERSION,
    IMRD_MODULE_TYPE_LIBRARY,
    {
	IMRD_LIB_STATUS_CLOSED,
	( IMRT_Lib_Open    *) &IMR_IOM_Library_Open,
	( IMRT_Lib_Close   *) &IMR_IOM_Library_Close
    }
};

#   else

extern IMRT_IO_Base  IMRG_IOM_Base;
extern IMRT_Module   IMRG_IOM_Module;

#   endif





/*----------------------------------------------------------------------------*/

#endif
