/*----------------------------------------------------------------------------*/
/* File: log_mng.h 				         		      */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#ifndef _LOG_MNG_H_
#define _LOG_MNG_H_



#include <imr.h>





/*----------------------------------------------------------------------------*/
/* IMRD_LM_API definition                                                     */
/*----------------------------------------------------------------------------*/

#   ifdef _LIBLOG_MNG_
#       define IMRD_LM_API  
#   else
#       define IMRD_LM_API  extern
#   endif





/*----------------------------------------------------------------------------*/
/* Status definition                                                          */
/*----------------------------------------------------------------------------*/

#define IMRS_LM_LOG (IMRT_Status) 2



/*----------------------------------------------------------------------------*/
/* Log Type definition                                                        */
/*----------------------------------------------------------------------------*/

#define IMRD_LOG_TYPE_NB               (short) 5



typedef short  IMRT_Log_Type_Id;

#define IMRD_LOG_TYPE_ID_UNKNOWN  (IMRT_Log_Type_Id) 0
#define IMRD_LOG_TYPE_ID_TRACE    (IMRT_Log_Type_Id) 1
#define IMRD_LOG_TYPE_ID_INFO     (IMRT_Log_Type_Id) 2
#define IMRD_LOG_TYPE_ID_WARNING  (IMRT_Log_Type_Id) 3
#define IMRD_LOG_TYPE_ID_ERROR    (IMRT_Log_Type_Id) 4



typedef char  *IMRT_Log_Type_Name;

#define IMRD_LOG_TYPE_NAME_UNKNOWN  (IMRT_Log_Type_Name) "???"
#define IMRD_LOG_TYPE_NAME_TRACE    (IMRT_Log_Type_Name) "TRA"
#define IMRD_LOG_TYPE_NAME_INFO     (IMRT_Log_Type_Name) "INF"
#define IMRD_LOG_TYPE_NAME_WARNING  (IMRT_Log_Type_Name) "WRN"
#define IMRD_LOG_TYPE_NAME_ERROR    (IMRT_Log_Type_Name) "ERR"





/*----------------------------------------------------------------------------*/
/* Log Level definition                                                       */
/*----------------------------------------------------------------------------*/

typedef short  IMRT_Log_Level;

#define IMRD_LOG_LEVEL_UNSET      ( IMRT_Log_Level) -1

#define IMRD_LOG_LEVEL_MIN        ( IMRT_Log_Level) 0
#define IMRD_LOG_LEVEL_MAX        ( IMRT_Log_Level) 999

#define IMRD_LOG_LEVEL_VERBOSE_0  ( IMRT_Log_Level) 10
#define IMRD_LOG_LEVEL_VERBOSE_1  ( IMRT_Log_Level) 20
#define IMRD_LOG_LEVEL_VERBOSE_2  ( IMRT_Log_Level) 30

#define IMRD_LOG_LEVEL_DEFAULT    ( IMRT_Log_Level) 10





/*----------------------------------------------------------------------------*/
/* IMRT_File_Name definition                                                  */
/*----------------------------------------------------------------------------*/

typedef char  *IMRT_File_Name;





/*----------------------------------------------------------------------------*/
/* IMRT_Line_Id definition                                                    */
/*----------------------------------------------------------------------------*/

typedef long  IMRT_Line_Id;





/*----------------------------------------------------------------------------*/
/* Log Fmt definition                                                         */
/*----------------------------------------------------------------------------*/

typedef char  *IMRT_Log_Fmt;





/*----------------------------------------------------------------------------*/
/* IMRT_Log_Id_Bit definition                                                 */
/*----------------------------------------------------------------------------*/

typedef size_t  IMRT_Log_Id_Bit_Offset;
typedef size_t  IMRT_Log_Id_Bit_Size;



typedef struct IMRT_Log_Id_Bit
{
	IMRT_Log_Id_Bit_Offset  Offset;
	IMRT_Log_Id_Bit_Size    Size;
}  IMRT_Log_Id_Bit;





/*----------------------------------------------------------------------------*/
/* IMRT_Log_Id_Struct definition                                              */
/*----------------------------------------------------------------------------*/

typedef short   IMRT_Bit_Nb;
typedef size_t  IMRT_Log_Size;



typedef struct IMRT_Log_Id_Struct
{
	IMRT_Bit_Nb       Bit_Nb;
	IMRT_Log_Size     Log_Size;
        IMRT_Log_Id_Bit  *Log_Id_Bit_Ptr;
} IMRT_Log_Id_Struct;





/*----------------------------------------------------------------------------*/
/* IMRT_Log_Msg_Header definition                                             */
/*----------------------------------------------------------------------------*/

typedef char  *IMRT_Log_Id_Header;





/*----------------------------------------------------------------------------*/
/* LM_Config definition                                                       */
/*----------------------------------------------------------------------------*/

typedef struct IMRT_LM_Config
{
	IMRT_Path       Log_File;
	IMRT_Module_Id  Module_Nb;
	IMRT_Log_Level  Level_Tab[ IMRD_LOG_TYPE_NB][ IMRD_MODULE_NB_MAX];
}  IMRT_LM_Config;







/*----------------------------------------------------------------------------*/
/* Global Variables                                                           */
/*----------------------------------------------------------------------------*/

#   ifdef _LIBLOG_MNG_

#   else

extern IMRT_Module  IMRG_LM_Module;

#   endif







/******************************************************************************/
/* Log Manager API                                                            */
/******************************************************************************/

/*----------------------------------------------------------------------------*/
/* Log Trace Macros                                                           */
/*----------------------------------------------------------------------------*/

#define IMRD_LM_LOG_TRACE_0( Level, Fmt)                                  if( IMR_LM_Log_Write( IMRD_LOG_TYPE_ID_TRACE,   (Level), IMRD_MODULE_PTR, __FILE__, __LINE__, IMRD_TRUE, (Fmt)) != IMRS_OK) return( IMRS_KO);
#define IMRD_LM_LOG_TRACE_1( Level, Fmt, Arg1)                            if( IMR_LM_Log_Write( IMRD_LOG_TYPE_ID_TRACE,   (Level), IMRD_MODULE_PTR, __FILE__, __LINE__, IMRD_TRUE, (Fmt), (Arg1)) != IMRS_OK) return(IMRS_KO);
#define IMRD_LM_LOG_TRACE_2( Level, Fmt, Arg1, Arg2)                      if( IMR_LM_Log_Write( IMRD_LOG_TYPE_ID_TRACE,   (Level), IMRD_MODULE_PTR, __FILE__, __LINE__, IMRD_TRUE, (Fmt), (Arg1), (Arg2)) != IMRS_OK) return(IMRS_KO);
#define IMRD_LM_LOG_TRACE_3( Level, Fmt, Arg1, Arg2, Arg3)                if( IMR_LM_Log_Write( IMRD_LOG_TYPE_ID_TRACE,   (Level), IMRD_MODULE_PTR, __FILE__, __LINE__, IMRD_TRUE, (Fmt), (Arg1), (Arg2), (Arg3)) != IMRS_OK) return(IMRS_KO);
#define IMRD_LM_LOG_TRACE_4( Level, Fmt, Arg1, Arg2, Arg3, Arg4)          if( IMR_LM_Log_Write( IMRD_LOG_TYPE_ID_TRACE,   (Level), IMRD_MODULE_PTR, __FILE__, __LINE__, IMRD_TRUE, (Fmt), (Arg1), (Arg2), (Arg3), (Arg4)) != IMRS_OK) return(IMRS_KO);
#define IMRD_LM_LOG_TRACE_5( Level, Fmt, Arg1, Arg2, Arg3, Arg4, Arg5)    if( IMR_LM_Log_Write( IMRD_LOG_TYPE_ID_TRACE,   (Level), IMRD_MODULE_PTR, __FILE__, __LINE__, IMRD_TRUE, (Fmt), (Arg1), (Arg2), (Arg3), (Arg4), (Arg5)) != IMRS_OK) return(IMRS_KO);



/*----------------------------------------------------------------------------*/
/* Log Info Macros                                                            */
/*----------------------------------------------------------------------------*/

#define IMRD_LM_LOG_INFO_0( Fmt)                                   if(IMR_LM_Log_Write( IMRD_LOG_TYPE_ID_INFO,    IMRD_LOG_LEVEL_DEFAULT, IMRD_MODULE_PTR, __FILE__, __LINE__, IMRD_TRUE, (Fmt)) != IMRS_OK ) return(IMRS_KO);
#define IMRD_LM_LOG_INFO_1( Fmt, Arg1)                              if( IMR_LM_Log_Write( IMRD_LOG_TYPE_ID_INFO,    IMRD_LOG_LEVEL_DEFAULT, IMRD_MODULE_PTR, __FILE__, __LINE__, IMRD_TRUE, (Fmt), (Arg1)) != IMRS_OK) return(IMRS_KO) ;
#define IMRD_LM_LOG_INFO_2( Fmt, Arg1, Arg2)                       if( IMR_LM_Log_Write( IMRD_LOG_TYPE_ID_INFO,    IMRD_LOG_LEVEL_DEFAULT, IMRD_MODULE_PTR, __FILE__, __LINE__, IMRD_TRUE, (Fmt), (Arg1), (Arg2)) != IMRS_OK) return(IMRS_KO);
#define IMRD_LM_LOG_INFO_3( Fmt, Arg1, Arg2, Arg3)                 if( IMR_LM_Log_Write( IMRD_LOG_TYPE_ID_INFO,    IMRD_LOG_LEVEL_DEFAULT, IMRD_MODULE_PTR, __FILE__, __LINE__, IMRD_TRUE, (Fmt), (Arg1), (Arg2), (Arg3)) != IMRS_OK) return(IMRS_KO);
#define IMRD_LM_LOG_INFO_4( Fmt, Arg1, Arg2, Arg3, Arg4)           if( IMR_LM_Log_Write( IMRD_LOG_TYPE_ID_INFO,    IMRD_LOG_LEVEL_DEFAULT, IMRD_MODULE_PTR, __FILE__, __LINE__, IMRD_TRUE, (Fmt), (Arg1), (Arg2), (Arg3), (Arg4)) != IMRS_OK) return(IMRS_KO);
#define IMRD_LM_LOG_INFO_5( Fmt, Arg1, Arg2, Arg3, Arg4, Arg5)     if( IMR_LM_Log_Write( IMRD_LOG_TYPE_ID_INFO,    IMRD_LOG_LEVEL_DEFAULT, IMRD_MODULE_PTR, __FILE__, __LINE__, IMRD_TRUE, (Fmt), (Arg1), (Arg2), (Arg3), (Arg4), (Arg5)) != IMRS_OK) return(IMRS_KO);



/*----------------------------------------------------------------------------*/
/* Log Warning Macros                                                         */
/*----------------------------------------------------------------------------*/

#define IMRD_LM_LOG_WARNING_0( Fmt)                                if(IMR_LM_Log_Write( IMRD_LOG_TYPE_ID_WARNING, IMRD_LOG_LEVEL_DEFAULT, IMRD_MODULE_PTR, __FILE__, __LINE__, IMRD_TRUE, (Fmt)) != IMRS_OK) return(IMRS_KO);
#define IMRD_LM_LOG_WARNING_1( Fmt, Arg1)                          if(IMR_LM_Log_Write( IMRD_LOG_TYPE_ID_WARNING, IMRD_LOG_LEVEL_DEFAULT, IMRD_MODULE_PTR, __FILE__, __LINE__, IMRD_TRUE, (Fmt), (Arg1)) != IMRS_OK) return(IMRS_KO);
#define IMRD_LM_LOG_WARNING_2( Fmt, Arg1, Arg2)                    if( IMR_LM_Log_Write( IMRD_LOG_TYPE_ID_WARNING, IMRD_LOG_LEVEL_DEFAULT, IMRD_MODULE_PTR, __FILE__, __LINE__, IMRD_TRUE, (Fmt), (Arg1), (Arg2)) != IMRS_OK) return(IMRS_KO);
#define IMRD_LM_LOG_WARNING_3( Fmt, Arg1, Arg2, Arg3)              if( IMR_LM_Log_Write( IMRD_LOG_TYPE_ID_WARNING, IMRD_LOG_LEVEL_DEFAULT, IMRD_MODULE_PTR, __FILE__, __LINE__, IMRD_TRUE, (Fmt), (Arg1), (Arg2), (Arg3)) != IMRS_OK) return(IMRS_KO);
#define IMRD_LM_LOG_WARNING_4( Fmt, Arg1, Arg2, Arg3, Arg4)        if( IMR_LM_Log_Write( IMRD_LOG_TYPE_ID_WARNING, IMRD_LOG_LEVEL_DEFAULT, IMRD_MODULE_PTR, __FILE__, __LINE__, IMRD_TRUE, (Fmt), (Arg1), (Arg2), (Arg3), (Arg4)) != IMRS_OK) return(IMRS_KO);
#define IMRD_LM_LOG_WARNING_5( Fmt, Arg1, Arg2, Arg3, Arg4, Arg5)  if( IMR_LM_Log_Write( IMRD_LOG_TYPE_ID_WARNING, IMRD_LOG_LEVEL_DEFAULT, IMRD_MODULE_PTR, __FILE__, __LINE__, IMRD_TRUE, (Fmt), (Arg1), (Arg2), (Arg3), (Arg4), (Arg5)) != IMRS_OK) return(IMRS_KO);



/*----------------------------------------------------------------------------*/
/* Log Error Macros                                                           */
/*----------------------------------------------------------------------------*/

#define IMRD_LM_LOG_ERROR_0( Fmt)                                  if( IMR_LM_Log_Write( IMRD_LOG_TYPE_ID_ERROR,   IMRD_LOG_LEVEL_DEFAULT, IMRD_MODULE_PTR, __FILE__, __LINE__, IMRD_TRUE, (Fmt)) != IMRS_OK) return(IMRS_KO);
#define IMRD_LM_LOG_ERROR_1( Fmt, Arg1)                            if(IMR_LM_Log_Write( IMRD_LOG_TYPE_ID_ERROR,   IMRD_LOG_LEVEL_DEFAULT, IMRD_MODULE_PTR, __FILE__, __LINE__, IMRD_TRUE, (Fmt), (Arg1)) != IMRS_OK) return(IMRS_KO) ;
#define IMRD_LM_LOG_ERROR_2( Fmt, Arg1, Arg2)                      if( IMR_LM_Log_Write( IMRD_LOG_TYPE_ID_ERROR,   IMRD_LOG_LEVEL_DEFAULT, IMRD_MODULE_PTR, __FILE__, __LINE__, IMRD_TRUE, (Fmt), (Arg1), (Arg2)) != IMRS_OK) return(IMRS_KO);
#define IMRD_LM_LOG_ERROR_3( Fmt, Arg1, Arg2, Arg3)                if( IMR_LM_Log_Write( IMRD_LOG_TYPE_ID_ERROR,   IMRD_LOG_LEVEL_DEFAULT, IMRD_MODULE_PTR, __FILE__, __LINE__, IMRD_TRUE, (Fmt), (Arg1), (Arg2), (Arg3)) != IMRS_OK) return(IMRS_KO);
#define IMRD_LM_LOG_ERROR_4( Fmt, Arg1, Arg2, Arg3, Arg4)          if(IMR_LM_Log_Write( IMRD_LOG_TYPE_ID_ERROR,   IMRD_LOG_LEVEL_DEFAULT, IMRD_MODULE_PTR, __FILE__, __LINE__, IMRD_TRUE, (Fmt), (Arg1), (Arg2), (Arg3), (Arg4)) != IMRS_OK) return(IMRS_KO);
#define IMRD_LM_LOG_ERROR_5( Fmt, Arg1, Arg2, Arg3, Arg4, Arg5)    if( IMR_LM_Log_Write( IMRD_LOG_TYPE_ID_ERROR,   IMRD_LOG_LEVEL_DEFAULT, IMRD_MODULE_PTR, __FILE__, __LINE__, IMRD_TRUE, (Fmt), (Arg1), (Arg2), (Arg3), (Arg4), (Arg5)) != IMRS_OK ) return(IMRS_KO);



/*----------------------------------------------------------------------------*/
/* Exec Trace Macros                                                          */
/*----------------------------------------------------------------------------*/

#define IMRD_LM_STREAM_OUT  stream_out

#define IMRD_LM_EXEC_TRACE( Level, Msg_Pre, Func, Msg_Post)		\
    {									\
	FILE  *stream_out;						\
									\
									\
	if( IMR_LM_Log_Trigger( IMRD_LOG_TYPE_ID_TRACE, (Level), IMRD_MODULE_PTR) == IMRS_LM_LOG) \
	{								\
	    stream_out = IMR_LM_Stream_Out_Get();			\
									\
	    IMR_LM_Log_Write( IMRD_LOG_TYPE_ID_TRACE,   (Level), IMRD_MODULE_PTR, __FILE__, __LINE__, IMRD_FALSE, (Msg_Pre)); \
	    (Func);							\
	    fprintf( stream_out, "%s%s", (Msg_Post), "\n");		\
	    fflush( stream_out);					\
	}								\
    }



/*----------------------------------------------------------------------------*/
/* Log Buffer Id Macro                                                        */
/*----------------------------------------------------------------------------*/

#define IMRD_LM_LOG_BUFFER_ID( Buffer_Ptr, Log_Id_Struct_Ptr, Log_Id_Header)  if( IMR_LM_Log_Buffer_Id( (Buffer_Ptr), (Log_Id_Struct_Ptr), (Log_Id_Header), IMRD_LOG_TYPE_ID_INFO, IMRD_LOG_LEVEL_DEFAULT, IMRD_MODULE_PTR, __FILE__, __LINE__, IMRD_TRUE) != IMRS_OK) return(IMRS_KO);





/*----------------------------------------------------------------------------*/
/* Open Log Manager                                                           */
/*----------------------------------------------------------------------------*/
/* LM_Config_Ptr:   Setup LM init mode                                        */
/*----------------------------------------------------------------------------*/

IMRD_LM_API IMRT_Status  IMR_LM_Library_Open( IMRT_LM_Config  *);



/*----------------------------------------------------------------------------*/
/* Close Log Manager                                                          */
/*----------------------------------------------------------------------------*/

IMRD_LM_API IMRT_Status  IMR_LM_Library_Close( void);



/*----------------------------------------------------------------------------*/
/* LM_Stream_Out_Get                                                          */
/*----------------------------------------------------------------------------*/

IMRD_LM_API FILE  *IMR_LM_Stream_Out_Get( void);



/*----------------------------------------------------------------------------*/
/* LM_Log_Trigger                                                             */
/*----------------------------------------------------------------------------*/
/* Log_Type_Id:  Type id of log                                               */
/* Log_Level:    Level of log                                                 */
/* Module_Ptr:   Loger module pointer                                         */
/*----------------------------------------------------------------------------*/

IMRD_LM_API IMRT_Status  IMR_LM_Log_Trigger( IMRT_Log_Type_Id, IMRT_Log_Level, IMRT_Module  *);



/*----------------------------------------------------------------------------*/
/* LM_Log_Write                                                               */
/*----------------------------------------------------------------------------*/
/* Log_Type_Id:  Type id of log                                               */
/* Log_Level:    Level of log                                                 */
/* Module_Ptr:   Loger module pointer                                         */
/* File_Name:    Loger file name                                              */
/* Line_Id:      Loger line id                                                */
/* LF_Flag:      Add or not a LF at EOF                                       */
/* Log_Fmt:      Format string a la printf                                    */
/* ...:          Optional args                                                */
/*----------------------------------------------------------------------------*/

IMRD_LM_API IMRT_Status  IMR_LM_Log_Write( IMRT_Log_Type_Id, IMRT_Log_Level, IMRT_Module  *, IMRT_File_Name, IMRT_Line_Id, IMRT_Boolean, IMRT_Log_Fmt, ...);



/*----------------------------------------------------------------------------*/
/* Log_Buffer_Id                                                              */
/*----------------------------------------------------------------------------*/
/* Buffer_Ptr:         buffer to identify                                     */
/* Log_Id_Struct_Ptr:  Log_Id_Heap to use                                     */
/* Log_Id_Header_Ptr:  message to log as header                               */
/* Log_Type_Id:        Type id of log                                         */
/* Log_Level:          Level of log                                           */
/* Module_Ptr:         Loger module pointer                                   */
/* File_Name:          Loger file name                                        */
/* Line_Id:            Loger line id                                          */
/* LF_Flag:            Add or not a LF at EOF                                 */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_LM_Log_Buffer_Id( IMRT_Buffer  *, IMRT_Log_Id_Struct  *, IMRT_Log_Id_Header, IMRT_Log_Type_Id, IMRT_Log_Level, IMRT_Module  *, IMRT_File_Name, IMRT_Line_Id, IMRT_Boolean);







/*----------------------------------------------------------------------------*/

#endif
