/*----------------------------------------------------------------------------*/
/* $Workfile:   liblog_mng.c  $                                               */
/*----------------------------------------------------------------------------*/
/* $Author: agibert $                                                      */
/* $Date: 2008/11/12 02:25:22 $                                           */
/* $Revision: 1.1 $                                                       */
/* $Label:  $                                                         */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#define _LIBLOG_MNG_
#define _LIBLOG_MNG_C_

#include "liblog_mng.h"







/*----------------------------------------------------------------------------*/
/* Open Log Manager                                                           */
/*----------------------------------------------------------------------------*/
/* Lib_Flag:   Setup IOM init mode                                            */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_LM_Library_Open( IMRT_LM_Config  *Config_Ptr)
{
    IMRT_Module_Id    module_id;
    IMRT_Log_Type_Id  log_type_id;


    if( IMRG_LM_Module.Lib.Status != IMRD_LIB_STATUS_CLOSED)
    {
	IMRD_LM_LOG_ERROR_0( "Library not closed !");

	return( IMRS_KO);
    }
    else
    {
	strcpy( IMRG_LM_Base.Config.Log_File, Config_Ptr->Log_File);

	if( strcmp( Config_Ptr->Log_File, "-") == 0)
	{
	    IMRG_LM_Base.Stream_Out = stderr;
	}
	else
	{
	    if( ( IMRG_LM_Base.Stream_Out = fopen( Config_Ptr->Log_File, "a")) == NULL)
	    {
		IMRD_LM_LOG_ERROR_2( "Can't open log file [%s] (%d) !", IMRG_LM_Base.Config.Log_File, errno);

		return( IMRS_KO);
	    }
	}



	IMRG_LM_Base.Config.Module_Nb = Config_Ptr->Module_Nb;

	for( module_id = 0; module_id < IMRG_LM_Base.Config.Module_Nb; module_id++)
	{
	    for( log_type_id = 0; log_type_id < IMRD_LOG_TYPE_NB; log_type_id++)
	    {
		IMRG_LM_Base.Config.Level_Tab[ log_type_id][ module_id] = Config_Ptr->Level_Tab[ log_type_id][ module_id];
	    }
	}

	IMRG_LM_Module.Lib.Status = IMRD_LIB_STATUS_OPENED;

	IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_1, "Library opened !");

	return( IMRS_OK);
    }
}





/*----------------------------------------------------------------------------*/
/* Close Log Manager                                                          */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_LM_Library_Close()
{
    if( IMRG_LM_Module.Lib.Status != IMRD_LIB_STATUS_OPENED)
    {
	IMRD_LM_LOG_ERROR_0( "Library not opened !");

	return( IMRS_KO);
    }
    else
    {
	if( ( IMRG_LM_Base.Stream_Out != stderr) && ( fclose( IMRG_LM_Base.Stream_Out) == EOF))
	{
	    IMRD_LM_LOG_ERROR_2( "Can't close log file [%s] (%d) !", IMRG_LM_Base.Config.Log_File, errno);

	    return( IMRS_KO);
	}

	IMRG_LM_Module.Lib.Status = IMRD_LIB_STATUS_CLOSED;

	IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_1, "Library closed !");

	return( IMRS_OK);
    }
}





/*----------------------------------------------------------------------------*/
/* LM_Stream_Out_Get                                                          */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

FILE  *IMR_LM_Stream_Out_Get( void)
{
    if( IMRG_LM_Base.Stream_Out == NULL)
    {
	return( stderr);
    }
    else
    {
	return( IMRG_LM_Base.Stream_Out);
    }
}





/*----------------------------------------------------------------------------*/
/* LM_Log_Trigger                                                             */
/*----------------------------------------------------------------------------*/
/* Log_Type_Id:  Type id of log                                               */
/* Log_Level:    Level of log                                                 */
/* Module_Ptr:   Loger module pointer                                         */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_LM_Log_Trigger( IMRT_Log_Type_Id  Log_Type_Id, IMRT_Log_Level  Log_Level, IMRT_Module  *Module_Ptr)
{
    FILE     *Stream_Out;


    Stream_Out = IMR_LM_Stream_Out_Get();

    if( ( Log_Type_Id < 0) || ( Log_Type_Id >= IMRD_LOG_TYPE_NB))
    {
	fprintf( Stream_Out, "liblog_mng: %s: Invalid Log_Type_Id (%d) !\n", __FILE__, errno);

	return( IMRS_KO);
    }
    
    if( IMRG_LM_Module.Lib.Status == IMRD_LIB_STATUS_OPENED)
    {
	if ( IMRG_LM_Base.Config.Level_Tab[ Log_Type_Id][ Module_Ptr->Id] != IMRD_LOG_LEVEL_UNSET)
	{
	    if( Log_Level > IMRG_LM_Base.Config.Level_Tab[ Log_Type_Id][ Module_Ptr->Id])
	    {
		return( IMRS_OK);
	    }
	}
	else
	{
	    if( Log_Level > IMRG_LM_Base.Config.Level_Tab[ Log_Type_Id][ IMRD_MODULE_ID_UNKNOWN])
	    {
		return( IMRS_OK);
	    }
	}
    }
    else
    {
	if( Log_Type_Id == IMRD_LOG_TYPE_ID_TRACE)
	{
	    return( IMRS_OK);
	}
    }

    return( IMRS_LM_LOG);
}





/*----------------------------------------------------------------------------*/
/* LM_Log_Write                                                               */
/*----------------------------------------------------------------------------*/
/* Log_Type_Id:  Type id of log                                               */
/* Log_Level:    Level of log                                                 */
/* Module_Ptr:   Loger module pointer                                         */
/* File_Name:    Loger file name                                              */
/* Line_Id:      Loger line id                                                */
/* LF_Flag:      Add or not a LF at EOF                                       */
/* Log_Fmt:      Format string a la printf                                    */
/* ...:          Optional args                                                */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_LM_Log_Write( IMRT_Log_Type_Id  Log_Type_Id, IMRT_Log_Level  Log_Level, IMRT_Module  *Module_Ptr, IMRT_File_Name  File_Name, IMRT_Line_Id  Line_Id, IMRT_Boolean  LF_Flag, IMRT_Log_Fmt  Log_Fmt, ...)
{
           va_list       args;
	   FILE         *Stream_Out;
	   IMRT_Status   status;
           char          fmt[255];
    struct timeval       cur_timeval;
    struct tm           *tm_ptr;
    int  rc;


    Stream_Out = IMR_LM_Stream_Out_Get();

    if( ( status = IMR_LM_Log_Trigger( Log_Type_Id, Log_Level, Module_Ptr)) != IMRS_LM_LOG)
    {
	return( status);
    }

    gettimeofday( &cur_timeval, NULL);

    if( ( tm_ptr = localtime( &( cur_timeval.tv_sec))) == NULL)
    {
	fprintf( Stream_Out, "liblog_mng: %s: can't convert localtime (%d) !\n", __FILE__, errno);

	return( IMRS_KO);
    }

    va_start( args, Log_Fmt);

    sprintf( fmt, "%02d/%02d/%02d %02d:%02d:%02d.%04d %3s %03d %-4s %-21s %-4d %s%s",
	     ( tm_ptr->tm_year - 100), tm_ptr->tm_mon, tm_ptr->tm_mday, tm_ptr->tm_hour, tm_ptr->tm_min, tm_ptr->tm_sec, ( cur_timeval.tv_usec / 100),
	     IMRG_LM_Base.Log_Type_Tab[ Log_Type_Id], Log_Level, Module_Ptr->Name, File_Name, Line_Id,
	     Log_Fmt,
	     ( ( LF_Flag == IMRD_TRUE) ? "\n" : ""));

    rc = vfprintf( Stream_Out, fmt, args);
    
    if( rc  < 0)
    {
	fprintf(stderr, "liblog_mng: Can't log messages (%d) !!!\n", errno);
	ND_Library_StdErr_Set( NULL);
	return(IMRS_KO);
    }

    fflush( Stream_Out);

    va_end( args);

    return( IMRS_OK);
}





/*----------------------------------------------------------------------------*/
/* Log_Buffer_Id                                                              */
/*----------------------------------------------------------------------------*/
/* Buffer_Ptr:         buffer to identify                                     */
/* Log_Id_Struct_Ptr:  Log_Id_Heap to use                                     */
/* Log_Id_Header_Ptr:  message to log as header                               */
/* Log_Type_Id:        Type id of log                                         */
/* Log_Level:          Level of log                                           */
/* Module_Ptr:         Loger module pointer                                   */
/* File_Name:          Loger file name                                        */
/* Line_Id:            Loger line id                                          */
/* LF_Flag:            Add or not a LF at EOF                                 */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_LM_Log_Buffer_Id( IMRT_Buffer  *Buffer_Ptr, IMRT_Log_Id_Struct  *Log_Id_Struct_Ptr, IMRT_Log_Id_Header  Log_Id_Header, IMRT_Log_Type_Id  Log_Type_Id, IMRT_Log_Level  Log_Level, IMRT_Module  *Module_Ptr, IMRT_File_Name  File_Name, IMRT_Line_Id  Line_Id, IMRT_Boolean  LF_Flag)
{
  IMRT_Status   status;
  char         *log_string, *log_ptr, *buf_ptr;
  int           idx;
   

  if( Log_Id_Struct_Ptr->Bit_Nb > 0 )
  {
      if( Log_Id_Struct_Ptr->Log_Id_Bit_Ptr != NULL)
      {
	  if( Buffer_Ptr->Data_Buf_Ptr != NULL)
	  {
	      /* allocation = taille total des Id + ( deux crochets + un espace entre chaque id ) * le nombre d'Id */

	      if( ( log_string = malloc( Log_Id_Struct_Ptr->Log_Size + ( 3 * Log_Id_Struct_Ptr->Bit_Nb) + 1)) != NULL)
	      {
		  for( idx = 0, log_ptr = log_string; idx < Log_Id_Struct_Ptr->Bit_Nb; idx++)
		  {
		      *(log_ptr++) = ' ';
		      *(log_ptr++) = '[';

		      for( buf_ptr = Buffer_Ptr->Data_Buf_Ptr + Log_Id_Struct_Ptr->Log_Id_Bit_Ptr[ idx].Offset;
			   ( *buf_ptr != '\0') && ( buf_ptr < ( Buffer_Ptr->Data_Buf_Ptr
								+ Log_Id_Struct_Ptr->Log_Id_Bit_Ptr[ idx].Offset
								+ Log_Id_Struct_Ptr->Log_Id_Bit_Ptr[ idx].Size));
			   buf_ptr++)
		      {
			  *(log_ptr++) = isprint( *buf_ptr) ? *buf_ptr : '.';
		      }

		      *(log_ptr++) = ']';
		  }

		  *log_ptr = '\0';

		  IMR_LM_Log_Write( Log_Type_Id, Log_Level, Module_Ptr, File_Name, Line_Id, LF_Flag, "%s%s", Log_Id_Header, log_string);

		  free( log_string);

		  status = IMRS_OK;
	      }
	      else
	      {
		  IMRD_LM_LOG_ERROR_1( "Allocation error ! (%d)", errno);
		  status = IMRS_KO;
	      }
	      
	  }
	  else
	  {
	      IMRD_LM_LOG_ERROR_0( "Buffer data NULL !");
	      status = IMRS_KO;
	  }
      }
      else
      {
	  IMRD_LM_LOG_ERROR_0( "Log_Id_Bit_Ptr Null !");
	  status = IMRS_KO;
      }    
  }
  else
  {
      status = IMRS_OK;
  }    

  return( status);
}
