/*----------------------------------------------------------------------------*/
/* File: buffer_mng.c					   		      */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#define _LIBMAPPING_MNG_
#define _BUFFER_MNG_C_

#include "buffer_mng.h" 


IMRT_Status	MM_Buffer_DeInit()
{
	IMRT_Status status;
	char *func_name = "MM_Buffer_DeInit";
	int indice;
	IMRT_StrHdl_Fmt_Func *StrHdl_Fmt_dynamic_ptr;
	IMRT_StrHdl_Cnv_Func *StrHdl_Cnv_dynamic_ptr;
	

	status = IMRS_OK;
	indice = 0;
	StrHdl_Fmt_dynamic_ptr = NULL;
	StrHdl_Cnv_dynamic_ptr = NULL;
	
	if ((IMRG_MM_StrHdl_Fmt_Func_Tab != NULL) && (IMRG_MM_StrHdl_Cnv_Func_Tab != NULL))
	{
		while (   (status == IMRS_OK)
		       && (indice < IMRG_Taille_Fmt_Max) )
		{
		        StrHdl_Fmt_dynamic_ptr = &(IMRG_MM_StrHdl_Fmt_Func_Tab[indice]);
		
	        	if (( StrHdl_Fmt_dynamic_ptr != NULL) && (StrHdl_Fmt_dynamic_ptr->DeInit_Ptr != NULL))
	        	{
	        		status = StrHdl_Fmt_dynamic_ptr->DeInit_Ptr();
	        	}
	        	indice++;
	        }
	        
	
		indice = 0;
		
		while (   (status == IMRS_OK)
		       && (indice < IMRG_Taille_Cnv_Max) )
		{
		        StrHdl_Cnv_dynamic_ptr = &(IMRG_MM_StrHdl_Cnv_Func_Tab[indice]);
		
	        	if ((StrHdl_Cnv_dynamic_ptr != NULL) && (StrHdl_Cnv_dynamic_ptr->DeInit_Ptr != NULL))
	        	{
	        		status = StrHdl_Cnv_dynamic_ptr->DeInit_Ptr();
	        	}
	        	indice++;
	        }
	}
	
	if ( IMRG_MM_StrHdl_Fmt_Func_Tab != NULL)
	{
		free( IMRG_MM_StrHdl_Fmt_Func_Tab );
	}
        
	if ( IMRG_MM_StrHdl_Cnv_Func_Tab != NULL)
	{
		free( IMRG_MM_StrHdl_Cnv_Func_Tab );
	}

	IMRG_Input_Buffer_Ptr    = NULL;
	IMRG_Output_Buffer_Ptr   = NULL;
	IMRG_Taille_Fmt_Max          = 0;
	IMRG_Taille_Cnv_Max          = 0;


	return ( status );
}

IMRT_Status	MM_Buffer_Init()
{
        
	IMRT_Status status;
	char *func_name = "MM_Buffer_Init";

	
	status = IMRS_OK;
	IMRG_Input_Buffer_Ptr    = NULL;
	IMRG_Output_Buffer_Ptr   = NULL;
	IMRG_Taille_Fmt_Max          = 0;
	IMRG_Taille_Cnv_Max          = 0;
	IMRG_MM_StrHdl_Fmt_Func_Tab = NULL;
	IMRG_MM_StrHdl_Cnv_Func_Tab = NULL;
	
	handle = dlopen (NULL , RTLD_LAZY);
	if (handle == NULL)
	{
	        IMRD_LM_LOG_ERROR_1("dlopen failed  [%s]" , dlerror());
	        status = IMRS_KO;
	}
	
	
	status = IMR_SDM_Fmt_Proceed(IMR_MM_Fmt_Add);

	if ( status != IMRS_OK)
	{
		IMRD_LM_LOG_ERROR_1("Format proceed ended with status (%d)" , status);
	}
	else
	{
	        status = IMR_SDM_Cnv_Proceed(IMR_MM_Cnv_Add);
		if ( status != IMRS_OK)
		{
		        IMRD_LM_LOG_ERROR_1("Conversion functions proceed ended with status (%d)" , status);
		}

	}
	

	return ( status );
	
}


IMRT_Status IMR_MM_Fmt_Add(IMRT_Fmt_Id Format_Id , IMRT_Fmt_Name Format_Name , IMRT_Fmt_Id Fmt_Id_Nb)
{
	IMRT_Status                     status;
	char                            *func_name = "IMR_MM_Fmt_Add";
	size_t                          tableau_size;
	IMRT_StrHdl_Fmt_Func            *StrHdl_Fmt_dynamic_ptr;
        char                            func_ptr_name[100];
        char                            ptr_name[100];
        IMRT_Boolean                    fmt_is_unknown;
        IMRT_Boolean                    it_is_OK;
        
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_2 , "============================================================================")
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_2 , "Initialisation format demand : Begin");
	IMRD_LM_LOG_TRACE_2(IMRD_LOG_LEVEL_VERBOSE_2 , "Format Id : (%d) name :  (%s)" , Format_Id , Format_Name);	

        fmt_is_unknown = IMRD_FALSE;        
        it_is_OK = IMRD_TRUE;
        status = IMRS_OK;
        StrHdl_Fmt_dynamic_ptr = NULL;
        
        memset( func_ptr_name, '\0', sizeof(func_ptr_name) );
        memset( ptr_name, '\0', sizeof(ptr_name) );
        
        if (strcmp(Format_Name,IMRD_FMT_NAME_UNKNOWN) == 0)
                fmt_is_unknown = IMRD_TRUE;
                
        if (IMRG_MM_StrHdl_Fmt_Func_Tab == NULL)
        {
		IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_2 , "Format_Id Max  : (%d)" , Fmt_Id_Nb);	
                tableau_size = (sizeof(IMRT_StrHdl_Fmt_Func) * Fmt_Id_Nb);
                IMRG_MM_StrHdl_Fmt_Func_Tab = (IMRT_StrHdl_Fmt_Func *) malloc( tableau_size );
                if (IMRG_MM_StrHdl_Fmt_Func_Tab== NULL)
                {
                        status = IMRS_KO;
                        IMRD_LM_LOG_ERROR_0("malloc IMRG_MM_StrHdl_Fmt_Func_Tab failed" );
                }
                else
                {
                        memset( IMRG_MM_StrHdl_Fmt_Func_Tab, '\0', tableau_size );
                        IMRG_Taille_Fmt_Max = Fmt_Id_Nb;
                }
        }
        
        if (status == IMRS_OK)
        {
                StrHdl_Fmt_dynamic_ptr = &(IMRG_MM_StrHdl_Fmt_Func_Tab[Format_Id]);
                strcpy( StrHdl_Fmt_dynamic_ptr->Nom , Format_Name);

                strcpy(ptr_name , "Init");
                sprintf(func_ptr_name,"MM_%s_StrHdl_%s",Format_Name,ptr_name);
                StrHdl_Fmt_dynamic_ptr->Init_Ptr = ( IMRT_MM_Fmt_Hdl_Init  *)dlsym(handle , func_ptr_name);
                if ((StrHdl_Fmt_dynamic_ptr->Init_Ptr == NULL) && (!fmt_is_unknown))
                        it_is_OK = IMRD_FALSE;
                        
                if (it_is_OK)
                {                
                        strcpy(ptr_name , "DeInit");
                        sprintf(func_ptr_name,"MM_%s_StrHdl_%s",Format_Name,ptr_name);
                        StrHdl_Fmt_dynamic_ptr->DeInit_Ptr = ( IMRT_MM_Fmt_Hdl_DeInit  *)dlsym(handle , func_ptr_name);
                        if ((StrHdl_Fmt_dynamic_ptr->DeInit_Ptr == NULL) &&( !fmt_is_unknown))
                                it_is_OK = IMRD_FALSE;
                }
                
                if (it_is_OK)
                {                
                        strcpy(ptr_name , "Data_Free");
                        sprintf(func_ptr_name,"MM_%s_StrHdl_%s",Format_Name,ptr_name);                
                        StrHdl_Fmt_dynamic_ptr->Data_Free_Ptr   = (IMRT_MM_Fmt_Hdl_Data_Free *)dlsym(handle , func_ptr_name);
                        if ((StrHdl_Fmt_dynamic_ptr->Data_Free_Ptr == NULL) && (!fmt_is_unknown))
                                it_is_OK = IMRD_FALSE;
                }
                
                if (it_is_OK)
                {
                        strcpy(ptr_name , "Ident");
                        sprintf(func_ptr_name,"MM_%s_StrHdl_%s",Format_Name,ptr_name);        	
                        StrHdl_Fmt_dynamic_ptr->Ident_Ptr        = (IMRT_MM_Fmt_Hdl_Ident *)dlsym(handle , func_ptr_name);
                        if ((StrHdl_Fmt_dynamic_ptr->Ident_Ptr == NULL) && (!fmt_is_unknown))
                                it_is_OK = IMRD_FALSE;
                }
                
                if (it_is_OK)
                {                
                        strcpy(ptr_name , "Open");
                        sprintf(func_ptr_name,"MM_%s_StrHdl_%s",Format_Name,ptr_name);        	
                        StrHdl_Fmt_dynamic_ptr->Open_Ptr        = (IMRT_MM_Fmt_Hdl_Open *)dlsym(handle , func_ptr_name);
                        if ((StrHdl_Fmt_dynamic_ptr->Open_Ptr == NULL) && (!fmt_is_unknown))
                                it_is_OK = IMRD_FALSE;
                }
                
                if (it_is_OK)
                {                
                        strcpy(ptr_name , "Close");
                        sprintf(func_ptr_name,"MM_%s_StrHdl_%s",Format_Name,ptr_name);		
                        StrHdl_Fmt_dynamic_ptr->Close_Ptr       = (IMRT_MM_Fmt_Hdl_Close *)dlsym(handle , func_ptr_name);
                        if ((StrHdl_Fmt_dynamic_ptr->Close_Ptr == NULL) && (!fmt_is_unknown))
                                it_is_OK = IMRD_FALSE;
                }
                
                if (it_is_OK)
                {                
                        strcpy(ptr_name , "Data_List");
                        sprintf(func_ptr_name,"MM_%s_StrHdl_%s",Format_Name,ptr_name);		
                        StrHdl_Fmt_dynamic_ptr->Data_List_Ptr   = (IMRT_MM_Fmt_Hdl_Data_List *)dlsym(handle , func_ptr_name);
                        if ((StrHdl_Fmt_dynamic_ptr->Data_List_Ptr == NULL) && (!fmt_is_unknown))
                                it_is_OK = IMRD_FALSE;
                }
                
                if (it_is_OK)
                {                
                        strcpy(ptr_name , "Copy");
                        sprintf(func_ptr_name,"MM_%s_StrHdl_%s",Format_Name,ptr_name);		
                        StrHdl_Fmt_dynamic_ptr->Copy_Ptr        = (IMRT_MM_Fmt_Hdl_Copy *)dlsym(handle , func_ptr_name);
                        if ((StrHdl_Fmt_dynamic_ptr->Copy_Ptr == NULL) && (!fmt_is_unknown))
                                it_is_OK = IMRD_FALSE;
                }
                
                if (it_is_OK)
                {                
                        strcpy(ptr_name , "Flush");
                        sprintf(func_ptr_name,"MM_%s_StrHdl_%s",Format_Name,ptr_name);		
                        StrHdl_Fmt_dynamic_ptr->Flush_Ptr       = (IMRT_MM_Fmt_Hdl_Flush *)dlsym(handle , func_ptr_name);
                        if ((StrHdl_Fmt_dynamic_ptr->Flush_Ptr == NULL) && (!fmt_is_unknown))
                                it_is_OK = IMRD_FALSE;
                }
                
                if (it_is_OK)
                {                
                        strcpy(ptr_name , "Get");
                        sprintf(func_ptr_name,"MM_%s_StrHdl_%s",Format_Name,ptr_name);		
                        StrHdl_Fmt_dynamic_ptr->Get_Ptr         = (IMRT_MM_Fmt_Hdl_Get *)dlsym(handle , func_ptr_name);
                        if ((StrHdl_Fmt_dynamic_ptr->Get_Ptr == NULL) && (!fmt_is_unknown))
                                it_is_OK = IMRD_FALSE;
                }
                
                if (it_is_OK)
                {                
                        strcpy(ptr_name , "Convert");
                        sprintf(func_ptr_name,"MM_%s_StrHdl_%s",Format_Name,ptr_name);		
                        StrHdl_Fmt_dynamic_ptr->Cnv_Ptr         = (IMRT_MM_Fmt_Hdl_Cnv *)dlsym(handle , func_ptr_name);
                        if ((StrHdl_Fmt_dynamic_ptr->Cnv_Ptr == NULL) && (!fmt_is_unknown))
                                it_is_OK = IMRD_FALSE;
                }
                
                if (it_is_OK)
                {                
                        strcpy(ptr_name , "Put");
                        sprintf(func_ptr_name,"MM_%s_StrHdl_%s",Format_Name,ptr_name);		
                        StrHdl_Fmt_dynamic_ptr->Put_Ptr         = (IMRT_MM_Fmt_Hdl_Put *)dlsym(handle , func_ptr_name);
                        if ((StrHdl_Fmt_dynamic_ptr->Put_Ptr == NULL) && (!fmt_is_unknown))
                                it_is_OK = IMRD_FALSE;
                }
                        
                if (it_is_OK && StrHdl_Fmt_dynamic_ptr->Init_Ptr != NULL)
                {
                        status = StrHdl_Fmt_dynamic_ptr->Init_Ptr();
                        if (status == IMRS_KO)
                                IMRD_LM_LOG_ERROR_0("Initialisation handlers failed" );
                }
                
                if (!it_is_OK)
                {
                        status = IMRS_KO;
                        IMRD_LM_LOG_ERROR_0("Pointeur null et format different de unknown" );
                        IMRD_LM_LOG_ERROR_1("dlsym failed [%s]", dlerror());
                }

        }

	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_2 , "Initialisation format demand : End with status :  (%d)" , status);
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_2 ,  "============================================================================")

	return ( status );

}

IMRT_Status IMR_MM_Cnv_Add(IMRT_Cnv_Id Conv_Id , IMRT_Cnv_Name Conv_Name , IMRT_Cnv_Id Conv_Id_Nb)
{
	IMRT_Status                     status;
	char                            *func_name = "IMR_MM_Cnv_Add";
	size_t                          tableau_size;
	IMRT_StrHdl_Cnv_Func            *StrHdl_Cnv_dynamic_ptr;
        char                            func_ptr_name[100];
        char                            ptr_name[100];
        IMRT_Boolean                    cnv_is_unknown;
        IMRT_Boolean                    it_is_OK;
        
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_2 ,  "============================================================================")
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_2 , "Initialisation conversion function demand : Begin");
	IMRD_LM_LOG_TRACE_2(IMRD_LOG_LEVEL_VERBOSE_2 , "Conversion  Id : (%d) name :  (%s)" , Conv_Id , Conv_Name);	

	
        cnv_is_unknown = IMRD_FALSE;
        it_is_OK = IMRD_TRUE;
        status = IMRS_OK;
        StrHdl_Cnv_dynamic_ptr = NULL;
        
        memset( func_ptr_name, '\0', sizeof(func_ptr_name) );
        memset( ptr_name, '\0', sizeof(ptr_name) );

        if (strcmp(Conv_Name,IMRD_CNV_NAME_UNKNOWN)==0)
                cnv_is_unknown = IMRD_TRUE;
        
        if (IMRG_MM_StrHdl_Cnv_Func_Tab == NULL)
        {
	        IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_2 , "Conv_Id Max  : (%d)" , Conv_Id_Nb);	
                tableau_size = (sizeof(IMRT_StrHdl_Cnv_Func) * Conv_Id_Nb);
                IMRG_MM_StrHdl_Cnv_Func_Tab = (IMRT_StrHdl_Cnv_Func *) malloc( tableau_size );
                if (IMRG_MM_StrHdl_Cnv_Func_Tab== NULL)
                {
                        status = IMRS_KO;
                        IMRD_LM_LOG_ERROR_0("malloc IMRG_MM_StrHdl_Cnv_Func_Tab failed" );
                }
                else
                {
                        memset( IMRG_MM_StrHdl_Cnv_Func_Tab, '\0', tableau_size );
                        IMRG_Taille_Cnv_Max = Conv_Id_Nb;
                }
        }
        
        if (status == IMRS_OK)
        {
                StrHdl_Cnv_dynamic_ptr = &(IMRG_MM_StrHdl_Cnv_Func_Tab[Conv_Id]);
                strcpy( StrHdl_Cnv_dynamic_ptr->Nom , Conv_Name);
                strcpy(ptr_name , "Init");
                sprintf(func_ptr_name,"MM_%s_%s",Conv_Name,ptr_name);
                StrHdl_Cnv_dynamic_ptr->Init_Ptr = ( IMRT_MM_Cnv_Init  *)dlsym(handle , func_ptr_name);
                if ((StrHdl_Cnv_dynamic_ptr->Init_Ptr == NULL) && (!cnv_is_unknown))
                        it_is_OK = IMRD_FALSE;
                
                if (it_is_OK)
                {                
                        strcpy(ptr_name , "DeInit");
                        sprintf(func_ptr_name,"MM_%s_%s",Conv_Name,ptr_name);
                        StrHdl_Cnv_dynamic_ptr->DeInit_Ptr = ( IMRT_MM_Cnv_DeInit  *)dlsym(handle , func_ptr_name);
                        if ((StrHdl_Cnv_dynamic_ptr->DeInit_Ptr == NULL) && ( !cnv_is_unknown))
                                it_is_OK = IMRD_FALSE;
                }
                
                if (it_is_OK)
                {                
                        strcpy(ptr_name , "Convert");
                        sprintf(func_ptr_name,"MM_%s_%s",Conv_Name,ptr_name);                
                        StrHdl_Cnv_dynamic_ptr->Convert_Ptr   = (IMRT_MM_Convert *)dlsym(handle , func_ptr_name);
                        if ((StrHdl_Cnv_dynamic_ptr->Convert_Ptr == NULL) && ( !cnv_is_unknown))
                                it_is_OK = IMRD_FALSE;
                }
                
                
                if (it_is_OK && StrHdl_Cnv_dynamic_ptr->Init_Ptr != NULL)
                {
                        status = StrHdl_Cnv_dynamic_ptr->Init_Ptr();
                        if (status == IMRS_KO)
                                IMRD_LM_LOG_ERROR_0("Initialisation convert failed" );
                }
                
                if (!it_is_OK)
                {
                        status = IMRS_KO;
                        IMRD_LM_LOG_ERROR_0("Pointeur null et Fonction de conversion differente de unknown" );
                        IMRD_LM_LOG_ERROR_1("dlsym failed [%s]", dlerror());
                }

        }


	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_2 , "Initialisation conversion function demand : End with status :  (%d)" , status);
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_2 ,  "============================================================================")
        
	return ( status );

}


IMRT_Status  MM_Buffer_List( IMRT_Buffer *Buffer_Ptr, char *texte)
{
    char         status_str[10];
    char         flag_str[10];
    IMRT_Fmt_Id  fmt_id;
    IMRT_Status	 status;


    status = IMRS_OK;
    fmt_id = IMRD_FMT_ID_UNKNOWN;

    if (Buffer_Ptr == NULL)
    {
    	IMRD_LM_LOG_WARNING_0("Buffer pointer NULL");
    }
    else
    {
	switch( Buffer_Ptr->Status)
	{
		case IMRD_BUFFER_STATUS_ALLOCATED : strcpy( status_str, "ALLOCATED"); break;
		case IMRD_BUFFER_STATUS_FREED     : strcpy( status_str, "FREED");     break;
		default                           : strcpy( status_str, "UNKNOWN");   break;
	}
	
	switch (Buffer_Ptr->Flag)
	{
		case IMRD_BUFFER_FLAG_OPENED : strcpy( flag_str, "OPENED");  break;
		case IMRD_BUFFER_FLAG_CLOSED : strcpy( flag_str, "CLOSED");  break;
		default                      : strcpy( flag_str, "UNKNOWN"); break;
	}
	    
	IMRD_LM_LOG_TRACE_2( IMRD_LOG_LEVEL_VERBOSE_1, "Buffer_List: Buffer: [%s]   Address: (0x%X)", texte, Buffer_Ptr);

	if( ( status = Buffer_Format_Check( Buffer_Ptr->Fmt_Id)) == IMRS_OK)
	{
	    fmt_id = Buffer_Ptr->Fmt_Id;
	}
	else
	{
	    fmt_id = IMRD_FMT_ID_UNKNOWN;
	}

	IMRD_LM_LOG_TRACE_3( IMRD_LOG_LEVEL_VERBOSE_1, "Buffer_List: Appl_Id_Input: (%2d)               Appl_Id_Output: (%2d)   Dir_Id: (%2d)", Buffer_Ptr->Appl_Id_Input, Buffer_Ptr->Appl_Id_Output, Buffer_Ptr->Dir_Id);
	IMRD_LM_LOG_TRACE_3( IMRD_LOG_LEVEL_VERBOSE_1, "Buffer_List: Fmt_Id:        (%2d) [%9s]     Msg_Id:         (%2d)", Buffer_Ptr->Fmt_Id, IMRG_MM_StrHdl_Fmt_Func_Tab[fmt_id].Nom, Buffer_Ptr->Msg_Id);
	IMRD_LM_LOG_TRACE_4( IMRD_LOG_LEVEL_VERBOSE_1, "Buffer_List: Status:        (%2d) [%9s]   Flag:           (%2d) [%7s]", Buffer_Ptr->Status, status_str, Buffer_Ptr->Flag, flag_str);
	IMRD_LM_LOG_TRACE_1( IMRD_LOG_LEVEL_VERBOSE_1, "Buffer_List: Format attribute:        [%s]" , Buffer_Ptr->FmtAttr_Value);
	
	if (IMRG_MM_StrHdl_Fmt_Func_Tab != NULL)
	{
		if ( IMRG_MM_StrHdl_Fmt_Func_Tab[fmt_id].Data_List_Ptr == NULL)
		{
		    IMRD_LM_LOG_WARNING_1( "Null Fonction List for [%s] format", IMRG_MM_StrHdl_Fmt_Func_Tab[fmt_id].Nom);
		}
		else
		{
		    status = IMRG_MM_StrHdl_Fmt_Func_Tab[Buffer_Ptr->Fmt_Id].Data_List_Ptr(Buffer_Ptr);
		}
	}
    }

    return ( status );
}

IMRT_Status MM_Buffer_Alloc (IMRT_Buffer **Buffer_Ptr_Ptr)
{
	IMRT_Buffer *buffer_Ptr;
	IMRT_Status status;
	char *func_name = "MM_Buffer_Alloc";

	
	status          = IMRS_OK;
	buffer_Ptr      = (IMRT_Buffer *) malloc( sizeof(IMRT_Buffer) );
	
	
	if (buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	if (buffer_Ptr->FmtAttr_Value == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("buffer_Ptr->FmtAttr_Value NULL" );
	}
	else
	if (Buffer_Ptr_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	{
		*Buffer_Ptr_Ptr = buffer_Ptr;
		buffer_Ptr->Appl_Id_Input = IMRD_APPL_ID_UNKNOWN;
		buffer_Ptr->Appl_Id_Output = IMRD_APPL_ID_UNKNOWN;
		buffer_Ptr->Dir_Id          = IMRD_DIR_ID_UNKNOWN;
		buffer_Ptr->Fmt_Id             = IMRD_FMT_ID_UNKNOWN;
		strcpy(buffer_Ptr->FmtAttr_Value , IMRD_FMTATTR_NAME_UNKNOWN);
		buffer_Ptr->Msg_Id             = IMRD_MSG_ID_UNKNOWN;
		buffer_Ptr->Status             = IMRD_BUFFER_STATUS_UNKNOWN;
		buffer_Ptr->Flag               = IMRD_BUFFER_FLAG_UNKNOWN;
		buffer_Ptr->Data_Buf_Ptr       = NULL;
		buffer_Ptr->Data_Tmp_Ptr      = NULL;
		buffer_Ptr->Data_Tmp_Changed  = IMRD_FALSE;
		buffer_Ptr->Data_Buf_Size      = 0;
		buffer_Ptr->Data_Buf_Len       = 0;
	}                       

	return ( status );
}     

IMRT_Status Buffer_Data_Free( IMRT_Buffer *Buffer_Ptr)
{
	IMRT_Status status;
	char *func_name = "Buffer_Data_Free";
	
	status = IMRS_OK;

	/* recherche du handler */
	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	if (IMRG_MM_StrHdl_Fmt_Func_Tab == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Format tab pointer NULL" );
	}
	else
	{
		status = Buffer_Format_Check( Buffer_Ptr->Fmt_Id );
	
		if (status == IMRS_OK)
		{
			if (IMRG_MM_StrHdl_Fmt_Func_Tab[Buffer_Ptr->Fmt_Id].Data_Free_Ptr == NULL)
			{
				status = IMRS_KO;
				IMRD_LM_LOG_ERROR_1("fonction Data_Free = NULL pour format %s" ,
							IMRG_MM_StrHdl_Fmt_Func_Tab[Buffer_Ptr->Fmt_Id].Nom);
			}
			else
			{
				status = IMRG_MM_StrHdl_Fmt_Func_Tab[Buffer_Ptr->Fmt_Id].Data_Free_Ptr(Buffer_Ptr);
			}
		}
	}


	return ( status );
}

IMRT_Status MM_Buffer_Free (IMRT_Buffer **Buffer_Ptr_Ptr)
{
	IMRT_Status status;
	char *func_name = "MM_Buffer_Free";


	status = IMRS_OK;
	
	if (*Buffer_Ptr_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else	
	{
		/* il y a des donnees dans Data_Buf_Ptr ou dans Data_Tmp_Ptr */
		if ( ((*Buffer_Ptr_Ptr)->Status == IMRD_BUFFER_STATUS_ALLOCATED) ||
		     ((*Buffer_Ptr_Ptr)->Flag   == IMRD_BUFFER_FLAG_OPENED) )
		{
			status = Buffer_Data_Free( *Buffer_Ptr_Ptr );
		}
		free(*Buffer_Ptr_Ptr);
		*Buffer_Ptr_Ptr = NULL;
	}

	return ( status ) ;
}     

IMRT_Status MM_Buffer_Clear (IMRT_Buffer *Buffer_Ptr)
{
	IMRT_Status status;
	char *func_name = "MM_Buffer_Clear";

	
	status = IMRS_OK;
	
	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	{
		/* il y a des donnees dans Data_Buf_Ptr ou dans Data_Tmp_Ptr */
		if ( (Buffer_Ptr->Status == IMRD_BUFFER_STATUS_ALLOCATED) ||
		     (Buffer_Ptr->Flag   == IMRD_BUFFER_FLAG_OPENED) )
		{
			status = Buffer_Data_Free( Buffer_Ptr);
		}
		if (status == IMRS_OK)
		{
			Buffer_Ptr->Fmt_Id             = IMRD_FMT_ID_UNKNOWN;
			Buffer_Ptr->Msg_Id             = IMRD_MSG_ID_UNKNOWN;
			Buffer_Ptr->Status             = IMRD_BUFFER_STATUS_UNKNOWN;
			Buffer_Ptr->Flag               = IMRD_BUFFER_FLAG_UNKNOWN;
		}
	}

	return ( status );
}     

IMRT_Status Buffer_Format_Check( IMRT_Fmt_Id Format )
{
	IMRT_Status status;
	char *func_name = "Buffer_Format_Check";
	
	status = IMRS_OK;

	if ( (Format >= 0) && (Format < IMRG_Taille_Fmt_Max))
		status =IMRS_OK;
        else
        {
   	        status = IMRS_KO;
   	        IMRD_LM_LOG_ERROR_1("Format %d incorrect" , Format );
        }

	return ( status );
}

IMRT_Status MM_Buffer_Ident_Update( IMRT_Buffer *Buffer_Ptr, 
                                    IMRT_Fmt_Id Format,
                                    IMRT_FmtAttr_Value Fmt_Attr ,
                                    IMRT_Msg_Id Type)
{
	IMRT_Status status;
	char *func_name = "MM_Buffer_Ident_Update";
	
	status = IMRS_OK;

	
	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	if (Buffer_Ptr->FmtAttr_Value == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer_Ptr->FmtAttr_Value NULL" );
	}
	if (Fmt_Attr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("FmtAttr NULL" );
	}
	else
	{
		status = Buffer_Format_Check( Format );
		if (status == IMRS_OK)
		{
			/* si le buffer est ouvert le fermer avant de changer de format */
			if (  (Buffer_Ptr->Flag   == IMRD_BUFFER_FLAG_OPENED)
			      && ( (Buffer_Ptr->Fmt_Id != Format) || 
				   (strcmp(Buffer_Ptr->FmtAttr_Value,Fmt_Attr) !=0) ) )
			{
			   status = MM_Buffer_Close(Buffer_Ptr);
			}
		}
		if (status == IMRS_OK)
		{
			Buffer_Ptr->Fmt_Id = Format;
			strcpy(Buffer_Ptr->FmtAttr_Value , Fmt_Attr);
			Buffer_Ptr->Msg_Id = Type;
		}
	}

	return ( status );
}

IMRT_Status MM_Buffer_Ident(IMRT_Msg_Id Msg_Id_Ident , IMRT_Buffer *Buffer_Ptr)
{
	IMRT_Status status;
	char *func_name = "MM_Buffer_Ident";
        
        status = IMRS_OK;
        
        if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
    {
       	if (IMRG_MM_StrHdl_Fmt_Func_Tab == NULL)
       	{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("Format tab pointer NULL" );        		
       	}
       	else
       	{
			if (Buffer_Ptr->Flag != IMRD_BUFFER_FLAG_OPENED)
			{
	        	status = IMRS_KO;
	        	IMRD_LM_LOG_ERROR_0("Buffer not open" );
			}
			else
			{
				status = Buffer_Format_Check(Buffer_Ptr->Fmt_Id);
				if (status == IMRS_OK)
				{
					if (IMRG_MM_StrHdl_Fmt_Func_Tab[Buffer_Ptr->Fmt_Id].Ident_Ptr == NULL)
					{
						IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_1 , "fonction Ident = NULL pour format %s" , 
								IMRG_MM_StrHdl_Fmt_Func_Tab[Buffer_Ptr->Fmt_Id].Nom);
					}
					else
					{	
						status = IMRG_MM_StrHdl_Fmt_Func_Tab[Buffer_Ptr->Fmt_Id].Ident_Ptr(&Msg_Id_Ident, Buffer_Ptr);
					}      
		    	}
		       	if (status == IMRS_OK)
		       	{
		   	        Buffer_Ptr->Msg_Id = Msg_Id_Ident;
		       	}
			}
		}
	}
        
		
	return ( status );
        
}


IMRT_Status MM_Buffer_Open( IMRT_Buffer *Buffer_Ptr)
{
	IMRT_Fmt_Id   format_temp;
	short indice;
	IMRT_Status status;
	char *func_name = "MM_Buffer_Open";
	
	
	status = IMRS_OK;
	indice = 0;
	format_temp = IMRD_FMT_ID_UNKNOWN;
	
	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	{
		format_temp = Buffer_Ptr->Fmt_Id;
		if (Buffer_Ptr->Flag !=IMRD_BUFFER_FLAG_OPENED)
		{
					
			/* controle validite du format */
			status = Buffer_Format_Check( format_temp );
			
			if (IMRG_MM_StrHdl_Fmt_Func_Tab == NULL)
			{
				status = IMRS_KO;
				IMRD_LM_LOG_ERROR_0("Format tab pointer NULL" );
			}
			else
			{
				/* format non identifie */
				if ( format_temp == IMRD_FMT_ID_UNKNOWN )
				{
					if ( Buffer_Ptr->Status != IMRD_BUFFER_STATUS_ALLOCATED )
					{
						status = IMRS_KO;
						IMRD_LM_LOG_ERROR_0("Buffer not allocated" );
					}
					else
					{
						indice = 0;
						status = IMRS_BAD_FORMAT;
						while (  (status == IMRS_BAD_FORMAT) 
						      && (indice < IMRG_Taille_Fmt_Max) )
						{
							if (IMRG_MM_StrHdl_Fmt_Func_Tab[indice].Open_Ptr == NULL)
							{
								IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_1 , "fonction Open = NULL pour format %s" ,
								             IMRG_MM_StrHdl_Fmt_Func_Tab[indice].Nom);
							}
							else
							{	format_temp = indice;
								status = IMRG_MM_StrHdl_Fmt_Func_Tab[indice].Open_Ptr( Buffer_Ptr);
							}
							indice++;
						}
					}
				}
				/* format identifie */
				else
				{
					if (status == IMRS_OK)
					{
						if (IMRG_MM_StrHdl_Fmt_Func_Tab[Buffer_Ptr->Fmt_Id].Open_Ptr == NULL)
						{
							status = IMRS_KO;
							IMRD_LM_LOG_ERROR_1("fonction Open = NULL pour format %s" ,
						          IMRG_MM_StrHdl_Fmt_Func_Tab[Buffer_Ptr->Fmt_Id].Nom);
						}
						else
						{
							status = IMRG_MM_StrHdl_Fmt_Func_Tab[Buffer_Ptr->Fmt_Id].Open_Ptr( Buffer_Ptr);
						}
					}
				}
			}
	        	
	        	if (status == IMRS_OK)
	        	{
	        		Buffer_Ptr->Flag = IMRD_BUFFER_FLAG_OPENED;
	        		if (Buffer_Ptr->Fmt_Id != format_temp)
	        		        Buffer_Ptr->Fmt_Id = format_temp;
	        		
	        	}
	        }
        }

		
	return ( status );
}

IMRT_Status MM_Buffer_Close(IMRT_Buffer *Buffer_Ptr)
{
	IMRT_Status status;
	char *func_name = "MM_Buffer_Close";
	
	
	status = IMRS_OK;

	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	{
		if (Buffer_Ptr->Flag !=IMRD_BUFFER_FLAG_CLOSED)
		{
			status = Buffer_Format_Check( Buffer_Ptr->Fmt_Id );
			if (status == IMRS_OK)
			{
				if (Buffer_Ptr->Flag == IMRD_BUFFER_FLAG_OPENED)
				{
					if (IMRG_MM_StrHdl_Fmt_Func_Tab == NULL)
					{
						status = IMRS_KO;
						IMRD_LM_LOG_ERROR_0("Format tab pointer NULL" );
					}
					else
					{
						if (IMRG_MM_StrHdl_Fmt_Func_Tab[Buffer_Ptr->Fmt_Id].Close_Ptr == NULL)
						{
							status = IMRS_KO;
							IMRD_LM_LOG_ERROR_1("fonction Close = NULL pour format %s" ,
								          IMRG_MM_StrHdl_Fmt_Func_Tab[Buffer_Ptr->Fmt_Id].Nom);
						}
						else
						{
							status = IMRG_MM_StrHdl_Fmt_Func_Tab[Buffer_Ptr->Fmt_Id].Close_Ptr(Buffer_Ptr);
						}
					}
				}
			}
			if (status == IMRS_OK)
			{
				Buffer_Ptr->Flag = IMRD_BUFFER_FLAG_CLOSED;
			}
		}
	}

	return ( status );
}

/* La fonction suivante n'est pas implementee pour le moment */
/* Le nom des applications n'est connue que du static data manager */
IMRT_Status	Buffer_Application_Check( IMRT_Appl_Id Application )
{
	IMRT_Status status;
	char *func_name = "Buffer_Application_Check";
	
	
	status =IMRS_OK;
	

           	
	return ( status );
}

IMRT_Status	Buffer_Direction_Check( IMRT_Dir_Id Direction )
{
	IMRT_Status status;
	char *func_name = "Buffer_Direction_Check";
	

	if ((Direction == IMRD_DIR_ID_UNKNOWN) ||
	    (Direction == IMRD_DIR_ID_IN     ) ||
	    (Direction == IMRD_DIR_ID_OUT    ) )
		status =IMRS_OK;
        else
        {
           	status = IMRS_KO;
           	IMRD_LM_LOG_ERROR_1("Incorrect direction %d" , Direction );
        }


	return ( status );
}

IMRT_Status	Buffer_Statut_Check( IMRT_Buffer_Status Statut )
{
	IMRT_Status status;
	char *func_name = "Buffer_Statut_Check";
	

	if ((Statut == IMRD_BUFFER_STATUS_UNKNOWN  ) ||
	    (Statut == IMRD_BUFFER_STATUS_ALLOCATED) ||
	    (Statut == IMRD_BUFFER_STATUS_FREED    ) )
		status =IMRS_OK;
        else
        {
           	status = IMRS_KO;
           	IMRD_LM_LOG_ERROR_1("Incorrect status %d" , Statut );
        }

	return ( status );
}

IMRT_Status	Buffer_Etat_Check( IMRT_Buffer_Flag Flag )
{
	IMRT_Status status;
	char *func_name = "Buffer_Etat_Check";
	

	if ((Flag == IMRD_BUFFER_FLAG_UNKNOWN) ||
	    (Flag == IMRD_BUFFER_FLAG_OPENED ) ||
	    (Flag == IMRD_BUFFER_FLAG_CLOSED ) )
		status =IMRS_OK;
        else
        {
           	status = IMRS_KO;
           	IMRD_LM_LOG_ERROR_1("Incorrect Flag %d" , Flag )
        }

	return ( status );
}

IMRT_Status	MM_Buffer_Check( IMRT_Buffer	*Buffer_Ptr)
{
	IMRT_Status status;
	char *func_name = "MM_Buffer_Check";
	
	status = IMRS_OK;

	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	{
		if ( ( Buffer_Application_Check(Buffer_Ptr->Appl_Id_Input  ) == IMRS_OK ) &&
		     ( Buffer_Application_Check(Buffer_Ptr->Appl_Id_Output ) == IMRS_OK ) &&
		     ( Buffer_Direction_Check  (Buffer_Ptr->Dir_Id          ) == IMRS_OK ) &&
		     ( Buffer_Format_Check     (Buffer_Ptr->Fmt_Id             ) == IMRS_OK ) &&
		     ( Buffer_Statut_Check     (Buffer_Ptr->Status             ) == IMRS_OK ) &&
		     ( Buffer_Etat_Check       (Buffer_Ptr->Flag               ) == IMRS_OK ) ) 
			status = IMRS_OK;
	        else
	        {
	           	status = IMRS_KO;
	           	IMRD_LM_LOG_ERROR_0("Buffer check failed" );
	        }
        }

	return ( status );
}

IMRT_Status MM_Buffer_Copy( IMRT_Buffer *Output_Buffer_Ptr, IMRT_Buffer *Input_Buffer_Ptr)
{
	IMRT_Status status;
	char *func_name = "MM_Buffer_Copy";
	

	status  = IMRS_OK;
	
	
	if (Input_Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Input buffer pointer NULL" );
	}
	else
	if (Output_Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Output buffer pointer NULL" );
	}
	else
	if (IMRG_MM_StrHdl_Fmt_Func_Tab == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Format tab pointer NULL" );
	}
	else
	{
		/* buffer non identifie => demande identification */	
		if ( (Input_Buffer_Ptr->Fmt_Id == IMRD_FMT_ID_UNKNOWN) || (Input_Buffer_Ptr->Msg_Id == IMRD_MSG_ID_UNKNOWN) )
		{
		   status = MM_Buffer_Open( Input_Buffer_Ptr );
		}
		
		/* s'assurer que les donnees sont dans Data_Buf */
		if ( status  == IMRS_OK )
		{
			if (IMRG_MM_StrHdl_Fmt_Func_Tab[Input_Buffer_Ptr->Fmt_Id].Flush_Ptr == NULL)
			{
				status = IMRS_KO;
				IMRD_LM_LOG_ERROR_1("fonction Flush = NULL pour format %s" , 
					    IMRG_MM_StrHdl_Fmt_Func_Tab[Input_Buffer_Ptr->Fmt_Id].Nom);
			}
			else
			{
				status = IMRG_MM_StrHdl_Fmt_Func_Tab[Input_Buffer_Ptr->Fmt_Id].Flush_Ptr(Input_Buffer_Ptr);
			}
		}
		
		/* reinitialiser le buffer destinataire */
		if (status == IMRS_OK)
		{
			status = MM_Buffer_Clear( Output_Buffer_Ptr );
		}
		
		/* mise a jour du format et du type du buffer destinataire */
		if (status == IMRS_OK)
		{
			status = MM_Buffer_Ident_Update(Output_Buffer_Ptr, 
			                                Input_Buffer_Ptr->Fmt_Id, 
			                                Input_Buffer_Ptr->FmtAttr_Value, 
			                                Input_Buffer_Ptr->Msg_Id);
		}
	
		/* mise a jour des applications et direction du buffer destinataire */
		if (status == IMRS_OK)
		{
			Output_Buffer_Ptr->Appl_Id_Input = Input_Buffer_Ptr->Appl_Id_Input ;
			Output_Buffer_Ptr->Appl_Id_Output = Input_Buffer_Ptr->Appl_Id_Output ;
			Output_Buffer_Ptr->Dir_Id = Input_Buffer_Ptr->Dir_Id ;
		}
		
		
		/* mise a jour de l'attribut du format du buffer destinataire */
		if (status == IMRS_OK)
		{
			strcpy(Output_Buffer_Ptr->FmtAttr_Value , Input_Buffer_Ptr->FmtAttr_Value) ;
		}
	
		
		/* copie des donnees du buffer source dans le buffer destinataire */
		if (status == IMRS_OK)
		{
			if (IMRG_MM_StrHdl_Fmt_Func_Tab[Input_Buffer_Ptr->Fmt_Id].Copy_Ptr == NULL)
			{
				status = IMRS_KO;
				IMRD_LM_LOG_ERROR_1("fonction Copy = NULL pour format %s" , 
					    IMRG_MM_StrHdl_Fmt_Func_Tab[Input_Buffer_Ptr->Fmt_Id].Nom);
			}
			else
			{
				status = IMRG_MM_StrHdl_Fmt_Func_Tab[Input_Buffer_Ptr->Fmt_Id].Copy_Ptr(Output_Buffer_Ptr,
				                                                                Input_Buffer_Ptr);
			}
		}
		if (status == IMRS_OK)
		{
			Output_Buffer_Ptr->Flag = IMRD_BUFFER_FLAG_CLOSED;
		}
	}


	return ( status );
}

IMRT_Status MM_Buffer_Map(IMRT_Buffer *Output_Buffer_Ptr, 
                          IMRT_Buffer *Input_Buffer_Ptr, 
                          IMRT_MsgMap *MsgMap_Ptr)
{
	IMRT_Status status;
	char *func_name = "MM_Buffer_Map";
	
	status = IMRS_OK;
	
	if (Input_Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Input buffer pointer NULL" );
	}
	else
	if (Output_Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Output buffer pointer NULL" );
	}
	else
	{
		if ( ( Input_Buffer_Ptr->Flag  != IMRD_BUFFER_FLAG_OPENED ) ||
		     ( Output_Buffer_Ptr->Flag != IMRD_BUFFER_FLAG_OPENED ) )
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("Buffer input or output not open" );
		}
		else
		{
			IMRG_Input_Buffer_Ptr  = Input_Buffer_Ptr;
			IMRG_Output_Buffer_Ptr = Output_Buffer_Ptr;
			status = IMR_SDM_MsgMap_Proceed( MsgMap_Ptr, IMR_MM_Buffer_FieldMap);
			if (status != IMRS_OK)
				IMRD_LM_LOG_ERROR_1("MsgMap proceed ended with status (%d)" , status);
			IMRG_Input_Buffer_Ptr  = NULL;
			IMRG_Output_Buffer_Ptr = NULL;
		}
	}


	return ( status );
}

IMRT_Status IMR_MM_Buffer_FieldMap( IMRT_Field *Field_Output_Ptr , IMRT_Field *Field_Input_Ptr )
{
	IMRT_Buffer      *buffer_ptr;
	IMRT_Buffer_Data *data_In_ptr;
	IMRT_Buffer_Data *data_Out_ptr;
	IMRT_Status status;
	size_t field_len;
	IMRT_Boolean cnv_OK;
	char *func_name = "IMR_MM_Buffer_FieldMap";
	
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 ,  "============================================================================")
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 , "Field map demand : Begin");
	IMRD_LM_LOG_TRACE_2(IMRD_LOG_LEVEL_VERBOSE_1 , "Field Input name : (%s) Field Output name :  (%s)" , Field_Input_Ptr->Name , Field_Output_Ptr->Name);	

	status  = IMRS_OK;
	cnv_OK =IMRD_TRUE;
	data_In_ptr   = NULL;
	data_Out_ptr   = NULL;
	buffer_ptr = NULL;
	field_len = 0L;
	
	if (IMRG_Input_Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Input buffer pointer NULL" );
	}
	else
	if (IMRG_Output_Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Output buffer pointer NULL" );
	}
	else
	if (IMRG_MM_StrHdl_Fmt_Func_Tab == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Format tab pointer NULL" );
	}
	else
	{
		buffer_ptr = IMRG_Input_Buffer_Ptr;
		if (IMRG_MM_StrHdl_Fmt_Func_Tab[buffer_ptr->Fmt_Id].Get_Ptr == NULL)
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_1("fonction Get = NULL pour format %s" , 
					 IMRG_MM_StrHdl_Fmt_Func_Tab[buffer_ptr->Fmt_Id].Nom);
		}
		else
		{
			status = IMRG_MM_StrHdl_Fmt_Func_Tab[buffer_ptr->Fmt_Id].Get_Ptr(&data_In_ptr, 
			                                                         Field_Input_Ptr,
			                                                         buffer_ptr ,
			                                                         &field_len );
		        IMRD_LM_LOG_TRACE_2(IMRD_LOG_LEVEL_VERBOSE_2 , "Data and len after get [%s] : (%ld)",data_In_ptr,field_len);	
		}
		
	
		if (status == IMRS_OK)
		{
			buffer_ptr = IMRG_Output_Buffer_Ptr;
			if (IMRG_MM_StrHdl_Fmt_Func_Tab[buffer_ptr->Fmt_Id].Cnv_Ptr == NULL)
			{
				cnv_OK = IMRD_FALSE;
				IMRD_LM_LOG_WARNING_1("fonction de convertion nulle pour handler de format %s" , 
					    IMRG_MM_StrHdl_Fmt_Func_Tab[buffer_ptr->Fmt_Id].Nom);
			}
			else
			{
				status = IMRG_MM_StrHdl_Fmt_Func_Tab[buffer_ptr->Fmt_Id].Cnv_Ptr(&data_Out_ptr,
				                                                         data_In_ptr, 
				                                                         Field_Output_Ptr,
				                                                         &field_len,
				                                                         IMRG_MM_StrHdl_Cnv_Func_Tab);
		               IMRD_LM_LOG_TRACE_2(IMRD_LOG_LEVEL_VERBOSE_2 , "Data and len after convert [%s] : (%ld)",data_Out_ptr,field_len);
			}
		}
	
		if (status == IMRS_OK)
		{
			if (IMRG_MM_StrHdl_Fmt_Func_Tab[buffer_ptr->Fmt_Id].Put_Ptr == NULL)
			{
				status = IMRS_KO;
				IMRD_LM_LOG_ERROR_1("fonction Put = NULL pour format %s" , 
					    IMRG_MM_StrHdl_Fmt_Func_Tab[buffer_ptr->Fmt_Id].Nom);
			        if (cnv_OK)
			        {
			                free(data_Out_ptr);
			                IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_2 , "liberation du data_Out_ptr" );
			        }
			        else
			        {
			                free(data_In_ptr);
			                IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_2 , "liberation du data_In_ptr" );
			        }
			}
			else
			{
	       		        if (cnv_OK)
	       		        {
	        			status = IMRG_MM_StrHdl_Fmt_Func_Tab[buffer_ptr->Fmt_Id].Put_Ptr(data_Out_ptr, 
				                                                                        Field_Output_Ptr,
				                                                                        buffer_ptr,
				                                                                        field_len);
	                        }
	                        else
	                        {
	        			status = IMRG_MM_StrHdl_Fmt_Func_Tab[buffer_ptr->Fmt_Id].Put_Ptr(data_In_ptr, 
				                                                                        Field_Output_Ptr,
				                                                                        buffer_ptr,
				                                                                        field_len);
	                        }
			}
		}
	}

	MM_Buffer_List( buffer_ptr, " Buffer after put");
	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_2 , "Field map demand : End with status :  (%d)" , status);
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_2 ,  "============================================================================")

	return ( status );
}
