/*----------------------------------------------------------------------------*/
/* File: buffer_mng.h							      */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#ifndef _BUFFER_MNG_H_
#define _BUFFER_MNG_H_

/*----------------------------------------------------------------------------*/
/* Global Include                                                             */
/*----------------------------------------------------------------------------*/

#include <dlfcn.h>
#include "libmapping_mng.h"

/*----------------------------------------------------------------------------*/
/* Local Includes                                                             */
/*----------------------------------------------------------------------------*/

#ifdef _BUFFER_MNG_C_

#   include "fmt_hdl_fixed.h"
#   include "fmt_hdl_xml.h"

#   ifdef TUX_SUPPORT
#       include "fmt_hdl_fml.h"
#   endif /* TUX_SUPPORT */

#    include "cnv_hdl_pad_right.h"
#    include "cnv_hdl_pad_left.h"
#    include "cnv_hdl_strip_right.h"
#    include "cnv_hdl_strip_left.h"
#    include "cnv_hdl_cut_right.h"
#    include "cnv_hdl_strip_num.h"

#endif

/*----------------------------------------------------------------------------*/
/* Local definition                                                           */
/*----------------------------------------------------------------------------*/

#ifdef _BUFFER_MNG_C_


        typedef IMRT_Status  (IMRT_MM_Fmt_Hdl_Init)( void );
        typedef IMRT_Status  (IMRT_MM_Fmt_Hdl_DeInit)( void );
        typedef IMRT_Status  (IMRT_MM_Fmt_Hdl_Data_Free)( IMRT_Buffer * );
        typedef IMRT_Status  (IMRT_MM_Fmt_Hdl_Ident)( IMRT_Msg_Id *, IMRT_Buffer * );
        typedef IMRT_Status  (IMRT_MM_Fmt_Hdl_Open)( IMRT_Buffer * );
        typedef IMRT_Status  (IMRT_MM_Fmt_Hdl_Close)( IMRT_Buffer * );
        typedef IMRT_Status  (IMRT_MM_Fmt_Hdl_Data_List)( IMRT_Buffer * );
        typedef IMRT_Status  (IMRT_MM_Fmt_Hdl_Copy)( IMRT_Buffer * , IMRT_Buffer * );
        typedef IMRT_Status  (IMRT_MM_Fmt_Hdl_Flush)( IMRT_Buffer * );
        typedef IMRT_Status  (IMRT_MM_Fmt_Hdl_Get)( IMRT_Buffer_Data ** , IMRT_Field * , IMRT_Buffer * , size_t * );
        typedef IMRT_Status  (IMRT_MM_Fmt_Hdl_Cnv)( IMRT_Buffer_Data ** , IMRT_Buffer_Data * , IMRT_Field * , size_t *, IMRT_StrHdl_Cnv_Func * );
        typedef IMRT_Status  (IMRT_MM_Fmt_Hdl_Put)( IMRT_Buffer_Data * , IMRT_Field * , IMRT_Buffer * , size_t );

    typedef struct IMRT_StrHdl_Fmt_Func
	
	{   IMRT_Fmt_Name        	Nom;
	    IMRT_MM_Fmt_Hdl_Init	*Init_Ptr;
	    IMRT_MM_Fmt_Hdl_DeInit	*DeInit_Ptr;
	    IMRT_MM_Fmt_Hdl_Data_Free	*Data_Free_Ptr;
	    IMRT_MM_Fmt_Hdl_Ident	*Ident_Ptr;
	    IMRT_MM_Fmt_Hdl_Open	*Open_Ptr;
	    IMRT_MM_Fmt_Hdl_Close	*Close_Ptr;
	    IMRT_MM_Fmt_Hdl_Data_List	*Data_List_Ptr;
	    IMRT_MM_Fmt_Hdl_Copy	*Copy_Ptr;
	    IMRT_MM_Fmt_Hdl_Flush	*Flush_Ptr;
	    IMRT_MM_Fmt_Hdl_Get	        *Get_Ptr;
	    IMRT_MM_Fmt_Hdl_Cnv	        *Cnv_Ptr;
	    IMRT_MM_Fmt_Hdl_Put	        *Put_Ptr;
	    
	} IMRT_StrHdl_Fmt_Func;

		


#endif

/*----------------------------------------------------------------------------*/
/*	Global Variables						      */
/*----------------------------------------------------------------------------*/

#ifdef _BUFFER_MNG_C_

        void                    *handle;
	IMRT_Buffer             *IMRG_Input_Buffer_Ptr;
	IMRT_Buffer             *IMRG_Output_Buffer_Ptr;

        
	IMRT_StrHdl_Fmt_Func    *IMRG_MM_StrHdl_Fmt_Func_Tab;
	IMRT_Fmt_Id             IMRG_Taille_Fmt_Max;

	IMRT_Cnv_Id             IMRG_Taille_Cnv_Max;     


#endif



/*----------------------------------------------------------------------------*/
/*	Prototypes																							 */
/*----------------------------------------------------------------------------*/

#ifdef _BUFFER_MNG_C_

	IMRT_Status	MM_Buffer_Init	        ( void );
	IMRT_Status	MM_Buffer_DeInit	( void );		
	IMRT_Status     IMR_MM_Fmt_Add          ( IMRT_Fmt_Id , IMRT_Fmt_Name , IMRT_Fmt_Id );
        IMRT_Status     IMR_MM_Cnv_Add          ( IMRT_Cnv_Id , IMRT_Cnv_Name , IMRT_Cnv_Id ); 
	IMRT_Status	MM_Buffer_List		( IMRT_Buffer * , char * );
	IMRT_Status	MM_Buffer_Alloc		( IMRT_Buffer ** );
	IMRT_Status	MM_Buffer_Free 		( IMRT_Buffer ** );
	IMRT_Status     MM_Buffer_Clear		( IMRT_Buffer * );
        IMRT_Status     MM_Buffer_Ident         ( IMRT_Msg_Id , IMRT_Buffer *);
	IMRT_Status     MM_Buffer_Open		( IMRT_Buffer * );
        IMRT_Status     MM_Buffer_Close		( IMRT_Buffer * );
	IMRT_Status     MM_Buffer_Copy		( IMRT_Buffer * , IMRT_Buffer * );
	IMRT_Status     MM_Buffer_Map		( IMRT_Buffer * , IMRT_Buffer * , IMRT_MsgMap * );
	IMRT_Status     MM_Buffer_Ident_Update	( IMRT_Buffer * , IMRT_Fmt_Id , IMRT_FmtAttr_Value  , IMRT_Msg_Id );
	IMRT_Status	Buffer_Application_Check( IMRT_Appl_Id );
	IMRT_Status	Buffer_Direction_Check	( IMRT_Dir_Id );
	IMRT_Status     Buffer_Format_Check	( IMRT_Fmt_Id );
	IMRT_Status	Buffer_Statut_Check	( IMRT_Buffer_Status );
	IMRT_Status	Buffer_Etat_Check	( IMRT_Buffer_Flag );
	IMRT_Status	MM_Buffer_Check		( IMRT_Buffer * );
	IMRT_Status     Buffer_Data_Free 	( IMRT_Buffer * );
	IMRT_Status     IMR_MM_Buffer_FieldMap	( IMRT_Field * , IMRT_Field * );
		
#else

	extern IMRT_Status	MM_Buffer_Init	        ( void );
	extern IMRT_Status	MM_Buffer_DeInit	( void );
	extern IMRT_Status	MM_Buffer_List		( IMRT_Buffer * , char * );
	extern IMRT_Status	MM_Buffer_Alloc		( IMRT_Buffer ** );
	extern IMRT_Status	MM_Buffer_Free 		( IMRT_Buffer ** );
	extern IMRT_Status 	MM_Buffer_Clear		( IMRT_Buffer * );
	extern IMRT_Status      MM_Buffer_Ident         ( IMRT_Msg_Id , IMRT_Buffer *);
	extern IMRT_Status 	MM_Buffer_Open		( IMRT_Buffer * );
	extern IMRT_Status	MM_Buffer_Close		( IMRT_Buffer * );
	extern IMRT_Status	MM_Buffer_Check		( IMRT_Buffer * );
	extern IMRT_Status 	MM_Buffer_Copy		( IMRT_Buffer * , IMRT_Buffer * );
	extern IMRT_Status 	MM_Buffer_Map		( IMRT_Buffer * , IMRT_Buffer * , IMRT_MsgMap * );
	extern IMRT_Status      MM_Buffer_Ident_Update	( IMRT_Buffer * , IMRT_Fmt_Id , IMRT_FmtAttr_Value , IMRT_Msg_Id );

#endif


/*----------------------------------------------------------------------------*/

#endif                                   
