/*----------------------------------------------------------------------------*/
/* File: cnv_hdl_cut_right.c    				         	      */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#define _LIBMAPPING_MNG_
#define _CNV_HANDLER_CUT_RIGHT_C_

#include "cnv_hdl_cut_right.h" 

IMRT_Status	MM_Cut_Right_Init()
{
	return (IMRS_OK);
}

IMRT_Status	MM_Cut_Right_DeInit()
{

	return (IMRS_OK);
}

IMRT_Status	MM_Cut_Right_Convert(IMRT_Buffer_Data *Data_Out_Ptr , IMRT_Buffer_Data *Data_In_Ptr , size_t * Len_Ptr , size_t size , IMRT_Cnv_Attr Cnv_Attr )
{
	IMRT_Status status;
	int nb_char_to_rm;
	int cpt;
	/*char attr;*/
	size_t len_tmp;
	IMRT_Buffer_Data *data_tmp_ptr;
	char *func_name = "MM_Cut_Right_Convert";


        status = IMRS_OK;
        cpt = 0;
        
       	if (Len_Ptr == NULL)
       	{
       		IMRD_LM_LOG_ERROR_0("Len pointer NULL" );
       		status = IMRS_KO;
       	}
       	else if (Data_In_Ptr == NULL)
       	{
       		IMRD_LM_LOG_ERROR_0("Input pointer NULL" );
       		status = IMRS_KO;
       	}
       	else
       	if (Data_Out_Ptr == NULL)
       	{
       		IMRD_LM_LOG_ERROR_0("Output pointer NULL" );
       		status = IMRS_KO;
       	}
       	else
       	{
	        len_tmp = *Len_Ptr ;
	        nb_char_to_rm = atoi(Cnv_Attr);
	        if (len_tmp > size)
	        {
	        	IMRD_LM_LOG_ERROR_0("Field Len > size" );
	        	status = IMRS_KO;
	        }
	        else
	        if ((size_t)nb_char_to_rm > len_tmp)
	        {
	        	IMRD_LM_LOG_ERROR_2("Nb of char : (%d) > len (%ld)" , nb_char_to_rm , len_tmp);
	        	status = IMRS_KO;
	        }
	        else
	        if (nb_char_to_rm <= 0)
	        {
	        	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_1 , "Nb of char : (%d) Nothing to delete" , nb_char_to_rm);
	        	status = IMRD_STATUS_NOTHING_TO_DO;
	        }
	}

	if (status == IMRS_OK)
	{
		
		memcpy(Data_Out_Ptr , Data_In_Ptr , (len_tmp - nb_char_to_rm));
		
	        Data_Out_Ptr[(len_tmp - nb_char_to_rm)] = '\0';       
	        
	        *Len_Ptr = (len_tmp - nb_char_to_rm) ;
	}

	if (status == IMRD_STATUS_NOTHING_TO_DO)
		status = IMRS_OK;

	return (status);
}

