/*----------------------------------------------------------------------------*/
/* File: fmt_hdl_dltd.c    				         	      */
/* Comptage des occurences : la premiere est numerotee 0        	      */
/* La zone de donnees Data_Tmp_Ptr du buffer contient la prochaine occurence  */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#define _LIBMAPPING_MNG_
#define _DELIMITED_HANDLER_C_

#include "fmt_hdl_dltd.h" 

IMRT_Status	MM_Delimited_StrHdl_Init()
{
	IMRG_Delimited_Buffer_Ident_Ptr = NULL;
	return (IMRS_OK);
}

IMRT_Status	MM_Delimited_StrHdl_DeInit()
{
	return (IMRS_OK);
}

IMRT_Status Delimited_Hdl_DataBuf_Alloc( IMRT_Buffer *Buffer_Ptr, size_t Taille_Alloc )
{
	IMRT_Buffer_Data *data_Ptr;
	IMRT_Status status;
	IMRT_Delimited_TmpId * TmpId_Ptr;
	char *func_name = "Delimited_Hdl_DataBuf_Alloc";

	
	status   = IMRS_OK;
	data_Ptr = NULL;
	
	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	if (Buffer_Ptr->Status == IMRD_BUFFER_STATUS_ALLOCATED)
	{
		if (Taille_Alloc > Buffer_Ptr->Data_Buf_Size)
		{
			data_Ptr = (IMRT_Buffer_Data *) realloc ( (void *) Buffer_Ptr->Data_Buf_Ptr, Taille_Alloc );
			if (data_Ptr == NULL)
			{
				status = IMRS_KO;
				IMRD_LM_LOG_ERROR_0("Realloc error");
			}
			else
			{
				memset( data_Ptr + Buffer_Ptr->Data_Buf_Size, '\0', Taille_Alloc - Buffer_Ptr->Data_Buf_Size );
				Buffer_Ptr->Data_Buf_Ptr  = data_Ptr;
				Buffer_Ptr->Data_Buf_Size = Taille_Alloc;
			}
		}
	}
	else
	{
		data_Ptr = (IMRT_Buffer_Data *) malloc ( Taille_Alloc );
		if (data_Ptr == NULL)
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("Malloc error");
		}
		else
		{
			memset( data_Ptr, '\0', Taille_Alloc );
			Buffer_Ptr->Data_Buf_Ptr  = data_Ptr;
			Buffer_Ptr->Status        = IMRD_BUFFER_STATUS_ALLOCATED;
			Buffer_Ptr->Data_Buf_Size = Taille_Alloc;
			Buffer_Ptr->Data_Buf_Len  = 0;
		}
		
		if (status == IMRS_OK)
		{
		        TmpId_Ptr = (IMRT_Delimited_TmpId *) malloc ( sizeof(IMRT_Delimited_TmpId));
        		if (TmpId_Ptr == NULL)
        		{
        			status = IMRS_KO;
        			IMRD_LM_LOG_ERROR_0("Malloc error");
        			free(Buffer_Ptr->Data_Buf_Ptr);
        			Buffer_Ptr->Status = IMRD_BUFFER_STATUS_FREED;
        			Buffer_Ptr->Data_Buf_Size = 0;
        		}
        		else
        		{
        		        /* Initialisation de la premiere occurence a 0 */
        			memset( TmpId_Ptr, '\0', sizeof(IMRT_Delimited_TmpId) );
        			TmpId_Ptr->TmpId = 0;
        			Buffer_Ptr->Data_Tmp_Ptr  = (IMRT_Buffer_Data *)(TmpId_Ptr);		
        		}
	        }

	}
	

	return ( status );
}

IMRT_Status MM_Delimited_StrHdl_Data_Free( IMRT_Buffer  *Buffer_Ptr)
{
	IMRT_Status status;
	char *func_name = "MM_Delimited_StrHdl_Data_Free";

	
	status = IMRS_OK;

	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	if (Buffer_Ptr->Status == IMRD_BUFFER_STATUS_ALLOCATED)
	{
		if(Buffer_Ptr->Data_Buf_Ptr != NULL)
		{
			free(Buffer_Ptr->Data_Buf_Ptr);
		}
		Buffer_Ptr->Data_Buf_Ptr  = NULL;	
		Buffer_Ptr->Status        = IMRD_BUFFER_STATUS_FREED;
		Buffer_Ptr->Data_Buf_Size = 0;
		Buffer_Ptr->Data_Buf_Len  = 0;
		if(Buffer_Ptr->Data_Tmp_Ptr != NULL)
			free(Buffer_Ptr->Data_Tmp_Ptr);
		Buffer_Ptr->Data_Tmp_Ptr  = NULL;
		Buffer_Ptr->Data_Tmp_Changed = IMRD_FALSE;
		Buffer_Ptr->Data_Tmp_Ptr  = NULL;
	}

	return ( status );
}

IMRT_Status Delimited_Hdl_Format_Ident( IMRT_Buffer *Buffer_Ptr )
{
	IMRT_Status status;
	char *func_name = "Delimited_Hdl_Format_Ident";


	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	if (Buffer_Ptr->Status == IMRD_BUFFER_STATUS_ALLOCATED)
	{
		status = IMRS_OK;
	}
	else
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer not allocated");
	}
	

	return ( status );
}

IMRT_Status MM_Delimited_StrHdl_Ident( IMRT_Msg_Id *Msg_Id_Ident_Ptr, 
                                        IMRT_Buffer *Buffer_Ptr )
{
        
        IMRT_Fmt_Id Format;
        IMRT_Status status;
        IMRT_Msg_Id *Msg_Id_Tmp;
        char *func_name = "MM_Delimited_StrHdl_Ident";
        

	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	if (Msg_Id_Ident_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Msg_Id_Ident_Ptr NULL" );
	}
	else
        {
        	Format = Buffer_Ptr->Fmt_Id;
	        Msg_Id_Tmp = Msg_Id_Ident_Ptr;
	        status = IMRS_OK;
	
		IMRG_Delimited_Buffer_Ident_Ptr = Buffer_Ptr;
		/* Le Msg_Id transmis au SDM est le point de depart de l'identification */
		/* C'est egalement le resultat qui sera inclus dans le buffer */
		status = IMR_SDM_MsgIdent_Proceed(Format, 
		                                  Msg_Id_Tmp,
		                                  IMR_MM_Delimited_StrHdl_Field_Value,
		                                  IMR_MM_Delimited_StrHdl_Field_Exist,
		                                  IMR_MM_Delimited_StrHdl_Msg_Size);
		IMRG_Delimited_Buffer_Ident_Ptr = NULL;
	
	        Msg_Id_Ident_Ptr = Msg_Id_Tmp;
	}

	return ( status );

}

IMRT_Status Delimited_Hdl_TmpId_Init( IMRT_Buffer *Buffer_Ptr , short nb_occ)
{
	IMRT_Status status;
	IMRT_Delimited_TmpId * TmpId_Ptr;
	char *func_name = "Delimited_Hdl_TmpId_Init";

	status = IMRS_OK;
	TmpId_Ptr = NULL;
        TmpId_Ptr = (IMRT_Delimited_TmpId *) malloc ( sizeof(IMRT_Delimited_TmpId));
        if (TmpId_Ptr == NULL)
        {
                status = IMRS_KO;
        	IMRD_LM_LOG_ERROR_0("Malloc error");
        }
        else
        if (Buffer_Ptr == NULL)
        {
                status = IMRS_KO;
        	IMRD_LM_LOG_ERROR_0("Buffer pointer null");
        }
        else
        {
                /* Initialisation de la premiere occurence a 0 */
        	memset( TmpId_Ptr, '\0', sizeof(IMRT_Delimited_TmpId) );
        	TmpId_Ptr->TmpId = nb_occ;
        	Buffer_Ptr->Data_Tmp_Ptr  = (IMRT_Buffer_Data *)(TmpId_Ptr);
        }


        return status;        
}



IMRT_Status Delimited_Hdl_Occ_Count( IMRT_Buffer_Data *data_ptr , short *nb_occ , IMRT_FmtAttr_Value attr)
{
	IMRT_Status status;
	IMRT_Buffer_Data *deb_data_ptr;
	IMRT_Boolean loop_end;
	short cpt;
	char indicateur[2];
	char *delim_ptr;
	IMRT_FmtAttr_Value delimiteur;
	char *func_name = "Delimited_Hdl_Occ_Count";

	status = IMRS_OK;
	deb_data_ptr = NULL;
	loop_end = IMRD_FALSE;
	
	
        if (nb_occ == NULL)
        {
                status = IMRS_KO;
        	IMRD_LM_LOG_ERROR_0("Nb occ pointer null");
        }
        else
	if ((data_ptr == NULL) || (strcmp(attr , "")==0))
	{
	        status = IMRS_KO;
	        IMRD_LM_LOG_ERROR_0("Error : data pointer NULL or attribute is an empty string");
	}
	
	if ( status == IMRS_OK )
	{
		strcpy(delimiteur , "");
		strncpy(indicateur , attr , 1);
		indicateur[1] = '\0';
		delim_ptr = NULL;
		delim_ptr = (attr);
		delim_ptr++;
		strcpy(delimiteur , delim_ptr);
	
	
	/* Controle de validite de l'indicateur  */
		if ((strcmp(indicateur , "S") != 0) && (strcmp(indicateur , "T") != 0))
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_1("Indicateur non correct [%s]",indicateur);
		}
	}



	if (status == IMRS_OK)
	{
		if (strcmp(data_ptr , "") == 0)
		{
			cpt = 0;
		}
		else
		{
	        	/* La premire occurence est la 0 */        
	        	cpt = 0;
	                /* positionnement au debut de la chaine */
	                deb_data_ptr = data_ptr;
	                /* boucle de comptage des occurrences */
	                while (!loop_end)
	                {
	                        deb_data_ptr = (IMRT_Buffer_Data *)strstr((const char *)deb_data_ptr , (const char *)delimiteur);
	                        IMRD_LM_LOG_TRACE_1( IMRD_LOG_LEVEL_VERBOSE_2, "attr [%s]" , delimiteur);
	                        IMRD_LM_LOG_TRACE_1( IMRD_LOG_LEVEL_VERBOSE_2, "deb_data_ptr [%s]" , deb_data_ptr);
	                        
	                        if (deb_data_ptr != NULL)
	                        {
	                                cpt++;
	                                deb_data_ptr++;
	                        }
	                        else
	                                loop_end = IMRD_TRUE;
	                }
	                
	        }
	        /* si l'indicateur est S il faut rajouter une occurence */
	        if ((cpt != 0)&&(strcmp(indicateur , "S") == 0))
	        	cpt++;
	        	
	        *nb_occ = cpt;
        }
	
        return status;        
}


IMRT_Status MM_Delimited_StrHdl_Open( IMRT_Buffer *Buffer_Ptr)
{
	IMRT_Msg_Id type_temp;
	IMRT_Status status;
	IMRT_Delimited_TmpId *TmpId_Ptr;
	IMRT_Boolean flag_init;
	short nb_occ;
	char *func_name = "MM_Delimited_StrHdl_Open";
	


	status = IMRS_OK;
	nb_occ = 0;
	flag_init = IMRD_FALSE;

	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	{
		type_temp = Buffer_Ptr->Msg_Id;
	        if (Buffer_Ptr->Status != IMRD_BUFFER_STATUS_ALLOCATED)
	        {
	                
	                IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_2, "Le buffer n'a pas le status allocated");
		        /* Cas d'open du buffer en ecriture */
		        /* le type du message est passe */
		        /* par le SDM : il doit donc etre connu */
			if (type_temp == IMRD_MSG_ID_UNKNOWN) 
			{
				status = IMRS_KO;
				IMRD_LM_LOG_ERROR_0("Msg Id unknown");
			}
			else
			{
			        status = Delimited_Hdl_DataBuf_Alloc( Buffer_Ptr, DELIMITED_TAILLE_ALLOC );
			}
	        }
	        else /* le status est allocated */
	        {
	                IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_2, "Le buffer a  le status allocated");
	                
	                /* comptage du nombre d'occurences */
	                status = Delimited_Hdl_Occ_Count( Buffer_Ptr->Data_Buf_Ptr , &nb_occ , Buffer_Ptr->FmtAttr_Value);
	                if (Buffer_Ptr->Data_Tmp_Ptr != NULL)
	                {
	                	TmpId_Ptr = (IMRT_Delimited_TmpId *)Buffer_Ptr->Data_Tmp_Ptr;
	                	if (TmpId_Ptr->TmpId != nb_occ)
	                	{
	                		flag_init = IMRD_TRUE;
	                		free(Buffer_Ptr->Data_Tmp_Ptr);
	                	}
	                }
                	else
                	{
                		flag_init = IMRD_TRUE;
                	}	

	                if ((status == IMRS_OK) && (flag_init == IMRD_TRUE))
	                {
	                        /* initialisation de Data_Tmp_Ptr avec le nombre d'occurence */
	                        status = Delimited_Hdl_TmpId_Init(Buffer_Ptr , nb_occ);
	                }
	                
	                
	                if (status == IMRS_OK)
	                {
	                        /* controle du format (inutile actuellement) */
	                        status = Delimited_Hdl_Format_Ident( Buffer_Ptr ) ;
	                        if (status != IMRS_OK)
	                        {
	                                status = IMRS_BAD_FORMAT;
	                        }
	                }
	        }
	}

	return ( status );
}



IMRT_Status MM_Delimited_StrHdl_Close( IMRT_Buffer *Buffer_Ptr)
{
	char *func_name = "MM_Delimited_StrHdl_Close";
	IMRT_Status	status;

	status = IMRS_OK;

	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	{
		status = MM_Delimited_StrHdl_Flush( Buffer_Ptr ); 
		if ((status == IMRS_OK) && (Buffer_Ptr->Data_Tmp_Ptr != NULL))
		{
			free(Buffer_Ptr->Data_Tmp_Ptr);
			Buffer_Ptr->Data_Tmp_Ptr = NULL;
		}
	}

	return ( status );
}

IMRT_Status	MM_Delimited_StrHdl_Flush( IMRT_Buffer  *Buffer_Ptr)
{
	char *func_name = "MM_Delimited_StrHdl_Flush";

	return (IMRS_OK);
}



IMRT_Status MM_Delimited_StrHdl_Data_List( IMRT_Buffer *Buffer_Ptr)
{

	if (Buffer_Ptr == NULL)
	{
		IMRD_LM_LOG_WARNING_0("Buffer pointer NULL" );
	}
	else
	{
	    IMRD_LM_LOG_TRACE_3( IMRD_LOG_LEVEL_VERBOSE_1, "Buffer_List: Data_Buf_Ptr_Size: (%d)   Data_Buf_Ptr_Len: (%d)   Data_Buf_Ptr: (0x%X)", Buffer_Ptr->Data_Buf_Size, Buffer_Ptr->Data_Buf_Len, Buffer_Ptr->Data_Buf_Ptr);
	    IMRD_LM_LOG_TRACE_1( IMRD_LOG_LEVEL_VERBOSE_2, "Buffer_List: Data_Buf: [%s]", Buffer_Ptr->Data_Buf_Ptr);
	    IMRD_LM_LOG_TRACE_2( IMRD_LOG_LEVEL_VERBOSE_1, "Buffer_List: Data_Tmp_Changed: (%d)   Data_Tmp_Ptr: (0x%X)", Buffer_Ptr->Data_Tmp_Changed, Buffer_Ptr->Data_Tmp_Ptr);
	    if (Buffer_Ptr->Data_Tmp_Ptr != NULL)
	    	IMRD_LM_LOG_TRACE_1( IMRD_LOG_LEVEL_VERBOSE_2, "Buffer_List: TmpId: (%d)", (short )( ( (IMRT_Delimited_TmpId  *)(Buffer_Ptr->Data_Tmp_Ptr))->TmpId));
	}

    return (IMRS_OK);
}



IMRT_Status MM_Delimited_StrHdl_Copy( IMRT_Buffer *Output_Buffer_Ptr, IMRT_Buffer *Input_Buffer_Ptr )
{
	IMRT_Status status;
	char *func_name = "MM_Delimited_StrHdl_Copy";

	
	status = IMRS_OK;
	
	if (Input_Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Input buffer pointer NULL" );
	}
	else
	if (Output_Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Output buffer pointer NULL" );
	}
	else
	/* controle donnees dans Input_Buffer */
	if (Input_Buffer_Ptr->Status != IMRD_BUFFER_STATUS_ALLOCATED)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer input not allocated");
	}
	/* le buffer destinataire ne doit pas comporter de donnes */ 
	else if ( ( Output_Buffer_Ptr->Status == IMRD_BUFFER_STATUS_ALLOCATED) ||
	          ( Output_Buffer_Ptr->Data_Tmp_Ptr != NULL) )
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer output not allocated or Data_Tmp_Ptr no NULL");
	}
	else
	{
		status = Delimited_Hdl_DataBuf_Alloc(Output_Buffer_Ptr, Input_Buffer_Ptr->Data_Buf_Size);
		if (status == IMRS_OK)
		{
			memcpy( Output_Buffer_Ptr->Data_Buf_Ptr,
			        Input_Buffer_Ptr->Data_Buf_Ptr,
			        Input_Buffer_Ptr->Data_Buf_Size);
			Output_Buffer_Ptr->Data_Buf_Len = Input_Buffer_Ptr->Data_Buf_Len;
			if (Input_Buffer_Ptr->Data_Tmp_Ptr != NULL)
			{
			        memcpy( Output_Buffer_Ptr->Data_Tmp_Ptr,
			                Input_Buffer_Ptr->Data_Tmp_Ptr,
			                sizeof(IMRT_Delimited_TmpId));
			}
		}
	}
 

	return ( status );
}

IMRT_Status	Delimited_Hdl_Field_Exist(IMRT_Field *Field_Ptr , IMRT_Buffer *Buffer_Ptr)
{
        IMRT_Status status;
        char *func_name = "Delimited_Hdl_Field_Exist";
        
	
	status   = IMRS_OK;


	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	if (Field_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Field pointer NULL" );
	}
	else
        if (Field_Ptr->Delimited.Id >= ((IMRT_Delimited_TmpId *)(Buffer_Ptr->Data_Tmp_Ptr))->TmpId)
        {
                status   = IMRS_NO_IDENT;
                IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 , "Field not exist");
        }
        

	return ( status );

}

IMRT_Status	MM_Delimited_StrHdl_Get(IMRT_Buffer_Data **Data_Ptr_Ptr, IMRT_Field *Field_Ptr,  
                                IMRT_Buffer *Buffer_Ptr , size_t *len )
{
	IMRT_Buffer_Data *data_ptr;
	IMRT_Buffer_Data *deb_data_ptr;
	IMRT_Buffer_Data *end_data_ptr;
	IMRT_Status status;
	IMRT_Status status_ret;
	char *delim_ptr;
	IMRT_FmtAttr_Value delimiteur;
	size_t sizeTmp;
	size_t delimLen;
	char indicateur[2];
	short indice;
	
	char *func_name = "MM_Delimited_StrHdl_Get";

	
	status   = IMRS_OK;
	deb_data_ptr = NULL;
	end_data_ptr = NULL;
	sizeTmp = 0L;
	indice = 0;
	
	if (Data_Ptr_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Data_Ptr_Ptr NULL");
	}
	else if (len == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("len pointer NULL");
	}
	else if (Buffer_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer_Ptr NULL");
	}
	else if (Field_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Field_Ptr NULL");
	}
	else if (strcmp((Buffer_Ptr->FmtAttr_Value) , "") == 0)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Delimiteur vide");
	}
		

	if ( status == IMRS_OK )
	{
		strncpy(indicateur , Buffer_Ptr->FmtAttr_Value , 1);
		indicateur[1] = '\0';
		strcpy(delimiteur , "");
		delim_ptr = (Buffer_Ptr->FmtAttr_Value);
		delim_ptr++;
		strcpy(delimiteur , delim_ptr);
		delimLen = strlen(delimiteur);
	}

	/* Controle de validite de l'indicateur  */
	if ((strcmp(indicateur , "S") != 0) && (strcmp(indicateur , "T") != 0))
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_1("Indicateur non correct [%s]",indicateur);
	}


        /* Controle de validite de l'Id */
        if (status == IMRS_OK)
        {
	        if (Field_Ptr->Delimited.Id == IMRD_DELIMITED_ID_UNKNOWN)
	        {
	            status   = IMRS_KO;
	            IMRD_LM_LOG_ERROR_0("Occurence Delimited unknown");
	        }
	}

        /* Controle de l'existence du champ */
        if (status == IMRS_OK)
                status = Delimited_Hdl_Field_Exist(Field_Ptr , Buffer_Ptr);
        /* Le champ n'existe pas => on prend la valeur par defaut */
        if (status == IMRS_NO_IDENT)
        {
                sizeTmp = strlen(Field_Ptr->Delimited.Def_Val);
                data_ptr = (IMRT_Buffer_Data *) malloc ( sizeTmp + 1);
                if (data_ptr == NULL)
                {
                        status   = IMRS_KO;
                        IMRD_LM_LOG_ERROR_0("Malloc error");
                }
                else
                {
                        memset( data_ptr, '\0', sizeTmp + 1 );
                        memcpy(data_ptr , Field_Ptr->Delimited.Def_Val , sizeTmp );
                        *(data_ptr + sizeTmp) = '\0';
                        *Data_Ptr_Ptr = data_ptr;
                        status   = IMRS_OK;
                }
        }       
        /* le champ existe => on extrait la valeur */
        else if (status == IMRS_OK)
        {
                /* positionnement au debut de la chaine */
                deb_data_ptr = Buffer_Ptr->Data_Buf_Ptr;
                /* Placement sur la bonne occurence */
                while ((indice < Field_Ptr->Delimited.Id)&&(status == IMRS_OK))
                {
                        deb_data_ptr = (IMRT_Buffer_Data *)strstr((const char *)deb_data_ptr , (const char *)delimiteur);
                        if (deb_data_ptr == NULL)
                        {
                                status   = IMRS_KO;
                                IMRD_LM_LOG_ERROR_0("Search occurence error");
                        }
                        else
                        {
                                deb_data_ptr = deb_data_ptr + delimLen ;
                                indice++;
                        }
                }
                /*positionnement a la fin de la chaine */
                end_data_ptr = (IMRT_Buffer_Data *)strstr((const char *)deb_data_ptr , (const char *)delimiteur);
                if (end_data_ptr == NULL)
                {
                	/* Si l'indicateur est S */
                        /* Si c'est la derniere occurence il n'y a pas de delimiteur */
                        if (strcmp(indicateur , "S") == 0)
                        	sizeTmp = (Buffer_Ptr->Data_Buf_Len - (deb_data_ptr - (Buffer_Ptr->Data_Buf_Ptr)));
                        else
                        {
                        	status   = IMRS_KO;
                        	IMRD_LM_LOG_ERROR_0("Incorrect format");
                        }
                }
                else
                {
                        sizeTmp = end_data_ptr - deb_data_ptr ;
                }
                data_ptr = (IMRT_Buffer_Data *) malloc ( sizeTmp + 1);
                if (data_ptr == NULL)
                {
                        status   = IMRS_KO;
                        IMRD_LM_LOG_ERROR_0("Malloc error");
                }
                else
                {
                        memset( data_ptr, '\0', sizeTmp + 1 );
                        memcpy(data_ptr , deb_data_ptr , sizeTmp );
                        *(data_ptr + sizeTmp) = '\0';
                        *Data_Ptr_Ptr = data_ptr;
                }                
        }

        if (status == IMRS_OK)
        {
                *len = sizeTmp;
        }
	

	return ( status );
}


IMRT_Status     IMR_MM_Delimited_StrHdl_Msg_Size( size_t Msg_Size )
{
	IMRT_Status status;
	char *func_name = "IMR_MM_Delimited_StrHdl_Msg_Size";

	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 ,  "============================================================================")
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 , "Size msg demand : Begin");
	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_1 , "Msg_Size  : (%ld)" , Msg_Size);	


        /* Erreur de referentiel */
        status = IMRS_KO;
        IMRD_LM_LOG_WARNING_0("Referential error ; this should never happen");

	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_2 , "Size msg demand : End with status :  (%d)" , status);
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_2 ,  "============================================================================")

	return ( status );
}


IMRT_Status IMR_MM_Delimited_StrHdl_Field_Exist( IMRT_Field *Field_Ptr )
{
	IMRT_Status status;
	char *func_name = "IMR_MM_Delimited_StrHdl_Field_Exist";

        status = IMRS_OK;

	if (Field_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Field_Ptr NULL");
	}
	else
	{
		IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 ,  "============================================================================")
		IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 , "Existing field demand : Begin");
		IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_1 , "Field Input name  : [%s]" , Field_Ptr->Name);	
	
	 	if (IMRG_Delimited_Buffer_Ident_Ptr == NULL)
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("Buffer pointer NULL");
		}
		else
		{
			status = Delimited_Hdl_Field_Exist( Field_Ptr, IMRG_Delimited_Buffer_Ident_Ptr);
		}
	}


	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_2 , "Existing field demand : End with status :  (%d)" , status);
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_2 ,  "============================================================================")


	return ( status );
}


IMRT_Status IMR_MM_Delimited_StrHdl_Field_Value( IMRT_Field *Field_Ptr , IMRT_Field_Value_Id Value_Id )
{
	IMRT_Buffer_Data *data_ptr;
	IMRT_Status status;
	size_t field_len;
	char *func_name = "IMR_MM_Delimited_StrHdl_Field_Value";

        status = IMRS_OK;

	if (Field_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Field_Ptr NULL");
	}
	else
	{
		IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 ,  "============================================================================")
		IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 , "Field Value demand : Begin");
		IMRD_LM_LOG_TRACE_2(IMRD_LOG_LEVEL_VERBOSE_1 , "Field Input name  : [%s] Value Id : [%s]" , Field_Ptr->Name , Value_Id);	
	
	
		field_len = 0L;
		
		if (IMRG_Delimited_Buffer_Ident_Ptr == NULL)
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("Buffer pointer NULL");
		}
		else
		{
			status = MM_Delimited_StrHdl_Get(&data_ptr, Field_Ptr, IMRG_Delimited_Buffer_Ident_Ptr , &field_len);
		}
		if (status == IMRS_OK)
		{
			IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_2 , "Value extracted :  [%s]" , data_ptr);
			if (strcmp(data_ptr, Value_Id) != 0)
			{
				status = IMRS_NO_IDENT;
			}
			free(data_ptr);
		}
	}
	
	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_2 , "Field Value demand : End with status :  (%d)" , status);
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_2 ,  "============================================================================")

	return ( status );
}

IMRT_Status	MM_Delimited_StrHdl_Convert(IMRT_Buffer_Data **Data_Output_Ptr , IMRT_Buffer_Data *Data_Input_Ptr , IMRT_Field *Field_Ptr,  
                                        size_t *len , IMRT_StrHdl_Cnv_Func *Cnv_Func_Tab )
{
	IMRT_Status status;
	IMRT_Buffer_Data *data_tmp_Ptr = NULL;
        size_t size;
	char *func_name = "MM_Delimited_StrHdl_Convert";


	
	status = IMRS_OK;

	if (Data_Output_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Data_Output_Ptr NULL");
	}
	else if (Cnv_Func_Tab == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Cnv_Func_Tab NULL");
	}
	else if (len == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("len pointer NULL");
	}
	else if (Field_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Field_Ptr NULL");
	}
	else
	if (Data_Input_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Data_Input_Ptr NULL");
	}
	else
	{
 		size = Field_Ptr->Fixed.Size;

		data_tmp_Ptr = (IMRT_Buffer_Data *)malloc(size + 1L);
	        memset( data_tmp_Ptr, '\0' , (size + 1L) );
	
	
	        /* Appel de la fonction de convertion */
	
		if ((Field_Ptr->Delimited.Cnv_Ptr == NULL) ||(Cnv_Func_Tab[Field_Ptr->Delimited.Cnv_Ptr->Id].Convert_Ptr == NULL))
		{
			IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_1 , "Pas de fonction de conversion");
	
	                if (Data_Input_Ptr != NULL)
	                {
	                   	*Data_Output_Ptr = data_tmp_Ptr;
		        	memcpy(*Data_Output_Ptr,Data_Input_Ptr,*len);
		        }
		}
		else
		{
			status = Cnv_Func_Tab[Field_Ptr->Delimited.Cnv_Ptr->Id].Convert_Ptr( data_tmp_Ptr, 
			                                                         Data_Input_Ptr, 
			                                                         len ,
			                                                         size ,
			                                                         Field_Ptr->Delimited.Cnv_Attr );
	                *Data_Output_Ptr = data_tmp_Ptr;
		}
	}

        if (Data_Input_Ptr != NULL)
                free(Data_Input_Ptr);
                

	return ( status );

}

IMRT_Status	MM_Delimited_StrHdl_Put(IMRT_Buffer_Data *Data_Ptr, IMRT_Field *Field_Ptr,  
                                IMRT_Buffer *Buffer_Ptr , size_t len )
{
	IMRT_Buffer_Data *data_buf_ptr;
	size_t data_len;
	size_t delim_len;
	size_t data_max_len;
	IMRT_Delimited_TmpId *TmpId_Ptr;
	char *delim_ptr;
	int occ;
	IMRT_FmtAttr_Value delimiteur;
	char indicateur[2];
	IMRT_Status status;
	char *func_name = "MM_Delimited_StrHdl_Put";

	
	status = IMRS_OK;
	
	if (Buffer_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer_Ptr NULL");
	}
	else if (strcmp((Buffer_Ptr->FmtAttr_Value) , "") == 0)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Delimiteur vide");
	}
	else if (Field_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Field_Ptr NULL");
	}
	else if (Data_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("data pointer NULL");
	}
		
	if ( status == IMRS_OK )
	{
		strcpy(delimiteur , "");
		strncpy(indicateur , Buffer_Ptr->FmtAttr_Value , 1);
		indicateur[1] = '\0';
		delim_ptr = NULL;
		delim_ptr = (Buffer_Ptr->FmtAttr_Value);
		delim_ptr++;
		strcpy(delimiteur , delim_ptr);
		delim_len = strlen(delimiteur);
	
	
	/* Controle de validite de l'indicateur  */
		if ((strcmp(indicateur , "S") != 0) && (strcmp(indicateur , "T") != 0))
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_1("Indicateur non correct [%s]",indicateur);
		}
	}
	
	/* Controle de validite du delimiteur */
	if ( status == IMRS_OK )
	{
		if (strcmp(delimiteur , "") == 0)
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("Delimiteur vide");
		}
	}
	 
	if ( status == IMRS_OK )
	{	
	        /* Controle de validite de l'Id */
	        if (Field_Ptr->Delimited.Id == IMRD_DELIMITED_ID_UNKNOWN)
	        {
	            status   = IMRS_KO;
	            IMRD_LM_LOG_ERROR_0("Occurence Delimited unknown");
	        }
	}

	if ( status == IMRS_OK )
	{
        	occ = (((IMRT_Delimited_TmpId *)(Buffer_Ptr->Data_Tmp_Ptr))->TmpId);
        	/* la len est recuperee de la fonction de convertion */
        	data_len     = len;
        	
        	TmpId_Ptr = (IMRT_Delimited_TmpId *)Buffer_Ptr->Data_Tmp_Ptr;
        
        
                /* Controle de l' occurence */
                if (Field_Ptr->Delimited.Id > occ)
        	{
        		status = IMRS_KO;
        		IMRD_LM_LOG_ERROR_2("Occurence (%d) not valid in [%s]",Field_Ptr->Delimited.Id,func_name);
        	}
        }

	if ( status == IMRS_OK )
	{
                /* Controle de la taille */
                data_max_len = Buffer_Ptr->Data_Buf_Len + data_len + delim_len;
                if (Buffer_Ptr->Data_Buf_Size <= data_max_len)
                        status = Delimited_Hdl_DataBuf_Alloc( Buffer_Ptr, data_max_len + 1 ); 
        }               


	/* l'indicateur est 'S' */

	if (( status == IMRS_OK )&&(strcmp(indicateur , "S")) == 0)
	{
		/* insertion de la valeur dans les donnees du buffer */
		
		data_buf_ptr = Buffer_Ptr->Data_Buf_Ptr + Buffer_Ptr->Data_Buf_Len ;
		
		/* on n'insere le dlimiteur qu'a partir de la deuxieme occurence (1) */
		if (occ != 0)
		{
        		memcpy(data_buf_ptr, delimiteur, delim_len);
        		memcpy(data_buf_ptr + delim_len , Data_Ptr , data_len) ;				
        		Buffer_Ptr->Data_Buf_Len = data_max_len;
		}
		else
		{
		        memcpy(data_buf_ptr , Data_Ptr , data_len) ;
		        Buffer_Ptr->Data_Buf_Len = Buffer_Ptr->Data_Buf_Len + data_len ;
		}
		occ++;
		TmpId_Ptr->TmpId = (IMRT_Delimited_Id)occ;
	}
	else /* l'indicateur est 'T' */
	if (( status == IMRS_OK )&&(strcmp(indicateur , "T") == 0))
	{
		/* insertion de la valeur dans les donnees du buffer */
		data_buf_ptr = Buffer_Ptr->Data_Buf_Ptr + Buffer_Ptr->Data_Buf_Len ;
		
		/* on insere systematiquement le dlimiteur en fin de chaine */
       		memcpy(data_buf_ptr , Data_Ptr , data_len) ;				
       		memcpy(data_buf_ptr + data_len , delimiteur, delim_len);
       		Buffer_Ptr->Data_Buf_Len = data_max_len;
		occ++;
		TmpId_Ptr->TmpId = (IMRT_Delimited_Id)occ;
		 
	}

	IMRD_LM_LOG_TRACE_1( IMRD_LOG_LEVEL_VERBOSE_2 , "Prochaine occurence apres put (%d)",TmpId_Ptr->TmpId); 
	
	if (Data_Ptr != NULL)
                free(Data_Ptr);

	

	return ( status );
}
