/*----------------------------------------------------------------------------*/
/* File: fmt_hdl_fixed.c    				         	      */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#define _LIBMAPPING_MNG_
#define _FIXED_HANDLER_C_

#include "fmt_hdl_fixed.h" 

IMRT_Status	MM_Fixed_StrHdl_Init()
{
	IMRG_Fixed_Buffer_Ident_Ptr = NULL;
	return (IMRS_OK);
}

IMRT_Status	MM_Fixed_StrHdl_DeInit()
{

	return (IMRS_OK);
}
IMRT_Status Fixed_Hdl_DataBuf_Alloc( IMRT_Buffer *Buffer_Ptr, size_t Taille_Alloc )
{
	IMRT_Buffer_Data *data_Ptr;
	IMRT_Status status;
	char *func_name = "Fixed_Hdl_DataBuf_Alloc";

	
	status   = IMRS_OK;
	data_Ptr = NULL;

	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	if (Buffer_Ptr->Status == IMRD_BUFFER_STATUS_ALLOCATED)
	{
		if (Taille_Alloc > Buffer_Ptr->Data_Buf_Size)
		{
			data_Ptr = (IMRT_Buffer_Data *) realloc ( (void *) Buffer_Ptr->Data_Buf_Ptr, Taille_Alloc );
			if (data_Ptr == NULL)
			{
				status = IMRS_KO;
				IMRD_LM_LOG_ERROR_0("Realloc error");
			}
			else
			{
				memset( data_Ptr + Buffer_Ptr->Data_Buf_Size, '\0', Taille_Alloc - Buffer_Ptr->Data_Buf_Size );
				Buffer_Ptr->Data_Buf_Ptr  = data_Ptr;
				Buffer_Ptr->Data_Buf_Size = Taille_Alloc;
			}
		}
	}
	else
	{
		data_Ptr = (IMRT_Buffer_Data *) malloc ( Taille_Alloc );
		if (data_Ptr == NULL)
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("Malloc error");
		}
		else
		{
			memset( data_Ptr, '\0', Taille_Alloc );
			Buffer_Ptr->Data_Buf_Ptr  = data_Ptr;
			Buffer_Ptr->Status        = IMRD_BUFFER_STATUS_ALLOCATED;
			Buffer_Ptr->Data_Buf_Size = Taille_Alloc;
			Buffer_Ptr->Data_Buf_Len  = 0;
		}
	}
	

	return ( status );
}

IMRT_Status MM_Fixed_StrHdl_Data_Free( IMRT_Buffer  *Buffer_Ptr)
{
	IMRT_Status status;
	char *func_name = "MM_Fixed_StrHdl_Data_Free";

	
	status = IMRS_OK;

	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	if (Buffer_Ptr->Status == IMRD_BUFFER_STATUS_ALLOCATED)
	{
		free(Buffer_Ptr->Data_Buf_Ptr);
		Buffer_Ptr->Data_Buf_Ptr  = NULL;	
		Buffer_Ptr->Status        = IMRD_BUFFER_STATUS_FREED;
		Buffer_Ptr->Data_Buf_Size = 0;
		Buffer_Ptr->Data_Buf_Len  = 0;		
	}

	return ( status );
}

IMRT_Status Fixed_Hdl_Format_Ident( IMRT_Buffer *Buffer_Ptr )
{
	IMRT_Status status;
	char *func_name = "Fixed_Hdl_Format_Ident";


	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	if (Buffer_Ptr->Status == IMRD_BUFFER_STATUS_ALLOCATED)
	{
		status = IMRS_OK;
	}
	else
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer not allocated");
	}
	

	return ( status );
}

IMRT_Status MM_Fixed_StrHdl_Ident( IMRT_Msg_Id *Msg_Id_Ident_Ptr, 
                                  IMRT_Buffer *Buffer_Ptr )
{
        
        IMRT_Fmt_Id Format;
        IMRT_Status status;
        IMRT_Msg_Id *Msg_Id_Tmp;
        char *func_name = "MM_Fixed_StrHdl_Ident";
        

	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	if (Msg_Id_Ident_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Msg_Id_Ident_Ptr NULL" );
	}
	else
	{
	        Format = Buffer_Ptr->Fmt_Id;
	        Msg_Id_Tmp = Msg_Id_Ident_Ptr;
	        status = IMRS_OK;
	
		IMRG_Fixed_Buffer_Ident_Ptr = Buffer_Ptr;
		/* Le Msg_Id transmis au SDM est le point de depart de l'identification */
		/* C'est egalement le resultat qui sera inclus dans le buffer */
		status = IMR_SDM_MsgIdent_Proceed(Format, 
		                                  Msg_Id_Tmp,
		                                  IMR_MM_Fixed_StrHdl_Field_Value,
		                                  IMR_MM_Fixed_StrHdl_Field_Exist,
		                                  IMR_MM_Fixed_StrHdl_Msg_Size);
		IMRG_Fixed_Buffer_Ident_Ptr = NULL;
	
	        Msg_Id_Ident_Ptr = Msg_Id_Tmp;
	}


	return ( status );

}

IMRT_Status MM_Fixed_StrHdl_Open( IMRT_Buffer *Buffer_Ptr)
{
	IMRT_Msg_Id type_temp;
	IMRT_Status status;
	char *func_name = "MM_Fixed_StrHdl_Open";


	status = IMRS_OK;
	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	{
		type_temp = Buffer_Ptr->Msg_Id;
		/* absence de donnes */
		if (Buffer_Ptr->Status != IMRD_BUFFER_STATUS_ALLOCATED)
		{
		        /* Cas d'open du buffer en ecriture */
		        /* le type du message est passe */
		        /* par le SDM : il doit donc etre connu */
			if (type_temp == IMRD_MSG_ID_UNKNOWN) 
			{
				status = IMRS_KO;
				IMRD_LM_LOG_ERROR_0("Msg Id unknown");
			}
			else
			{
				status = Fixed_Hdl_DataBuf_Alloc( Buffer_Ptr, FIXED_TAILLE_ALLOC );
			}
		}
		else 
	        {
	                /* controle du format */
	                status = Fixed_Hdl_Format_Ident( Buffer_Ptr ) ;
	                if (status != IMRS_OK)
	                {
	                        status = IMRS_BAD_FORMAT;
	                }       
	        }
	}
	
	

	return ( status );
}


IMRT_Status MM_Fixed_StrHdl_Close( IMRT_Buffer *Buffer_Ptr)
{
	char *func_name = "MM_Fixed_StrHdl_Close";


	return (IMRS_OK);
}

IMRT_Status	MM_Fixed_StrHdl_Flush( IMRT_Buffer  *Buffer_Ptr)
{
	char *func_name = "MM_Fixed_StrHdl_Flush";

	return (IMRS_OK);
}



IMRT_Status MM_Fixed_StrHdl_Data_List( IMRT_Buffer *Buffer_Ptr)
{
	if (Buffer_Ptr == NULL)
	{
		IMRD_LM_LOG_WARNING_0("Buffer pointer NULL" );
	}
	else
	{
	    IMRD_LM_LOG_TRACE_3( IMRD_LOG_LEVEL_VERBOSE_1, "Buffer_List: Data_Buf_Ptr_Size: (%d)   Data_Buf_Ptr_Len: (%d)   Data_Buf_Ptr: (0x%X)", Buffer_Ptr->Data_Buf_Size, Buffer_Ptr->Data_Buf_Len, Buffer_Ptr->Data_Buf_Ptr);
	    IMRD_LM_LOG_TRACE_1( IMRD_LOG_LEVEL_VERBOSE_2, "Buffer_List: Data_Buf: [%s]", Buffer_Ptr->Data_Buf_Ptr);
	
	    IMRD_LM_LOG_TRACE_2( IMRD_LOG_LEVEL_VERBOSE_1, "Buffer_List: Data_Tmp_Changed: (%d)   Data_Tmp_Ptr: (0x%X)", Buffer_Ptr->Data_Tmp_Changed, Buffer_Ptr->Data_Tmp_Ptr);
	    if (Buffer_Ptr->Data_Tmp_Ptr != NULL)
		    IMRD_LM_LOG_TRACE_1( IMRD_LOG_LEVEL_VERBOSE_2, "Buffer_List: Data_Tmp: [%s]", Buffer_Ptr->Data_Tmp_Ptr);
	}

    return (IMRS_OK);
}



IMRT_Status MM_Fixed_StrHdl_Copy( IMRT_Buffer *Output_Buffer_Ptr, IMRT_Buffer *Input_Buffer_Ptr )
{
	IMRT_Status status;
	char *func_name = "MM_Fixed_StrHdl_Copy";

	
	status = IMRS_OK;
	
	if (Input_Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Input buffer pointer NULL" );
	}
	else
	if (Output_Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Output buffer pointer NULL" );
	}
	else
	/* controle donnees dans Input_Buffer */
	if (Input_Buffer_Ptr->Status != IMRD_BUFFER_STATUS_ALLOCATED)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer input not allocated");
	}
	/* le buffer destinataire ne doit pas comporter de donnes */ 
	else if ( ( Output_Buffer_Ptr->Status == IMRD_BUFFER_STATUS_ALLOCATED) ||
	          ( Output_Buffer_Ptr->Data_Tmp_Ptr != NULL) )
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer output not allocated or Data_Tmp_Ptr no NULL");
	}
	else
	{
		status = Fixed_Hdl_DataBuf_Alloc(Output_Buffer_Ptr, Input_Buffer_Ptr->Data_Buf_Size);
		if (status == IMRS_OK)
		{
			memcpy( Output_Buffer_Ptr->Data_Buf_Ptr,
			        Input_Buffer_Ptr->Data_Buf_Ptr,
			        Input_Buffer_Ptr->Data_Buf_Size);
			Output_Buffer_Ptr->Data_Buf_Len = Input_Buffer_Ptr->Data_Buf_Len;
		}
	}
 

	return ( status );
}

IMRT_Status	Fixed_Hdl_Field_Exist(IMRT_Field *Field_Ptr , IMRT_Buffer *Buffer_Ptr)
{
        IMRT_Status status;
        char *func_name = "Fixed_Hdl_Field_Exist";
        
        /* En fixed, pas de cas de champ inexistant */      
	
	status   = IMRS_OK;

	return ( status );

}


IMRT_Status	MM_Fixed_StrHdl_Get(IMRT_Buffer_Data **Data_Ptr_Ptr, IMRT_Field *Field_Ptr,  
                                IMRT_Buffer *Buffer_Ptr , size_t *len )
{
	IMRT_Buffer_Data *data_ptr;
	IMRT_Status status;
	int offsetTmp;
	size_t sizeTmp;
	char *func_name = "MM_Fixed_StrHdl_Get";

	
	status   = IMRS_OK;
	data_ptr = NULL;
	offsetTmp = 0;
	sizeTmp = 0L;
	
	if (Data_Ptr_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Data_Ptr_Ptr NULL");
	}
	else if (len == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("len pointer NULL");
	}
	else if (Buffer_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer_Ptr NULL");
	}
	else if (Field_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Field_Ptr NULL");
	}
	else 
	   /* Controle de validite de la taille */
        if (Field_Ptr->Fixed.Size == IMRD_FIXED_SIZE_UNKNOWN)
        {
            status   = IMRS_KO;
            IMRD_LM_LOG_ERROR_0("Size Fixed unknown");
        }


        if (status == IMRS_OK)
        {
	/* controle sur l'offset */
        	if (Field_Ptr->Fixed.Offset > Buffer_Ptr->Data_Buf_Len )
        	{
        		status   = IMRS_KO;
        		IMRD_LM_LOG_ERROR_0("Field offset out of data len");
        	}
        }
	
        /* Controle de l'existence du champ */
	if (status == IMRS_OK)
	{
	        status = Fixed_Hdl_Field_Exist(Field_Ptr , Buffer_Ptr);
	}
	
        /* Le champ n'existe pas => on prend la valeur par defaut */
        /* Actuellement, ca ne peut pas arriver en Fixed */
        if (status == IMRS_NO_IDENT)
        {
                sizeTmp = strlen(Field_Ptr->Fixed.Def_Val);
                data_ptr = (IMRT_Buffer_Data *) malloc ( sizeTmp + 1);
                if (data_ptr == NULL)
                {
                        status   = IMRS_KO;
                        IMRD_LM_LOG_ERROR_0("Malloc error");
                }
                else
                {
                        memset( data_ptr, '\0', sizeTmp + 1 );
                        memcpy(data_ptr , Field_Ptr->Fixed.Def_Val , sizeTmp );
                        *(data_ptr + sizeTmp) = '\0';
                        *Data_Ptr_Ptr = data_ptr;
                        status   = IMRS_OK;
                }                

        }
	else if (status == IMRS_OK)
	{
		/* cas ou Field_Ptr->Fixed.Size = 0 => on prend tout le champ */
		/* idem si offset + size > len du message */
		if((Field_Ptr->Fixed.Size == 0L) ||( Field_Ptr->Fixed.Offset + Field_Ptr->Fixed.Size > Buffer_Ptr->Data_Buf_Len ) )
		{
			sizeTmp = Buffer_Ptr->Data_Buf_Len - Field_Ptr->Fixed.Offset;
		}
		else
		{
			sizeTmp = Field_Ptr->Fixed.Size;
		}
		data_ptr = (IMRT_Buffer_Data *) malloc ( sizeTmp + 1);
		if (data_ptr == NULL)
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("Malloc error");
		}
		else
		{
			memcpy( data_ptr, 
			        Buffer_Ptr->Data_Buf_Ptr + Field_Ptr->Fixed.Offset,
			       sizeTmp);
			*(data_ptr + sizeTmp) = '\0';
			*Data_Ptr_Ptr = data_ptr;
		}
		
	}			     
	
        if (status == IMRS_OK)
        {
                *len = sizeTmp;
        }

	

	return ( status );
}


IMRT_Status     IMR_MM_Fixed_StrHdl_Msg_Size    ( size_t Msg_Size)
{
	IMRT_Status status;
	char *func_name = "IMR_MM_Fixed_StrHdl_Msg_Size";

 	if (IMRG_Fixed_Buffer_Ident_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL");
	}
	else
	{
		IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 ,  "============================================================================")
		IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 , "Size msg demand : Begin");
		IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_1 , "Msg_Size  : (%ld)" , Msg_Size);	
		IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_1 , "Length of the buffer data  : (%ld)" , IMRG_Fixed_Buffer_Ident_Ptr->Data_Buf_Len);	
	
	        status = IMRS_OK;
	
	        if (Msg_Size != IMRG_Fixed_Buffer_Ident_Ptr->Data_Buf_Len)
	                status = IMRS_NO_IDENT;
	}

	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_2 , "Size msg demand : End with status :  (%d)" , status);
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_2 ,  "============================================================================")

	return ( status );
}


IMRT_Status IMR_MM_Fixed_StrHdl_Field_Exist( IMRT_Field *Field_Ptr )
{
	IMRT_Status status;
	char *func_name = "IMR_MM_Fixed_StrHdl_Field_Exist";

        status = IMRS_OK;

	if (Field_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Field_Ptr NULL");
	}
	else
	{
		IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 ,  "============================================================================")
		IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 , "Existing field demand : Begin");
		IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_1 , "Field Input name  : [%s]" , Field_Ptr->Name);	
	
		if (IMRG_Fixed_Buffer_Ident_Ptr == NULL)
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("Buffer pointer NULL");
		}
		else
		{
			status = Fixed_Hdl_Field_Exist( Field_Ptr, IMRG_Fixed_Buffer_Ident_Ptr);
		}
	}

	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_2 , "Existing field demand : End with status :  (%d)" , status);
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_2 ,  "============================================================================")

	return ( status );
}

IMRT_Status IMR_MM_Fixed_StrHdl_Field_Value( IMRT_Field *Field_Ptr , IMRT_Field_Value_Id Value_Id )
{
	IMRT_Buffer_Data *data_ptr;
	IMRT_Status status;
	size_t field_len;
	char *func_name = "IMR_MM_Fixed_StrHdl_Field_Value";

        status = IMRS_OK;

	if (Field_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Field_Ptr NULL");
	}
	else
	{
		IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 ,  "============================================================================")
		IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 , "Field Value demand : Begin");
		IMRD_LM_LOG_TRACE_2(IMRD_LOG_LEVEL_VERBOSE_1 , "Field Input name  : [%s] Value Id : [%s]" , Field_Ptr->Name , Value_Id);	
	        
	        field_len = 0L;
	        status = IMRS_OK;
		
		if (IMRG_Fixed_Buffer_Ident_Ptr == NULL)
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("Buffer pointer NULL");
		}
		else
		{
			status = MM_Fixed_StrHdl_Get(&data_ptr, Field_Ptr, IMRG_Fixed_Buffer_Ident_Ptr , &field_len );
		}
		
		if (status == IMRS_OK)
		{
			IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_2 , "Value extracted :  [%s]" , data_ptr);
			if (strcmp(data_ptr, Value_Id) != 0)
			{
				status = IMRS_NO_IDENT;
			}
			free(data_ptr);
		}
	}

	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_2 , "Field Value demand : End with status :  (%d)" , status);
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_2 ,  "============================================================================")

	return ( status );
}

IMRT_Status	MM_Fixed_StrHdl_Convert(IMRT_Buffer_Data **Data_Output_Ptr , IMRT_Buffer_Data *Data_Input_Ptr , IMRT_Field *Field_Ptr,  
                                        size_t *len , IMRT_StrHdl_Cnv_Func *Cnv_Func_Tab )
{
	IMRT_Status status;
	size_t size;
	char *func_name = "MM_Fixed_StrHdl_Convert";



	
	status       = IMRS_OK;

	if (Cnv_Func_Tab == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Cnv_Func_Tab NULL");
	}
	else if (Data_Output_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Data_Output_Ptr NULL");
	}
	else if (len == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("len pointer NULL");
	}
	else if (Field_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Field_Ptr NULL");
	}
	else
	if (Data_Input_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Data_Input_Ptr NULL");
	}
	else
	{
		size = Field_Ptr->Fixed.Size ;
	
		*Data_Output_Ptr = (IMRT_Buffer_Data *)malloc(size + 1L);
		if (*Data_Output_Ptr == NULL)
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("*Data_Output_Ptr NULL");
		}
		else
		{
		    memset(*Data_Output_Ptr, '\0' , (size + 1L) );
		
		        /* Appel de la fonction de convertion */
			if ((Field_Ptr->Fixed.Cnv_Ptr == NULL) || (Cnv_Func_Tab[Field_Ptr->Fixed.Cnv_Ptr->Id].Convert_Ptr == NULL))
			{
				IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_1 , "Pas de fonction de conversion");
		
		                if (Data_Input_Ptr != NULL)
			                memcpy(*Data_Output_Ptr,Data_Input_Ptr,*len);
			}
			else
			{
				status = Cnv_Func_Tab[Field_Ptr->Fixed.Cnv_Ptr->Id].Convert_Ptr(*Data_Output_Ptr, 
				                                                         Data_Input_Ptr, 
				                                                         len ,
				                                                         size ,
				                                                         Field_Ptr->Fixed.Cnv_Attr);
			}
		}
	}

    if (Data_Input_Ptr != NULL)
		free(Data_Input_Ptr);


	return ( status );

}


IMRT_Status	MM_Fixed_StrHdl_Put(IMRT_Buffer_Data *Data_Ptr, IMRT_Field *Field_Ptr,  
                                IMRT_Buffer *Buffer_Ptr , size_t len )
{
	IMRT_Buffer_Data *data_buf_ptr;
	size_t data_len;
	size_t data_max_len;
	IMRT_Status status;
	char *func_name = "MM_Fixed_StrHdl_Put";

	
	status       = IMRS_OK;

	if (Buffer_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer_Ptr NULL");
	}
	else if (Field_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Field_Ptr NULL");
	}
	else if (Data_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("data pointer NULL");
	}
	else
        /* Controle de validite de la taille */
        if (Field_Ptr->Fixed.Size == IMRD_FIXED_SIZE_UNKNOWN)
        {
            status   = IMRS_KO;
            IMRD_LM_LOG_ERROR_0("Size Fixed unknown");
        }

        if (status == IMRS_OK)
        {
	        if (strcmp(Data_Ptr , "") == 0)
	                status = IMRD_STATUS_NOTHING_TO_DO;
	}


	if ( status == IMRS_OK )
	{	
	        data_len     = len ;
	        data_max_len = Field_Ptr->Fixed.Offset + data_len ;

        	/* controle sur la valeur */
        	if ( data_len > Field_Ptr->Fixed.Size )
        	{
        		status = IMRS_KO;
        		IMRD_LM_LOG_ERROR_2("Data len (%ld) superior to data size (%ld)",data_len , Field_Ptr->Fixed.Size);
        	}
        	else if ( Buffer_Ptr->Data_Buf_Size <= data_max_len )
        	{
        		status = Fixed_Hdl_DataBuf_Alloc( Buffer_Ptr, data_max_len + 1 );
        	}
        }
        
	if ( status == IMRS_OK )
	{
		/* insertion de la valeur dans les donnees du buffer */
		data_buf_ptr = Buffer_Ptr->Data_Buf_Ptr + Field_Ptr->Fixed.Offset;
		memcpy(data_buf_ptr, Data_Ptr, data_len);

		if ( Buffer_Ptr->Data_Buf_Len < data_max_len )
		{
			Buffer_Ptr->Data_Buf_Len = data_max_len;
		}
	}			     
	
	if (status == IMRD_STATUS_NOTHING_TO_DO)
	        status = IMRS_OK;

	if (Data_Ptr != NULL)
                free(Data_Ptr);


	return ( status );
}

