/*----------------------------------------------------------------------------*/
/* File: fmt_hdl_fml.c      				         	      */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#define _LIBMAPPING_MNG_
#define _FML_HANDLER_C_

#include "fmt_hdl_fml.h"


IMRT_Status MM_FML_StrHdl_Init()
{
	IMRG_FML_Buffer_Ident_Ptr = NULL;
	return (IMRS_OK);
}

IMRT_Status MM_FML_StrHdl_DeInit()
{

	return (IMRS_OK);
}

IMRT_Status FML_Hdl_DataBuf_Alloc( IMRT_Buffer *Buffer_Ptr )
{
	IMRT_Status status;
	int retour;
	char *func_name = "FML_Hdl_DataBuf_Alloc";
	FBFR32  *tpFBbuf;
	


	tpFBbuf = NULL;
	status = IMRS_OK;
	retour = 0;
	

	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	if (Buffer_Ptr->Status != IMRD_BUFFER_STATUS_ALLOCATED)
	{
		tpFBbuf = (FBFR32 *)tpalloc((char *) IMRD_FML_TYPE , NULL, Fneeded32(IMRD_FML_MAX_FLD, IMRD_FML_SIZE) );

		if (tpFBbuf == NULL)
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("Error tpalloc");
		}
	
		if (status == IMRS_OK)
		{
			
			retour = Finit32(tpFBbuf, (FLDLEN32) Fsizeof32(tpFBbuf));

			if (retour == -1)
			{
				if (Ferror32 == FNOSPACE)
				{
					IMRD_LM_LOG_ERROR_0("Ferror32 = FNOSPACE");
				}
				else if (Ferror32 == FALIGNERR)
				{
					IMRD_LM_LOG_ERROR_0("Ferror32 = FALIGNERR");
				}
				else if (Ferror32 == FNOTFLD)
				{
					IMRD_LM_LOG_ERROR_0("Ferror32 = FNOTFLD");
				}
				else if (Ferror32 == FEINVAL)
				{
					IMRD_LM_LOG_ERROR_0("Ferror32 = FEINVAL");
				}
				else if (Ferror32 == FBADFLD)
				{
					IMRD_LM_LOG_ERROR_0("Ferror32 = FBADFLD");
				}
		        status = IMRS_KO;
				tpfree((char *)tpFBbuf);
			}
		}
		

		if (status == IMRS_OK)
		{
			Buffer_Ptr->Data_Buf_Ptr  = (IMRT_Buffer_Data *)tpFBbuf;
			Buffer_Ptr->Status        = IMRD_BUFFER_STATUS_ALLOCATED;
			Buffer_Ptr->Data_Buf_Size = IMRD_FML_SIZE;
			Buffer_Ptr->Data_Buf_Len  = 0;
	
		}
	}
	
	

	return status;

}


IMRT_Status MM_FML_StrHdl_Data_Free( IMRT_Buffer  *Buffer_Ptr)
{
	IMRT_Status status;
	char *func_name = "MM_FML_StrHdl_Data_Free";

	
	status = IMRS_OK;

	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	if (Buffer_Ptr->Status == IMRD_BUFFER_STATUS_ALLOCATED)
	{
		tpfree((char *)Buffer_Ptr->Data_Buf_Ptr);
		Buffer_Ptr->Data_Buf_Ptr  = NULL;	
		Buffer_Ptr->Status        = IMRD_BUFFER_STATUS_FREED;
		Buffer_Ptr->Data_Buf_Size = 0;
		Buffer_Ptr->Data_Buf_Len  = 0;		
	}

	return ( status );
}


IMRT_Status FML_Hdl_Format_Ident( IMRT_Buffer *Buffer_Ptr )
{
	IMRT_Status status;
	char *func_name = "FML_Hdl_Format_Ident";


	status = IMRS_KO;

	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	if (Buffer_Ptr->Status == IMRD_BUFFER_STATUS_ALLOCATED)
	{
		/* Controle du format FML */
		if (Fnum32((FBFR32 *)Buffer_Ptr->Data_Buf_Ptr) != -1)
		{
			status = IMRS_OK;
		}
	}
	else
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer not allocated");
	}


	return ( status );
}

IMRT_Status MM_FML_StrHdl_Ident( IMRT_Msg_Id *Msg_Id_Ident_Ptr, 
                                  IMRT_Buffer *Buffer_Ptr )
{
        
        IMRT_Fmt_Id Format;
        IMRT_Status status;
        IMRT_Msg_Id *Msg_Id_Tmp;
        char *func_name = "MM_FML_StrHdl_Ident";
        

	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	if (Msg_Id_Ident_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Msg_Id_Ident_Ptr NULL" );
	}
	else
	{
	        Format = Buffer_Ptr->Fmt_Id;
	        Msg_Id_Tmp = Msg_Id_Ident_Ptr;
	        status = IMRS_OK;
	
		IMRG_FML_Buffer_Ident_Ptr = Buffer_Ptr;
		/* Le Msg_Id transmis au SDM est le point de depart de l'identification */
		/* C'est egalement le resultat qui sera inclus dans le buffer */
		status = IMR_SDM_MsgIdent_Proceed(Format, 
		                                  Msg_Id_Tmp,
		                                  IMR_MM_FML_StrHdl_Field_Value,
		                                  IMR_MM_FML_StrHdl_Field_Exist,
		                                  IMR_MM_FML_StrHdl_Msg_Size);
		IMRG_FML_Buffer_Ident_Ptr = NULL;
	
	        Msg_Id_Ident_Ptr = Msg_Id_Tmp;
	}


	return ( status );

}

IMRT_Status MM_FML_StrHdl_Open( IMRT_Buffer *Buffer_Ptr)
{
	IMRT_Msg_Id type_temp;
	IMRT_Status status;
	char *func_name = "MM_FML_StrHdl_Open";

	status = IMRS_OK;

	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	{
		type_temp = Buffer_Ptr->Msg_Id;
	
		/* absence de donnes */
		if (Buffer_Ptr->Status != IMRD_BUFFER_STATUS_ALLOCATED)
		{
		        /* Cas d'open du buffer en ecriture */
		        /* le type du message est passe */
		        /* par le SDM : il doit donc etre connu */
			if (type_temp == IMRD_MSG_ID_UNKNOWN)
			{
				status = IMRS_KO;
				IMRD_LM_LOG_ERROR_0("Msg Id unknown");
			}
			else
			{
				status = FML_Hdl_DataBuf_Alloc( Buffer_Ptr );
			}
		}
		else  
		{
		        /* controle du format */
		        status = FML_Hdl_Format_Ident( Buffer_Ptr );
		        /* format non reconnu */
		        if (status != IMRS_OK)
		                status = IMRS_BAD_FORMAT;
		}
	}
	


	return ( status );
}

IMRT_Status MM_FML_StrHdl_Close( IMRT_Buffer *Buffer_Ptr)
{
	char *func_name = "MM_FML_StrHdl_Close";

	return (IMRS_OK);
}

IMRT_Status	MM_FML_StrHdl_Flush( IMRT_Buffer  *Buffer_Ptr)
{
	char *func_name = "MM_FML_StrHdl_Flush";

	return (IMRS_OK);
}

IMRT_Status MM_FML_StrHdl_Data_List( IMRT_Buffer *Buffer_Ptr)
{
	if (Buffer_Ptr == NULL)
	{
		IMRD_LM_LOG_WARNING_0("Buffer pointer NULL" );
	}
	else
	{
	    IMRD_LM_LOG_TRACE_3( IMRD_LOG_LEVEL_VERBOSE_1, "Buffer_List: Data_Buf_Ptr_Size: (%d)   Data_Buf_Ptr_Len: (%d)   Data_Buf_Ptr: (0x%X)", Buffer_Ptr->Data_Buf_Size, Buffer_Ptr->Data_Buf_Len, Buffer_Ptr->Data_Buf_Ptr);
	
	    IMRD_LM_EXEC_TRACE( IMRD_LOG_LEVEL_VERBOSE_2, "Buffer_List: Data_Tmp: [", Ffprint32( (FBFR32 *)Buffer_Ptr->Data_Buf_Ptr, IMRD_LM_STREAM_OUT), "]");
	
	    IMRD_LM_LOG_TRACE_2( IMRD_LOG_LEVEL_VERBOSE_1, "Buffer_List: Data_Tmp_Changed: (%d)   Data_Tmp_Ptr: (0x%X)", Buffer_Ptr->Data_Tmp_Changed, Buffer_Ptr->Data_Tmp_Ptr);

	    if (Buffer_Ptr->Data_Tmp_Ptr != NULL)
		    IMRD_LM_LOG_TRACE_1( IMRD_LOG_LEVEL_VERBOSE_2, "Buffer_List: Data_Tmp: [%s]", Buffer_Ptr->Data_Tmp_Ptr);
	}

    return( IMRS_OK);
}

IMRT_Status MM_FML_StrHdl_Copy( IMRT_Buffer *Output_Buffer_Ptr, IMRT_Buffer *Input_Buffer_Ptr )
{
	IMRT_Status status;
	char *func_name = "MM_FML_StrHdl_Copy";

	
	status = IMRS_OK;
	
	if (Input_Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Input buffer pointer NULL" );
	}
	else
	if (Output_Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Output buffer pointer NULL" );
	}
	else
	/* controle donnees dans Input_Buffer */
	if (Input_Buffer_Ptr->Status != IMRD_BUFFER_STATUS_ALLOCATED)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0( "Buffer input not allocated");
	}
	/* le buffer destinataire ne doit pas comporter de donnees */ 
	else if ( ( Output_Buffer_Ptr->Status == IMRD_BUFFER_STATUS_ALLOCATED) ||
	          ( Output_Buffer_Ptr->Data_Tmp_Ptr != NULL) )
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer output not allocated or Data_Tmp_Ptr no NULL");
	}
	else
	{
		status = FML_Hdl_DataBuf_Alloc(Output_Buffer_Ptr);
		if (status == IMRS_OK)
		{
			/* copie de buffer FML */
			Fcpy32((FBFR32 *)Output_Buffer_Ptr->Data_Buf_Ptr , (FBFR32 *)Input_Buffer_Ptr->Data_Buf_Ptr);
			Output_Buffer_Ptr->Data_Buf_Len = Fused32((FBFR32 *)Output_Buffer_Ptr->Data_Buf_Ptr);
			
			/* doit on faire la maj du size et du len pour le FML ??? */
		}
	}
 

	return ( status );
}


IMRT_Status	FML_Hdl_Field_Exist(IMRT_Field *Field_Ptr , IMRT_Buffer *Buffer_Ptr , FLDID32 FML_Field)
{
        IMRT_Status status;	
	FLDOCC32 retour;
        char *func_name = "FML_Hdl_Field_Exist";
        
	
	status   = IMRS_OK;
	retour = 0;

	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	if (Field_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Field pointer NULL" );
	}
	else
	{
	        retour = Foccur32((FBFR32 *)Buffer_Ptr->Data_Buf_Ptr, FML_Field);
	
	        if (retour == 0 )
	        {
	                status   = IMRS_NO_IDENT;
	                IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 , "Field not exist");
	        }
	        else if (retour == -1 )
	        {
	               status   = IMRS_KO;
	               IMRD_LM_LOG_ERROR_1("Foccur32 error [%s]" , Fstrerror32(Ferror32));
	        }
	}
        

	return ( status );

}


/* dans la fonction get, il conviendra d'etudier la necessite ou non de developper : 
	- la conversion des types
	- la possibilite de champ multi-occurrence
*/
IMRT_Status	MM_FML_StrHdl_Get(IMRT_Buffer_Data **Data_Ptr_Ptr, IMRT_Field *Field_Ptr,  
                                IMRT_Buffer *Buffer_Ptr , size_t *len )
{
	IMRT_Buffer_Data *data_ptr;
	IMRT_Status status;
	int retour;
	FLDID32 FML_Field;
	size_t sizeTmp;
	char *func_name = "MM_FML_StrHdl_Get";


	
	status   = IMRS_OK;
	retour = 0;
	data_ptr = NULL;
	sizeTmp = 0L;

	if (Data_Ptr_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Data_Ptr_Ptr NULL");
	}
	else if (len == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("len pointer NULL");
	}
	else if (Buffer_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer_Ptr NULL");
	}
	else if (Field_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Field_Ptr NULL");
	}
	else 
    	/* Controle de validite du tag */
    	if (strcmp(Field_Ptr->FML.Tag ,IMRD_FML_TAG_UNKNOWN) == 0)
    	{
	        status   = IMRS_KO;
	        IMRD_LM_LOG_ERROR_0("Tag FML unknown");
    	}

	if (status == IMRS_OK)
	{
	        FML_Field = Fldid32( Field_Ptr->FML.Tag );
	
		if ( FML_Field == BADFLDID )
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_1("Fldid32 %s BADFLDID error",Field_Ptr->FML.Tag );
		}
	}

    	if (status == IMRS_OK)
    	{
            status = FML_Hdl_Field_Exist(Field_Ptr , Buffer_Ptr , FML_Field);
    	}
    	/* Le champ n'existe pas => on prend la valeur par defaut */
	if (status == IMRS_NO_IDENT)
    	{
            sizeTmp = strlen(Field_Ptr->FML.Def_Val);
            data_ptr = (IMRT_Buffer_Data *) malloc ( sizeTmp + 1);
            if (data_ptr == NULL)
            {
                    status   = IMRS_KO;
                    IMRD_LM_LOG_ERROR_0("Malloc error");
	    }
            else
            {
                    memset( data_ptr, '\0', sizeTmp + 1 );
                    memcpy(data_ptr , Field_Ptr->FML.Def_Val , sizeTmp );
                    *(data_ptr + sizeTmp) = '\0';
                    *Data_Ptr_Ptr = data_ptr;
                    status   = IMRS_OK;
            }                
    	}       
    	/* le champ existe => on extrait la valeur */
    	else if (status == IMRS_OK)
	{
		/*data_ptr = (IMRT_Buffer_Data *) malloc ( (Field_Ptr->Fixed.Size + 1));
		memset( data_ptr, '\0', Field_Ptr->Fixed.Size + 1 );*/
		data_ptr = (IMRT_Buffer_Data *) malloc ( IMRD_FML_SIZE);
		memset( data_ptr, '\0', IMRD_FML_SIZE );
		retour = (IMRT_Status)Fget32((FBFR32 *)Buffer_Ptr->Data_Buf_Ptr, 
										FML_Field,
										(FLDOCC32 )0L,
										(char *)data_ptr,
										0);


		if (retour == -1)
		{
	        	IMRD_LM_LOG_ERROR_3("Fget32 failed with error [%s] Field (%d)  [%s]" , Fstrerror32(Ferror32) , FML_Field, Fname32(FML_Field));
	        	status = IMRS_KO;
		}
		else
		{
		    *Data_Ptr_Ptr = data_ptr;
		    sizeTmp = (size_t) strlen(data_ptr);
		}    
	}
									
	if (status == IMRS_OK)
	{
		*len = sizeTmp;
	}


	return ( status );
}


IMRT_Status     IMR_MM_FML_StrHdl_Msg_Size    ( size_t Msg_Size)
{
	IMRT_Status status;
	char *func_name = "IMR_MM_FML_StrHdl_Msg_Size";

	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 ,  "============================================================================")
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 , "Size msg demand : Begin");
	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_1 , "Msg_Size  : (%ld)" , Msg_Size);	

        /* Erreur de referentiel */
        status = IMRS_KO;
        IMRD_LM_LOG_WARNING_0("Referential error ; this should never happen");


	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_2 , "Size msg demand : End with status :  (%d)" , status);
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_2 ,  "============================================================================")

	return ( status );
}


IMRT_Status IMR_MM_FML_StrHdl_Field_Exist( IMRT_Field *Field_Ptr )
{
	IMRT_Status status;
	FLDID32 FML_Field;
	char *func_name = "IMR_MM_FML_StrHdl_Field_Exist";

        status = IMRS_OK;

	if (Field_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Field_Ptr NULL");
	}
	else
	{
		IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 ,  "============================================================================")
		IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 , "Existing field demand : Begin");
		IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_1 , "Field Input name  : [%s]" , Field_Ptr->Name);	
	
		if (IMRG_FML_Buffer_Ident_Ptr == NULL)
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("Buffer pointer NULL");
		}
		
		if (status == IMRS_OK)
		{
		        FML_Field = Fldid32( Field_Ptr->FML.Tag );
		}
			
		if ( FML_Field == BADFLDID )
		{
			status = IMRS_NO_IDENT;
			IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_1 , "Field [%s] BADFLDID error",Field_Ptr->FML.Tag );
		}
	
	        if (status == IMRS_OK)
	        {
	                status = FML_Hdl_Field_Exist(Field_Ptr , IMRG_FML_Buffer_Ident_Ptr , FML_Field);
	        }
	}

	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_2 , "Existing field demand : End with status :  (%d)" , status);
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_2 ,  "============================================================================")

	return ( status );
}

IMRT_Status IMR_MM_FML_StrHdl_Field_Value( IMRT_Field *Field_Ptr , IMRT_Field_Value_Id Value_Id )
{
	IMRT_Buffer_Data *data_ptr;
	IMRT_Status status;
	size_t field_len;
	char *func_name = "IMR_MM_FML_StrHdl_Field_Value";

        status = IMRS_OK;

	if (Field_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Field_Ptr NULL");
	}
	else
	{
		IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 ,  "============================================================================")
		IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 , "Field Value demand : Begin");
		IMRD_LM_LOG_TRACE_2(IMRD_LOG_LEVEL_VERBOSE_1 , "Field Input name  : [%s] Value Id : [%s]" , Field_Ptr->Name , Value_Id);	
		
		field_len = 0L;
		
		if (IMRG_FML_Buffer_Ident_Ptr == NULL)
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("Buffer pointer NULL");
		}
		else
		{
			status = MM_FML_StrHdl_Get(&data_ptr, Field_Ptr, IMRG_FML_Buffer_Ident_Ptr , &field_len );
		}
		if (status == IMRS_OK)
		{
			IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_2 , "Value extracted :  [%s]" , data_ptr);
			if (strcmp(data_ptr, Value_Id) != 0)
			{			
				status = IMRS_NO_IDENT;
			}
			free(data_ptr);
		}
	}

	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_2 , "Field Value demand : End with status :  (%d)" , status);
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_2 ,  "============================================================================")

	return ( status );
}

IMRT_Status	MM_FML_StrHdl_Convert(IMRT_Buffer_Data **Data_Output_Ptr , IMRT_Buffer_Data *Data_Input_Ptr , IMRT_Field *Field_Ptr,  
                                       size_t *len , IMRT_StrHdl_Cnv_Func *Cnv_Func_Tab )
{
	IMRT_Status status;
        size_t size;
	char *func_name = "MM_FML_StrHdl_Convert";
	
	status       = IMRS_OK;

	if (len == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("len pointer NULL");
	}
	else if (Data_Output_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Data_Output_Ptr NULL");
	}
	else
	{
		size = *len ;	
		*Data_Output_Ptr = (IMRT_Buffer_Data *)malloc(size + 1L);
		if (*Data_Output_Ptr == NULL)
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("*Data_Output_Ptr NULL");
		}
		else
		{
	        memset( *Data_Output_Ptr, '\0' , (size + 1L) );
	    }

		if (status == IMRS_OK)
		{
			if (Field_Ptr == NULL)
			{
				status = IMRS_KO;
				IMRD_LM_LOG_ERROR_0("Field_Ptr NULL");
			}
			else
			if (Data_Input_Ptr == NULL) 
			{
				status = IMRS_KO;
				IMRD_LM_LOG_ERROR_0("Data_Input_Ptr NULL");
			}
			else if (Cnv_Func_Tab == NULL) 
			{
				status = IMRS_KO;
				IMRD_LM_LOG_ERROR_0("Cnv_Func_Tab NULL");
			}
			else
			{
			        /* Appel de la fonction de convertion */
				if ((Field_Ptr->FML.Cnv_Ptr == NULL) || (Cnv_Func_Tab[Field_Ptr->FML.Cnv_Ptr->Id].Convert_Ptr == NULL))
				{
					IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_1 , "Pas de fonction de conversion");
			
			                if (Data_Input_Ptr != NULL)
			                        memcpy(*Data_Output_Ptr,Data_Input_Ptr,*len);
				}
				else
				{
					status = Cnv_Func_Tab[Field_Ptr->FML.Cnv_Ptr->Id].Convert_Ptr(*Data_Output_Ptr, 
					                                                         Data_Input_Ptr, 
					                                                         len,
					                                                         size ,
					                                                         Field_Ptr->FML.Cnv_Attr );
				}
			    	
			}
		}
	}

    if (Data_Input_Ptr != NULL)
        free(Data_Input_Ptr);


	return ( status );

}

IMRT_Status	MM_FML_StrHdl_Put(IMRT_Buffer_Data *Data_Ptr, IMRT_Field *Field_Ptr,  
                                IMRT_Buffer *Buffer_Ptr , size_t len )
{
	IMRT_Buffer_Data *data_buf_ptr;
	FLDID32 FML_Field;
	FLDLEN32 FML_maxlen;
	IMRT_Status status;
	int retour;
	char *func_name = "MM_FML_StrHdl_Put";
	size_t   longueur;
	int      FML_Field_Type;
	
	status     = IMRS_OK;
	retour	  = 0;
	
	if (Buffer_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer_Ptr NULL");
	}
	else if (Field_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Field_Ptr NULL");
	}
	else if (Data_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("data pointer NULL");
	}
	else
        /* Controle de validite du tag */
        if (strcmp(Field_Ptr->FML.Tag ,IMRD_FML_TAG_UNKNOWN) == 0)
        {
            status   = IMRS_KO;
            IMRD_LM_LOG_ERROR_0("Tag FML unknown");
        }

	if ((status == IMRS_OK) && (strcmp(Data_Ptr , "") == 0))
	        status = IMRD_STATUS_NOTHING_TO_DO;

	if (status == IMRS_OK)
	{
        	FML_maxlen = (FLDLEN32)0L;
        	FML_Field  = Fldid32( Field_Ptr->FML.Tag );
        	if ( FML_Field == BADFLDID )
        	{
        		status = IMRS_KO;
        		IMRD_LM_LOG_ERROR_1("Fldid32 %s BADFLDID error",Field_Ptr->FML.Tag );
        	}
        	else
        	{
        		FML_Field_Type = Fldtype32(FML_Field);
        		if (FML_Field_Type == FLD_CARRAY)
        		{
        			longueur = len;
        		}
        		else
        		{
        			longueur = 0L;
        		}

        		retour = Fadd32((FBFR32 *)Buffer_Ptr->Data_Buf_Ptr, 
        				FML_Field,
        				(char *)Data_Ptr,
        				(FLDLEN32) longueur);
        
        		if (retour == -1)
        		{
        			if (Ferror32 == FNOSPACE)
        				/* prevoir la realloc */
        			{
        				IMRD_LM_LOG_ERROR_0("Ferror32 = FNOSPACE");
        			}
        			else if (Ferror32 == FALIGNERR)
        			{
        				IMRD_LM_LOG_ERROR_0("Ferror32 = FALIGNERR");
        			}
        			else if (Ferror32 == FNOTFLD)
        			{
        				IMRD_LM_LOG_ERROR_0("Ferror32 = FNOTFLD");
        			}
        			else if (Ferror32 == FEINVAL)
        			{
        				IMRD_LM_LOG_ERROR_0("Ferror32 = FEINVAL");
        			}
        			else if (Ferror32 == FBADFLD)
        			{
        				IMRD_LM_LOG_ERROR_0("Ferror32 = FBADFLD");
        			}
        			status = IMRS_KO;
        		}
        	}
        }
        
	if (status == IMRS_OK)
	{
	        /* Mise a jour de la taille du buffer */
	        Buffer_Ptr->Data_Buf_Len = (Buffer_Ptr->Data_Buf_Len) + Fsizeof32((FBFR32 *)Buffer_Ptr->Data_Buf_Ptr);
	}

	if (status == IMRD_STATUS_NOTHING_TO_DO)
	        status = IMRS_OK;

	if (Data_Ptr != NULL)
                free(Data_Ptr);


	return ( status );
}
