/*----------------------------------------------------------------------------*/
/* File: fmt_hdl_.c      				         	      */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#define _LIBMAPPING_MNG_
#define _XML_HANDLER_C_

#include "fmt_hdl_xml.h" 

IMRT_Status MM_XML_StrHdl_Init()
{
	IMRG_XML_Buffer_Ident_Ptr = NULL;
	return (IMRS_OK);
}

IMRT_Status MM_XML_StrHdl_DeInit()
{

	return (IMRS_OK);
}

IMRT_Status XML_Hdl_DataTemp_Alloc( IMRT_Buffer *Buffer_Ptr )
{
	xmlDocPtr 	xml_doc_ptr=NULL;
	IMRT_Status status;
	char *func_name = "XML_Hdl_DataTemp_Alloc";


	status = IMRS_OK;

	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	if (Buffer_Ptr->Data_Tmp_Ptr == NULL)
	{
		if ( Buffer_Ptr->Status == IMRD_BUFFER_STATUS_ALLOCATED)
		{
			xml_doc_ptr = xmlParseMemory( (char *) Buffer_Ptr->Data_Buf_Ptr, Buffer_Ptr->Data_Buf_Size);
		}
		else
		{
			xml_doc_ptr = xmlNewDoc ((xmlChar *) IMRD_XML_VERSION);
		}
		if ( xml_doc_ptr == NULL)
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("Pointer xml_doc_ptr NULL");
		}
		else
		{
			Buffer_Ptr->Data_Tmp_Ptr     = (IMRT_Buffer_Data *) xml_doc_ptr;
			Buffer_Ptr->Data_Tmp_Changed = IMRD_FALSE;
		}
	}

	return ( status );
}

IMRT_Status XML_Hdl_DataTemp_Free( IMRT_Buffer *Buffer_Ptr )
{
	IMRT_Status status;
	char *func_name = "XML_Hdl_DataTemp_Free";


	status = IMRS_OK;
	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	if (Buffer_Ptr->Data_Tmp_Ptr != NULL)
	{
		xmlFreeDoc( (xmlDocPtr) Buffer_Ptr->Data_Tmp_Ptr);
		xmlCleanupParser();
		Buffer_Ptr->Data_Tmp_Ptr     = NULL;
		Buffer_Ptr->Data_Tmp_Changed = IMRD_FALSE;
	}

	return ( status );
}

IMRT_Status XML_Hdl_DataBuf_Free( IMRT_Buffer *Buffer_Ptr )
{
	IMRT_Status status;
	char *func_name = "XML_Hdl_DataBuf_Free";


	status = IMRS_OK;
	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	if (Buffer_Ptr->Status == IMRD_BUFFER_STATUS_ALLOCATED)
	{
		xmlFree(Buffer_Ptr->Data_Buf_Ptr);
		Buffer_Ptr->Data_Buf_Ptr  = NULL;	
		Buffer_Ptr->Status        = IMRD_BUFFER_STATUS_FREED;
		Buffer_Ptr->Data_Buf_Size = 0;
		Buffer_Ptr->Data_Buf_Len  = 0;
	}

	return ( status );
}

IMRT_Status MM_XML_StrHdl_Data_Free( IMRT_Buffer  *Buffer_Ptr)
{
	IMRT_Status status;
	char *func_name = "MM_XML_StrHdl_Data_Free";


	status = IMRS_OK;
	
	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	{
		if ( XML_Hdl_DataBuf_Free( Buffer_Ptr ) != IMRS_OK )
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("Error XML_Hdl_DataBuf_Free");
			
		}
		if ( XML_Hdl_DataTemp_Free( Buffer_Ptr ) != IMRS_OK )
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("Error XML_Hdl_DataTemp_Free");
		}
	}
	

	return ( status );
}

IMRT_Status XML_Hdl_Format_Ident( IMRT_Buffer *Buffer_Ptr )
{
	IMRT_Status status;
	char *func_name = "XML_Hdl_Format_Ident";


	status = IMRS_KO;
	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	if (Buffer_Ptr->Status != IMRD_BUFFER_STATUS_ALLOCATED)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer not allocated");
	}
	else
	{
		if (*Buffer_Ptr->Data_Buf_Ptr == (IMRT_Buffer_Data) '<')
		{
			status = IMRS_OK;
		}
        }
	

	return ( status );
}

IMRT_Status MM_XML_StrHdl_Ident( IMRT_Msg_Id *Msg_Id_Ident_Ptr, 
                                  IMRT_Buffer *Buffer_Ptr )
{
        
        IMRT_Fmt_Id Format;
        IMRT_Status status;
        IMRT_Msg_Id *Msg_Id_Tmp;
        char *func_name = "MM_XML_StrHdl_Ident";
        
        status = IMRS_OK;

	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	if (Msg_Id_Ident_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Msg_Id_Ident_Ptr NULL" );
	}
	else
	{
	        Format = Buffer_Ptr->Fmt_Id;
	        Msg_Id_Tmp = Msg_Id_Ident_Ptr;
	
		IMRG_XML_Buffer_Ident_Ptr = Buffer_Ptr;
		/* Le Msg_Id transmis au SDM est le point de depart de l'identification */
		/* C'est egalement le resultat qui sera inclus dans le buffer */
		status = IMR_SDM_MsgIdent_Proceed(Format, 
		                                  Msg_Id_Tmp,
		                                  IMR_MM_XML_StrHdl_Field_Value,
		                                  IMR_MM_XML_StrHdl_Field_Exist,
		                                  IMR_MM_XML_StrHdl_Msg_Size);
		IMRG_XML_Buffer_Ident_Ptr = NULL;
	
	        Msg_Id_Ident_Ptr = Msg_Id_Tmp;
	}


	return ( status );

}

IMRT_Status MM_XML_StrHdl_Open( IMRT_Buffer  *Buffer_Ptr)
{
	IMRT_Msg_Id type_temp;
	IMRT_Status status;
	char *func_name = "MM_XML_StrHdl_Open";


	status = IMRS_OK;

	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	if (Buffer_Ptr->Data_Tmp_Ptr != NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Pointer Data_Tmp_Ptr NULL");
	}
	else
	{
		type_temp = Buffer_Ptr->Msg_Id;
		/* absence de donnees */
		if (Buffer_Ptr->Status != IMRD_BUFFER_STATUS_ALLOCATED)
		{
		        /* Cas d'open du buffer en ecriture */
		        /* le type du message est passe */
		        /* par le SDM : il doit donc etre connu */
			if (type_temp == IMRD_MSG_ID_UNKNOWN)
			{
				status = IMRS_KO;
				IMRD_LM_LOG_ERROR_0("Msg Id unknown");
			}
			else
			{
				status = XML_Hdl_DataTemp_Alloc( Buffer_Ptr );
				Buffer_Ptr->Data_Tmp_Changed = IMRD_TRUE;
			}
		}
		else 
		{
		        /* controle du format */
		        status =  XML_Hdl_Format_Ident( Buffer_Ptr );
		        if (status != IMRS_OK)
		                status = IMRS_BAD_FORMAT;
		        else
			/* acces aux donnees */
			        status = XML_Hdl_DataTemp_Alloc( Buffer_Ptr );
		}
	}

	return ( status );
}

IMRT_Status MM_XML_StrHdl_Close( IMRT_Buffer *Buffer_Ptr)
{
	IMRT_Status	status;
	char *func_name = "MM_XML_StrHdl_Close";

	status = IMRS_OK;

	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	{
		status = MM_XML_StrHdl_Flush( Buffer_Ptr ); 
		if ((status == IMRS_OK) && (Buffer_Ptr->Data_Tmp_Ptr != NULL))
		{
			status = XML_Hdl_DataTemp_Free( Buffer_Ptr );
		}
	}

	return ( status );
}

IMRT_Status MM_XML_StrHdl_Flush( IMRT_Buffer *Buffer_Ptr)
{
	xmlChar *mem_Ptr;
	int     size;
	IMRT_Status status;
	char *func_name = "MM_XML_StrHdl_Flush";


	status = IMRS_OK;
	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	if ((Buffer_Ptr->Data_Tmp_Ptr != NULL) && (Buffer_Ptr->Data_Tmp_Changed == IMRD_TRUE))
	{			
		if ( Buffer_Ptr->Status == IMRD_BUFFER_STATUS_ALLOCATED )
		{
			status = XML_Hdl_DataBuf_Free( Buffer_Ptr );
		}
		
		if ( status == IMRS_OK )
		{

			/* Dump an XML document in memory and return the #xmlChar * and it's size in bytes. 
			It's up to the caller to free the memory with xmlFree(). 
			The resulting byte array is zero terminated, though the last 0 is not included 
			in the returned size. */
		    xmlDocDumpFormatMemory( (xmlDocPtr) Buffer_Ptr->Data_Tmp_Ptr, &mem_Ptr, &size,
					    ( ( IMRG_MM_Base.Config.XML_Format == IMRD_TRUE) ? 1 : 0));

			if (mem_Ptr != NULL)
			{
				Buffer_Ptr->Data_Buf_Ptr  = (IMRT_Buffer_Data *) mem_Ptr;
				Buffer_Ptr->Status        = IMRD_BUFFER_STATUS_ALLOCATED;
				Buffer_Ptr->Data_Buf_Size = size + 1;
				Buffer_Ptr->Data_Buf_Len  = size;
				Buffer_Ptr->Data_Tmp_Changed = IMRD_FALSE;
			}
			else
			{
				status == IMRS_KO;
				IMRD_LM_LOG_ERROR_0("Pointer mem_Ptr = NULL after xmlDocDumpMemory");
			}
		}
	}
 

	return ( status );
}

IMRT_Status  MM_XML_StrHdl_Data_List( IMRT_Buffer  *Buffer_Ptr)
{

	if (Buffer_Ptr == NULL)
	{
		IMRD_LM_LOG_WARNING_0("Buffer pointer NULL" );
	}
	else
	{
	    IMRD_LM_LOG_TRACE_3( IMRD_LOG_LEVEL_VERBOSE_1, "Buffer_List: Data_Buf_Ptr_Size: (%d)   Data_Buf_Ptr_Len: (%d)   Data_Buf_Ptr: (0x%X)", Buffer_Ptr->Data_Buf_Size, Buffer_Ptr->Data_Buf_Len, Buffer_Ptr->Data_Buf_Ptr);
	    IMRD_LM_LOG_TRACE_1( IMRD_LOG_LEVEL_VERBOSE_2, "Buffer_List: Data_Buf: [%s]", Buffer_Ptr->Data_Buf_Ptr);
	
	    IMRD_LM_LOG_TRACE_2( IMRD_LOG_LEVEL_VERBOSE_1, "Buffer_List: Data_Tmp_Changed: (%d)   Data_Tmp_Ptr: (0x%X)", Buffer_Ptr->Data_Tmp_Changed, Buffer_Ptr->Data_Tmp_Ptr);
	
	    if (Buffer_Ptr->Data_Tmp_Ptr != NULL)
	    {
		IMRD_LM_EXEC_TRACE( IMRD_LOG_LEVEL_VERBOSE_2, "Buffer_List: Data_Tmp: [", xmlDocFormatDump( IMRD_LM_STREAM_OUT, (xmlDocPtr) Buffer_Ptr->Data_Tmp_Ptr, 1), "]");
	    }
	}
	
    return ( IMRS_OK);
}



IMRT_Status MM_XML_StrHdl_Copy( IMRT_Buffer *Output_Buffer_Ptr, IMRT_Buffer *Input_Buffer_Ptr )
{
	IMRT_Status status;
	char *func_name = "MM_XML_StrHdl_Copy";

	
	status = IMRS_OK;
	
	if (Input_Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Input buffer pointer NULL" );
	}
	else
	if (Output_Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Output buffer pointer NULL" );
	}
	else
	/* controle donnees dans Input_Buffer */
	if (Input_Buffer_Ptr->Status != IMRD_BUFFER_STATUS_ALLOCATED)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer input not allocated");
	}
	/* le buffer destinataire ne doit pas comporter de donnees */ 
	else if ( ( Output_Buffer_Ptr->Status == IMRD_BUFFER_STATUS_ALLOCATED) ||
	          ( Output_Buffer_Ptr->Data_Tmp_Ptr != NULL) )
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer output not allocated or Data_Tmp_Ptr no NULL");
	}
	else
	{
		Output_Buffer_Ptr->Data_Buf_Ptr = (IMRT_Buffer_Data *) malloc ( Input_Buffer_Ptr->Data_Buf_Size );
		if (Output_Buffer_Ptr->Data_Buf_Ptr != NULL)
		{
			memcpy( Output_Buffer_Ptr->Data_Buf_Ptr,
			        Input_Buffer_Ptr->Data_Buf_Ptr,
			        Input_Buffer_Ptr->Data_Buf_Size);
			Output_Buffer_Ptr->Status        = IMRD_BUFFER_STATUS_ALLOCATED;
			Output_Buffer_Ptr->Data_Buf_Size = Input_Buffer_Ptr->Data_Buf_Size;
			Output_Buffer_Ptr->Data_Buf_Len  = Input_Buffer_Ptr->Data_Buf_Len;
		}
		else
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("Pointer Data_Buf_Ptr of output buffer is NULL");
		}
	}
 

	return ( status );
}

IMRT_Status	XML_Hdl_Field_Exist(IMRT_Field *Field_Ptr , IMRT_Buffer *Buffer_Ptr)
{
        IMRT_Status status;
	xmlXPathContextPtr context;
	xmlNodeSetPtr      nodeset;
	xmlXPathObjectPtr  result;
	xmlDocPtr          xml_doc_ptr; 
	xmlChar            *xpath;

        char *func_name = "XML_Hdl_Field_Exist";
        
	
	status   = IMRS_OK;

	if (Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer pointer NULL" );
	}
	else
	if (Field_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Field pointer NULL" );
	}
	else
	if (Buffer_Ptr->Data_Tmp_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Data_Tmp_Ptr of buffer is NULL");
	}
	else
	{
		xml_doc_ptr = (xmlDocPtr) Buffer_Ptr->Data_Tmp_Ptr;
		xpath       = (xmlChar *) Field_Ptr->XML.Tag;
		context     = xmlXPathNewContext(xml_doc_ptr);
		result      = xmlXPathEvalExpression(xpath, context);
		if (result == NULL)
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("XPath eval expression pointer NULL" );			
		}
		else
		{

			if(xmlXPathNodeSetIsEmpty(result->nodesetval))
			{
				status = IMRS_NO_IDENT;
			}
			else
			{			
				nodeset = result->nodesetval;
				
				if (nodeset->nodeNr != 1)
				{
				        status = IMRS_NO_IDENT;
				        IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_1 , "number of occurences not equal to one for %s",Field_Ptr->Name);
				}
			}
			xmlXPathFreeContext(context);			        
			xmlXPathFreeObject(result);
		}
	}

        


	return ( status );

}


IMRT_Status	MM_XML_StrHdl_Get(IMRT_Buffer_Data **Data_Ptr_Ptr, IMRT_Field *Field_Ptr,
                                        IMRT_Buffer *Buffer_Ptr , size_t *len )
{
	IMRT_Buffer_Data   *data_ptr;
	int                sizeTmp;
	xmlXPathContextPtr context;
	xmlXPathObjectPtr  result;
	xmlNodeSetPtr      nodeset;
	xmlDocPtr          xml_doc_ptr; 
	xmlChar            *xpath, 
	                   *xml_data_ptr;
	IMRT_Status status;
	char *func_name = "MM_XML_StrHdl_Get";

	
	status = IMRS_OK;
	*Data_Ptr_Ptr = NULL;
	data_ptr = NULL;
	sizeTmp = 0;

	if (Data_Ptr_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Data_Ptr_Ptr NULL");
	}
	else if (len == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("len pointer NULL");
	}
	else if (Buffer_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer_Ptr NULL");
	}
	else if (Field_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Field_Ptr NULL");
	}

        /* Controle de validite du tag */
        if ((status == IMRS_OK) && (strcmp(Field_Ptr->XML.Tag ,IMRD_XML_TAG_UNKNOWN) == 0))
        {
            status   = IMRS_KO;
            IMRD_LM_LOG_ERROR_0("Tag XML unknown");
        }

	/* controle donnees dans le buffer */
	if ((status == IMRS_OK) && (Buffer_Ptr->Data_Tmp_Ptr == NULL))
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Data_Tmp_Ptr of buffer is NULL");
	}
	else
	{
		xml_doc_ptr = (xmlDocPtr) Buffer_Ptr->Data_Tmp_Ptr;
		xpath       = (xmlChar *) Field_Ptr->XML.Tag;
		context     = xmlXPathNewContext(xml_doc_ptr);
		result      = xmlXPathEvalExpression(xpath, context);
		if (result == NULL)
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("XPath eval expression pointer NULL" );
		}
		else if(xmlXPathNodeSetIsEmpty(result->nodesetval))
		{
			status = IMRS_NO_IDENT;
		}
		xmlXPathFreeContext(context);
	}
	
	
	
        /* le champ existe => on extrait la valeur */
        if (status == IMRS_OK)
	{
		nodeset = result->nodesetval;
		/* nombre d'occurences */
		if (nodeset->nodeNr != 1)
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_1("number of occurences not equal to one for %s",Field_Ptr->Name);
		}
		else
		{
			xml_data_ptr = xmlNodeListGetString(xml_doc_ptr, nodeset->nodeTab[0]->xmlChildrenNode, 1);
			if (xml_data_ptr == NULL)
			{
				status = IMRS_NO_IDENT;
			}
			else
			{
			    
			    sizeTmp = strlen((char *)xml_data_ptr) + 1;

//			    IMRD_LM_LOG_TRACE_2( IMRD_LOG_LEVEL_VERBOSE_1, "[%s] (%d)", xml_data_ptr, sizeTmp);
			    
			    if( ( *Data_Ptr_Ptr = (IMRT_Buffer_Data *)malloc( sizeTmp)) == NULL)
			    {
				status = IMRS_KO;
				IMRD_LM_LOG_ERROR_0 ( "Can't allocate memory...");
			    }
			    else
			    {
				UTF8Toisolat1( (unsigned char  *)*Data_Ptr_Ptr, &sizeTmp, xml_data_ptr, &sizeTmp);

//				IMRD_LM_LOG_TRACE_3( IMRD_LOG_LEVEL_VERBOSE_1, "[%s] (%d) (%d)", xml_data_ptr, sizeTmp, strlen( *Data_Ptr_Ptr));

				sizeTmp = strlen( *Data_Ptr_Ptr);
			    }
			    
			    xmlFree( xml_data_ptr);
			}
		}
		
	}

        /* Le champ n'existe pas => on prend la valeur par defaut */
        if (status == IMRS_NO_IDENT)
        {
                sizeTmp = strlen(Field_Ptr->XML.Def_Val);
                data_ptr = (IMRT_Buffer_Data *) malloc ( sizeTmp + 1);
                if (data_ptr == NULL)
                {
                        status   = IMRS_KO;
                        IMRD_LM_LOG_ERROR_0("Malloc error on data_ptr");
                }
                else
                {
                        memset( data_ptr, '\0', sizeTmp + 1 );
                        memcpy(data_ptr , Field_Ptr->XML.Def_Val , sizeTmp );
                        *(data_ptr + sizeTmp) = '\0';
                        *Data_Ptr_Ptr = data_ptr;
                        status   = IMRS_OK;
                }                
        }       

	if (status == IMRS_OK)
	{
		*len = sizeTmp;
	}
	
	if(result != NULL)
		xmlXPathFreeObject(result);

	return ( status );
}


IMRT_Status     IMR_MM_XML_StrHdl_Msg_Size    ( size_t Msg_Size)
{
	IMRT_Status status;
	char *func_name = "IMR_MM_XML_StrHdl_Msg_Size";

	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 ,  "============================================================================")
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 , "Size msg demand : Begin");
	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_1 , "Msg_Size  : (%ld)" , Msg_Size);	

        /* Erreur de referentiel */
        status = IMRS_KO;
        IMRD_LM_LOG_WARNING_0("Referential error ; this should never happen");

	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_2 , "Size msg demand : End with status :  (%d)" , status);
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_2 ,  "============================================================================")

	return ( status );
}


IMRT_Status IMR_MM_XML_StrHdl_Field_Exist( IMRT_Field *Field_Ptr )
{
	IMRT_Status status;
	char *func_name = "IMR_MM_XML_StrHdl_Field_Exist";

        status = IMRS_OK;

	if (Field_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Field_Ptr NULL");
	}
	else
	{
		IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 ,  "============================================================================")
		IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 , "Existing field demand : Begin");
		IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_1 , "Field Input name  : [%s]" , Field_Ptr->Name);	
	
		if (IMRG_XML_Buffer_Ident_Ptr == NULL)
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("Buffer pointer NULL");
		}
		else
		{
			status = XML_Hdl_Field_Exist( Field_Ptr, IMRG_XML_Buffer_Ident_Ptr);
		}
	}

	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_2 , "Existing field demand : End with status :  (%d)" , status);
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_2 ,  "============================================================================")

	return ( status );
}

IMRT_Status IMR_MM_XML_StrHdl_Field_Value( IMRT_Field *Field_Ptr , IMRT_Field_Value_Id Value_Id )
{
	IMRT_Buffer_Data *data_ptr;
	IMRT_Status status;
	size_t field_len;
	char *func_name = "IMR_MM_XML_StrHdl_Field_Value";

        status = IMRS_OK;

	if (Field_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Field_Ptr NULL");
	}
	else
	{
		IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 ,  "============================================================================")
		IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 , "Field Value demand : Begin");
		IMRD_LM_LOG_TRACE_2(IMRD_LOG_LEVEL_VERBOSE_1 , "Field Input name  : [%s] Value Id : [%s]" , Field_Ptr->Name , Value_Id);	
	
	        field_len = 0L;
		
		if (IMRG_XML_Buffer_Ident_Ptr == NULL)
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("Buffer pointer NULL");
		}
		else
		{
			status = MM_XML_StrHdl_Get(&data_ptr, Field_Ptr, IMRG_XML_Buffer_Ident_Ptr , &field_len );
		}
		
		if (status == IMRS_OK)
		{
			IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_2 , "Value extracted :  [%s]" , data_ptr);
			if (strcmp(data_ptr, Value_Id) != 0)
			{
				status = IMRS_NO_IDENT;
			}
			free(data_ptr);
		}
	}

	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_2 , "Field Value demand : End with status :  (%d)" , status);
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_2 ,  "============================================================================")

	return ( status );
}

IMRT_Status	MM_XML_StrHdl_Convert(IMRT_Buffer_Data **Data_Output_Ptr , IMRT_Buffer_Data *Data_Input_Ptr , IMRT_Field *Field_Ptr,  
                                       size_t *len , IMRT_StrHdl_Cnv_Func *Cnv_Func_Tab )
{
	IMRT_Status status;
        size_t size;
	char *func_name = "MM_XML_StrHdl_Convert";



	
	status = IMRS_OK;

	if (len == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("len pointer NULL");
	}
	else if (Data_Output_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Data_Output_Ptr NULL");
	}
	else if (Cnv_Func_Tab == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Cnv_Func_Tab NULL");
	}
	else
	{
		size = *len ;
	
		*Data_Output_Ptr = (IMRT_Buffer_Data *)malloc(size + 1L);
		if (*Data_Output_Ptr == NULL)
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("*Data_Output_Ptr NULL");
		}
		else
		{
        	memset( *Data_Output_Ptr, '\0' , (size + 1L) );

			if (Field_Ptr == NULL) 
			{
				status = IMRS_KO;
				IMRD_LM_LOG_ERROR_0("Field_Ptr NULL");
			}
			else
			if (Data_Input_Ptr == NULL) 
			{
				status = IMRS_KO;
				IMRD_LM_LOG_ERROR_0("Data_Input_Ptr NULL");
			}
			else
			{
		                /* Appel de la fonction de convertion */
				if ((Field_Ptr->XML.Cnv_Ptr == NULL) || (Cnv_Func_Tab[Field_Ptr->XML.Cnv_Ptr->Id].Convert_Ptr == NULL))
				{
					IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_1 , "Pas de fonction de conversion");
			
			                if (Data_Input_Ptr != NULL)
			                        memcpy(*Data_Output_Ptr,Data_Input_Ptr,*len);
				}
				else
				{
					status = Cnv_Func_Tab[Field_Ptr->XML.Cnv_Ptr->Id].Convert_Ptr(*Data_Output_Ptr, 
					                                                         Data_Input_Ptr, 
					                                                         len,
					                                                         size ,
					                                                         Field_Ptr->XML.Cnv_Attr );
				}
			}
		}
	}

    if (Data_Input_Ptr != NULL)
        free(Data_Input_Ptr);


	return ( status );

}

IMRT_Status  MM_XML_DInit_StructPath(IMRT_PARSE_PATH_XML * Parse_PathStruct_Ptr)
{
   int indexe = 0;
   IMRT_Status status = IMRS_OK;

   if(Parse_PathStruct_Ptr == NULL)
   {
	status = IMRS_KO;
	IMRD_LM_LOG_ERROR_0("Parse_PathStruct_Ptr NULL");
   }
   else
   {
    if(Parse_PathStruct_Ptr->Father_Table != NULL)
    {
	for(indexe=0; indexe < Parse_PathStruct_Ptr->Nb_Father; indexe++)
	{
	    if(Parse_PathStruct_Ptr->Father_Table[indexe] != NULL)
	    {
	       free(Parse_PathStruct_Ptr->Father_Table[indexe]);
	    }
	}
	free(Parse_PathStruct_Ptr->Father_Table);
    }
    Parse_PathStruct_Ptr->Nb_Father = 0; 
   }
   return(status);
}

IMRT_Status MM_XML_Proceed_Path( IMRT_PARSE_PATH_XML * PathXml_Ptr, IMRT_Buffer_Data * value, xmlDocPtr xml_doc_ptr)
{
    xmlNodePtr xml_cur_node_ptr;
    xmlNodePtr root_node_ptr;
    xmlNodePtr xml_node1_ptr; 
    xmlNodePtr xml_father_node_ptr;
    xmlNodePtr xml_brother_node_ptr;
    xmlAttrPtr Attribute_Ptr;
    
    int indexe = 0;
    IMRT_Status status;
  
    
    status = IMRS_OK;

	if (PathXml_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("PathXml_Ptr NULL");
	}
	else if (value == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("value NULL");
	}
	else
	if (xml_doc_ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("xml_doc_ptr NULL");
	}
	else
	{

	    xml_cur_node_ptr = xmlDocGetRootElement(xml_doc_ptr);  
	
	    for(indexe = 0; indexe < PathXml_Ptr->Nb_Father; indexe++)
	    {
	   
		while( ( xml_cur_node_ptr != NULL) && 
		       (strcmp((const char*)xml_cur_node_ptr->name ,
			       (const char*)PathXml_Ptr->Father_Table[indexe]) != 0)
		    )
		{
		    xml_brother_node_ptr = xml_cur_node_ptr;
		    xml_cur_node_ptr = xml_cur_node_ptr->next;
	
		}
	
		if(xml_cur_node_ptr == NULL)
		{
		    /*il faut creer le root**/
		    if(indexe == IMRD_FIRST_NODE )
		    {
			xml_father_node_ptr = xmlNewNode ( NULL, (const xmlChar *) PathXml_Ptr->Father_Table[indexe]);
			if(xml_father_node_ptr != NULL)
	                {
			    xmlDocSetRootElement( xml_doc_ptr, xml_father_node_ptr );
			    xml_cur_node_ptr  = xml_father_node_ptr; 
	                }
	                else
			{
			    /*echec de la creation du nouveau noeud on sort*/
			    status = IMRS_KO;
	                    break;  
	                }
		    }
		    else /*il faut creer un fils*/
		    {
			 xml_cur_node_ptr = xmlNewChild( xml_father_node_ptr, NULL, (const xmlChar *) (const xmlChar *) PathXml_Ptr->Father_Table[indexe], NULL);
			 if(xml_cur_node_ptr != NULL)
			 {    
			     xml_father_node_ptr= xml_cur_node_ptr;
		
			 }
			 else
			 {
			     /*echec de la creation du nouveau noeud on sort*/
			    status = IMRS_KO;
	                    break;   
			 }
		    }
		}
		else
		{
		    xml_father_node_ptr = xml_cur_node_ptr;
		    xml_cur_node_ptr = xml_cur_node_ptr->xmlChildrenNode;
		
		}      
	    }
	    /*Tout les pere existaient ou on ete crees*/
	    /*creation d'un fils (feuille et insertion de sa valeur*/
	    /*le pere vien d'tre cree c'est le noeud courant*/
	    /*si le mode est egale  0 alors on est mode creation 
	      le neoud pere vient d'tre cree c'est le noeud courant*/
	         xml_cur_node_ptr = xmlNewChild(  xml_father_node_ptr, NULL, (const xmlChar *) PathXml_Ptr->Soon_Name, NULL);
		if(xml_cur_node_ptr != NULL)
	        {
		    xml_node1_ptr = xmlNewText( (const xmlChar *) value );
		
		    if( PathXml_Ptr->Attribute_present == IMRS_OK )
		    {
			Attribute_Ptr = xmlNewProp(xml_cur_node_ptr,(const xmlChar *)PathXml_Ptr->Attribute_Name,(const xmlChar *)PathXml_Ptr->Attribute_Value);
	            }
	    
		    xmlAddChild(xml_cur_node_ptr,xml_node1_ptr);
	        }
		else
		{
		    status = IMRS_KO;
		}    
	}
	
  
    return(status);
}

IMRT_Status MM_XML_SplitAttribute( IMRT_PARSE_PATH_XML * PathXml_Ptr )
{
    char * pos_deb_attribute_ptr = NULL;
    char * pos_fin_attribute_ptr = NULL;
    size_t len_attribute_value;
    static const char SEP_XPATH_DEB [] = "[";
    static const char SEP_XPATH_FIN [] = "]";
    static const char SEP_ATTRIBUTE [] = "@";
    IMRT_Status status;


	if (PathXml_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("PathXml_Ptr NULL");
	}
	else
	{
	    memset(PathXml_Ptr->Attribute_Name, '\0',sizeof(PathXml_Ptr->Attribute_Name));
	    memset(PathXml_Ptr->Attribute_Value, '\0',sizeof(PathXml_Ptr->Attribute_Value));
	    len_attribute_value = 0;
	    status = IMRS_OK;  
	  
	    pos_deb_attribute_ptr = strstr((const char*)PathXml_Ptr->Soon_Name,SEP_XPATH_DEB);
	    pos_fin_attribute_ptr = strstr((const char*)PathXml_Ptr->Soon_Name,SEP_XPATH_FIN);
	    PathXml_Ptr->Attribute_present = IMRS_KO;
	              
	  
	    if( (pos_deb_attribute_ptr != NULL) && (pos_fin_attribute_ptr != NULL) )
	    {
		
	                   
		if(PathXml_Ptr->Soon_Name[(pos_deb_attribute_ptr -PathXml_Ptr->Soon_Name)+1 ] == '@')
		{
		    /*on se positionne sur le debut de l'occurence*/
		    pos_deb_attribute_ptr = pos_deb_attribute_ptr +2;
	
		    pos_fin_attribute_ptr = strstr(PathXml_Ptr->Soon_Name,"=");                       
		    strncpy(PathXml_Ptr->Attribute_Name,pos_deb_attribute_ptr,(pos_fin_attribute_ptr -  pos_deb_attribute_ptr));
		    PathXml_Ptr->Attribute_Name[(pos_fin_attribute_ptr -  pos_deb_attribute_ptr)] = '\0';
	
		    /*on se positionne au debut de la valeur*/
		    pos_deb_attribute_ptr = pos_fin_attribute_ptr+1;
	
		    len_attribute_value = (size_t) ( (PathXml_Ptr->Soon_Name + strlen(PathXml_Ptr->Soon_Name)) - pos_deb_attribute_ptr ) -3; 
		    strncpy(PathXml_Ptr->Attribute_Value,pos_deb_attribute_ptr+1,len_attribute_value);
	            PathXml_Ptr->Attribute_present = IMRS_OK ;  
		}
	        
	        pos_deb_attribute_ptr = strstr((const char*)PathXml_Ptr->Soon_Name,SEP_XPATH_DEB);        
		PathXml_Ptr->Soon_Name[(pos_deb_attribute_ptr - PathXml_Ptr->Soon_Name)] = '\0';     
	    }
	}
	
    return(status);
}

IMRT_Status  MM_XML_Split_Path( IMRT_Buffer_Data * Chaine_Ptr , IMRT_PARSE_PATH_XML * PathXml_Ptr )
{
    int indexe  = 0;
    size_t len  =0;
    const char SEP_XML [] = "/";
    char * sep_position_prec = NULL;
    char * sep_position_suiv = NULL;
    char SEP_XML_CHAR  = '/';
    IMRT_Status status;

/*initialisation */
    

    if( Chaine_Ptr == NULL)
    {
	IMRD_LM_LOG_ERROR_0("Le pointeur chaine a traiter est NULL");
	status = IMRS_KO;
    }
    else
    if( PathXml_Ptr == NULL)
    {
	IMRD_LM_LOG_ERROR_0("PathXml_Ptr NULL");
	status = IMRS_KO;
    }
    else
    {
	PathXml_Ptr->Father_Table = NULL;
        memset(PathXml_Ptr->Soon_Name, '\0', sizeof(PathXml_Ptr->Soon_Name));
        PathXml_Ptr->Nb_Father = 0;
	len =  strlen(Chaine_Ptr);
	status = IMRS_OK;
  
	/*calcul du nombre de pere */
	for( indexe=0; indexe <len ; indexe++)
	{
	    if(Chaine_Ptr[indexe] ==  SEP_XML_CHAR)
	    {
		PathXml_Ptr->Nb_Father++;
	    }
	}
	/*si la chane commence par un / on en tien pas compte*/
	if(Chaine_Ptr[0] == SEP_XML_CHAR)
	{
	    sep_position_suiv = Chaine_Ptr +1;
	    sep_position_prec = Chaine_Ptr +1;
	    PathXml_Ptr->Nb_Father = PathXml_Ptr->Nb_Father -1;
	}  
	else
	{
	    sep_position_suiv = Chaine_Ptr ;
	    sep_position_prec = Chaine_Ptr;
	}
	/* allocation du tableau de peres */
	PathXml_Ptr->Father_Table =  (IMRT_FATHER_TABLE)malloc(sizeof(char *) * PathXml_Ptr->Nb_Father);
	if( PathXml_Ptr->Father_Table == NULL)
	{
	    /*erreur d'allocation memoire**/
	    IMRD_LM_LOG_ERROR_0("Erreur allocation memoire");
	    status = IMRS_KO;
	}
	else
	{
	    for(indexe = 0 ; indexe < PathXml_Ptr->Nb_Father; indexe++)
	    {
		PathXml_Ptr->Father_Table[indexe] = (char *)malloc(IMRD_FATHER_SIZE);
		if(PathXml_Ptr->Father_Table[indexe] == NULL)
		{
		    /*erreur allocation */
		    IMRD_LM_LOG_ERROR_0("Erreur allocation memoire");
		    status = IMRS_KO;
		    break ;
		}

	    }
      
	    /*eclatement de la chane  sur le n elemment on a 
	      n-1 noeud et le n ieme elment est une feuille*/  
	    if(status == IMRS_OK)
	    {
		for(indexe =0; indexe <= PathXml_Ptr->Nb_Father; indexe++)
		{
		    /*on est a la fin de la chaine c'est une feuille */
		    if( indexe == PathXml_Ptr->Nb_Father )
		    {
			strncpy(PathXml_Ptr->Soon_Name,sep_position_suiv,((Chaine_Ptr +len) - sep_position_suiv));
                        PathXml_Ptr->Soon_Name[((Chaine_Ptr +len) - sep_position_suiv)] = '\0';
                        status = MM_XML_SplitAttribute(PathXml_Ptr);
		    }
		    else
		    {
			/*recopie des pere dans le tableau */
			sep_position_suiv = strstr((const char *)sep_position_suiv, SEP_XML);
			if( sep_position_suiv != NULL)
			{
			    strncpy(PathXml_Ptr->Father_Table[indexe], sep_position_prec, (sep_position_suiv - sep_position_prec));
			    PathXml_Ptr->Father_Table[indexe][( sep_position_suiv - sep_position_prec)] = '\0';

			    sep_position_suiv  = sep_position_suiv+1; 
			    sep_position_prec  = sep_position_suiv; 
			}
		    }
		}
         
	    }
	}
    }
    return(status); 
}




IMRT_Status   MM_XML_StrHdl_Put(IMRT_Buffer_Data *Data_Ptr, IMRT_Field *Field_Ptr,  
				  IMRT_Buffer *Buffer_Ptr , size_t len )
{
 char *func_name = "MM_XML_StrHdl_Put";
 IMRT_Status status;
 xmlDocPtr  xml_doc_ptr;
 IMRT_Buffer_Data tagxml_tab[IMRD_XML_TAG_LEN];
 IMRT_PARSE_PATH_XML Path_XML_Struct;


 status = IMRS_OK;

	if (Buffer_Ptr == NULL) 
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer_Ptr NULL");
	}
	else if (Field_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Field_Ptr NULL");
	}
	else if (Data_Ptr == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("data pointer NULL");
	}
	else
	{ 
		 /* Controle de validite du tag */
		 if (strcmp(Field_Ptr->XML.Tag ,IMRD_XML_TAG_UNKNOWN) == 0)
		 {
		     status   = IMRS_KO;
		     IMRD_LM_LOG_ERROR_0("Tag XML unknown");
		 }
	}


 if ((status == IMRS_OK) && (strcmp(Data_Ptr , "") == 0))
     status = IMRD_STATUS_NOTHING_TO_DO;

 if (status == IMRS_OK)
 {
     /* controle sur le buffer et la valeur */
     if (Buffer_Ptr->Data_Tmp_Ptr == NULL)
     {
	 status = IMRS_KO;
	 IMRD_LM_LOG_ERROR_0("Buffer pointer NULL");
     }
     else 
     {
	 xml_doc_ptr = (xmlDocPtr) Buffer_Ptr->Data_Tmp_Ptr;
	 strcpy( tagxml_tab, (IMRT_Buffer_Data *) Field_Ptr->XML.Tag);
	 //status = MM_XML_Init_StructPath(&Path_XML_Struct);

         if(status == IMRS_OK)
	 {
	     IMRD_LM_LOG_TRACE_1( IMRD_LOG_LEVEL_VERBOSE_2 , "Tag xml [%s] ",tagxml_tab);
            status =  MM_XML_Split_Path( tagxml_tab ,&Path_XML_Struct  );
	    if(status == IMRS_OK)
            {
		status = MM_XML_Proceed_Path( &Path_XML_Struct,Data_Ptr, xml_doc_ptr);
            }		
         }
	 status = MM_XML_DInit_StructPath(&Path_XML_Struct);
     }
 }

	if (status == IMRS_OK)
	{
	        /* Mise a jour de la taille du buffer */
	        Buffer_Ptr->Data_Buf_Len = (Buffer_Ptr->Data_Buf_Len) + len;
	}

 if (status == IMRD_STATUS_NOTHING_TO_DO)
        status = IMRS_OK;

	if (Data_Ptr != NULL)
                free(Data_Ptr);

 return(status);
}


