/*----------------------------------------------------------------------------*/
/* $Workfile:   libmapping_mng.c  $                                           */
/*----------------------------------------------------------------------------*/
/* $Author: agibert $                                                      */
/* $Date: 2008/11/12 02:25:23 $                                           */
/* $Revision: 1.1 $                                                       */
/* $Label:  $                                                         */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#define _LIBMAPPING_MNG_
#define _LIBMAPPING_MNG_C_

#include "libmapping_mng.h"







/******************************************************************************/
/* Mapping Manager API                                                        */
/******************************************************************************/



/*----------------------------------------------------------------------------*/
/* Open Mapping Manager                                                       */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_MM_Library_Open( IMRT_MM_Config  *Config)
{
    IMRT_Status	status;

	
    if( IMRG_MM_Module.Lib.Status != IMRD_LIB_STATUS_CLOSED)
    {
	IMRD_LM_LOG_ERROR_0( "Library not closed !");
	
	return( IMRS_KO);
    }
    else
    {
	IMRG_MM_Base.Config.XML_Format = Config->XML_Format;

        status  = IMRS_OK;
        
        status = MM_Schedul_Init( );
        
        if (status != IMRS_OK)
        {
	    IMRD_LM_LOG_ERROR_0("Error MM_Schedul_Init in IMR_MM_Library_Open");
	}
	else
	{
	    status =   MM_Buffer_Init( ) ;
	}
	    

        if( status != IMRS_OK)
	{
	    IMRD_LM_LOG_ERROR_0( "Can't open library !");
	}
	else
	{
	    IMRG_MM_Module.Lib.Status = IMRD_LIB_STATUS_OPENED;
	    IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_1, "Library opened !");
	}
                
	return( status);
    }
}





/*----------------------------------------------------------------------------*/
/* Close Mapping Manager                                                      */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_MM_Library_Close()
{
    IMRT_Status	status;
        

    if( IMRG_MM_Module.Lib.Status != IMRD_LIB_STATUS_OPENED)
    {
	IMRD_LM_LOG_ERROR_0( "Library not opened !");

	return( IMRS_KO);
    }
    else
    {
        status  = IMRS_OK;
        
        status = MM_Schedul_DeInit( );
        
        if (status != IMRS_OK)
        {
	        IMRD_LM_LOG_ERROR_0("Error MM_Schedul_DeInit in IMR_MM_Library_Close");
	}
	else
	{
	        status =   MM_Buffer_DeInit( ) ;
	}
	        
        if( status != IMRS_OK)
	{
	    IMRD_LM_LOG_ERROR_0( "Can't close library !");
	}
	else
	{
	    IMRG_MM_Module.Lib.Status = IMRD_LIB_STATUS_CLOSED;

	    IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_1, "Library closed !");
	}

	return( status) ;
    }
}





/*----------------------------------------------------------------------------*/
/* Buffer allocation	                                                      */
/*----------------------------------------------------------------------------*/
/* (O) Buffer_Ptr_Ptr :   Buffer pointer pointer		    	      */
/*----------------------------------------------------------------------------*/

IMRT_Status	IMR_MM_Buffer_Alloc( IMRT_Buffer **Buffer_Ptr_Ptr)
{
        IMRT_Status	status;
        
        status  = IMRS_OK;

	status = MM_Buffer_Alloc( Buffer_Ptr_Ptr ) ;

        if (status != IMRS_OK)
        {
	        IMRD_LM_LOG_ERROR_0("Error MM_Buffer_Alloc");
	}
	else
	{
                IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_1, "Allocated buffer success");
        }

	return (status) ;
}


/*----------------------------------------------------------------------------*/
/* Buffer free			                                              */
/*----------------------------------------------------------------------------*/
/* (IO) Buffer_Ptr_Ptr :   Buffer pointer pointer		    	      */
/*----------------------------------------------------------------------------*/

IMRT_Status	IMR_MM_Buffer_Free( IMRT_Buffer **Buffer_Ptr_Ptr)
{
        IMRT_Status	status;
        
        status  = IMRS_OK;

	status = MM_Buffer_Free( Buffer_Ptr_Ptr ) ;

        if (status != IMRS_OK)
        {
	        IMRD_LM_LOG_ERROR_0("Error MM_Buffer_Free");
	}
	else
	{
                IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_1, "Freed buffer success");
        }

	return (status) ;
	
}


/*----------------------------------------------------------------------------*/
/* Buffer check			                                              */
/*----------------------------------------------------------------------------*/
/* (I) Buffer_Ptr :   Buffer pointer 					      */
/*----------------------------------------------------------------------------*/

IMRT_Status	IMR_MM_Buffer_Check (IMRT_Buffer *Buffer_Ptr)
{

        IMRT_Status	status;
        
        status  = IMRS_OK;

	status = MM_Buffer_Check( Buffer_Ptr )  ;

        if (status != IMRS_OK)
        {
	        IMRD_LM_LOG_ERROR_0("Error MM_Buffer_Check in IMR_MM_Buffer_Check");
	}
	else
	{
                IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_1, "Check buffer success");
        }

	return (status) ;

}


/*----------------------------------------------------------------------------*/
/* Buffer list			                                              */
/*----------------------------------------------------------------------------*/
/* (I) Buffer_Ptr :   Buffer pointer 					      */
/* (I) texte 	  :   Text to print		 		              */
/*----------------------------------------------------------------------------*/

IMRT_Status	IMR_MM_Buffer_List( IMRT_Buffer *Buffer_Ptr, char *texte)
{
	return( MM_Buffer_List( Buffer_Ptr, texte ) );
}


/*----------------------------------------------------------------------------*/
/* Buffer mapping		                                              */
/*----------------------------------------------------------------------------*/
/* (I) Output_Buffer_Ptr :   Output buffer pointer 			      */
/* (I) Input_Buffer_Ptr  :   Input buffer pointer		 	      */
/*----------------------------------------------------------------------------*/

IMRT_Status IMR_MM_Buffer_Map( IMRT_Buffer *Output_Buffer_Ptr,  IMRT_Buffer *Input_Buffer_Ptr)
{
	return ( MM_Schedul_Buffer_Map( Output_Buffer_Ptr, Input_Buffer_Ptr ) );
}


