/*----------------------------------------------------------------------------*/
/* File: map_scheduler.c					   		         		  	      */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#define _LIBMAPPING_MNG_
#define _MAP_SCHEDULER_C_

#include "map_scheduler.h" 

IMRT_Status MM_Schedul_Buffer_Map( IMRT_Buffer *Output_Buffer_Ptr, IMRT_Buffer *Input_Buffer_Ptr)
{
	IMRT_Status status;
	short indice;
	IMRT_Fmt_Id   format_temp;
	char *func_name = "MM_Schedul_Buffer_Map";

	status = IMRS_OK;
	indice = 0;
	format_temp = Input_Buffer_Ptr->Fmt_Id;
	
	/* controles sur les buffers */
	status = Schedul_Buffers_Check(Input_Buffer_Ptr, Output_Buffer_Ptr);

	/* fermeture des buffers */
	if (status == IMRS_OK)
	{
		status = MM_Buffer_Close(Input_Buffer_Ptr);
	}
	if (status != IMRS_OK)
	{
		IMRD_LM_LOG_ERROR_0("Buffer input close failed");
	}
	else
	{
		status = MM_Buffer_Close(Output_Buffer_Ptr);
	}

	if (status != IMRS_OK)
	{
		IMRD_LM_LOG_ERROR_0("Buffer output close failed");
	}
	else
	{
		/* initialisation du tableau */
		Buffer_Tab[0] = Input_Buffer_Ptr;
		Buffer_Tab[1] = Output_Buffer_Ptr;
		for (indice = 2; indice < IMRT_BUFFER_ID_MAX_NB; indice++)
			Buffer_Tab[indice] = NULL;
			
		Input_Buffer_Ptr->Fmt_Id = IMRD_FMT_ID_UNKNOWN ;
		status = MM_Buffer_Open( Input_Buffer_Ptr );

		IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_2 , "Values before MapMath proceed begin :");
		IMRD_LM_LOG_TRACE_3(IMRD_LOG_LEVEL_VERBOSE_2 , "Output Application : Id (%d) Fmt_Id (%d) Msg type Id (%d)" , Output_Buffer_Ptr->Appl_Id_Output , Output_Buffer_Ptr->Fmt_Id , Output_Buffer_Ptr->Msg_Id);
		IMRD_LM_LOG_TRACE_3(IMRD_LOG_LEVEL_VERBOSE_2 , "Input Application : Id (%d) Fmt_Id (%d) Msg type Id (%d)" , Input_Buffer_Ptr->Appl_Id_Input , Input_Buffer_Ptr->Fmt_Id , Input_Buffer_Ptr->Msg_Id);
	
	
		if (status != IMRS_OK)
		{
			IMRD_LM_LOG_ERROR_0("Buffer input open failed");
			Input_Buffer_Ptr->Fmt_Id = format_temp;
		}
		else
		{		
			status = IMR_SDM_MapPath_Proceed( Input_Buffer_Ptr->Appl_Id_Output,
                                          	Output_Buffer_Ptr->Fmt_Id,
                                          	Output_Buffer_Ptr->Msg_Id,
		                                Input_Buffer_Ptr->Appl_Id_Input,                                        	
                                          	Input_Buffer_Ptr->Fmt_Id,
                                          	Input_Buffer_Ptr->Msg_Id,
                                          	IMR_MM_Schedul_Map,
                                          	IMR_MM_Schedul_Copy,
                                                IMR_MM_Schedul_Ident);
			if (status != IMRS_OK)
			{
				IMRD_LM_LOG_ERROR_1("MapPath proceed ended with status (%d)", status);
			}
        }                         	
	}

	indice = 2;
	while (indice < IMRT_BUFFER_ID_MAX_NB)
	{
		if (Buffer_Tab[indice] != NULL)
		{
			if (MM_Buffer_Free(&Buffer_Tab[indice]) != IMRS_OK)
			{
				status = IMRS_KO;
			        IMRD_LM_LOG_ERROR_1("Buffer (%d) free failed" , indice);
			}
		}
		indice++;
	}
	
	if (MM_Buffer_Close(Input_Buffer_Ptr) != IMRS_OK)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer input close failed");
	}
	if (MM_Buffer_Close(Output_Buffer_Ptr) != IMRS_OK)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_0("Buffer output close failed");
	}
	
	Buffer_Tab[0] = NULL;
	Buffer_Tab[1] = NULL;

	return ( status );
}

IMRT_Status	Schedul_Buffers_Check( IMRT_Buffer *Input_Buffer_Ptr, IMRT_Buffer *Output_Buffer_Ptr)
{
	IMRT_Status	status;
	char *func_name = "Schedul_Buffers_Check";

	
	status = IMRS_OK;
	
	if (Input_Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
	        IMRD_LM_LOG_ERROR_0("Input buffer pointer NULL");		
	}
	else
	if (Output_Buffer_Ptr == NULL)
	{
		status = IMRS_KO;
	        IMRD_LM_LOG_ERROR_0("Output buffer pointer NULL");		
	}
	else
	{
		if (MM_Buffer_Check(Input_Buffer_Ptr)  != IMRS_OK)
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("Check buffer input failed");
		}
		if (MM_Buffer_Check(Output_Buffer_Ptr) != IMRS_OK)
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("Check buffer output failed");
		}
		if (Input_Buffer_Ptr->Appl_Id_Input == IMRD_APPL_ID_UNDEFINEDED)
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("Application input buffer equal to undefineded");
		}
		if (Input_Buffer_Ptr->Status != IMRD_BUFFER_STATUS_ALLOCATED)
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("Status input buffer equal to unknown");
		}
		if (Input_Buffer_Ptr->Data_Buf_Ptr == NULL)
		{
			status = IMRS_KO;
			IMRD_LM_LOG_ERROR_0("Data_Buf_Ptr input buffer equal to NULL");
		}
	}

	return ( status );
}


IMRT_Status IMR_MM_Schedul_Ident( IMRT_Fmt_Id *Format_Ptr,
                                  IMRT_Msg_Id *Type,
                                  IMRT_Buffer_Id Ident_Buffer_Indice,
                                  IMRT_Boolean Ident_Total )
{
	IMRT_Status	status;
	IMRT_Msg_Id     TypeTmp;
	char *func_name = "IMR_MM_Schedul_Ident";

	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 ,  "============================================================================")
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 , "Ident buffer demand : Begin");
	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_1 , "Ident buffer demand : Buffer       :  (%d)" , Ident_Buffer_Indice);	
	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_1 , "Ident buffer demand : Format Id    :  (%d)" , *Format_Ptr);	
	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_1 , "Ident buffer demand : Msg type     :  (%d)" , *Type);	
	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_1 , "Ident buffer demand : Total ident  :  (%d)" , Ident_Total);	
	
	status = IMRS_OK;
	TypeTmp = IMRD_MSG_ID_UNKNOWN;

	/* controle valeur de l'indice */
	if ( ( Ident_Buffer_Indice < 0 ) || ( Ident_Buffer_Indice >= IMRT_BUFFER_ID_MAX_NB	) )
	{
		status = IMRS_KO;	
		IMRD_LM_LOG_ERROR_1("Buffer (%d) out of bounds" , Ident_Buffer_Indice);
	}
	/* controle allocation d'un buffer pour l'indice */
	else if ( Buffer_Tab[Ident_Buffer_Indice] == NULL)
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_1("Buffer (%d) not allocated" , Ident_Buffer_Indice);
	}
	else
	{
	        TypeTmp = Buffer_Tab[Ident_Buffer_Indice]->Msg_Id;
	        
	        if (Ident_Total == IMRD_TRUE) // identification totale
	        {
		        status = MM_Buffer_Ident_Update(Buffer_Tab[Ident_Buffer_Indice],
        		                                IMRD_FMT_ID_UNKNOWN,
        		                                IMRD_FMTATTR_NAME_UNKNOWN,
        		                                IMRD_MSG_ID_UNKNOWN);
                }
	}

	if ( status != IMRS_OK )
	{
	        IMRD_LM_LOG_ERROR_0("MM_Buffer_Ident_Update failed");
	}
	else
	{
		status = MM_Buffer_Open ( Buffer_Tab[Ident_Buffer_Indice] );
	}
	
	if ( status != IMRS_OK )
	{
	        IMRD_LM_LOG_ERROR_1("Buffer (%d) not open" , Ident_Buffer_Indice);
	}
	else
 	{
		status = MM_Buffer_Ident ( *Type , Buffer_Tab[Ident_Buffer_Indice] );
	}

      	if (status != IMRS_OK)
	{
	        IMRD_LM_LOG_ERROR_0("MM_Buffer_Ident failed");
	}
	else
       	{
	        *Format_Ptr = Buffer_Tab[Ident_Buffer_Indice]->Fmt_Id;
	        *Type = Buffer_Tab[Ident_Buffer_Indice]->Msg_Id;
	}

	
       	if ((status != IMRS_OK) && (status != IMRS_KO))
       	{
		status = IMRS_OK; 
	}

	MM_Buffer_List( Buffer_Tab[Ident_Buffer_Indice], " CB Ident end");

	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_1 , "Ident buffer demand end : Format Id    :  (%d)" , *Format_Ptr);	
	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_1 , "Ident buffer demand end : Msg type     :  (%d)" , *Type);	
	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_1 , "Ident buffer demand : End with status :  (%d)" , status);
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 , "============================================================================")

	return ( status );
}
	
IMRT_Status IMR_MM_Schedul_Copy( IMRT_Buffer_Id Output_Buffer_Indice, IMRT_Buffer_Id Input_Buffer_Indice)
{
	IMRT_Status	status;
	IMRT_Buffer *buffer_ptr;
	char *func_name = "IMR_MM_Schedul_Copy";


	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 ,  "============================================================================")
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 , "Copy buffer demand : Begin");
	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_1 , "Copy buffer demand : Buffer Target  :  (%d)" , Output_Buffer_Indice);	
	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_1 , "Copy buffer demand : Buffer Source  :  (%d)" , Input_Buffer_Indice);	

        
	status = IMRS_OK;
	buffer_ptr = NULL;
	
	/* controle la validite des indices */
	if ( ( Input_Buffer_Indice  < 0 ) || ( Input_Buffer_Indice  >= IMRT_BUFFER_ID_MAX_NB ) ||
	     ( Output_Buffer_Indice < 0 ) || ( Output_Buffer_Indice >= IMRT_BUFFER_ID_MAX_NB ) )
	{
	        IMRD_LM_LOG_ERROR_2("Buffer input with indice (%d) or output with indice (%d) out of bounds" , Input_Buffer_Indice , Output_Buffer_Indice);
		status = IMRS_KO;	
	}
	else if ( Buffer_Tab[Input_Buffer_Indice] == NULL)
	{
		status = IMRS_KO;	
	        IMRD_LM_LOG_ERROR_1("Buffer with indice (%d) not initialized" , Input_Buffer_Indice);
	}
	/* allocation du buffer destinataire si necessaire */
	else if ( Buffer_Tab[Output_Buffer_Indice] == NULL) 
	{
		status = MM_Buffer_Alloc(&buffer_ptr);
		if (status != IMRS_OK)
		{
		        IMRD_LM_LOG_ERROR_0("Error MM_Buffer_Alloc");
		}
		else
		{
			Buffer_Tab[Output_Buffer_Indice] = buffer_ptr;
		}
	}
	MM_Buffer_List( Buffer_Tab[Input_Buffer_Indice], " Before Copy Buffer In");	
	if (status == IMRS_OK)
	{
		status = MM_Buffer_Copy( Buffer_Tab[Output_Buffer_Indice], Buffer_Tab[Input_Buffer_Indice]);
	}
	MM_Buffer_List( Buffer_Tab[Output_Buffer_Indice], " After Copy Buffer Out");	

	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_1 , "Copy buffer demand : End with status :  (%d)" , status);
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 , "============================================================================")

	return ( status );
}

/*----------------------------------------------------------------------------*/
/* Buffer Mapping CallBack                                                    */
/*----------------------------------------------------------------------------*/
/* (I) Buffer_Id_Target:   Target buffer id                                   */
/* (I) Fmt_Target:         Target format                                      */
/* (I) Buffer_Id_Src:      Source buffer id                                   */
/* (I) Fmt_Src:            Source format                                      */
/* (I) MsgMap:             Message Map to use                                 */
/* (I) Clear_Flag:         Clear or not output buffer                         */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_MM_Schedul_Map( IMRT_Buffer_Id Buffer_Id_Target ,
                                         IMRT_Fmt_Id Fmt_Target, 
                                         IMRT_FmtAttr_Value Fmt_Target_Attr ,
                                         IMRT_Buffer_Id Buffer_Id_Src , 
                                         IMRT_Fmt_Id Fmt_Src , 
                                         IMRT_FmtAttr_Value Fmt_Src_Attr ,
                                         IMRT_MsgMap *MsgMap_ptr , 
                                         IMRT_MapCmd_Flag Clear_Flag)

{
	IMRT_Buffer *input_buffer_ptr,
	            *output_buffer_ptr;
	IMRT_Status	status;
	char *func_name = "IMR_MM_Schedul_Map";

	
	status  = IMRS_OK;
	input_buffer_ptr = NULL;
	output_buffer_ptr = NULL;


	/* controle buffers */
	if ( ( Buffer_Id_Src  < 0 ) || ( Buffer_Id_Src  >= IMRT_BUFFER_ID_MAX_NB ) ||
	     ( Buffer_Id_Target < 0 ) || ( Buffer_Id_Target >= IMRT_BUFFER_ID_MAX_NB ) )	
	{
		status = IMRS_KO;
		IMRD_LM_LOG_ERROR_2("Indices buffer input (%d) or output (%d) out of bounds" , Buffer_Id_Src,Buffer_Id_Target);
	}
	else if ( Buffer_Tab[Buffer_Id_Src] == NULL)
	{
		status = IMRS_KO;	
		IMRD_LM_LOG_ERROR_1("Buffer with indice (%d) not allocated ",Buffer_Id_Src);
	}
	else if (Fmt_Src_Attr == NULL)
	{
		status = IMRS_KO;	
		IMRD_LM_LOG_ERROR_0("Source format attribute NULL");
	}
	else if (Fmt_Target_Attr == NULL)
	{
		status = IMRS_KO;	
		IMRD_LM_LOG_ERROR_0("Target format attribute NULL");
	}
	else if (MsgMap_ptr == NULL)
	{
		status = IMRS_KO;	
		IMRD_LM_LOG_ERROR_0("Message map pointer NULL");
	}
	else if (MsgMap_ptr->Msg_Input_Ptr == NULL)
	{
		status = IMRS_KO;	
		IMRD_LM_LOG_ERROR_0("Message input pointer NULL");
	}
	else if (MsgMap_ptr->Msg_Output_Ptr == NULL)
	{
		status = IMRS_KO;	
		IMRD_LM_LOG_ERROR_0("Message output pointer NULL");
	}
	
	if ( status == IMRS_OK )
	{
		IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 , "============================================================================")
		IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 , "Mapping buffer demand : Begin");
		IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_1 , "Mapping buffer demand : MsgMap :  (%s)" , (char *) MsgMap_ptr->Name);	
		IMRD_LM_LOG_TRACE_2(IMRD_LOG_LEVEL_VERBOSE_1 , "Mapping buffer demand : indices Buffers : Src (%d) : Tgt (%d)" , Buffer_Id_Src , Buffer_Id_Target);
		IMRD_LM_LOG_TRACE_2(IMRD_LOG_LEVEL_VERBOSE_1 , "Mapping buffer demand : formats         : Src (%d) : Tgt (%d)" , Fmt_Src , Fmt_Target);
		IMRD_LM_LOG_TRACE_2(IMRD_LOG_LEVEL_VERBOSE_1 , "Mapping buffer demand : Attributs       : Src (%s) : Tgt (%s)" , Fmt_Src_Attr , Fmt_Target_Attr);
		IMRD_LM_LOG_TRACE_2(IMRD_LOG_LEVEL_VERBOSE_1 , "Mapping buffer demand : Types de msg    : Src (%s) : Tgt (%s)" , (char *) MsgMap_ptr->Msg_Input_Ptr->Name , (char *) MsgMap_ptr->Msg_Output_Ptr->Name);
		IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_1 , "Mapping buffer demand : Flag buffer clear :  (%d)" , Clear_Flag);
		
		input_buffer_ptr = Buffer_Tab[Buffer_Id_Src];
		if ( Buffer_Tab[Buffer_Id_Target] == NULL)
		{
			status = MM_Buffer_Alloc(&output_buffer_ptr);		
			if ( status != IMRS_OK )
			{
			        IMRD_LM_LOG_ERROR_1("MM_Buffer_Alloc failed for buffer output (%d)",Buffer_Id_Target);
			}
			else
			{
				Buffer_Tab[Buffer_Id_Target] = output_buffer_ptr;
			}
			
		}
		else if ( Clear_Flag == IMRD_TRUE )
		{
			output_buffer_ptr = Buffer_Tab[Buffer_Id_Target];
			status = MM_Buffer_Clear(output_buffer_ptr);		
		}
		else
		{
		        output_buffer_ptr = Buffer_Tab[Buffer_Id_Target];
		}
	}
	/* mise a jour des formats et types des buffers */
	if ( status == IMRS_OK )
	{
		status = MM_Buffer_Ident_Update(input_buffer_ptr,
		                                Fmt_Src,
		                                Fmt_Src_Attr,
		                                MsgMap_ptr->Msg_Input_Ptr->Id);
	}
	if ( status == IMRS_OK )
	{
		status =MM_Buffer_Ident_Update( output_buffer_ptr,
		                                Fmt_Target,
		                                Fmt_Target_Attr,
		                                MsgMap_ptr->Msg_Output_Ptr->Id);
	}
	/* ouverture des buffers */
	if ( status == IMRS_OK )
	{
		status = MM_Buffer_Open( input_buffer_ptr);
	}
	if ( status == IMRS_OK )
	{
		MM_Buffer_Open( output_buffer_ptr);
	}
	
	MM_Buffer_List( input_buffer_ptr, " Input Buffer Before Mapping");	
	
	if ( status == IMRS_OK )
	{
		status = MM_Buffer_Map( output_buffer_ptr, input_buffer_ptr, MsgMap_ptr);
	}                          
	
	MM_Buffer_List( output_buffer_ptr, " Output Buffer after Mapping");	
	
	IMRD_LM_LOG_TRACE_1(IMRD_LOG_LEVEL_VERBOSE_1 , "Mapping buffer demand : End with status :  (%d)" , status);
	IMRD_LM_LOG_TRACE_0(IMRD_LOG_LEVEL_VERBOSE_1 ,  "============================================================================")
	
	return ( status );
}

IMRT_Status MM_Schedul_Init()
{
      return (IMRS_OK);  
}

IMRT_Status MM_Schedul_DeInit()
{
      return (IMRS_OK);  
}
