/*----------------------------------------------------------------------------*/
/* $Workfile:   libmsg_router.c  $                                            */                                            
/*----------------------------------------------------------------------------*/
/* $Author: agibert $                                                      */
/* $Date: 2008/11/12 02:25:23 $                                           */
/* $Revision: 1.1 $                                                       */
/* $Label:  $                                                         */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#define _LIBMSG_ROUTER_
#define _LIBMSG_ROUTER_C_

#include "libmsg_router.h"







/******************************************************************************/
/* Msg Router API                                                             */
/******************************************************************************/



/*----------------------------------------------------------------------------*/
/* Open Msg Router                                                            */
/*----------------------------------------------------------------------------*/
/* Lib_Flag:   Setup MR init mode                                             */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_MR_Library_Open( IMRT_MR_Config  *Config_Ptr)
{
    int  idx;


    if( IMRG_MR_Module.Lib.Status != IMRD_LIB_STATUS_CLOSED)
    {
	IMRD_LM_LOG_ERROR_0( "Library not closed !");

	return( IMRS_KO);
    }
    else
    {
	IMRG_MR_Base.Config.Mapping_Forced  = Config_Ptr->Mapping_Forced;
	IMRG_MR_Base.Config.Continue_Forced = Config_Ptr->Continue_Forced;
	IMRG_MR_Base.Config.Read_Mode       = Config_Ptr->Read_Mode;

	if( Config_Ptr->Log_Id_Struct_Input.Bit_Nb != 0)
        {
	    IMRG_MR_Base.Config.Log_Id_Struct_Input.Bit_Nb   = Config_Ptr->Log_Id_Struct_Input.Bit_Nb;
	    IMRG_MR_Base.Config.Log_Id_Struct_Input.Log_Size = Config_Ptr->Log_Id_Struct_Input.Log_Size;

	    if( ( IMRG_MR_Base.Config.Log_Id_Struct_Input.Log_Id_Bit_Ptr =
		  malloc( IMRG_MR_Base.Config.Log_Id_Struct_Input.Bit_Nb * sizeof( IMRT_Log_Id_Bit))) == NULL)
	    {
		IMRD_LM_LOG_ERROR_1( "Can't allocate Input Log_Id_Bit_Ptr (%d) !", errno);

		return( IMRS_OK);
	    }

	    for( idx = 0; idx < IMRG_MR_Base.Config.Log_Id_Struct_Input.Bit_Nb; idx++)
	    {
		IMRG_MR_Base.Config.Log_Id_Struct_Input.Log_Id_Bit_Ptr[idx].Offset = Config_Ptr->Log_Id_Struct_Input.Log_Id_Bit_Ptr[idx].Offset;
		IMRG_MR_Base.Config.Log_Id_Struct_Input.Log_Id_Bit_Ptr[idx].Size   = Config_Ptr->Log_Id_Struct_Input.Log_Id_Bit_Ptr[idx].Size;
	    }
        }

	if( Config_Ptr->Log_Id_Struct_Output.Bit_Nb != 0)
        {
	    IMRG_MR_Base.Config.Log_Id_Struct_Output.Bit_Nb   = Config_Ptr->Log_Id_Struct_Output.Bit_Nb;
	    IMRG_MR_Base.Config.Log_Id_Struct_Output.Log_Size = Config_Ptr->Log_Id_Struct_Output.Log_Size;

	    if( ( IMRG_MR_Base.Config.Log_Id_Struct_Output.Log_Id_Bit_Ptr =
		  malloc( IMRG_MR_Base.Config.Log_Id_Struct_Output.Bit_Nb * sizeof( IMRT_Log_Id_Bit))) == NULL)
	    {
		IMRD_LM_LOG_ERROR_1( "Can't allocate Output Log_Id_Bit_Ptr (%d) !", errno);

		return( IMRS_OK);
	    }

	    for( idx = 0; idx < IMRG_MR_Base.Config.Log_Id_Struct_Output.Bit_Nb; idx++)
	    {
		IMRG_MR_Base.Config.Log_Id_Struct_Output.Log_Id_Bit_Ptr[idx].Offset = Config_Ptr->Log_Id_Struct_Output.Log_Id_Bit_Ptr[idx].Offset;
		IMRG_MR_Base.Config.Log_Id_Struct_Output.Log_Id_Bit_Ptr[idx].Size   = Config_Ptr->Log_Id_Struct_Output.Log_Id_Bit_Ptr[idx].Size;
	    }
        }
          
	IMRG_MR_Module.Lib.Status = IMRD_LIB_STATUS_OPENED;

	IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_1, "Library opened !");

	return( IMRS_OK);
    }
}





/*----------------------------------------------------------------------------*/
/* Close Msg Router                                                           */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_MR_Library_Close()
{
    if( IMRG_MR_Module.Lib.Status != IMRD_LIB_STATUS_OPENED)
    {
	IMRD_LM_LOG_ERROR_0( "Library not opened !");

	return( IMRS_KO);
    }
    else
    {
	if( IMRG_MR_Base.Config.Log_Id_Struct_Input.Bit_Nb != 0)
        {
	    free( IMRG_MR_Base.Config.Log_Id_Struct_Input.Log_Id_Bit_Ptr);
        }
	
	if( IMRG_MR_Base.Config.Log_Id_Struct_Output.Bit_Nb != 0)
        {
	    free( IMRG_MR_Base.Config.Log_Id_Struct_Output.Log_Id_Bit_Ptr);
        }
	
	IMRG_MR_Module.Lib.Status = IMRD_LIB_STATUS_CLOSED;
	IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_1, "Library closed !");

	return( IMRS_OK);
    }
}





/*----------------------------------------------------------------------------*/
/* Close all allocated resources                                              */
/*----------------------------------------------------------------------------*/
/* Buffer_Input_Ptr:    Input buffer                                          */
/* Buffer_Output_Ptr:   Output buffer                                         */
/* Medium_Input_Ptr:    Input Medium                                          */
/* Medium_Output_Ptr:   Output Medium                                         */
/* Medium_Trash_Ptr:    Output Medium                                         */
/*----------------------------------------------------------------------------*/

IMRT_Status  Close_All( IMRT_Buffer  *Buffer_Input_Ptr, IMRT_Buffer  *Buffer_Output_Ptr,
			IMRT_Medium  *Medium_Input_Ptr, IMRT_Medium  *Medium_Output_Ptr, IMRT_Medium  *Medium_Trash_Ptr)
{
    IMRT_Status  status;


    if( Buffer_Output_Ptr != NULL)
    {
	IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Freeing output buffer...");
	
	if( ( status = IMR_MM_Buffer_Free( &Buffer_Output_Ptr)) != IMRS_OK)
	{
	    IMRD_LM_LOG_ERROR_1( "Can't free output buffer (%d) !", status);
	}
    }
    
    if( Buffer_Input_Ptr != NULL)
    {
	IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Freeing input buffer...");
	
	if( ( status = IMR_MM_Buffer_Free( &Buffer_Input_Ptr)) != IMRS_OK)
	{
	    IMRD_LM_LOG_ERROR_1( "Can't free input buffer (%d) !", status);
	}
    }
    
    if( Medium_Trash_Ptr != NULL)
    {
	IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Closing trash medium...");
	
	if( ( status = IMR_IOM_Medium_Close( Medium_Trash_Ptr)) != IMRS_OK)
	{
	    IMRD_LM_LOG_ERROR_1( "Can't close trash medium (%d) !", status);
	}
    }

    if( Medium_Output_Ptr != NULL)
    {
	IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Closing output medium...");
	
	if( ( status = IMR_IOM_Medium_Close( Medium_Output_Ptr)) != IMRS_OK)
	{
	    IMRD_LM_LOG_ERROR_1( "Can't close output medium (%d) !", status);
	}
    }
    
    if( Medium_Input_Ptr != NULL)
    {
	IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Closing input medium...");
	
	if( ( status = IMR_IOM_Medium_Close( Medium_Input_Ptr)) != IMRS_OK)
	{
	    IMRD_LM_LOG_ERROR_1( "Can't close input medium (%d) !", status);
	}
    }

    return( IMRS_OK);
}





/*----------------------------------------------------------------------------*/
/* Proceed Message Routing                                                    */
/*----------------------------------------------------------------------------*/
/* Medium_URL_Output:   Output medium URL                                     */
/* Appl_Id_Output:      Output application Id                                 */
/* Medium_URL_Input:    Input Medium URL                                      */
/* Appl_Id_Input:       Input Application Id                                  */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_MR_Proceed( IMRT_Medium_URL  Medium_URL_Output, IMRT_Appl_Id  Appl_Id_Output,
			     IMRT_Medium_URL  Medium_URL_Trash,
                             IMRT_Medium_URL  Medium_URL_Input,  IMRT_Appl_Id  Appl_Id_Input)
{
    IMRT_Status    status, return_status = IMRS_OK;
    IMRT_Boolean   data             = IMRD_TRUE;
    IMRT_Appl_Id   appl_id_raw;
    IMRT_Buffer   *buffer_input_ptr = NULL,        *buffer_output_ptr = NULL;
    IMRT_Medium   *medium_input_ptr = NULL,        *medium_output_ptr = NULL,   *medium_trash_ptr = NULL;
    IMRT_Tran     *tran_ptr         = NULL;


    if( sigsetjmp( IMRG_Base.SigLongJmp_Env, 1) != 0 )
    {
	IMRD_LM_LOG_TRACE_1( IMRD_LOG_LEVEL_VERBOSE_0, "Exiting on signal (%d)...", IMRG_Base.Sig_Trapped);

	Close_All( buffer_input_ptr, buffer_output_ptr, medium_input_ptr, medium_output_ptr, medium_trash_ptr);

	return( IMRS_OK);
    }

    if( IMR_SDM_Appl_Id_Get( &appl_id_raw, "raw") != IMRS_OK)
    {
	IMRD_LM_LOG_ERROR_1( "Can't get Appl_Id from [%s] !", "raw");

	return( IMRS_KO);
    }

    IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Opening input medium...");

    if( ( status = IMR_IOM_Medium_Open( &medium_input_ptr, "Input", Medium_URL_Input, Appl_Id_Input, Appl_Id_Output, IMRD_DIR_ID_IN)) != IMRS_OK)
    {
	IMRD_LM_LOG_ERROR_1( "Can't open input medium (%d) !", status);

	return_status = IMRS_KO;
    }
    else
    {
	IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Opening output medium...");

	if( ( status = IMR_IOM_Medium_Open( &medium_output_ptr, "Output", Medium_URL_Output, Appl_Id_Input, Appl_Id_Output, IMRD_DIR_ID_OUT)) != IMRS_OK)
	{
	    IMRD_LM_LOG_ERROR_1( "Can't open output medium (%d) !", status);

	    return_status = IMRS_KO;
	}
	else
	{
	    if( IMRG_MR_Base.Config.Continue_Forced == IMRD_TRUE)
	    {
		IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Opening trash medium...");

		if( ( status = IMR_IOM_Medium_Open( &medium_trash_ptr, "Trash", Medium_URL_Trash, appl_id_raw, appl_id_raw, IMRD_DIR_ID_OUT)) != IMRS_OK)
		{
		    IMRD_LM_LOG_ERROR_1( "Can't open trash medium (%d) !", status);
		    
		    return_status = IMRS_KO;
		}
	    }

	    if( return_status == IMRS_OK)
	    {
		do
		{
		    IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Allocating input buffer...");
		    
		    if( ( status = IMR_MM_Buffer_Alloc( &buffer_input_ptr)) != IMRS_OK)
		    {
			IMRD_LM_LOG_ERROR_1( "Can't allocate input buffer (%d) !", status);
			
			return_status = IMRS_KO;
		    }
		    else
		    {
			IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Begining transaction...");
			
			if( ( status = IMR_IOM_Tran_Begin( &tran_ptr, 2, medium_output_ptr, medium_input_ptr)) == IMRS_KO)
			{
			    IMRD_LM_LOG_ERROR_1( "Can't begin transaction (%d) !", status);
			    
			    return_status = IMRS_KO;
			}
			else
			{
			    IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Reading input medium...");
			    
			    if( ( status = IMR_IOM_Medium_Read( buffer_input_ptr, medium_input_ptr, IMRG_MR_Base.Config.Read_Mode)) == IMRS_KO)
			    {
				IMRD_LM_LOG_ERROR_1( "Can't read input buffer (%d) !", status);
				
				return_status = IMRS_KO;
			    }
			    else
			    {
				if( status == IMRS_NO_DATA)
				{

				    
				    
				    IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "No more data to read...");
				    
				    IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Rollbaking transaction...");
						
				    if( ( status = IMR_IOM_Tran_Rollback( tran_ptr)) == IMRS_KO)
				    {
					IMRD_LM_LOG_ERROR_1( "Can't rollback transaction (%d) !", status);
					
					return_status = IMRS_KO;
				    }
				    else
				    {
					return_status = IMRS_NO_DATA;
				    }
				}
				else
				{
				    if( status == IMRS_SIGNAL)
				    {
					IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Read interupted by a signal !");
				    
					return_status = status;
				    }
				    else
				    {
					IMR_MM_Buffer_List( buffer_input_ptr, "Input Buffer");
				    
					IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Allocating output buffer...");
				    
					if( ( status = IMR_MM_Buffer_Alloc( &buffer_output_ptr)) != IMRS_OK)
					{
					    IMRD_LM_LOG_ERROR_1( "Can't allocate output buffer (%d) !", status);
					    
					    return_status = IMRS_KO;
					}
					else
					{
					    IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Maping input buffer to output buffer...");
					    
					    if( ( status = IMR_MM_Buffer_Map( buffer_output_ptr, buffer_input_ptr)) != IMRS_OK)
					    {
						IMRD_LM_LOG_ERROR_1( "Can't map buffers (%d) !", status);
						
						if( IMRG_MR_Base.Config.Mapping_Forced == IMRD_TRUE)
						{
						    data = IMRD_TRUE;
						}
						else
						{
						    data = IMRD_FALSE;
						}
						
						if( IMRG_MR_Base.Config.Continue_Forced == IMRD_TRUE)
						{
						    IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Writing trash medium...");
						    
						    if( ( status = IMR_IOM_Medium_Write( medium_trash_ptr, buffer_input_ptr)) != IMRS_OK)
						    {
							IMRD_LM_LOG_ERROR_1( "Can't write input buffer (%d) !", status);
							
							return_status = IMRS_KO;
						    }
						}
						else
						{
						    return_status = IMRS_KO;
						}
					    }
					    else
					    {
						data = IMRD_TRUE;
					    }
					    
					    IMR_MM_Buffer_List( buffer_output_ptr, "Output Buffer");
					    
					    if( data == IMRD_TRUE)
					    {
						IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Writing output medium...");
						
						if( ( status = IMR_IOM_Medium_Write( medium_output_ptr, buffer_output_ptr)) != IMRS_OK)
						{
						    IMRD_LM_LOG_ERROR_1( "Can't write output buffer (%d) !", status);

						    return_status = IMRS_KO;
						}
					    }
					    
					    if( return_status != IMRS_OK)
					    {
						IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Rollbaking transaction...");
						
						if( ( status = IMR_IOM_Tran_Rollback( tran_ptr)) == IMRS_KO)
						{
						    IMRD_LM_LOG_ERROR_1( "Can't rollback transaction (%d) !", status);
						    
						    return_status = IMRS_KO;
						}
					    }
					    else
					    {
						IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Comiting transaction...");
						
						if( ( status = IMR_IOM_Tran_Commit( tran_ptr)) == IMRS_KO)
						{
						    IMRD_LM_LOG_ERROR_1( "Can't commit transaction (%d) !", status);
						    
						    return_status = IMRS_KO;
						}

						IMRD_LM_LOG_BUFFER_ID( buffer_input_ptr,  &IMRG_MR_Base.Config.Log_Id_Struct_Input,  IMRD_MR_LOG_ID_HEADER_MSG_INPUT);
						IMRD_LM_LOG_BUFFER_ID( buffer_output_ptr, &IMRG_MR_Base.Config.Log_Id_Struct_Output, IMRD_MR_LOG_ID_HEADER_MSG_OUTPUT);
					    }
				    
					    IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Freeing output buffer...");
					    
					    if( ( status = IMR_MM_Buffer_Free( &buffer_output_ptr)) != IMRS_OK)
					    {
						IMRD_LM_LOG_ERROR_1( "Can't free output buffer (%d) !", status);
						
						return_status = IMRS_KO;
					    }
					    
					    buffer_output_ptr = NULL;
					}
				    }
				}
			    }
			}
			
			IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Freeing input buffer...");
			
			if( ( status = IMR_MM_Buffer_Free( &buffer_input_ptr)) != IMRS_OK)
			{
			    IMRD_LM_LOG_ERROR_1( "Can't free input buffer (%d) !", status);
			    
			    return_status = IMRS_KO;
			}
			
			buffer_input_ptr = NULL;
		    }
		}
		while( ( return_status == IMRS_OK) && ( IMRG_Base.Sig_Trapped == 0));

		switch( return_status)
		{
		    case IMRS_OK:
		    case IMRS_NO_DATA:
		    case IMRS_SIGNAL:
		    {
			return_status = IMRS_OK;
			break;
		    }

		    case IMRS_KO:
		    default:
		    {
			return_status = IMRS_KO;
			break;
		    }
		}
		
		if( IMRG_MR_Base.Config.Continue_Forced == IMRD_TRUE)
		{
		    IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Closing trash medium...");
		    
		    if( ( status = IMR_IOM_Medium_Close( medium_trash_ptr)) != IMRS_OK)
		    {
			IMRD_LM_LOG_ERROR_1( "Can't close trash medium (%d) !", status);
			
			return_status = IMRS_KO;
		    }
		
		    medium_trash_ptr = NULL;
		}
	    }

	    IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Closing output medium...");
		    
	    if( ( status = IMR_IOM_Medium_Close( medium_output_ptr)) != IMRS_OK)
	    {
		IMRD_LM_LOG_ERROR_1( "Can't close output medium (%d) !", status);

		return_status = IMRS_KO;
	    }

	    medium_output_ptr = NULL;
	}
		
	IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Closing input medium...");
		
	if( ( status = IMR_IOM_Medium_Close( medium_input_ptr)) != IMRS_OK)
	{
	    IMRD_LM_LOG_ERROR_1( "Can't close input medium (%d) !", status);

	    return_status = IMRS_KO;
	}

	medium_input_ptr = NULL;
    }

    return( return_status);
}


