/*----------------------------------------------------------------------------*/
/* File: static_data_mng.h 				    		      */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#ifndef _STATIC_DATA_MNG_H_
#define _STATIC_DATA_MNG_H_



#include <imr.h>





/*----------------------------------------------------------------------------*/
/* IMRD_API definition                                                        */
/*----------------------------------------------------------------------------*/

#   ifdef _LIBSTATIC_DATA_MNG_C_
#       define IMRD_SDM_API  
#   else
#       define IMRD_SDM_API  extern
#   endif




/*----------------------------------------------------------------------------*/
/* IMRT_SDM_Config                                                            */
/*----------------------------------------------------------------------------*/

typedef short  IMRT_SDM_Flag;

#define IMRD_SDM_FLAG_UNKNOWN  ( IMRT_SDM_Flag) 0
#define IMRD_SDM_FLAG_XML      ( IMRT_SDM_Flag) 1
#define IMRD_SDM_FLAG_SQL      ( IMRT_SDM_Flag) 2



typedef struct
{
	IMRT_SDM_Flag  Flag;
	IMRT_Path      Lib_Path;
}  IMRT_SDM_Config;





/*----------------------------------------------------------------------------*/
/* IMRT_MsgMap_Id definition                                                */
/*----------------------------------------------------------------------------*/

typedef int  IMRT_MsgMap_Id;

#define IMRD_MSGMAP_ID_UNKNOWN  ( IMRT_MsgMap_Id) 0




/*----------------------------------------------------------------------------*/
/* IMRT_MsgMap definition                                                     */
/*----------------------------------------------------------------------------*/

typedef struct IMRT_MsgMap
{
	IMRT_MsgMap_Id  Id;
	char            Name[IMRD_NAME_SIZE];
	IMRT_Msg       *Msg_Input_Ptr;
	IMRT_Msg       *Msg_Output_Ptr;
	NDT_Root       *FieldMap_Struct_Ptr;
}  IMRT_MsgMap;





/*----------------------------------------------------------------------------*/
/* IMRT_Jump definition                                                       */
/*----------------------------------------------------------------------------*/

typedef struct IMRT_Jump
{
	IMRT_Fmt  *Fmt_Ptr;
	IMRT_Msg  *Msg_Ptr;
	int        Branche_Id;
}  IMRT_Jump;





/*----------------------------------------------------------------------------*/
/* IMRT_MapCmd                                                                */
/*----------------------------------------------------------------------------*/

/*#define IMRD_JUMP_NB_MAX   ( short) 40*/



typedef short  IMRT_MapCmd_Id;



typedef short  IMRT_MapCmd_Type;

#define IMRD_MAPCMD_TYPE_UNKNOWN  ( IMRT_MapCmd_Type) 0
#define IMRD_MAPCMD_TYPE_MAP      ( IMRT_MapCmd_Type) 1
#define IMRD_MAPCMD_TYPE_COPY     ( IMRT_MapCmd_Type) 2
#define IMRD_MAPCMD_TYPE_IDENT    ( IMRT_MapCmd_Type) 3
#define IMRD_MAPCMD_TYPE_GOTO     ( IMRT_MapCmd_Type) 4



typedef short  IMRT_MapCmd_Branch_Id;



typedef short  IMRT_MapCmd_Flag;



typedef struct IMRT_MapCmd
{
	IMRT_MapCmd_Id          MapCmd_Id;
	IMRT_MapCmd_Type        TypeCmd;
	IMRT_MapCmd_Branch_Id   Branche_Id; 
	IMRT_Boolean            Ident_Total;
	IMRT_Msg               *Ident_First_Msg_Ptr;
	IMRT_Fmt               *Fmt_Input_Ptr;
	IMRT_FmtAttr           *FmtAttr_Input_Ptr;
	IMRT_Fmt               *Fmt_Output_Ptr;
	IMRT_FmtAttr           *FmtAttr_Output_Ptr;
	IMRT_Buffer_Id          Buffer_Id_Input;
	IMRT_Buffer_Id          Buffer_Id_Output;
	IMRT_Buffer_Id          Buffer_Id_Ident;
	IMRT_MapCmd_Flag        Clear_Flag;
	IMRT_MsgMap            *MsgMap_Ptr;
	IMRT_MapCmd_Branch_Id   Goto_Branche_Id; 
	int						JmpTbleNbElts;
	IMRT_Jump               *Jump_Tab;
}  IMRT_MapCmd;







/*----------------------------------------------------------------------------*/
/* Global Variables                                                           */
/*----------------------------------------------------------------------------*/

#   ifdef _LIBSTATIC_DATA_MNG_

#   else

extern IMRT_Module  IMRG_SDM_Module;

#   endif







/******************************************************************************/
/* Static Data Manager Client CallBack                                        */
/******************************************************************************/



/*----------------------------------------------------------------------------*/
/* Format Add CallBack                                                        */
/*----------------------------------------------------------------------------*/
/* (I) Fmt_Id:      Format id                                                 */
/* (I) Fmt_Name:    Format name                                               */
/* (I) Fmt_Id_Nb:   Total number of format                                    */
/*----------------------------------------------------------------------------*/

typedef IMRT_Status  IMRT_SDMCB_Fmt_Add( IMRT_Fmt_Id, IMRT_Fmt_Name, IMRT_Fmt_Id);



/*----------------------------------------------------------------------------*/
/* Convert Add CallBack                                                       */
/*----------------------------------------------------------------------------*/
/* (I) Cnv_Id:      Convert id                                                */
/* (I) Cnv_Name:    Convert name                                              */
/* (I) Cnv_Id_Nb:   Total number of convert                                   */
/*----------------------------------------------------------------------------*/

typedef IMRT_Status  IMRT_SDMCB_Cnv_Add( IMRT_Cnv_Id, IMRT_Cnv_Name, IMRT_Cnv_Id);



/*----------------------------------------------------------------------------*/
/* Buffer Mapping CallBack                                                    */
/*----------------------------------------------------------------------------*/
/* (I) Buffer_Id_Target:   Target buffer id                                   */
/* (I) Fmt_Target:         Target format                                      */
/* (I) Fmt_Attr_Target:    Target format attribute                            */
/* (I) Buffer_Id_Src:      Source buffer id                                   */
/* (I) Fmt_Src:            Source format                                      */
/* (I) Fmt_Attr_Src:       Source format attribute                            */
/* (I) MsgMap:             Message Map to use                                 */
/* (I) Clear_Flag:         Clear or donn't clear output buffer                */
/*----------------------------------------------------------------------------*/

typedef IMRT_Status  IMRT_SDMCB_Buffer_Map( IMRT_Buffer_Id, IMRT_Fmt_Id, IMRT_FmtAttr_Value, IMRT_Buffer_Id, IMRT_Fmt_Id, IMRT_FmtAttr_Value, IMRT_MsgMap  *, IMRT_MapCmd_Flag);



/*----------------------------------------------------------------------------*/
/* Buffer Copy CallBack                                                       */
/*----------------------------------------------------------------------------*/
/* (I) Buffer_Id_Target:   Target buffer id                                   */
/* (I) Buffer_Id_Src:      Source buffer id                                   */
/*----------------------------------------------------------------------------*/

typedef IMRT_Status  IMRT_SDMCB_Buffer_Copy( IMRT_Buffer_Id, IMRT_Buffer_Id);



/*----------------------------------------------------------------------------*/
/* Buffer Identification CallBack                                             */
/*----------------------------------------------------------------------------*/
/* (O)   Fmt_Id_Ptr:      Buffer format to return                             */
/* (I/O) Msg_Id_Ptr:      Buffer message id to read (if not total) and return */
/* (I)   Buffer_Id_Ptr:   Id of the buffer to identify                        */
/* (I)   Ident_Total:     True if total identification                        */
/*----------------------------------------------------------------------------*/

typedef IMRT_Status  IMRT_SDMCB_Buffer_Ident( IMRT_Fmt_Id  *, IMRT_Msg_Id  *, IMRT_Buffer_Id, IMRT_Boolean);



/*----------------------------------------------------------------------------*/
/* Field Mapping CallBack                                                     */
/*----------------------------------------------------------------------------*/
/* (I) Field_Target_Ptr:   Target Field pointer                               */
/* (I) Field_Source_Ptr:   Source Field pointer                               */
/*----------------------------------------------------------------------------*/

typedef IMRT_Status  IMRT_SDMCB_Field_Map( IMRT_Field  *, IMRT_Field  *);



/*----------------------------------------------------------------------------*/
/* Field Ident CallBack                                                       */
/*----------------------------------------------------------------------------*/
/* (I) FieldMap_Ptr:   FielMap pointer                                        */
/*----------------------------------------------------------------------------*/

typedef IMRT_Status  IMRT_SDMCB_Ident_Field_Value( IMRT_Field  *, IMRT_Field_Value_Id);


typedef IMRT_Status  IMRT_SDMCB_Ident_Field_Exist( IMRT_Field *);


typedef IMRT_Status  IMRT_SDMCB_Ident_Msg_Size( size_t);





/******************************************************************************/
/* Static Data Manager API                                                    */
/******************************************************************************/



/*----------------------------------------------------------------------------*/
/* Open Static Data Manager                                                   */
/*----------------------------------------------------------------------------*/
/* Lib_Flag:   Setup SDM init mode (XML, SQL, ...)                            */
/*----------------------------------------------------------------------------*/

IMRD_SDM_API IMRT_Status  IMR_SDM_Library_Open( IMRT_SDM_Config  *);



/*----------------------------------------------------------------------------*/
/* Close Static Data Manager                                                  */
/*----------------------------------------------------------------------------*/

IMRD_SDM_API IMRT_Status  IMR_SDM_Library_Close( void);



/*----------------------------------------------------------------------------*/
/* Dump Data Structures to SQL files                                          */
/*----------------------------------------------------------------------------*/
IMRD_SDM_API IMRT_Status  IMR_SDM_SQL_Dump(void);



/*----------------------------------------------------------------------------*/
/* Dump Data Structures to XML files                                          */
/*----------------------------------------------------------------------------*/
IMRD_SDM_API IMRT_Status  IMR_SDM_XML_Dump(void);




/*----------------------------------------------------------------------------*/
/* Proceed Format                                                             */
/*----------------------------------------------------------------------------*/
/* (I) CCB_Fmt_Add_Ptr:   Format add callback                                 */
/*----------------------------------------------------------------------------*/

IMRD_SDM_API IMRT_Status  IMR_SDM_Fmt_Proceed( IMRT_SDMCB_Fmt_Add  *);



/*----------------------------------------------------------------------------*/
/* Proceed Convert                                                            */
/*----------------------------------------------------------------------------*/
/* (I) CCB_Cnv_Add_Ptr:   Convert add callback                                */
/*----------------------------------------------------------------------------*/
IMRD_SDM_API IMRT_Status  IMR_SDM_Cnv_Proceed( IMRT_SDMCB_Cnv_Add  *);



/*----------------------------------------------------------------------------*/
/* Proceed Convert                                                            */
/*----------------------------------------------------------------------------*/
/* (I) CCB_Cnv_Add_Ptr:   Convert add callback                                */
/*----------------------------------------------------------------------------*/

IMRD_SDM_API IMRT_Status  IMR_SDM_Cnv_Proceed( IMRT_SDMCB_Cnv_Add  *);



/*----------------------------------------------------------------------------*/
/* Proceed Mapping Path                                                       */
/*----------------------------------------------------------------------------*/
/* (I) Appl_Id_Target:         Target buffer application id                   */
/* (I) Fmt_Id_Target:          Target buffer format id                        */
/* (I) Msg_Id_Target:          Target buffer message id                       */
/* (I) ApplId_Src:             Source buffer application id                   */
/* (I) Fmt_Id_Src:             Source buffer format id                        */
/* (I) Msg_Id_Src:             Source buffer message id                       */
/* (I) CCB_Buffer_Map_Ptr:     Buffer mapping callback                        */
/* (I) CCB_Buffer_Copy_Ptr:    Buffer copy callback                           */
/* (I) CCB_Buffer_Ident_Ptr:   Buffer identification callback                 */
/*----------------------------------------------------------------------------*/

IMRD_SDM_API IMRT_Status  IMR_SDM_MapPath_Proceed( IMRT_Appl_Id, IMRT_Fmt_Id, IMRT_Msg_Id,
					       IMRT_Appl_Id, IMRT_Fmt_Id, IMRT_Msg_Id,
					       IMRT_SDMCB_Buffer_Map  *, IMRT_SDMCB_Buffer_Copy  *, IMRT_SDMCB_Buffer_Ident  *);



/*----------------------------------------------------------------------------*/
/* Proceed Buffer Mapping                                                     */
/*----------------------------------------------------------------------------*/
/* (I) MsgMap_Ptr:          Message map pointer                               */
/* (I) CCB_Field_Map_Ptr:   Field mapping callback                            */
/*----------------------------------------------------------------------------*/

IMRD_SDM_API IMRT_Status  IMR_SDM_MsgMap_Proceed( IMRT_MsgMap  *, IMRT_SDMCB_Field_Map  *);



/*----------------------------------------------------------------------------*/
/* Proceed Message Identification                                             */
/*----------------------------------------------------------------------------*/
/* (I) Fmt_Id:                Target buffer format id                         */
/* (I) Msg_Id:                Target buffer message id                        */
/* (I) CCB_Field_Ident_Ptr:   Field ident callback                            */
/*----------------------------------------------------------------------------*/

IMRD_SDM_API IMRT_Status  IMR_SDM_MsgIdent_Proceed( IMRT_Fmt_Id, IMRT_Msg_Id  *, IMRT_SDMCB_Ident_Field_Value  *, IMRT_SDMCB_Ident_Field_Exist *, IMRT_SDMCB_Ident_Msg_Size *);



/*----------------------------------------------------------------------------*/
/* Convert Buffer Format Name to Buffer Format Id                             */
/*----------------------------------------------------------------------------*/

IMRD_SDM_API IMRT_Status  IMR_SDM_Fmt_Id_Get( IMRT_Fmt_Id  *, const IMRT_Fmt_Name);



/*----------------------------------------------------------------------------*/
/* Convert Buffer Appli Name to Buffer Appli Id                               */
/*----------------------------------------------------------------------------*/
IMRD_SDM_API IMRT_Status  IMR_SDM_Appl_Id_Get( IMRT_Appl_Id  *, const IMRT_Appl_Name);



#endif
