/*----------------------------------------------------------------------------*/
/* File: buffer_info_xml.h 				         	      */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#ifndef _BUFFER_INFO_XML_H_
#   define _BUFFER_INFO_XML_H_



/*----------------------------------------------------------------------------*/
/* Global Include                                                             */
/*----------------------------------------------------------------------------*/

#   include <buffer_info_mng.h>

/*----------------------------------------------------------------------------*/
/* Locale Constantes                                                          */
/*----------------------------------------------------------------------------*/

#define IMRD_SDMD_TAG_FMTATTR        "fmtattr"
#define IMRD_SDMD_TAG_FMTATTR_NAME   "fmtattr_name"


/*definition des tags xml pour le chargement des structure Format et FmtMsg*/
#define IMRD_SDMD_TAG_FORMAT  "format"
#define IMRD_SDMD_TAG_FMTMSG  "fmtmsg"

/*definition des tags xml pour le chargement de la structure Application*/

#define IMRD_SDMD_TAG_APPLI        "appli"

/*definition des tags xml pour le chargement de les structures MapPath et MapCmd*/
#define IMRD_SDMD_TAG_MAPPATH            "mappath"
#define IMRD_SDMD_TAG_INPUTAPPLI         "inputappli"
#define IMRD_SDMD_TAG_OUTPUTAPPLI        "outputappli"
#define IMRD_SDMD_TAG_INPUTFORMAT        "inputformat"
#define IMRD_SDMD_TAG_INPUTFORMAT_ATTR   "inputformat_attr"
#define IMRD_SDMD_TAG_OUTPUTFORMAT       "outputformat"
#define IMRD_SDMD_TAG_OUTPUTFORMAT_ATTR  "outputformat_attr"
#define IMRD_SDMD_TAG_MAPCMD             "mapcmd"
#define IMRD_SDMD_TAG_BRANCHE_ID         "branche_id"
#define IMRD_SDMD_TAG_MAPPER             "mapper"
#define IMRD_SDMD_TAG_CLEARFLAG          "clearflag"
#define IMRD_SDMD_TAG_INPUTBUFFER        "inputbuffer"
#define IMRD_SDMD_TAG_OUTPUTBUFFER       "outputbuffer"
#define IMRD_SDMD_TAG_IDENTIFIER         "identifier"
#define IMRD_SDMD_TAG_NUMBUFFER          "numbuffer"
#define IMRD_SDMD_TAG_GO_TO              "go_to"
#define IMRD_SDMD_TAG_TYPE               "type"
#define IMRD_SDMD_TAG_COPIE              "copie"
#define IMRD_SDMD_TAG_IDENTTOTAL         "identtotal"
#define IMRD_SDMD_TAG_ATTRIBUTE          "attribute"
#define IMRD_SDMD_TAG_VALUE              "value"


/*----------------------------------------------------------------------------*/
/* Local definition                                                           */
/*----------------------------------------------------------------------------*/


typedef IMRT_Status (* xml_endeElement)(void * ctx, const xmlChar * localname, const xmlChar * prefix, const xmlChar * URI);

typedef struct SDMT_BufferInfo_Contexte
{
	char value[IMRD_SDMD_VALUE_LEN];
    IMRT_Array_Xml_Path PathArray;
    IMRT_Level_Tag PresentLevelTag;
	int  IdxJpTble;
	int  NbMapPath;
	int  NbMapCmd;
	IMRT_Appl_Id     CptAppl;
	IMRT_FmtAttr_Id  CptFmtAttr;
	IMRT_Fmt_Id      CptFmt;
	IMRT_MapPath   * CurrentMapPath;
	IMRT_MapCmd    * CurrentMapCmd;
	IMRT_Appl      * CurrentAppli;
	IMRT_FmtMsg    * CurrentFmtMsg;
	IMRT_Fmt       * CurrentFormat;
	IMRT_FmtAttr   * CurrentFmtAttr;
	NDT_Root       * RootFmtAttr;
	NDT_Root       * RootFormat;
	NDT_Root       * RootAppli;
	NDT_Root       * RootMapPath;
	xmlParserCtxt  * SaxCtxHdl; 
	IMRT_Error     FataError;
	IMRT_Status    flagerror;
}SDMT_BufferInfo_Contexte;


/*----------------------------------------------------------------------------*/
/* Prototypes                                                                 */
/*----------------------------------------------------------------------------*/

#   ifdef _BUFFER_INFO_XML_C_

static IMRT_Status error(SDMT_BufferInfo_Contexte *ctx , const char *msg, ...);

IMRT_Status startElementNsFmtAttr( SDMT_BufferInfo_Contexte * ctx , 
							            const xmlChar *localname,
							            const xmlChar *prefix,
							            const xmlChar *URI,
							            int nb_namespaces,
							            const xmlChar **namespaces,
							            int nb_attributes,
							            int nb_defaulted,
							            const xmlChar **attributes);

IMRT_Status startElementNsAppli( SDMT_BufferInfo_Contexte * ctx , 
							            const xmlChar *localname,
							            const xmlChar *prefix,
							            const xmlChar *URI,
							            int nb_namespaces,
							            const xmlChar **namespaces,
							            int nb_attributes,
							            int nb_defaulted,
							            const xmlChar **attributes);

IMRT_Status startElementNsFmt( SDMT_BufferInfo_Contexte * ctx , 
							          const xmlChar *localname,
							          const xmlChar *prefix,
							          const xmlChar *URI,
							          int nb_namespaces,
							          const xmlChar **namespaces,
							          int nb_attributes,
							          int nb_defaulted,
							          const xmlChar **attributes);

IMRT_Status startElementNsMapPath( SDMT_BufferInfo_Contexte * ctx , 
							              const xmlChar *localname,
							              const xmlChar *prefix,
							              const xmlChar *URI,
							              int nb_namespaces,
							              const xmlChar **namespaces,
							              int nb_attributes,
							              int nb_defaulted,
							              const xmlChar **attributes);

IMRT_Status endElementNsAppli( SDMT_BufferInfo_Contexte * ctx, 
							          const xmlChar *localname,
							          const xmlChar *prefix,
							          const xmlChar *URI);

IMRT_Status endElementNsFmtAttr( SDMT_BufferInfo_Contexte * ctx, 
							          const xmlChar *localname,
							          const xmlChar *prefix,
							          const xmlChar *URI);

IMRT_Status endElementNsFmt ( SDMT_BufferInfo_Contexte * ctx, 
							         const xmlChar *localname,
							         const xmlChar *prefix,
							         const xmlChar *URI);

IMRT_Status endElementNsMapPath( SDMT_BufferInfo_Contexte * ctx, 
							            const xmlChar *localname,
							            const xmlChar *prefix,
							            const xmlChar *URI);
							            
IMRT_Status Buff_Info_characters(SDMT_BufferInfo_Contexte * ctx , const xmlChar *ch, int len);

IMRT_Status  SDM_XML_Load_FmtAttr(   char *demo_file, SDMT_BufferInfo_Contexte *mycontexte);
IMRT_Status  SDM_XML_Load_Appli(   char *demo_file, SDMT_BufferInfo_Contexte *mycontexte);
IMRT_Status  SDM_XML_Load_Format(  char *demo_file, SDMT_BufferInfo_Contexte *mycontexte);
IMRT_Status  SDM_XML_Load_MapPath( char *demo_file, SDMT_BufferInfo_Contexte *mycontexte);
IMRT_Status  SDM_XML_BufferInfo_Init(IMRT_Path    Lib_Path);
IMRT_Status  SDM_Dump_XML_FmtAttr_Header(FILE *dump_file);
IMRT_Status  SDM_Dump_XML_FmtAttr_Body(FILE *dump_file);
IMRT_Status  SDM_Dump_XML_FmtAttr_Footer(FILE *dump_file);
IMRT_Status  SDM_Dump_XML_FmtAttr(void);
IMRT_Status  SDM_Dump_XML_Appli_Header(FILE *dump_file);
IMRT_Status  SDM_Dump_XML_Appli_Body(FILE *dump_file);
IMRT_Status  SDM_Dump_XML_Appli_Footer(FILE *dump_file);
IMRT_Status  SDM_Dump_XML_Appli(void);
IMRT_Status  SDM_Dump_XML_Format_Header(FILE *dump_file);
IMRT_Status  SDM_Dump_XML_Format_Body(FILE *dump_file);
IMRT_Status  SDM_Dump_XML_Format_Footer(FILE *dump_file);
IMRT_Status  SDM_XML_Dump_Format(void);
IMRT_Status  SDM_XML_Dump_MapPath_Header(FILE *dump_file);
IMRT_Status  SDM_XML_Dump_MapPath_Body(FILE *dump_file);
IMRT_Status  SDM_XML_Dump_MapPath_Footer(FILE *dump_file);
IMRT_Status  SDM_XML_Dump_MapPath(void);
IMRT_Status  SDM_XML_BufferInfo_Dump(void);
IMRT_Status  SDM_ConvertAttribute(char * Attrinute_ptr, char * Attribute_convert_ptr);
IMRT_Status  SDM_ConvertAttributeInv(char * Attrinute_Ptr, char * Attribute_convert_ptr);
IMRT_Status  SDM_Control_Fmt_Attr(IMRT_Msg *, NDT_Root *, IMRT_FmtAttr *);
#   else

extern  IMRT_Status  SDM_XML_Load_FmtAttr(   char *demo_file, SDMT_BufferInfo_Contexte *mycontexte);
extern  IMRT_Status  SDM_XML_Load_Appli( char *demo_file, SDMT_BufferInfo_Contexte *mycontexte);
extern  IMRT_Status  SDM_XML_Load_Format( char *demo_file, SDMT_BufferInfo_Contexte *mycontexte);
extern  IMRT_Status  SDM_XML_Load_MapPath( char *demo_file, SDMT_BufferInfo_Contexte *mycontexte);
extern  IMRT_Status  SDM_XML_BufferInfo_Init(IMRT_Path    Lib_Path);
extern  IMRT_Status  SDM_Dump_XML_FmtAttr_Header(FILE *dump_file);
extern  IMRT_Status  SDM_Dump_XML_FmtAttr_Body(FILE *dump_file);
extern  IMRT_Status  SDM_Dump_XML_FmtAttr_Footer(FILE *dump_file);
extern  IMRT_Status  SDM_Dump_XML_FmtAttr(void);
extern  IMRT_Status  SDM_Dump_XML_Appli_Header(FILE *dump_file);
extern  IMRT_Status  SDM_Dump_XML_Appli_Body(FILE *dump_file);
extern  IMRT_Status  SDM_Dump_XML_Appli_Footer(FILE *dump_file);
extern  IMRT_Status  SDM_Dump_XML_Appli(void);
extern  IMRT_Status  SDM_Dump_XML_Format_Header(FILE *dump_file);
extern  IMRT_Status  SDM_Dump_XML_Format_Body(FILE *dump_file);
extern  IMRT_Status  SDM_Dump_XML_Format_Footer(FILE *dump_file);
extern  IMRT_Status  SDM_XML_Dump_Format(void);
extern  IMRT_Status  SDM_XML_Dump_MapPath_Header(FILE *dump_file);
extern  IMRT_Status  SDM_XML_Dump_MapPath_Body(FILE *dump_file);
extern  IMRT_Status  SDM_XML_Dump_MapPath_Footer(FILE *dump_file);
extern  IMRT_Status  SDM_XML_Dump_MapPath(void);
extern  IMRT_Status  SDM_XML_BufferInfo_Dump(void);
extern  IMRT_Status  SDM_Control_Fmt_Attr(IMRT_Msg *, NDT_Root *, IMRT_FmtAttr *);
#   endif



/*----------------------------------------------------------------------------*/
/* Global Variables                                                           */
/*----------------------------------------------------------------------------*/
#   ifdef _BUFFER_INFO_XML_C_

/*xmlSAXHandler Buff_Info_Error_SAX_Hdl_Struct = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    1,
    NULL,
    NULL,
    NULL,
    NULL
    };*/
xmlSAXHandler SaxFmtHandlerStruct = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    (charactersSAXFunc)(Buff_Info_characters),
    NULL,
    NULL,
    NULL,
    NULL,
    (errorSAXFunc)(error),
    NULL,
    NULL,
    NULL,
    NULL,
    1,
    NULL,
    (startElementNsSAX2Func)(startElementNsFmt),
    (endElementNsSAX2Func)(endElementNsFmt),
    NULL
};


xmlSAXHandler SaxAppliHandlerStruct = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    (charactersSAXFunc)(Buff_Info_characters),
    NULL,
    NULL,
    NULL,
    NULL,
    (errorSAXFunc)error,
    NULL,
    NULL,
    NULL,
    NULL,
    1,
    NULL,
    (startElementNsSAX2Func)(startElementNsAppli),
    (endElementNsSAX2Func)(endElementNsAppli),
    NULL
};

xmlSAXHandler SaxFmtAttrHandlerStruct = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    (charactersSAXFunc)(Buff_Info_characters),
    NULL,
    NULL,
    NULL,
    NULL,
    (errorSAXFunc)error,
    NULL,
    NULL,
    NULL,
    NULL,
    1,
    NULL,
    (startElementNsSAX2Func)(startElementNsFmtAttr),
    (endElementNsSAX2Func)(endElementNsFmtAttr),
    NULL
};


xmlSAXHandler SaxMapPathHandlerStruct = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    (charactersSAXFunc)(Buff_Info_characters),
    NULL,
    NULL,
    NULL,
    NULL,
    (errorSAXFunc)error,
    NULL,
    NULL,
    NULL,
    NULL,
    1,
    NULL,
    (startElementNsSAX2Func)(startElementNsMapPath),
    (endElementNsSAX2Func)(endElementNsMapPath),
    NULL
};




#   endif

/*----------------------------------------------------------------------------*/

#endif
