/*----------------------------------------------------------------------------*/
/* $Workfile:   libstatic_data_mng.c  $                                       */
/*----------------------------------------------------------------------------*/
/* $Author: agibert $                                                      */
/* $Date: 2008/11/12 02:25:23 $                                           */
/* $Revision: 1.1 $                                                       */
/* $Label:  $                                                         */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/* File: libstatic_data_mng.c   				  	      */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#define _LIBSTATIC_DATA_MNG_
#define _LIBSTATIC_DATA_MNG_C_

#include "libstatic_data_mng.h"







/******************************************************************************/
/* Static Data Manager API                                                    */
/******************************************************************************/

/*----------------------------------------------------------------------------*/
/* Open Static Data Manager                                                   */
/*----------------------------------------------------------------------------*/
/* Lib_Flag:   Setup SDM init mode (XML, SQL, ...)                            */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_SDM_Library_Open( IMRT_SDM_Config * Lib_Config_Ptr)
{
  IMRT_Status  status, StatusTmp;
  NDT_Status   nd_status;


 
    if( IMRG_SDM_Module.Lib.Status != IMRD_LIB_STATUS_CLOSED)
    {
	IMRD_LM_LOG_ERROR_0( "Library not closed !");

	return( IMRS_KO);
    }
    else
    {
#ifdef ORACLE_SUPPORT
	if(( (status = SDM_HDR_SQL_Init(Lib_Config_Ptr->Flag)) != IMRS_OK)||( (status = SDM_HDR_XML_Init(Lib_Config_Ptr->Flag)) != IMRS_OK))
	{
	    status = IMRS_KO;
	}
#else
	    status = IMRS_OK;
#endif /* ORACLE_SUPPORT */

	if( status == IMRS_OK )
	{
	    status = SDM_StructInfo_Init(Lib_Config_Ptr);
	    if( status == IMRS_OK )
	    {
		status = SDM_MapInfo_Init(Lib_Config_Ptr);
		if( status == IMRS_OK )
		{
		    status = SDM_BufferInfo_Init(Lib_Config_Ptr);
		    if( status != IMRS_OK )
		    {
			StatusTmp = SDM_MapInfo_DInit();
			StatusTmp = SDM_StructInfo_DInit();
		    }
		}
		else
		{
		    StatusTmp = SDM_StructInfo_DInit();
		}
	    }

	}

	if( status == IMRS_OK)
	{
	    IMRG_SDM_Module.Lib.Status = IMRD_LIB_STATUS_OPENED;

	    IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_1, "Library opened !");
	}

	return(status);
    }
}





/*----------------------------------------------------------------------------*/
/* Close Static Data Manager                                                  */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_SDM_Library_Close( void)
{
  IMRT_Status  status;


    if( IMRG_SDM_Module.Lib.Status != IMRD_LIB_STATUS_OPENED)
    {
	IMRD_LM_LOG_ERROR_0( "Library not opened !");

	return( IMRS_KO);
    }
    else
    {
	status = IMRS_OK;

	if( ((status = SDM_BufferInfo_DInit()) != IMRS_OK)||
	    ((status = SDM_MapInfo_DInit())    != IMRS_OK)||
	    ((status = SDM_StructInfo_DInit()) != IMRS_OK)||

#ifdef ORACLE_SUPPORT
	    ((status = SDM_HDR_SQL_DInit())    != IMRS_OK)||
#endif /* ORACLE_SUPPORT */

	    ((status = SDM_HDR_XML_DInit())    != IMRS_OK) )
	{
	    status = IMRS_KO;
	}
	else
	{
	    IMRG_SDM_Module.Lib.Status = IMRD_LIB_STATUS_CLOSED;

	    IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_1, "Library closed !");
	}

	return(status);
    }
}





/*----------------------------------------------------------------------------*/
/* Proceed Format                                                             */
/*----------------------------------------------------------------------------*/
/* (I) CCB_Fmt_Add_Ptr:   Format add callback                                 */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_SDM_Fmt_Proceed( IMRT_SDMCB_Fmt_Add  *CCB_Fmt_Add_Ptr)
{
    IMRT_Status status;

    status = SDM_Fmt_Proceed(CCB_Fmt_Add_Ptr); 

    return(status);
}





/*----------------------------------------------------------------------------*/
/* Proceed Convert                                                            */
/*----------------------------------------------------------------------------*/
/* (I) CCB_Cnv_Add_Ptr:   Convert add callback                                */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_SDM_Cnv_Proceed( IMRT_SDMCB_Cnv_Add  *CCB_Cnv_Add_Ptr)
{
    IMRT_Status status;

    status = SDM_Cnv_Proceed(CCB_Cnv_Add_Ptr); 

    return(status);
}





/*----------------------------------------------------------------------------*/
/* Proceed Mapping Path                                                       */
/*----------------------------------------------------------------------------*/
/* (I) Appl_Id_Target:         Target buffer application id                   */
/* (I) Fmt_Id_Target:          Target buffer format id                        */
/* (I) Msg_Id_Target:          Target buffer message id                       */
/* (I) ApplId_Src:             Source buffer application id                   */
/* (I) Fmt_Id_Src:             Source buffer format id                        */
/* (I) Msg_Id_Src:             Source buffer message id                       */
/* (I) CCB_Buffer_Map_Ptr:     Buffer mapping callback                        */
/* (I) CCB_Buffer_Copy_Ptr:    Buffer copy callback                           */
/* (I) CCB_Buffer_Ident_Ptr:   Buffer identification callback                 */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_SDM_MapPath_Proceed( IMRT_Appl_Id  Appl_Id_Target, IMRT_Fmt_Id  Fmt_Id_Target, IMRT_Msg_Id  Msg_Id_Target,
				      IMRT_Appl_Id  ApplId_Src,     IMRT_Fmt_Id  Fmt_Id_Src,    IMRT_Msg_Id  Msg_Id_Src,
				      IMRT_SDMCB_Buffer_Map    *CCB_Buffer_Map_Ptr,
				      IMRT_SDMCB_Buffer_Copy   *CCB_Buffer_Copy_Ptr,
				      IMRT_SDMCB_Buffer_Ident  *CCB_Buffer_Ident_Ptr)
{
    IMRT_Status status;

    status = SDM_MapPath_Proceed(Appl_Id_Target,Fmt_Id_Target,Msg_Id_Target,
				 ApplId_Src,Fmt_Id_Src, Msg_Id_Src,
				 CCB_Buffer_Map_Ptr,
				 CCB_Buffer_Copy_Ptr,
				 CCB_Buffer_Ident_Ptr); 
  
}





/*----------------------------------------------------------------------------*/
/* Proceed Buffer Mapping                                                     */
/*----------------------------------------------------------------------------*/
/* (I) MsgMap_Ptr:          Message map pointer                               */
/* (I) CCB_Field_Map_Ptr:   Field mapping callback                            */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_SDM_MsgMap_Proceed( IMRT_MsgMap  *MsgMap_Ptr, IMRT_SDMCB_Field_Map  *CCB_Field_Map_Ptr)
{

    IMRT_Status status;

    status = IMRS_OK;
    
    if(SDM_MsgMap_Proceed(MsgMap_Ptr,CCB_Field_Map_Ptr) != NDS_OK)
    {
	status = IMRS_KO;
    }
    
    return(status);
   
}





/*----------------------------------------------------------------------------*/
/* Proceed Message Identification                                             */
/*----------------------------------------------------------------------------*/
/* (I) Fmt_Id:                Target buffer format id                         */
/* (O) Msg_Id_Ptr:            Target buffer message id to return              */
/* (I) CCB_Field_Ident_Ptr:   Field ident callback                            */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_SDM_MsgIdent_Proceed( IMRT_Fmt_Id  Fmt_Id, IMRT_Msg_Id  *Msg_Id, IMRT_SDMCB_Ident_Field_Value  *CCB_Ident_Field_Value_Ptr, 
									   IMRT_SDMCB_Ident_Field_Exist  *CCB_Ident_Field_Exist_Ptr, IMRT_SDMCB_Ident_Msg_Size  *CCB_Ident_Msg_Size_Ptr)
{
    IMRT_Status status;

    status = SDM_MsgIdent_Proceed( Fmt_Id, Msg_Id, CCB_Ident_Field_Value_Ptr, CCB_Ident_Field_Exist_Ptr, CCB_Ident_Msg_Size_Ptr);
    
}





/*----------------------------------------------------------------------------*/
/* Convert Buffer Format Name to Buffer Format Id                             */
/*----------------------------------------------------------------------------*/
/* (O) Fmt_Id_Ptr:   Buffer format id                                         */
/* (I) Fmt_Name:     Buffer format name                                       */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_SDM_Fmt_Id_Get( IMRT_Fmt_Id  *Fmt_Id_Ptr, const IMRT_Fmt_Name  Fmt_Name)
{
    IMRT_Status  status;
    NDT_Status   nd_status;
    NDT_Node    *node_ptr;
    IMRT_Fmt     Fmt;


	strcpy(Fmt.Name, Fmt_Name);

	if(((nd_status = ND_Index_Node_Find( &node_ptr, IMRG_SDM_Base.RootFmt,
					 IMRD_SDMD_FMT_IDX_NAME,&Fmt,NULL)) != NDS_OK)||
		(node_ptr == NULL))
	{
	    printf("\nErreur lors de la recherche de Format Name <%s>\n", Fmt_Name);
	    status = IMRS_KO;
	}
	else
	{
		*Fmt_Id_Ptr = ( ( IMRT_Fmt *)( node_ptr->Value))->Id;
		status = IMRS_OK;		
	}

	return(status);	
}





/*----------------------------------------------------------------------------*/
/* Convert Buffer Appli Name to Buffer Appli Id                               */
/*----------------------------------------------------------------------------*/
/* (O) Appl_Id_Ptr:   Buffer appli id                                         */
/* (I) Appl_Name:     Buffer appli name                                       */
/*----------------------------------------------------------------------------*/

IMRT_Status  IMR_SDM_Appl_Id_Get( IMRT_Appl_Id  *Appl_Id_Ptr, const IMRT_Appl_Name Appl_Name)
{
    IMRT_Status  status;
    NDT_Status   nd_status;
    NDT_Node    *node_ptr;
    IMRT_Appl    Appli;


	strcpy(Appli.Name, Appl_Name);

	if(((nd_status = ND_Index_Node_Find( &node_ptr, IMRG_SDM_Base.RootAppl,
					 IMRD_SDMD_APPL_IDX_NAME,&Appli,NULL)) != NDS_OK)||
		(node_ptr == NULL))
	{
	    printf("\nErreur lors de la recherche de Appli Name <%s>\n", Appl_Name);
	    status = IMRS_KO;
	}
	else
	{
		*Appl_Id_Ptr = ( ( IMRT_Appl *)( node_ptr->Value))->Id;
		status = IMRS_OK;		
	}

	return(status);	
}



void  SDM_DataStruct_Dump( NDT_Root  *ds_ptr )
{
    NDT_Status  status;


    printf( "Print DataStructure Info:\n");

    if( ( status = ND_DataStruct_Info_Print( stdout, ds_ptr, NDD_RECURSIVE_MODE_PARENT_CHILD, 0, 0)) != NDS_OK)
    {
        printf( "ND_DataStruct_Info_Print() failed (%d) !\n", status);
    }

    printf( "\n");
    printf( "Traverse DataStructure Index 0:\n");

    if( ( status = ND_DataStruct_Traverse( ds_ptr, IMRD_SDMD_CMD_INDEX0_PRINT, stdout, NDD_RECURSIVE_MODE_PARENT, 0)) != NDS_OK)
    {
        printf( "ND_DataStruct_Traverse() failed (%d) !\n", status);
    }
}





/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
IMRT_Status  IMR_SDM_SQL_Dump(void)
{
	IMRT_Status     status;

#ifdef ORACLE_SUPPORT

	IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "SQL Dump ...");

	status = SDM_SQL_StructInfo_Dump();
	if(status == IMRS_OK)
	{
		status = SDM_SQL_MapInfo_Dump();
		if(status == IMRS_OK)
		{
			status = SDM_SQL_BufferInfo_Dump();
		}
	}
#else
	status = IMRS_OK;
#endif /* ORACLE_SUPPORT */

	return(status);
}





/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
IMRT_Status  IMR_SDM_XML_Dump(void)
{
	IMRT_Status     status;

	IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "XML Dump ...");

	status = SDM_XML_StructInfo_Dump();
	if(status == IMRS_OK)
	{
		status = SDM_XML_MapInfo_Dump();
		if(status == IMRS_OK)
		{
			status = SDM_XML_BufferInfo_Dump();
		}
	}

	return(status);

}


/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
IMRT_Status  SDM_Set_Struct_Version(IMRT_Data_Version *Data_Version_Ptr, char *Version_String_Ptr)
{
	char    *Pos1_Ptr, *Pos2_Ptr;
	char    *Pos3_Ptr;
	char    Label[IMRD_SDMD_VERS_LABEL_LEN];
	char    Date[IMRD_SDMD_VERS_DATE_LEN];
	int    i;

	strcpy(Label, "");
	strcpy(Date, "");

	Pos1_Ptr = strstr( Version_String_Ptr, "Label:");
	if (Pos1_Ptr != NULL)
	{
		Pos2_Ptr = strstr( Pos1_Ptr, "$");
		if (Pos2_Ptr == NULL)
		{
			return(IMRS_KO);
		}

		strncpy(Label, Pos1_Ptr+6, Pos2_Ptr - Pos1_Ptr - 6);
		Label[Pos2_Ptr - Pos1_Ptr - 6]='\0';
		Pos3_Ptr = Label;
		i = strlen(Pos3_Ptr);
		while((Pos3_Ptr[0] == ' ') && (i >= 0))
		{
			Pos3_Ptr ++;
			i --;
		}

		i = strlen(Pos3_Ptr);
		while((Pos3_Ptr[i-1] == ' ') && (i > 0))
		{
			i --;
		}

		if (i > 0)
		{
			strncpy(Data_Version_Ptr->Label, Pos3_Ptr, i);
			Data_Version_Ptr->Label[i]= '\0';
		}

		Pos1_Ptr = strstr( Version_String_Ptr, "Date:");
		if (Pos1_Ptr != NULL)
		{
			Pos2_Ptr = strstr( Pos1_Ptr, "$");
			if (Pos2_Ptr == NULL)
			{
				return(IMRS_KO);
			}

			strncpy(Date, Pos1_Ptr+5, Pos2_Ptr - Pos1_Ptr - 5);
			Date[Pos2_Ptr - Pos1_Ptr - 5]='\0';
			Pos3_Ptr = Date;
			i = strlen(Pos3_Ptr);
			while((Pos3_Ptr[0] == ' ') && (i >= 0))
			{
				Pos3_Ptr ++;
				i --;
			}

			i = strlen(Pos3_Ptr);
			while((Pos3_Ptr[i-1] == ' ') && (i > 0))
			{
				i --;
			}

			if (i > 0)
			{
				strncpy(Data_Version_Ptr->Date, Pos3_Ptr, i);
				Data_Version_Ptr->Date[i]= '\0';
			}
		}	
		else
		{
			return(IMRS_KO);
		}
	}
	else
	{
		return(IMRS_KO);
	}

	return(IMRS_OK);

}
