/*----------------------------------------------------------------------------*/
/* File: libstatic_data_mng.h 				      		      */
/*----------------------------------------------------------------------------*/
/*                                                                             */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#ifndef _LIBSTATIC_DATA_MNG_H_
#define _LIBSTATIC_DATA_MNG_H_







/*----------------------------------------------------------------------------*/
/* Global Include                                                             */
/*----------------------------------------------------------------------------*/

#include <imr.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h> 
#include <libxml/uri.h> 





/*----------------------------------------------------------------------------*/
/* Lib Definitions                                                            */
/*----------------------------------------------------------------------------*/

#define IMRD_SDM_MODULE_ID                ( IMRT_Module_Id) 2
#define IMRD_SDM_MODULE_NAME            ( IMRT_Module_Name) "sdm"
#define IMRD_SDM_MODULE_NAME_LONG  ( IMRT_Module_Name_Long) "Static Data Manager"
#define IMRD_SDM_MODULE_VERSION      ( IMRT_Module_Version)  "@(#) VERSIONIMR: $Label:  $   $Date: 2008/11/12 02:25:23 $   $Workfile:   libstatic_data_mng.h  $"

#define IMRD_MODULE_PTR                                     &IMRG_SDM_Module





/*----------------------------------------------------------------------------*/
/* Local Definition                                                           */
/*----------------------------------------------------------------------------*/

#define IMRD_SDMD_FUNCTION_NAME_LEN   20
#define IMRD_SDMD_LIB_ERROR_LEN       60
#define IMRD_SDMD_VALUE_LEN           255
#define IMRD_SDMD_MARKUP_LEN          255
#define IMRD_SDMD_DESCERR_LEN         60

#define IMRD_SDMD_SEP_NAME "-"
#define IMRD_SDMD_EMPTY_STRING ""

#define IMRD_SDMD_CMD_INDEX0_PRINT    (NDT_Command)64
#define IMRD_SDMD_CMD_EXECUTE         (NDT_Command)68
#define NDD_CMD_EXECUTE               (NDT_Command)20

#define IMRD_SDMD_CMD_API_XML_FILE_DUMP   (NDT_Command)70
#define IMRD_SDMD_CMD_API_SQL_FILE_DUMP   (NDT_Command)71
#define IMRD_SDMD_CMD_API_SQL_DDBB_DUMP   (NDT_Command)72

#define IMRD_SDMD_CMD_API_XML_DUMP        (NDT_Command)80
#define IMRD_SDMD_CMD_API_SQL_DUMP        (NDT_Command)81
#define NDD_CMD_FIND_MAPPATH			  (NDT_Command)82

#define IMRD_SDMD_CMD_CHECK_FMT_ATTR      (NDT_Command)83

/*dfinition de code retour spcifique pour la lib_node*/

#define NDS_NODE_FOUND				      (NDT_Status)-1000
#define NDS_IDENT_OK                                  (NDT_Status)-1010 
#define NDS_NO_IDENT                                  (NDT_Status)-1020 
/**/


#define IMRD_SDMD_MAX_LINE_DUMP_FILE        500

#define IMRD_SDMD_INDEX_NB                (short)1

#define IMRD_SDMD_TAG_VERSION       "version"
#define IMRD_SDMD_TAG_NAME          "name"
#define IMRD_SDMD_TAG_MSGMAP        "msgmap"
#define IMRD_SDMD_MSGMAP_IDX_ID                (short)0
#define IMRD_SDMD_MSGMAP_IDX_NAME              (short)1

/*Liste des codes erreurs*/
#define SDMD_XML_CODE_ERROR_1   1
#define SDMD_XML_LIB_ERROR_1    "Balise non reconnu"
#define SDMD_XML_CODE_ERROR_2   2
#define SDMD_XML_LIB_ERROR_2    "Insertion de l'objet %s impossible"
#define SDMD_XML_CODE_ERROR_3   3
#define SDMD_XML_LIB_ERROR_3    "allocationde l'objet %s impossible"
#define SDMD_XML_CODE_ERROR_4   4
#define SDMD_XML_LIB_ERROR_4    "l'objet %s de valeur %s est absent"
#define SDMD_XML_CODE_ERROR_5   5
#define SDMD_XML_LIB_ERROR_5    "Impossible d'ouvrir la strcuture %s"
#define SDMD_CODE_ERROR_6        6
#define SDMD_LIB_ERROR_6         "Impossible de trouvers un chemin de mapping"
#define SDMD_CODE_ERROR_7        7
#define SDMD_LIB_ERROR_7         "La commande %d n'est pas une commande connut"
#define SDMD_CODE_ERROR_8        8
#define SDMD_LIB_ERROR_8         "Le format %s n'est pas un prsent dans la liste des formats"
#define SDMD_CODE_ERROR_9        9
#define SDMD_LIB_ERROR_9         "Indentification du message impossible"

#define SDMD_XML_CODE_ERROR_10   10
#define SDMD_XML_LIB_ERROR_10    "Erreur de chargement du fichier XML %s"

#define SDMD_CODE_ERROR_11       11
#define SDMD_LIB_ERROR_11        "Erreur de control de coherence de Format Attribute"


#define IMRD_SDMD_VERS_NAME_LEN           30
#define IMRD_SDMD_VERS_LABEL_LEN          30
#define IMRD_SDMD_VERS_DATE_LEN           30


/*----------------------------------------------------------------------------*/
/* IMRT_Mode_Traverse definition                                              */
/*----------------------------------------------------------------------------*/

typedef short  IMRT_Mode_Traverse;

#define IMRD_SKIP_MODE     ( IMRT_Mode_Traverse) 0
#define IMRD_NORMALE_MODE  ( IMRT_Mode_Traverse) 1
#define IMRD_ERREUR_MODE   ( IMRT_Mode_Traverse) 2


#define IMRD_LATEST_BRANCHE   ( IMRT_Branche_Id) 100

static NDT_Index_Type  index_type_initial_tab[IMRD_SDMD_INDEX_NB] = 
{
      (NDD_INDEX_STATUS_OPENED | NDD_INDEX_TYPE_LIST | NDD_INDEX_SUBTYPE_SORTED)
};

typedef struct IMRT_Error
{
 char function [IMRD_SDMD_FUNCTION_NAME_LEN];
 int  errorCode;
 char libError[IMRD_SDMD_LIB_ERROR_LEN];
} IMRT_Error;


typedef struct IMRT_SDM_Base
{
  NDT_Root       * RootMsg;
  NDT_Root       * RootMsgMap;
  NDT_Root       * RootFmtAttr;
  NDT_Root       * RootFmt;
  NDT_Root       * RootAppl;
  NDT_Root       * RootMapPath;
  NDT_Root       * RootCnv;
} IMRT_SDM_Base;

typedef struct IMRT_SDM_IdentMessage
{
 IMRT_Msg_Name Type;
 IMRT_Msg_Id   Id;
 IMRT_Mode_Traverse  Mode;
 IMRT_Branche_Id     Branche_Id;
 IMRT_SDMCB_Ident_Msg_Size     *Ident_Msg_SizeFunc_Ptr;
 IMRT_SDMCB_Ident_Field_Exist  *Ident_Field_ExistFunc_Ptr;
 IMRT_SDMCB_Ident_Field_Value  *Ident_Field_ValueFunc_Ptr;
} IMRT_SDM_IdentMessage;


typedef struct IMRT_Data_Version
{
 char Label [IMRD_SDMD_VERS_LABEL_LEN];
 char Date[IMRD_SDMD_VERS_DATE_LEN];
} IMRT_Data_Version;


/*----------------------------------------------------------------------------*/
/* IMRT_Array_Xml_Path definition                                             */
/*----------------------------------------------------------------------------*/

#define IMRD_SDMD_MIN_LEVEL_TAG           ( short ) -1
#define IMRD_SDMD_MAX_LEVEL_TAG           ( short ) 10
#define IMRD_SDMD_LEVEL_TAG_0             ( short ) 0
#define IMRD_SDMD_LEVEL_TAG_1             ( short ) 1
#define IMRD_SDMD_LEVEL_TAG_2             ( short ) 2
#define IMRD_SDMD_LEVEL_TAG_3             ( short ) 3
#define IMRD_SDMD_LEVEL_TAG_4             ( short ) 4
#define IMRD_SDMD_LEVEL_TAG_5             ( short ) 5
#define IMRD_SDMD_LEVEL_TAG_6             ( short ) 6
#define IMRD_SDMD_LEVEL_TAG_7             ( short ) 7
#define IMRD_SDMD_LEVEL_TAG_8             ( short ) 8
#define IMRD_SDMD_LEVEL_TAG_9             ( short ) 9
#define IMRD_SDMD_SIZE_TAG_LEN            ( short ) 30
#define IMRD_SDMD_SIZE_TAG_SIZE           ( short ) ( IMRD_SDMD_SIZE_TAG_LEN + 1)





/*----------------------------------------------------------------------------*/
/* IMRT_Array_Xml_Path definition                                             */
/*----------------------------------------------------------------------------*/

typedef char IMRT_Level_Name[IMRD_SDMD_SIZE_TAG_SIZE];

typedef IMRT_Level_Name  IMRT_Array_Xml_Path[IMRD_SDMD_MAX_LEVEL_TAG];





/*----------------------------------------------------------------------------*/
/* IMRT_Level_Tag definition                                                  */
/*----------------------------------------------------------------------------*/
typedef short IMRT_Level_Tag;






/*----------------------------------------------------------------------------*/
/* Prototypes                                                                 */
/*----------------------------------------------------------------------------*/

#   ifdef _LIBSTATIC_DATA_MNG_C_

void  SDM_DataStruct_Dump( NDT_Root  * );
IMRT_Status  SDM_Set_Struct_Version(IMRT_Data_Version *, char *);
#   else

extern  void  SDM_DataStruct_Dump( NDT_Root  * );
extern  IMRT_Status  SDM_Set_Struct_Version(IMRT_Data_Version *, char *);
#   endif 





/*----------------------------------------------------------------------------*/
/* Local Includes                                                             */
/*----------------------------------------------------------------------------*/

#include "arc_hdl_xml.h"

#ifdef ORACLE_SUPPORT
#   include "arc_hdl_sql.h"
#endif /* ORACLE_SUPPORT */

#include "buffer_info_mng.h"
#include "mapping_info_mng.h"
#include "struct_info_mng.h"





/*----------------------------------------------------------------------------*/
/* Global Variables                                                           */
/*----------------------------------------------------------------------------*/


#   ifdef _LIBSTATIC_DATA_MNG_C_

IMRT_SDM_Base  IMRG_SDM_Base;



IMRT_Module  IMRG_SDM_Module = 
{ 
    IMRD_SDM_MODULE_ID,
    IMRD_SDM_MODULE_NAME,
    IMRD_SDM_MODULE_NAME_LONG,
    IMRD_SDM_MODULE_VERSION,
    IMRD_MODULE_TYPE_LIBRARY,
    {
	IMRD_LIB_STATUS_CLOSED,
	( IMRT_Lib_Open    *) &IMR_SDM_Library_Open,
	( IMRT_Lib_Close   *) &IMR_SDM_Library_Close
    }
};

#   else

extern IMRT_SDM_Base  IMRG_SDM_Base;
extern IMRT_Module    IMRG_SDM_Module;

#   endif





/*----------------------------------------------------------------------------*/

#endif
