/*----------------------------------------------------------------------------*/
/* File: mapping_info_mng.c   		         		  	      */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#define _LIBSTATIC_DATA_MNG_
#define _MAPPING_INFO_MNG_C_

#include "mapping_info_mng.h" 




/*----------------------------------------------------------------------------*/
/* FieldMap Manager                                                           */
/*----------------------------------------------------------------------------*/

NDT_Status  SDM_FieldMap_Manager( NDT_Root  *Root_Ptr, NDT_Index_Id  Index_Id, NDT_Node  *Node_Ptr, NDT_Command  Command, va_list  Args)
{
    NDT_Command_Name  Command_Name;


    switch( Command)
    {
        case NDD_CMD_MANAGER_VERSION:
        {
            NDT_Version_Name  *Version_Name_Ptr = (NDT_Version_Name  *)va_arg( Args, NDT_Version_Name  *);


            Command_Name = "NDD_CMD_MANAGER_VERSION";

            *Version_Name_Ptr = "Revision: 2.1    Name: libnode-2_0_4-1    Date: 2002/02/28 22:00:35    Author: agibert ";

            return( NDS_OK);
        }

        case NDD_CMD_INDEX_GET:
        {
            /*
	      NDT_Index_Id  *Reply_Index_Id_Ptr = (NDT_Index_Id  *)va_arg( Args, NDT_Index_Id  *);
	      NDT_Command   *Reply_Command_Ptr  = (NDT_Command  *)va_arg( Args, NDT_Command  *);
	      NDT_Command    Cmd                = (NDT_Command)va_arg( Args, NDT_Command);
	      void          *Value_ptr          = (void  *)va_arg( Args, void  *);
            */
            NDT_Index_Id  *Reply_Index_Id_Ptr =  (NDT_Index_Id  *)va_arg( Args, NDT_Index_Id  *);
            NDT_Command   *Reply_Command_Ptr  =  (NDT_Command  *)va_arg( Args, NDT_Command  *);
            NDT_Command    Cmd                =  (NDT_Command)va_arg( Args, NDT_Command);
            IMRT_FieldMap    * Value_ptr      =  (IMRT_FieldMap  *)va_arg( Args, IMRT_FieldMap  *);


            Command_Name = "NDD_CMD_INDEX_GET";

            switch(Cmd)
            {
                case IMRD_SDMD_CMD_INDEX0_PRINT:
                {
                    *Reply_Index_Id_Ptr = 0;
                    *Reply_Command_Ptr = NDD_CMD_VALUE_PRINT;

                    break;
                }
                case IMRD_SDMD_CMD_EXECUTE:
                {
                    *Reply_Index_Id_Ptr = 0;
                    *Reply_Command_Ptr = NDD_CMD_EXECUTE;

                    break;
                }

                default:
                {
                    *Reply_Index_Id_Ptr = Index_Id;
                    *Reply_Command_Ptr = Cmd;

                    break;
                }
            }

            return( NDS_OK);
        }

        case NDD_CMD_VALUE_ALLOC:
        {
            /*
	      void       **Value_Ptr_Ptr = (void **)va_arg( Args, void  **);
	      va_list      user_args     = (va_list)va_arg( Args, va_list);
	      user_type    user_data     = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
            */
            IMRT_FieldMap  **Value_Ptr_Ptr = va_arg( Args, IMRT_FieldMap  **);


            Command_Name = "NDD_CMD_VALUE_ALLOC";

            if( ( *Value_Ptr_Ptr = (IMRT_FieldMap  *)malloc( sizeof(IMRT_FieldMap))) == NULL)
            {
                return( NDS_ERRMEM);
            }
            else
            {
                SDM_InitFieldMap( *Value_Ptr_Ptr);

                return( NDS_OK);
            }
        }

        case NDD_CMD_VALUE_FREE:
        {
            /*
	      void       *Value_Ptr = (void *)va_arg( Args, void  *);
	      va_list     user_args = (va_list)va_arg( Args, va_list);
	      user_type   user_data = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
            */
            IMRT_FieldMap  *Value_Ptr = (IMRT_FieldMap  *)va_arg( Args, IMRT_FieldMap  *);


            Command_Name = "NDD_CMD_VALUE_FREE";

            free( Value_Ptr);

            return( NDS_OK);
        }

        case NDD_CMD_VALUE_COMP:
        {
            /*
	      void       *Value1_Ptr = (void *)va_arg( Args, void  *);
	      void       *Value2_Ptr = (void *)va_arg( Args, void  *);
	      va_list     user_args  = (va_list)va_arg( Args, va_list);
	      user_type   user_data  = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
            */
            IMRT_FieldMap    *Value1_Ptr = va_arg( Args, IMRT_FieldMap  *);
            IMRT_FieldMap    *Value2_Ptr = va_arg( Args, IMRT_FieldMap  *);


            Command_Name = "NDD_CMD_VALUE_COMP";

            switch( Index_Id)
            {
                case 0:
                {
                    if( Value1_Ptr->Id < Value2_Ptr->Id)
                    {
                        return( NDS_LOWER);
                    }
                    else
                    {
                        if( Value1_Ptr->Id > Value2_Ptr->Id)
                        {
                            return( NDS_GREATER);
                        }
                        else
                        {
                            return( NDS_EQUAL);
                        }
                    }
                }
                
                default:
                {
                    printf( "Unknown COMP idx (%d) !\n", Index_Id);

                    return( NDS_KO);
                }
            }

            return( NDS_OK);
        }

        case NDD_CMD_VALUE_ADD:
        {
            /*
	      void       *Value_Ptr = (void *)va_arg( Args, void  *);
	      va_list     user_args = (va_list)va_arg( Args, va_list);
	      user_type   user_data = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
            */

            Command_Name = "NDD_CMD_VALUE_ADD";

            return( NDS_OK);
        }

        case NDD_CMD_VALUE_REMOVE:
        {
            /*
	      void       *Value_Ptr = (void *)va_arg( Args, void  *);
	      va_list     user_args = (va_list)va_arg( Args, va_list);
	      user_type   user_data = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
            */

            Command_Name = "NDD_CMD_VALUE_REMOVE";

            return( NDS_OK);
        }

        case NDD_CMD_VALUE_PRINT:
        {
            /*
	      NDT_Node             *Next_Node_Ptr   = (NDT_Node  *)va_arg( Args, NDT_Node  *);
	      va_list               lib_args        = (va_list)va_arg( Args, va_list);
	      FILE                 *Out             = (FILE  *)va_arg( lib_args, FILE  *);
	      NDT_Recursive_Mode    Recursive_Mode  = (NDT_Recursive_Mode)va_arg( lib_args, NDT_Recursive_Mode);
	      NDT_Recursive_Depth   Recursive_Depth = (NDT_Recursive_Depth)va_arg( lib_args, NDT_Recursive_Depth);
	      va_list               user_args       = (va_list)va_arg( lib_args, va_list);
	      user_type             user_data       = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
	      void                 *Value_Ptr       = Node_Ptr->Value;
            */
            NDT_Node             *Next_Node_Ptr   =         (NDT_Node  *)va_arg( Args, NDT_Node  *);
            va_list               lib_args        =             (va_list)va_arg( Args, va_list);
            FILE                 *Out             =             (FILE  *)va_arg( lib_args, FILE  *);
            NDT_Recursive_Mode    Recursive_Mode  =  (NDT_Recursive_Mode)va_arg( lib_args, NDT_Recursive_Mode);
            NDT_Recursive_Depth   Recursive_Depth = (NDT_Recursive_Depth)va_arg( lib_args, NDT_Recursive_Depth);

            IMRT_FieldMap           *Value_Ptr       = Node_Ptr->Value;


            Command_Name = "NDD_CMD_VALUE_PRINT";

            fprintf( Out, "Id: (%d) InputField: (%s)  OutputField: (%s)\n",
		     Value_Ptr->Id, Value_Ptr->Field_Input_Ptr->Name, Value_Ptr->Field_Output_Ptr->Name);

            return( NDS_OK);
        }

        case NDD_CMD_INFO_PRINT:
        {
            /*
	      NDT_Node             *Next_Node_Ptr   = (NDT_Node  *)va_arg( Args, NDT_Node  *);
	      va_list               lib_args        = (va_list)va_arg( Args, va_list);
	      FILE                 *Out             = (FILE  *)va_arg( lib_args, FILE  *);
	      NDT_Recursive_Mode    Recursive_Mode  = (NDT_Recursive_Mode)va_arg( lib_args, NDT_Recursive_Mode);
	      NDT_Recursive_Depth   Recursive_Depth = (NDT_Recursive_Depth)va_arg( lib_args, NDT_Recursive_Depth);
	      va_list               user_args       = (va_list)va_arg( lib_args, va_list);
	      user_type             user_data       = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
	      void                 *Value_Ptr       = Node_Ptr->Value;
            */

            Command_Name = "NDD_CMD_INFO_PRINT";

            return( NDS_OK);
        }

        case NDD_CMD_EXECUTE:
        { /*
            NDT_Node             *Next_Node_Ptr    = (NDT_Node  *)va_arg( Args, NDT_Node  *);
            va_list               lib_args         = (va_list)va_arg( Args, va_list);
            FILE                 *Out              = (FILE  *)va_arg( lib_args, FILE  *);
            NDT_Recursive_Mode    Recursive_Mode   = (NDT_Recursive_Mode)va_arg( lib_args, NDT_Recursive_Mode);
            NDT_Recursive_Depth   Recursive_Depth  = (NDT_Recursive_Depth)va_arg( lib_args, NDT_Recursive_Depth);
            NDT_Recursive_Offset  Recursive_Offset = (NDT_Recursive_Offset)va_arg( lib_args, NDT_Recursive_Offset);
            va_list               user_args        = (va_list)va_arg( lib_args, va_list);
            user_type             user_data        = (user_type)va_arg( user_args, user_type);
	    ...  = (...)va_arg( user_args, ...);
            void                 *Value_Ptr        = Node_Ptr->Value;
	  */
            IMRT_FieldMap *  Value_Ptr             = (IMRT_FieldMap*)Node_Ptr->Value;
            NDT_Node *Next_Node_Ptr                = (NDT_Node  *)va_arg( Args, NDT_Node  *);
            va_list lib_args                       = (va_list)va_arg( Args, va_list);
            FILE *Out                              = (FILE  *)va_arg( lib_args, FILE  *);
            NDT_Recursive_Mode    Recursive_Mode   = (NDT_Recursive_Mode)va_arg( lib_args, NDT_Recursive_Mode);
            va_list user_args                      = (va_list)va_arg( Args, va_list);
            IMRT_SDMCB_Field_Map * MappingFieldFunc      = ( IMRT_SDMCB_Field_Map *)va_arg(lib_args, IMRT_SDMCB_Field_Map *);
            IMRT_Status   status;

            status = MappingFieldFunc(Value_Ptr->Field_Output_Ptr, Value_Ptr->Field_Input_Ptr);
			if (status == IMRS_KO)
			{
				return( NDS_KO);
			}
            return( NDS_OK);
        }

		case IMRD_SDMD_CMD_API_SQL_DUMP:
		{
		/*
			NDT_Node   *Next_Node_Ptr = (NDT_Node  *)va_arg( Args, NDT_Node  *);
			va_list     user_args     =     (va_list)va_arg( Args, va_list);
			user_type   user_data     =   (user_type)va_arg( user_args, user_type);
								  ... =         (...)va_arg( user_args, ...);

  			void       *Value_Ptr     = Node_Ptr->Value;
		*/
			NDT_Node        *Next_Node_Ptr = (NDT_Node  *)va_arg( Args, NDT_Node  *);
			va_list          user_args     =     (va_list)va_arg( Args, va_list);
			FILE            *Out           =     (FILE  *)va_arg( user_args, FILE  *);
			IMRT_MsgMap_Id  MsgMap_Id      =     (IMRT_MsgMap_Id)va_arg( user_args, IMRT_MsgMap_Id);
			IMRT_FieldMap   *FieldMap_Ptr  = Node_Ptr->Value;


			Command_Name = "IMRD_SDMD_CMD_API_SQL_DUMP";
			
			fprintf( Out, "INSERT INTO IMR_FIELD_MAP(FIELD_MAP_ID, MSG_MAP_ID, FIELD_IN_ID, FIELD_OUT_ID) VALUES(%d, %d, %d, %d);\n",
					FieldMap_Ptr->Id, MsgMap_Id, FieldMap_Ptr->Field_Input_Ptr->Id, FieldMap_Ptr->Field_Output_Ptr->Id);

			return( NDS_OK);
		}

		case IMRD_SDMD_CMD_API_XML_DUMP:
		{
		/*
			NDT_Node   *Next_Node_Ptr = (NDT_Node  *)va_arg( Args, NDT_Node  *);
			va_list     user_args     =     (va_list)va_arg( Args, va_list);
			user_type   user_data     =   (user_type)va_arg( user_args, user_type);
								  ... =         (...)va_arg( user_args, ...);

  			void       *Value_Ptr     = Node_Ptr->Value;
		*/
			NDT_Node        *Next_Node_Ptr = (NDT_Node  *)va_arg( Args, NDT_Node  *);
			va_list          user_args     =     (va_list)va_arg( Args, va_list);
			FILE            *Out           =     (FILE  *)va_arg( user_args, FILE  *);

			IMRT_FieldMap   *FieldMap_Ptr  = Node_Ptr->Value;


			Command_Name = "IMRD_SDMD_CMD_API_XML_DUMP";

			fprintf( Out, "\t\t\t<%s>\n\t\t\t\t<%s>%s</%s>\n\t\t\t\t<%s>%s</%s>\n\t\t\t</%s>\n", IMRD_SDMD_TAG_FIELDMAP,
						IMRD_SDMD_TAG_INPUTFIELD, FieldMap_Ptr->Field_Input_Ptr->Name, IMRD_SDMD_TAG_INPUTFIELD, 
						IMRD_SDMD_TAG_OUTPUTFIELD, FieldMap_Ptr->Field_Output_Ptr->Name, IMRD_SDMD_TAG_OUTPUTFIELD, 
						IMRD_SDMD_TAG_FIELDMAP);

			return( NDS_OK);
		}

        default:
        {
            printf( "SDM_FieldMap_Manager() called with an undefined command %d\n", Command);

            return( NDS_ERRAPI);
        }
    }

    printf( "SDM_FieldMap_Manager() called with command %d (%s)\n", Command, Command_Name);

    return( NDS_OK);
}





/*----------------------------------------------------------------------------*/
/* MsgMap Manager                                                             */
/*----------------------------------------------------------------------------*/

NDT_Status  SDM_MsgMap_Manager( NDT_Root  *Root_Ptr, NDT_Index_Id  Index_Id, NDT_Node  *Node_Ptr, NDT_Command  Command, va_list  Args)
{
    NDT_Command_Name  Command_Name;
    NDT_Status        status;


    switch( Command)
    {
        case NDD_CMD_MANAGER_VERSION:
        {
            NDT_Version_Name  *Version_Name_Ptr = (NDT_Version_Name  *)va_arg( Args, NDT_Version_Name  *);


            Command_Name = "NDD_CMD_MANAGER_VERSION";

            *Version_Name_Ptr = "Revision: 2.1    Name: libnode-2_0_4-1    Date: 2002/02/28 22:00:35    Author: agibert ";

            return( NDS_OK);
        }

        case NDD_CMD_INDEX_GET:
        {
            /*
	      NDT_Index_Id  *Reply_Index_Id_Ptr = (NDT_Index_Id  *)va_arg( Args, NDT_Index_Id  *);
	      NDT_Command   *Reply_Command_Ptr  = (NDT_Command  *)va_arg( Args, NDT_Command  *);
	      NDT_Command    Cmd                = (NDT_Command)va_arg( Args, NDT_Command);
	      void          *Value_ptr          = (void  *)va_arg( Args, void  *);
            */
            NDT_Index_Id  *Reply_Index_Id_Ptr = (NDT_Index_Id  *)va_arg( Args, NDT_Index_Id  *);
            NDT_Command   *Reply_Command_Ptr  = (NDT_Command  *)va_arg( Args, NDT_Command  *);
            NDT_Command    Cmd                = (NDT_Command)va_arg( Args, NDT_Command);
            IMRT_MsgMap  *      Value_ptr     = (IMRT_MsgMap  *)va_arg( Args, IMRT_MsgMap  *);


            Command_Name = "NDD_CMD_INDEX_GET";

            switch(Cmd)
            {
                case IMRD_SDMD_CMD_INDEX0_PRINT:
                {
                    *Reply_Index_Id_Ptr = 0;
                    *Reply_Command_Ptr = NDD_CMD_VALUE_PRINT;

                    break;
                }

				case IMRD_SDMD_CMD_API_SQL_DUMP:
				{
                    *Reply_Index_Id_Ptr = IMRD_SDMD_MSGMAP_IDX_ID;
                    *Reply_Command_Ptr = IMRD_SDMD_CMD_API_SQL_DUMP;
                    break;
				}

				case IMRD_SDMD_CMD_API_XML_DUMP:
				{
                    *Reply_Index_Id_Ptr = IMRD_SDMD_MSGMAP_IDX_ID;
                    *Reply_Command_Ptr = IMRD_SDMD_CMD_API_XML_DUMP;
                    break;
				}

                default:
                {
                    *Reply_Index_Id_Ptr = Index_Id;
                    *Reply_Command_Ptr = Cmd;

                    break;
                }
            }

            return( NDS_OK);
        }

        case NDD_CMD_VALUE_ALLOC:
        {
            /*
	      void       **Value_Ptr_Ptr = (void **)va_arg( Args, void  **);
	      va_list      user_args     = (va_list)va_arg( Args, va_list);
	      user_type    user_data     = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
            */
            IMRT_MsgMap  **Value_Ptr_Ptr = va_arg( Args, IMRT_MsgMap  **);


            Command_Name = "NDD_CMD_VALUE_ALLOC";

            if( ( *Value_Ptr_Ptr = (IMRT_MsgMap  *)malloc( sizeof(IMRT_MsgMap))) == NULL)
            {
                return( NDS_ERRMEM);
            }
            else
            {
                SDM_InitMsgMap(*Value_Ptr_Ptr);

                if( ( status = ND_DataStruct_Open( &(*Value_Ptr_Ptr)->FieldMap_Struct_Ptr, IMRD_SDMD_INDEX_NB, index_type_initial_tab, "SDM_FieldMap_Manager", SDM_FieldMap_Manager, NULL, NULL, NULL, NULL,0,  NULL)) != NDS_OK)
		{
		    printf( "ND_DataStruct_Open() FieldMap failed (%d) !\n", status);
		}

                return( NDS_OK);
            }
        }

        case NDD_CMD_VALUE_FREE:
        {
            /*
	      void       *Value_Ptr = (void *)va_arg( Args, void  *);
	      va_list     user_args = (va_list)va_arg( Args, va_list);
	      user_type   user_data = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
            */
            IMRT_MsgMap  *Value_Ptr = (IMRT_MsgMap  *)va_arg( Args, IMRT_MsgMap  *);


            Command_Name = "NDD_CMD_VALUE_FREE";

            if(Value_Ptr->FieldMap_Struct_Ptr != NULL)
            {  if( ( status = ND_DataStruct_Close(Value_Ptr->FieldMap_Struct_Ptr)) != NDS_OK)
		{
		    printf( "ND_DataStruct_Close() failed (%d) !\n", status);

		    return(status);
		}
            }

            free( Value_Ptr);
            
            return( NDS_OK);
        }

        case NDD_CMD_VALUE_COMP:
        {
            /*
	      void       *Value1_Ptr = (void *)va_arg( Args, void  *);
	      void       *Value2_Ptr = (void *)va_arg( Args, void  *);
	      va_list     user_args  = (va_list)va_arg( Args, va_list);
	      user_type   user_data  = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
            */
            IMRT_MsgMap     *Value1_Ptr = va_arg( Args, IMRT_MsgMap  *);
            IMRT_MsgMap     *Value2_Ptr = va_arg( Args, IMRT_MsgMap  *);


            Command_Name = "NDD_CMD_VALUE_COMP";

            switch( Index_Id)
            {
                case IMRD_SDMD_MSGMAP_IDX_ID:
                {
                    if( Value1_Ptr->Id < Value2_Ptr->Id)
                    {
                        return( NDS_LOWER);
                    }
                    else
                    {
                        if( Value1_Ptr->Id > Value2_Ptr->Id)
                        {
                            return( NDS_GREATER);
                        }
                        else
                        {
                            return( NDS_EQUAL);
                        }
                    }
                }

                case IMRD_SDMD_MSGMAP_IDX_NAME:
                {
                    int     rc;


                    rc=strcmp( Value1_Ptr->Name, Value2_Ptr->Name);

                    if( rc <0)
                    {
                        return( NDS_LOWER);
                    }
                    else
                    {
                        if( rc > 0)
                        {
                            return( NDS_GREATER);
                        }
                        else
                        {
                            return( NDS_EQUAL);
                        }
                    }
                }

                default:
                {
                    printf( "Unknown COMP idx (%d) !\n", Index_Id);

                    return( NDS_KO);
                }
            }

            return( NDS_OK);
        }

        case NDD_CMD_VALUE_ADD:
        {
            /*
	      void       *Value_Ptr = (void *)va_arg( Args, void  *);
	      va_list     user_args = (va_list)va_arg( Args, va_list);
	      user_type   user_data = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
            */

            Command_Name = "NDD_CMD_VALUE_ADD";

            return( NDS_OK);
        }

        case NDD_CMD_VALUE_REMOVE:
        {
            /*
	      void       *Value_Ptr = (void *)va_arg( Args, void  *);
	      va_list     user_args = (va_list)va_arg( Args, va_list);
	      user_type   user_data = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
            */

            Command_Name = "NDD_CMD_VALUE_REMOVE";

            return( NDS_OK);
        }

        case NDD_CMD_VALUE_PRINT:
        {
            /*
	      NDT_Node             *Next_Node_Ptr    = (NDT_Node  *)va_arg( Args, NDT_Node  *);
	      va_list               lib_args         = (va_list)va_arg( Args, va_list);
	      FILE                 *Out              = (FILE  *)va_arg( lib_args, FILE  *);
	      NDT_Recursive_Mode    Recursive_Mode   = (NDT_Recursive_Mode)va_arg( lib_args, NDT_Recursive_Mode);
	      NDT_Recursive_Depth   Recursive_Depth  = (NDT_Recursive_Depth)va_arg( lib_args, NDT_Recursive_Depth);
	      NDT_Recursive_Offset  Recursive_Offset = (NDT_Recursive_Offset)va_arg( lib_args, NDT_Recursive_Offset);
	      va_list               user_args        = (va_list)va_arg( lib_args, va_list);
	      user_type             user_data        = (user_type)va_arg( user_args, user_type);
	      ...  = (...)va_arg( user_args, ...);
	      void                 *Value_Ptr       = Node_Ptr->Value;
            */
            NDT_Node             *Next_Node_Ptr    = (NDT_Node  *)va_arg( Args, NDT_Node  *);
            va_list               lib_args         = (va_list)va_arg( Args, va_list);
            FILE                 *Out              = (FILE  *)va_arg( lib_args, FILE  *);
            NDT_Recursive_Mode    Recursive_Mode   = (NDT_Recursive_Mode)va_arg( lib_args, NDT_Recursive_Mode);
            NDT_Recursive_Depth   Recursive_Depth  = (NDT_Recursive_Depth)va_arg( lib_args, NDT_Recursive_Depth);
            NDT_Recursive_Offset  Recursive_Offset = (NDT_Recursive_Offset)va_arg( lib_args, NDT_Recursive_Offset);

            IMRT_MsgMap           *Value_Ptr       = Node_Ptr->Value;


            Command_Name = "NDD_CMD_VALUE_PRINT";

            printf("Adresse avant traverse (%x)\n",Value_Ptr);

            fprintf( Out, "Name: (%s)   InputMsgName: (%s)   OutputMsgName: (%s)   \n",
		     Value_Ptr->Name, Value_Ptr->Msg_Input_Ptr->Name, Value_Ptr->Msg_Output_Ptr->Name);

            return( ND_DataStruct_Value_Print( Out, Value_Ptr->FieldMap_Struct_Ptr, NDD_RECURSIVE_MODE_PARENT_CHILD, --Recursive_Depth, ++Recursive_Offset));
        }

        case NDD_CMD_INFO_PRINT:
        {
            /*
	      NDT_Node             *Next_Node_Ptr    = (NDT_Node  *)va_arg( Args, NDT_Node  *);
	      va_list               lib_args         = (va_list)va_arg( Args, va_list);
	      FILE                 *Out              = (FILE  *)va_arg( lib_args, FILE  *);
	      NDT_Recursive_Mode    Recursive_Mode   = (NDT_Recursive_Mode)va_arg( lib_args, NDT_Recursive_Mode);
	      NDT_Recursive_Depth   Recursive_Depth  = (NDT_Recursive_Depth)va_arg( lib_args, NDT_Recursive_Depth);
	      NDT_Recursive_Offset  Recursive_Offset = (NDT_Recursive_Offset)va_arg( lib_args, NDT_Recursive_Offset);
	      va_list               user_args        = (va_list)va_arg( lib_args, va_list);
	      user_type             user_data        = (user_type)va_arg( user_args, user_type);
	      ...  = (...)va_arg( user_args, ...);
	      void                 *Value_Ptr        = Node_Ptr->Value;
            */

            NDT_Node             *Next_Node_Ptr    = (NDT_Node  *)va_arg( Args, NDT_Node  *);
	    va_list               lib_args         = (va_list)va_arg( Args, va_list);
	    FILE                 *Out              = (FILE  *)va_arg( lib_args, FILE  *);
	    NDT_Recursive_Mode    Recursive_Mode   = (NDT_Recursive_Mode)va_arg( lib_args, NDT_Recursive_Mode);
	    NDT_Recursive_Depth   Recursive_Depth  = (NDT_Recursive_Depth)va_arg( lib_args, NDT_Recursive_Depth);
	    NDT_Recursive_Offset  Recursive_Offset = (NDT_Recursive_Offset)va_arg( lib_args, NDT_Recursive_Offset);
			      
	    IMRT_MsgMap  *Msg_Ptr                    = Node_Ptr->Value;


	    Command_Name = "NDD_CMD_INFO_PRINT";

	    return( ND_DataStruct_Info_Print( Out, Msg_Ptr->FieldMap_Struct_Ptr, NDD_RECURSIVE_MODE_PARENT_CHILD, --Recursive_Depth, ++Recursive_Offset));
        }
        
		
		case IMRD_SDMD_CMD_API_SQL_DUMP:
		{
		/*
			NDT_Node   *Next_Node_Ptr = (NDT_Node  *)va_arg( Args, NDT_Node  *);
			va_list     user_args     =     (va_list)va_arg( Args, va_list);
			user_type   user_data     =   (user_type)va_arg( user_args, user_type);
								  ... =         (...)va_arg( user_args, ...);

  			void       *Value_Ptr     = Node_Ptr->Value;
		*/
			NDT_Node        *Next_Node_Ptr = (NDT_Node  *)va_arg( Args, NDT_Node  *);
			va_list          user_args     =     (va_list)va_arg( Args, va_list);
			FILE            *Out           =     (FILE  *)va_arg( user_args, FILE  *);

			IMRT_MsgMap     *MsgMap_Ptr    = Node_Ptr->Value;


			Command_Name = "IMRD_SDMD_CMD_API_SQL_DUMP";

			fprintf( Out, "INSERT INTO IMR_MSG_MAP(MSG_MAP_ID, MSG_MAP_NAME, MSG_IN_ID, MSG_OUT_ID) VALUES(%d, '%s', %d, %d);\n",
					MsgMap_Ptr->Id, MsgMap_Ptr->Name, MsgMap_Ptr->Msg_Input_Ptr->Id, MsgMap_Ptr->Msg_Output_Ptr->Id);

		    if( ( status = ND_DataStruct_Traverse( MsgMap_Ptr->FieldMap_Struct_Ptr, IMRD_SDMD_CMD_API_SQL_DUMP, Out, MsgMap_Ptr->Id)) != NDS_OK)
			{
				printf( "ND_DataStruct_Traverse failed (%d)...\n");
				return(NDS_KO);
			}

			return( NDS_OK);
		}

		case IMRD_SDMD_CMD_API_XML_DUMP:
		{
		/*
			NDT_Node   *Next_Node_Ptr = (NDT_Node  *)va_arg( Args, NDT_Node  *);
			va_list     user_args     =     (va_list)va_arg( Args, va_list);
			user_type   user_data     =   (user_type)va_arg( user_args, user_type);
								  ... =         (...)va_arg( user_args, ...);

  			void       *Value_Ptr     = Node_Ptr->Value;
		*/
			NDT_Node        *Next_Node_Ptr = (NDT_Node  *)va_arg( Args, NDT_Node  *);
			va_list          user_args     =     (va_list)va_arg( Args, va_list);
			FILE            *Out           =     (FILE  *)va_arg( user_args, FILE  *);

			IMRT_MsgMap     *MsgMap_Ptr    = Node_Ptr->Value;


			Command_Name = "IMRD_SDMD_CMD_API_XML_DUMP";

			fprintf( Out, "\t<%s>\n\t\t<%s>%s</%s>\n\t\t<%s>%s</%s>\n\t\t<%s>%s</%s>\n\t\t<%s>\n", IMRD_SDMD_TAG_MSGMAP, 
						IMRD_SDMD_TAG_NAME, MsgMap_Ptr->Name, IMRD_SDMD_TAG_NAME,
						IMRD_SDMD_TAG_INPUTMSG, MsgMap_Ptr->Msg_Input_Ptr->Name, IMRD_SDMD_TAG_INPUTMSG,
						IMRD_SDMD_TAG_OUTPUTMSG, MsgMap_Ptr->Msg_Output_Ptr->Name, IMRD_SDMD_TAG_OUTPUTMSG,
						IMRD_SDMD_TAG_FIELDMAPSTRUCT);

		    if( ( status = ND_DataStruct_Traverse( MsgMap_Ptr->FieldMap_Struct_Ptr, IMRD_SDMD_CMD_API_XML_DUMP, Out)) != NDS_OK)
			{
				printf( "ND_DataStruct_Traverse failed (%d)...\n");
				return(NDS_KO);
			}

			fprintf( Out, "\t\t</%s>\n\t</%s>\n", IMRD_SDMD_TAG_FIELDMAPSTRUCT, IMRD_SDMD_TAG_MSGMAP);

			return( NDS_OK);
		}


		default:
        {
            printf( "SDM_MsgMap_Manager() called with an undefined command %d\n", Command);

            return( NDS_ERRAPI);
        }
    }

    printf( "SDM_MsgMap_Manager() called with command %d (%s)\n", Command, Command_Name);

    return( NDS_OK);
}
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
IMRT_Status SDM_MapInfo_Init(IMRT_SDM_Config * Lib_Config_Ptr)
{
    IMRT_Status  status;
    if(Lib_Config_Ptr->Flag == IMRD_SDM_FLAG_XML)
    {
		status = SDM_XML_MapInfo_Init(Lib_Config_Ptr->Lib_Path);
    }
    else
    {
#ifdef ORACLE_SUPPORT
		status = SDM_SQL_MapInfo_Init();
#else
		status = IMRS_OK;
#endif /* ORACLE_SUPPORT */
    }
    return(status);
}
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
IMRT_Status SDM_Close_RootMsgMap(void)
{
	IMRT_Status  status;
	NDT_Status   nd_status;

	status = IMRS_OK;

	if(IMRG_SDM_Base.RootMsgMap != NULL)
	{
		if (IMRG_SDM_Base.RootMsgMap->User_Ptr != NULL)
		{
			free(IMRG_SDM_Base.RootMsgMap->User_Ptr);
		}

		if( ( nd_status = ND_DataStruct_Close(IMRG_SDM_Base.RootMsgMap)) != NDS_OK)
		{
			printf( "RootMsgMap ND_DataStruct_Close() failed (%d) !\n", nd_status);
			status = IMRS_KO;
		}
	}

	return(status);

}
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
IMRT_Status SDM_MapInfo_DInit(void)
{
	IMRT_Status  status, StatusMsgMap;
	NDT_Status   nd_status;

	status = IMRS_OK;

	StatusMsgMap = SDM_Close_RootMsgMap();

	if(StatusMsgMap == IMRS_KO)
	{
		status = IMRS_KO;
	}

	return(status);
}
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
IMRT_Status  SDM_MsgMap_Proceed( IMRT_MsgMap  *MsgMap_Ptr, IMRT_SDMCB_Field_Map  *CCB_Field_Map_Ptr)
{

    NDT_Status status;

    if( ( status = ND_DataStruct_Traverse( MsgMap_Ptr->FieldMap_Struct_Ptr, IMRD_SDMD_CMD_EXECUTE, stdout, NDD_RECURSIVE_MODE_PARENT,CCB_Field_Map_Ptr)) != NDS_OK)
    {
	printf( "ND_DataStruct_Traverse() failed (%d) !\n", status);
    }
    return((IMRT_Status)status);
}

void SDM_InitMsgMap(IMRT_MsgMap * mymsgmap)
{
  memset(mymsgmap->Name,0,sizeof(mymsgmap->Name));
  mymsgmap->Msg_Input_Ptr   = NULL;
  mymsgmap->Msg_Output_Ptr  = NULL;
  mymsgmap->FieldMap_Struct_Ptr   = NULL;
}

void SDM_InitFieldMap(IMRT_FieldMap * myfieldmap)
{
 myfieldmap->Id = IMRD_FIELDMAP_ID_UNKNOWN;
 myfieldmap->Field_Input_Ptr  = NULL;
 myfieldmap->Field_Output_Ptr = NULL;
}
