/*----------------------------------------------------------------------------*/
/* File: mapping_info_xml.c   		         		  	      */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#define _LIBSTATIC_DATA_MNG_
#define _MAPPING_INFO_XML_C_

#include "mapping_info_xml.h" 


IMRT_Status SDM_MsgMap_Ctxt_Close(SDMT_MsgMap_Contexte * MsgMap_Ctx_Ptr)
{
	NDT_Status   nd_status;

	if(MsgMap_Ctx_Ptr->RootMsgMap != NULL)
	{
		if( ( nd_status = ND_DataStruct_Close(MsgMap_Ctx_Ptr->RootMsgMap)) != NDS_OK)
		{
		    IMRD_LM_LOG_ERROR_1( "MsgMap Struct close failed: (%d) !", nd_status);
		}
	}

    if(MsgMap_Ctx_Ptr->SaxCtxHdl != NULL)
    {
    	xmlFreeParserCtxt(MsgMap_Ctx_Ptr->SaxCtxHdl);
    }

}





IMRT_Status SDM_XML_MapInfo_Init(IMRT_Path    Lib_Path)
{
    SDMT_MsgMap_Contexte MsgMap_Ctx;
    IMRT_Status  status;
    char	     Xml_File_Name[255];

    
    IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Loading MsgMap...");

    MsgMap_Ctx.flagerror = 0;

    MsgMap_Ctx.CptMsgMap = IMRD_MSGMAP_ID_UNKNOWN;
    MsgMap_Ctx.CptFieldMap = IMRD_FIELDMAP_ID_UNKNOWN;
    memset(MsgMap_Ctx.PathArray,0,sizeof(MsgMap_Ctx.PathArray));
    MsgMap_Ctx.PresentLevelTag = IMRD_SDMD_MIN_LEVEL_TAG;

    memset(Xml_File_Name,0,sizeof(Xml_File_Name));
    strcpy(Xml_File_Name, Lib_Path);
    strcat(Xml_File_Name, "/");
    strcat(Xml_File_Name, SDMD_MSGMAP_FILE);

    status = SDM_XML_Load_MsgMap(Xml_File_Name, &(MsgMap_Ctx));

    if(status != IMRS_OK)
    {
	IMRD_LM_LOG_ERROR_0( "Can't load MsgMap !");
    }
    else
    {
	IMRG_SDM_Base.RootMsgMap = MsgMap_Ctx.RootMsgMap;
	IMRD_LM_LOG_INFO_1( "MsgMap loaded: (%d)", IMRG_SDM_Base.RootMsgMap->Index_Tab[NDD_INDEX_PRIMARY].Node_Number);
    }
    
    return(status);
}





static IMRT_Status error(SDMT_MsgMap_Contexte *ctx , const char *msg, ...)
{
    xmlParserCtxt * SaxCtx;
  
    SaxCtx = ctx->SaxCtxHdl;
  
    IMRD_LM_LOG_ERROR_1( "CallBack error: <%s> !",msg);

    SaxCtx->sax = &ErrorSAXHandlerStruct;
    ctx->flagerror = 1;
    ctx->FataError.errorCode = SDMD_XML_CODE_ERROR_1;
    strcpy(ctx->FataError.libError,msg) ;
}





IMRT_Status startElementNsMap( SDMT_MsgMap_Contexte * ctx , 
			const xmlChar *localname,
			const xmlChar *prefix,
			const xmlChar *URI,
			int nb_namespaces,
			const xmlChar **namespaces,
			int nb_attributes,
			int nb_defaulted,
			const xmlChar **attributes)
{
    NDT_Status         nd_status;
    const char         functionname [] = "startElementNsMap";
    char               DescErr[IMRD_SDMD_DESCERR_LEN];
 

    if(strcmp((const char*)localname,IMRD_SDMD_TAG_FIELDMAP) == 0)
    {
	if( ( nd_status = ND_Value_Alloc( ctx->CurrentMsgMap->FieldMap_Struct_Ptr, (void  **)&(ctx->CurrentFieldMap)) != NDS_OK))
	{
	    IMRD_LM_LOG_ERROR_1( "ND_Value_Alloc() failed (%d) !", nd_status);

	    snprintf(DescErr,sizeof(DescErr),SDMD_XML_LIB_ERROR_3,IMRD_SDMD_TAG_FIELDMAP);
	    SDM_RaiseError(ctx->SaxCtxHdl,&(ctx->FataError),&(ctx->flagerror), functionname , DescErr, SDMD_XML_CODE_ERROR_3 );
	}
	else
	{
	    ctx->PresentLevelTag = ctx->PresentLevelTag + 1;
	    strcpy(ctx->PathArray[ctx->PresentLevelTag], (char*)localname);
	}
    }
    else if(strcmp((const char*)localname, IMRD_SDMD_TAG_MSGMAP) == 0)
    {
	if( ( nd_status = ND_Value_Alloc( ctx->RootMsgMap, (void  **)&(ctx->CurrentMsgMap)) != NDS_OK))
	{
	    IMRD_LM_LOG_ERROR_1( "ND_Value_Alloc() failed (%d) !", nd_status);
	    
	    snprintf(DescErr,sizeof(DescErr),SDMD_XML_LIB_ERROR_3,IMRD_SDMD_TAG_MSGMAP);
	    SDM_RaiseError(ctx->SaxCtxHdl,&(ctx->FataError),&(ctx->flagerror), functionname , DescErr, SDMD_XML_CODE_ERROR_3 );
	}
	else
	{
	    ctx->PresentLevelTag = ctx->PresentLevelTag + 1;
	    strcpy(ctx->PathArray[ctx->PresentLevelTag], (char*)localname);
	}
    }
    else if(strcmp((const char*)localname, IMRD_SDMD_TAG_FIELDMAPSTRUCT) == 0)
    {
	ctx->PresentLevelTag = ctx->PresentLevelTag + 1;
	strcpy(ctx->PathArray[ctx->PresentLevelTag], (char*)localname);
    }

}





IMRT_Status endElementNsMap( SDMT_MsgMap_Contexte * ctx, 
		      const xmlChar *localname,
		      const xmlChar *prefix,
		      const xmlChar *URI)
{
    IMRT_Status   status_tmp;
    NDT_Node *    node_ptr;
    NDT_Status    nd_status;
    IMRT_Msg      message;
    IMRT_Field    field;
    const char    functionname [] = "endElementNsMap";
    char          DescErr[IMRD_SDMD_DESCERR_LEN];


    if (strcmp((const char *)localname, IMRD_SDMD_TAG_NAME) == 0)
    {
	strcpy(ctx->CurrentMsgMap->Name, ctx->value);
    }
    else if (strcmp((const char*)localname,IMRD_SDMD_TAG_INPUTMSG) == 0)
    {
	strcpy(message.Name,ctx->value);
	if(((nd_status = ND_Index_Node_Find( &node_ptr, IMRG_SDM_Base.RootMsg, IMRD_SDMD_MSG_IDX_NAME, &message, NULL)) != NDS_OK)||
	   (node_ptr == NULL))
	{
	    IMRD_LM_LOG_ERROR_1( "IN Msg lookup failed: [%s] !", ctx->value);

	    snprintf(DescErr,sizeof(DescErr), SDMD_XML_LIB_ERROR_4,IMRD_SDMD_TAG_INPUTMSG,message.Name);
	    SDM_RaiseError(ctx->SaxCtxHdl,&(ctx->FataError),&(ctx->flagerror),functionname,DescErr,SDMD_XML_CODE_ERROR_4);
	}
	else
	{
	    ctx->CurrentMsgMap->Msg_Input_Ptr = node_ptr->Value;
	}
    }
    else if (strcmp((const char*)localname,IMRD_SDMD_TAG_OUTPUTMSG) == 0)
    {
	strcpy(message.Name,ctx->value);
	if( ( ( nd_status = ND_Index_Node_Find( &node_ptr, IMRG_SDM_Base.RootMsg, IMRD_SDMD_MSG_IDX_NAME,&message,NULL)) != NDS_OK)||
	    ( node_ptr == NULL))
	{
	    IMRD_LM_LOG_ERROR_1( "OUT Msg lookup failed: [%s] !", ctx->value);

	    snprintf(DescErr,sizeof(DescErr),SDMD_XML_LIB_ERROR_4,IMRD_SDMD_TAG_OUTPUTMSG,message.Name);
	    SDM_RaiseError(ctx->SaxCtxHdl,&(ctx->FataError),&(ctx->flagerror),functionname,DescErr,SDMD_XML_CODE_ERROR_4);
	}
	else
	{
	    ctx->CurrentMsgMap->Msg_Output_Ptr = node_ptr->Value;
	}
    }
    else if (strcmp((const char*)localname,IMRD_SDMD_TAG_INPUTFIELD) == 0)
    {
	strcpy(field.Name, ctx->value);
	if( ( ( nd_status = ND_Index_Node_Find(&node_ptr, ctx->CurrentMsgMap->Msg_Input_Ptr->Field_Struct_Ptr, IMRD_SDMD_FIELD_IDX_NAME,&field,NULL)) != NDS_OK)||
	    ( node_ptr == NULL))
	{
	    printf("\nErreur lors de la recherche du In Field <%s> %s\n",ctx->value, ctx->CurrentMsgMap->Msg_Input_Ptr->Name);
	    snprintf(DescErr,sizeof(DescErr),SDMD_XML_LIB_ERROR_4,IMRD_SDMD_TAG_INPUTFIELD,field.Name);
	    SDM_RaiseError(ctx->SaxCtxHdl,&(ctx->FataError),&(ctx->flagerror),functionname,DescErr,SDMD_XML_CODE_ERROR_4);
	}
	else
	{
	    ctx->CurrentFieldMap->Field_Input_Ptr = node_ptr->Value;
	}
    }
    else if (strcmp((const char*)localname,IMRD_SDMD_TAG_OUTPUTFIELD) == 0)
    {
	strcpy(field.Name, ctx->value);
	if(((nd_status = ND_Index_Node_Find(&node_ptr, ctx->CurrentMsgMap->Msg_Output_Ptr->Field_Struct_Ptr,
					    IMRD_SDMD_FIELD_IDX_NAME,&field,NULL)) != NDS_OK)||
	   (node_ptr == NULL))
	{
	    snprintf(DescErr,sizeof(DescErr),SDMD_XML_LIB_ERROR_4,IMRD_SDMD_TAG_OUTPUTFIELD, field.Name);
	    SDM_RaiseError(ctx->SaxCtxHdl,&(ctx->FataError),&(ctx->flagerror),functionname,DescErr,SDMD_XML_CODE_ERROR_4);
	}
	else
	{
	    ctx->CurrentFieldMap->Field_Output_Ptr = node_ptr->Value;
	}
    }
    else if (strcmp((const char*)localname,IMRD_SDMD_TAG_FIELDMAP) == 0)
    {
	ctx->CurrentFieldMap->Id = ctx->CptFieldMap;
	
	/*Creation de l'identifiant name de l'objet FieldMap*/
	if( ( nd_status = ND_DataStruct_Value_Add( ctx->CurrentMsgMap->FieldMap_Struct_Ptr, (void *)(ctx->CurrentFieldMap))) != NDS_OK)
	{
	    IMRD_LM_LOG_ERROR_1( "FieldMap add failed: (%d) !", nd_status);

	    snprintf(DescErr,sizeof(DescErr),SDMD_XML_LIB_ERROR_2,IMRD_SDMD_TAG_FIELDMAP );
	    SDM_RaiseError(ctx->SaxCtxHdl,&(ctx->FataError),&(ctx->flagerror), functionname , DescErr, SDMD_XML_CODE_ERROR_2 );
	}
	else
	{
	    ctx->CptFieldMap ++;
	    memset(ctx->PathArray[ctx->PresentLevelTag],0,sizeof(IMRD_SDMD_SIZE_TAG_SIZE));
	    ctx->PresentLevelTag = ctx->PresentLevelTag - 1;
	}
    }
    else if (strcmp((const char*)localname, IMRD_SDMD_TAG_MSGMAP) == 0)
    {
	ctx->CptMsgMap ++;
	ctx->CurrentMsgMap->Id = ctx->CptMsgMap;

	/*Creation de l'identifiant name de l'objet MsgMap*/
	if( ( nd_status = ND_DataStruct_Value_Add( ctx->RootMsgMap, (void  *)(ctx->CurrentMsgMap))) != NDS_OK)
	{
	    IMRD_LM_LOG_ERROR_1( "MsgMap add failed: (%d) !", nd_status);
	    
	    snprintf(DescErr,sizeof(DescErr),SDMD_XML_LIB_ERROR_2,IMRD_SDMD_TAG_MSGMAP );
	    SDM_RaiseError(ctx->SaxCtxHdl,&(ctx->FataError),&(ctx->flagerror), functionname , DescErr, SDMD_XML_CODE_ERROR_2 );
	}
	else
	{
	    memset(ctx->PathArray[ctx->PresentLevelTag],0,sizeof(IMRD_SDMD_SIZE_TAG_SIZE));
	    ctx->PresentLevelTag = ctx->PresentLevelTag - 1;
	}
    }
    else if (strcmp((const char*)localname, IMRD_SDMD_TAG_FIELDMAPSTRUCT) == 0)
    {
	memset(ctx->PathArray[ctx->PresentLevelTag],0,sizeof(IMRD_SDMD_SIZE_TAG_SIZE));
	ctx->PresentLevelTag = ctx->PresentLevelTag - 1;
    }
    else if (strcmp((const char *)localname, IMRD_SDMD_TAG_VERSION) == 0)
    {
		IMRD_LM_LOG_INFO_1( "[MsgMap]: [%s]", ctx->value);
		status_tmp = SDM_Set_Struct_Version((IMRT_Data_Version *)ctx->RootMsgMap->User_Ptr, (char *)ctx->value);
    }

}





IMRT_Status Map_Info_characters(SDMT_MsgMap_Contexte * ctx , const xmlChar *ch, int len)
{
    char output[40];
    int i;
    
    memset(ctx->value,0, sizeof(ctx->value));
    
    for (i = 0;(i<len) && (i < IMRD_SDMD_VALUE_LEN);i++)
	ctx->value[i] = ch[i];
    ctx->value[i] = 0;

}

IMRT_Status SDM_XML_Load_MsgMap( char *demo_file, SDMT_MsgMap_Contexte *mycontexte)
{
    IMRT_Status     status;
    NDT_Status      nd_status;
    char * docname = NULL;
    NDT_Index_Type  *index_type_ptr;
    xmlDocPtr doc = NULL;
    xmlDtdPtr dtd;
    xmlParserCtxtPtr ctxt =NULL;
    xmlSAXHandlerPtr SAXHandler;
    xmlSAXHandlerPtr SAXHandler_sav;
    const char functionname[] ="SDM_XML_Load_MsgMap";
    char DescErr[IMRD_SDMD_DESCERR_LEN];
	IMRT_Data_Version   *Data_Version_Ptr;

    LIBXML_TEST_VERSION
	xmlKeepBlanksDefault(0);	
    xmlSAXDefaultVersion(2);
    xmlLineNumbersDefault(1);
	
    status = IMRS_OK;
    docname = demo_file;
    ctxt = xmlNewParserCtxt();

    /*initialisation des indexes*/
    index_type_ptr =  MsgMap_Struct_initial_idx_tab;

    if(ctxt != NULL)
    {
	mycontexte->flagerror  = IMRS_OK;
	SAXHandler       = &SaxMapHandlerStruct;
	mycontexte->SaxCtxHdl  = ctxt;
	ctxt->userData        = mycontexte;
	SAXHandler_sav        = ctxt->sax;
	ctxt->sax             = SAXHandler;
	ctxt->sax2            = 1 ;
	    
	Data_Version_Ptr = (IMRT_Data_Version *) malloc(sizeof(IMRT_Data_Version));
	Data_Version_Ptr->Label[0] = '\0';
	Data_Version_Ptr->Date[0] = '\0';
	if( ( nd_status = ND_DataStruct_Open(&(mycontexte->RootMsgMap), IMRD_SDMD_MSGMAP_STRUCT_IDX_NB, index_type_ptr, "SDM_MsgMap_Manager",SDM_MsgMap_Manager, NULL, NULL, NULL, NULL,0, Data_Version_Ptr)) != NDS_OK)
	{
	    status = IMRS_KO;

	    IMRD_LM_LOG_ERROR_1( "Can't open RootMsgMap: (%d) !", nd_status);

	    snprintf(DescErr,sizeof(DescErr),SDMD_XML_LIB_ERROR_5,IMRD_SDMD_TAG_MSGMAP);
	    SDM_RaiseError(mycontexte->SaxCtxHdl,&(mycontexte->FataError),&(mycontexte->flagerror), functionname, DescErr, SDMD_XML_CODE_ERROR_5 );
	}
	else
	{
	    doc = xmlCtxtReadFile(ctxt,docname,NULL,XML_PARSE_DTDATTR+XML_PARSE_DTDVALID);
	    if (ctxt->lastError.code != XML_ERR_OK)
	    {
		status = IMRS_KO;
		printf("Erreur de parsing du fichier %s.\n%s\n", docname, ctxt->lastError.message);
	    }
	    else if(mycontexte->flagerror != IMRS_OK)
	    {
		status = IMRS_KO;
		printf("%s\n", mycontexte->FataError.libError);
	    }
	    else
	    {
		index_type_ptr = MsgMap_Struct_final_idx_tab;
		status = SDM_DataStruct_Finalize(mycontexte->RootMsgMap,index_type_ptr, IMRD_SDMD_MSGMAP_STRUCT_IDX_NB);
		/*SDM_DataStruct_Dump( mycontexte->RootMsgMap);*/
	    }
	}
	
	if((status != IMRS_OK)&&(mycontexte->RootMsgMap != NULL))
	{
	    if(mycontexte->CurrentMsgMap != NULL)
	    {
		if(mycontexte->CurrentMsgMap->FieldMap_Struct_Ptr != NULL)
		{
		    nd_status = ND_DataStruct_Close(mycontexte->CurrentMsgMap->FieldMap_Struct_Ptr);
		}
		if(nd_status = NDS_OK)
		{
		    free(mycontexte->CurrentMsgMap);
		}    
	    }
            
	    if(mycontexte->CurrentFieldMap != NULL)
	    {
		free(mycontexte->CurrentFieldMap);
	    }
	    if(mycontexte->RootMsgMap != NULL)
	    {
		nd_status = ND_DataStruct_Close(mycontexte->RootMsgMap);
	    }
	}
	
		  
	ctxt->sax = SAXHandler_sav;
	xmlFreeParserCtxt(ctxt);
    }
    else
    {
	status = IMRS_KO;
    }
    
    return(status);
}





/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
IMRT_Status  SDM_XML_Dump_MsgMap_Header(FILE *dump_file)
{
	fprintf(dump_file, "<?xml version=\"1.0\" ?>\n");
	fprintf(dump_file, "<!-- MsgMap Struct definitions -->\n");
	fprintf(dump_file, "<%s>\n", IMRD_SDMD_TAG_MSGMAP_STRUCT);
	fprintf(dump_file, "<version>@(#) VERSIONIMR: $Label:  $   $Date: 2008/11/12 02:25:23 $   $Workfile:   mapping_info_xml.c  $</version>\n", 
				((IMRT_Data_Version *)IMRG_SDM_Base.RootMsgMap->User_Ptr)->Label,
				((IMRT_Data_Version *)IMRG_SDM_Base.RootMsgMap->User_Ptr)->Date);
	return( IMRS_OK);

}





/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
IMRT_Status  SDM_XML_Dump_MsgMap_Body(FILE *dump_file)
{
    IMRT_Status  status;
    NDT_Status   nd_status;
    
    if( ( nd_status = ND_DataStruct_Traverse( IMRG_SDM_Base.RootMsgMap, IMRD_SDMD_CMD_API_XML_DUMP, dump_file)) != NDS_OK)
    {
	status = IMRS_KO;
	printf( "XML Dump RootMsgMap failed (%d)...\n");
    }
    else
    {
	status = IMRS_OK;		
    }
    
    return(status);
}





/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
IMRT_Status  SDM_XML_Dump_MsgMap_Footer(FILE *dump_file)
{
    fprintf(dump_file, "<!-- MsgMap Struct definitions footer -->\n");
    fprintf(dump_file, "</%s>\n", IMRD_SDMD_TAG_MSGMAP_STRUCT);

    return( IMRS_OK);
}





/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
IMRT_Status  SDM_XML_Dump_MsgMap(void)
{
    IMRT_Status     status;
    char	    dump_filename[255];
    FILE	   *dump_file;
    
    strcpy(dump_filename, SDMD_MSGMAP_FILE);
	
    if( ( dump_file = fopen( dump_filename, "w")) == NULL)
    {
	status = IMRS_KO;
	printf( "Can't open dump file (%s) !\n", dump_filename);
    }
    else
    {
	status = SDM_XML_Dump_MsgMap_Header(dump_file);
	if(status == IMRS_OK)
	{
	    status = SDM_XML_Dump_MsgMap_Body(dump_file);
	    if(status == IMRS_OK)
	    {
		status = SDM_XML_Dump_MsgMap_Footer(dump_file);
	    }
	}
	
	fclose( dump_file);
    }
    
    return(status);
}





/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
IMRT_Status SDM_XML_MapInfo_Dump(void)
{
	IMRT_Status     status;

	status = SDM_XML_Dump_MsgMap();

	return(status);
}





/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/
