/*----------------------------------------------------------------------------*/
/* File: mapping_info_xml.h 				         	      */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#ifndef _MAPPING_INFO_XML_H_
#   define _MAPPING_INFO_XML_H_



/*----------------------------------------------------------------------------*/
/* Global Include                                                             */
/*----------------------------------------------------------------------------*/

#   include <libstatic_data_mng.h>

/*----------------------------------------------------------------------------*/
/* Local constantes                                                           */
/*----------------------------------------------------------------------------*/

/*definition des tags xml pour le chargement des structure MsgMap et FieldMap*/

#define IMRD_SDMD_TAG_NAME              "name"
#define IMRD_SDMD_TAG_INPUTMSG          "inputmessage"
#define IMRD_SDMD_TAG_OUTPUTMSG         "outputmessage"
#define IMRD_SDMD_TAG_FIELDMAP          "fieldmap"
#define IMRD_SDMD_TAG_FIELDMAPSTRUCT    "fieldmapstruct"
#define IMRD_SDMD_TAG_INPUTFIELD        "inputfield"
#define IMRD_SDMD_TAG_OUTPUTFIELD       "outputfield"
#define IMRD_SDMD_TAG_MSGFIELDMAPSTRUCT "msgfieldmapstruct"



/*----------------------------------------------------------------------------*/
/* Local Definition                                                           */
/*----------------------------------------------------------------------------*/


typedef struct SDMT_MsgMap_Contexte
{
  char value[IMRD_SDMD_VALUE_LEN];
  IMRT_Array_Xml_Path PathArray;
  IMRT_Level_Tag PresentLevelTag;
  IMRT_MsgMap_Id    CptMsgMap;
  IMRT_FieldMap_Id  CptFieldMap;
  IMRT_FieldMap  * CurrentFieldMap;
  IMRT_MsgMap    * CurrentMsgMap;
  NDT_Root       * RootMsgMap;
  xmlParserCtxt  * SaxCtxHdl; 
  IMRT_Error     FataError;
  IMRT_Status    flagerror;
}SDMT_MsgMap_Contexte;

/*----------------------------------------------------------------------------*/
/* Prototypes                                                                 */
/*----------------------------------------------------------------------------*/

#   ifdef _MAPPING_INFO_XML_C_

static IMRT_Status error(SDMT_MsgMap_Contexte *ctx , const char *msg, ...);
IMRT_Status Map_Info_characters(SDMT_MsgMap_Contexte * ctx , const xmlChar *ch, int len);
IMRT_Status endElementNsMap( SDMT_MsgMap_Contexte * ctx, 
							        const xmlChar *localname,
							        const xmlChar *prefix,
							        const xmlChar *URI);
IMRT_Status startElementNsMap( SDMT_MsgMap_Contexte * ctx , 
							          const xmlChar *localname,
							          const xmlChar *prefix,
							          const xmlChar *URI,
							          int nb_namespaces,
							          const xmlChar **namespaces,
							          int nb_attributes,
							          int nb_defaulted,
							          const xmlChar **attributes);
IMRT_Status SDM_XML_Load_MsgMap( char *demo_file, SDMT_MsgMap_Contexte *mycontexte);
IMRT_Status SDM_MsgMap_Ctxt_Close(SDMT_MsgMap_Contexte * MsgMap_Ctx_Ptr);
IMRT_Status SDM_XML_MapInfo_Init(IMRT_Path    Lib_Path);
IMRT_Status  SDM_XML_Dump_MsgMap_Header(FILE *dump_file);
IMRT_Status  SDM_XML_Dump_MsgMap_Body(FILE *dump_file);
IMRT_Status  SDM_XML_Dump_MsgMap_Footer(FILE *dump_file);
IMRT_Status  SDM_XML_Dump_MsgMap(void);
IMRT_Status  SDM_XML_MapInfo_Dump(void);

#   else

extern IMRT_Status SDM_XML_Load_MsgMap( char *demo_file, SDMT_MsgMap_Contexte *mycontexte);
extern IMRT_Status SDM_XML_MapInfo_Init(IMRT_Path    Lib_Path);
extern IMRT_Status  SDM_XML_Dump_MsgMap_Header(FILE *dump_file);
extern IMRT_Status  SDM_XML_Dump_MsgMap_Body(FILE *dump_file);
extern IMRT_Status  SDM_XML_Dump_MsgMap_Footer(FILE *dump_file);
extern IMRT_Status  SDM_XML_Dump_MsgMap(void);
extern IMRT_Status  SDM_XML_MapInfo_Dump(void);

#   endif


#   ifdef _MAPPING_INFO_XML_C_


/*xmlSAXHandler Map_Info_Error_SAX_Hdl_Struct = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    1,
    NULL,
    NULL,
    NULL,
    NULL
    };*/

xmlSAXHandler SaxMapHandlerStruct = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    (charactersSAXFunc)(Map_Info_characters),
    NULL,
    NULL,
    NULL,
    NULL,
    (errorSAXFunc)(error),
    NULL,
    NULL,
    NULL,
    NULL,
    1,
    NULL,
    (startElementNsSAX2Func)(startElementNsMap),
    (endElementNsSAX2Func)(endElementNsMap),
    NULL
};


#   endif




/*----------------------------------------------------------------------------*/

#endif
