/*----------------------------------------------------------------------------*/
/* File: struct_info_mng.c   		         		  	      */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#define _LIBSTATIC_DATA_MNG_
#define _STRUCT_INFO_MNG_C_

#include "struct_info_mng.h" 




/*----------------------------------------------------------------------------*/
/* Cnv Manager                                                               */
/*----------------------------------------------------------------------------*/

NDT_Status  SDM_Cnv_Manager( NDT_Root  *Root_Ptr, NDT_Index_Id  Index_Id, NDT_Node  *Node_Ptr, NDT_Command  Command, va_list  Args)
{
    NDT_Command_Name  Command_Name;


    switch( Command)
    {
	case NDD_CMD_MANAGER_VERSION:
        {
            NDT_Version_Name  *Version_Name_Ptr = (NDT_Version_Name  *)va_arg( Args, NDT_Version_Name  *);


            Command_Name = "NDD_CMD_MANAGER_VERSION";
            *Version_Name_Ptr = "Revision: 2.1    Name: libnode-2_0_4-1    Date: 2002/02/28 22:00:35    Author: agibert ";

            return( NDS_OK);
        }

        case NDD_CMD_INDEX_GET:
        {
            /*
	      NDT_Index_Id  *Reply_Index_Id_Ptr = (NDT_Index_Id  *)va_arg( Args, NDT_Index_Id  *);
	      NDT_Command   *Reply_Command_Ptr  = (NDT_Command  *)va_arg( Args, NDT_Command  *);
	      NDT_Command    Cmd                = (NDT_Command)va_arg( Args, NDT_Command);
	      void          *Cnv_ptr            = (void  *)va_arg( Args, void  *);
            */
            NDT_Index_Id  *Reply_Index_Id_Ptr = (NDT_Index_Id  *)va_arg( Args, NDT_Index_Id  *);
            NDT_Command   *Reply_Command_Ptr  = (NDT_Command  *)va_arg( Args, NDT_Command  *);
            NDT_Command    Cmd                = (NDT_Command)va_arg( Args, NDT_Command);
            IMRT_Cnv      *Cnv_ptr            = (IMRT_Cnv  *)va_arg( Args, IMRT_Cnv  *);


            Command_Name = "NDD_CMD_INDEX_GET";

            switch(Cmd)
            {
                case IMRD_SDMD_CMD_INDEX0_PRINT:
                {
                    *Reply_Index_Id_Ptr = 0;
                    *Reply_Command_Ptr = NDD_CMD_VALUE_PRINT;

                    break;
                }

				case IMRD_SDMD_CMD_API_SQL_DUMP:
				{
                    *Reply_Index_Id_Ptr = IMRD_SDMD_CNV_IDX_ID;
                    *Reply_Command_Ptr = IMRD_SDMD_CMD_API_SQL_DUMP;
                    break;
				}

				case IMRD_SDMD_CMD_API_XML_DUMP:
				{
                    *Reply_Index_Id_Ptr = IMRD_SDMD_CNV_IDX_ID;
                    *Reply_Command_Ptr = IMRD_SDMD_CMD_API_XML_DUMP;
                    break;
				}

                default:
                {
                    *Reply_Index_Id_Ptr = Index_Id;
                    *Reply_Command_Ptr = Cmd;

                    break;
                }
            }

            return( NDS_OK);
        }

        case NDD_CMD_VALUE_ALLOC:
        {
            /*
	      void       **Cnv_ptr_Ptr   = (void **)va_arg( Args, void  **);
	      va_list      user_args     = (va_list)va_arg( Args, va_list);
	      user_type    user_data     = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
            */
            IMRT_Cnv  **Cnv_ptr_Ptr  = va_arg( Args, IMRT_Cnv  **);


            Command_Name = "NDD_CMD_VALUE_ALLOC";

            if( ( *Cnv_ptr_Ptr = (IMRT_Cnv  *)malloc( sizeof(IMRT_Cnv))) == NULL)
            {
                return( NDS_ERRMEM);
            }
            else
            {
                SDM_InitCnv(*Cnv_ptr_Ptr);
                return( NDS_OK);
            }
        }

        case NDD_CMD_VALUE_FREE:
        {
            /*
	      void       *Cnv_ptr   = (void *)va_arg( Args, void  *);
	      va_list     user_args = (va_list)va_arg( Args, va_list);
	      user_type   user_data = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
            */
            IMRT_Cnv  *Cnv_ptr = (IMRT_Cnv  *)va_arg( Args, IMRT_Cnv  *);
            Command_Name = "NDD_CMD_VALUE_FREE";

            free( Cnv_ptr);

            return( NDS_OK);
        }

        case NDD_CMD_VALUE_COMP:
        {
            /*
	      void       *Value1_Ptr = (void *)va_arg( Args, void  *);
	      void       *Value2_Ptr = (void *)va_arg( Args, void  *);
	      va_list     user_args  = (va_list)va_arg( Args, va_list);
	      user_type   user_data  = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
            */
            IMRT_Cnv    *Value1_Ptr = va_arg( Args, IMRT_Cnv  *);
            IMRT_Cnv    *Value2_Ptr = va_arg( Args, IMRT_Cnv  *);


            Command_Name = "NDD_CMD_VALUE_COMP";

            switch( Index_Id)
            {
                case IMRD_SDMD_CNV_IDX_ID:
                {
                    if( Value1_Ptr->Id < Value2_Ptr->Id)
                    {
                        return( NDS_LOWER);
                    }
                    else
                    {
                        if( Value1_Ptr->Id > Value2_Ptr->Id)
                        {
                            return( NDS_GREATER);
                        }
                        else
                        {
                            return( NDS_EQUAL);
                        }
                    }
                }

                case IMRD_SDMD_CNV_IDX_NAME:
                {
                    int     rc;


                    rc=strcmp( Value1_Ptr->Name, Value2_Ptr->Name);

                    if( rc <0)
                    {
                        return( NDS_LOWER);
                    }
                    else
                    {
                        if( rc > 0)
                        {
                            return( NDS_GREATER);
                        }
                        else
                        {
                            return( NDS_EQUAL);
                        }
                    }
                }

                default:
                {
                    printf( "Unknown COMP idx (%d) !\n", Index_Id);

                    return( NDS_KO);
                }
            }

            return( NDS_OK);
        }

        case NDD_CMD_VALUE_ADD:
        {
            /*
	      void       *Cnv_ptr   = (void *)va_arg( Args, void  *);
	      va_list     user_args = (va_list)va_arg( Args, va_list);
	      user_type   user_data = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
            */

            Command_Name = "NDD_CMD_VALUE_ADD";

            return( NDS_OK);
        }

        case NDD_CMD_VALUE_REMOVE:
        {
            /*
	      void       *Cnv_ptr   = (void *)va_arg( Args, void  *);
	      va_list     user_args = (va_list)va_arg( Args, va_list);
	      user_type   user_data = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
            */

            Command_Name = "NDD_CMD_VALUE_REMOVE";

            return( NDS_OK);
        }

        case NDD_CMD_VALUE_PRINT:
        {
            /*
	      NDT_Node             *Next_Node_Ptr   = (NDT_Node  *)va_arg( Args, NDT_Node  *);
	      va_list               lib_args        = (va_list)va_arg( Args, va_list);
	      FILE                 *Out             = (FILE  *)va_arg( lib_args, FILE  *);
	      NDT_Recursive_Mode    Recursive_Mode  = (NDT_Recursive_Mode)va_arg( lib_args, NDT_Recursive_Mode);
	      NDT_Recursive_Depth   Recursive_Depth = (NDT_Recursive_Depth)va_arg( lib_args, NDT_Recursive_Depth);
	      va_list               user_args       = (va_list)va_arg( lib_args, va_list);
	      user_type             user_data       = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
	      void                 *Cnv_ptr         = Node_Ptr->Value;
            */
            NDT_Node             *Next_Node_Ptr   = (NDT_Node  *)va_arg( Args, NDT_Node  *);
            va_list               lib_args        = (va_list)va_arg( Args, va_list);
            FILE                 *Out             = (FILE  *)va_arg( lib_args, FILE  *);
            NDT_Recursive_Mode    Recursive_Mode  = (NDT_Recursive_Mode)va_arg( lib_args, NDT_Recursive_Mode);
            NDT_Recursive_Depth   Recursive_Depth = (NDT_Recursive_Depth)va_arg( lib_args, NDT_Recursive_Depth);

            IMRT_Cnv             *Cnv_ptr         = Node_Ptr->Value;


            Command_Name = "NDD_CMD_VALUE_PRINT";

            fprintf( Out, "Id: (%d)   Name: (%s)\n", Cnv_ptr->Id, Cnv_ptr->Name);
            return( NDS_OK);

        }

        case NDD_CMD_INFO_PRINT:
        {
            /*
	      NDT_Node             *Next_Node_Ptr   = (NDT_Node  *)va_arg( Args, NDT_Node  *);
	      va_list               lib_args        = (va_list)va_arg( Args, va_list);
	      FILE                 *Out             = (FILE  *)va_arg( lib_args, FILE  *);
	      NDT_Recursive_Mode    Recursive_Mode  = (NDT_Recursive_Mode)va_arg( lib_args, NDT_Recursive_Mode);
	      NDT_Recursive_Depth   Recursive_Depth = (NDT_Recursive_Depth)va_arg( lib_args, NDT_Recursive_Depth);
	      va_list               user_args       = (va_list)va_arg( lib_args, va_list);
	      user_type             user_data       = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
	      void                 *Cnv_ptr         = Node_Ptr->Value;
            */

            Command_Name = "NDD_CMD_INFO_PRINT";

            return( NDS_OK);
        }

        case IMRD_SDMD_CMD_EXECUTE:
        { /*
            NDT_Node             *Next_Node_Ptr    = (NDT_Node  *)va_arg( Args, NDT_Node  *);
            va_list               lib_args         = (va_list)va_arg( Args, va_list);
            FILE                 *Out              = (FILE  *)va_arg( lib_args, FILE  *);
            NDT_Recursive_Mode    Recursive_Mode   = (NDT_Recursive_Mode)va_arg( lib_args, NDT_Recursive_Mode);
            NDT_Recursive_Depth   Recursive_Depth  = (NDT_Recursive_Depth)va_arg( lib_args, NDT_Recursive_Depth);
            NDT_Recursive_Offset  Recursive_Offset = (NDT_Recursive_Offset)va_arg( lib_args, NDT_Recursive_Offset);
            va_list               user_args        = (va_list)va_arg( lib_args, va_list);
            user_type             user_data        = (user_type)va_arg( user_args, user_type);
	    ...  = (...)va_arg( user_args, ...);
            void                 *Cnv_ptr          = Node_Ptr->Value;
	  */
            IMRT_Cnv   *  Cnv_ptr                  = (IMRT_Cnv*)Node_Ptr->Value;
            NDT_Node *Next_Node_Ptr                = (NDT_Node  *)va_arg( Args, NDT_Node  *);
            va_list lib_args                       = (va_list)va_arg( Args, va_list);
            FILE *Out                              = (FILE  *)va_arg( lib_args, FILE  *);
            NDT_Recursive_Mode    Recursive_Mode   = (NDT_Recursive_Mode)va_arg( lib_args, NDT_Recursive_Mode);
            IMRT_SDMCB_Cnv_Add   * CnvProceedFunc  = ( IMRT_SDMCB_Cnv_Add *)va_arg(lib_args, IMRT_SDMCB_Cnv_Add *);
            
            CnvProceedFunc(Cnv_ptr->Id, Cnv_ptr->Name, IMRG_SDM_Base.RootCnv->Index_Tab[NDD_INDEX_PRIMARY].Node_Number);

            return( NDS_OK);
        }

		case IMRD_SDMD_CMD_API_SQL_DUMP:
		{
		/*
			NDT_Node   *Next_Node_Ptr = (NDT_Node  *)va_arg( Args, NDT_Node  *);
			va_list     user_args     =     (va_list)va_arg( Args, va_list);
			user_type   user_data     =   (user_type)va_arg( user_args, user_type);
								  ... =         (...)va_arg( user_args, ...);

  			void       *Cnv_ptr      = Node_Ptr->Value;
		*/
			NDT_Node        *Next_Node_Ptr = (NDT_Node  *)va_arg( Args, NDT_Node  *);
			va_list          user_args     =     (va_list)va_arg( Args, va_list);
			FILE            *Out           =     (FILE  *)va_arg( user_args, FILE  *);

			IMRT_Cnv        *Cnv_Ptr       = Node_Ptr->Value;


			Command_Name = "IMRD_SDMD_CMD_API_SQL_DUMP";

			fprintf( Out, "INSERT INTO IMR_CNV(CNV_ID, CNV_NAME) VALUES(%d, '%s');\n",
					Cnv_Ptr->Id, Cnv_Ptr->Name);

			return( NDS_OK);
		}

		case IMRD_SDMD_CMD_API_XML_DUMP:
		{
		/*
			NDT_Node   *Next_Node_Ptr = (NDT_Node  *)va_arg( Args, NDT_Node  *);
			va_list     user_args     =     (va_list)va_arg( Args, va_list);
			user_type   user_data     =   (user_type)va_arg( user_args, user_type);
								  ... =         (...)va_arg( user_args, ...);

  			void       *Cnv_ptr       = Node_Ptr->Value;
		*/
			NDT_Node        *Next_Node_Ptr = (NDT_Node  *)va_arg( Args, NDT_Node  *);
			va_list          user_args     =     (va_list)va_arg( Args, va_list);
            FILE            *Out           =     (FILE  *)va_arg( user_args, FILE  *);

			IMRT_Cnv        *Cnv_Ptr       = Node_Ptr->Value;


			Command_Name = "IMRD_SDMD_CMD_API_XML_DUMP";

			fprintf( Out, "\t<%s>\n", IMRD_SDMD_TAG_CNV);

			fprintf( Out, "\t\t<%s>%s</%s>\n", IMRD_SDMD_TAG_NAME, Cnv_Ptr->Name, IMRD_SDMD_TAG_NAME);

			fprintf( Out, "\t</%s>\n", IMRD_SDMD_TAG_CNV);


			return( NDS_OK);
		}

        default:
        {
            printf( "SDM_Cnv_Manager() called with an undefined command %d\n", Command);
            return NDS_ERRAPI;
        }
    }

    printf( "SDM_Cnv_Manager() called with command %d (%s)\n", Command, Command_Name);
    return NDS_OK;
}


/*----------------------------------------------------------------------------*/
/* Field Manager                                                              */
/*----------------------------------------------------------------------------*/

NDT_Status SDM_Field_Manager( NDT_Root  *Root_Ptr, NDT_Index_Id  Index_Id, NDT_Node  *Node_Ptr, NDT_Command  Command, va_list  Args)
{
    NDT_Command_Name  Command_Name;
    NDT_Status         status;


    switch( Command)
    {
        case NDD_CMD_MANAGER_VERSION:
        {
            NDT_Version_Name  *Version_Name_Ptr = (NDT_Version_Name  *)va_arg( Args, NDT_Version_Name  *);


            Command_Name = "NDD_CMD_MANAGER_VERSION";

            *Version_Name_Ptr = "Revision: 2.1    Name: libnode-2_0_4-1    Date: 2002/02/28 22:00:35    Author: agibert ";

            return( NDS_OK);
        }

        case NDD_CMD_INDEX_GET:
        {
            /*
	      NDT_Index_Id  *Reply_Index_Id_Ptr = (NDT_Index_Id  *)va_arg( Args, NDT_Index_Id  *);
	      NDT_Command   *Reply_Command_Ptr  = (NDT_Command  *)va_arg( Args, NDT_Command  *);
	      NDT_Command    Cmd                = (NDT_Command)va_arg( Args, NDT_Command);
	      void          *Value_ptr          = (void  *)va_arg( Args, void  *);
            */
            NDT_Index_Id  *Reply_Index_Id_Ptr = (NDT_Index_Id  *)va_arg( Args, NDT_Index_Id  *);
            NDT_Command   *Reply_Command_Ptr  = (NDT_Command  *)va_arg( Args, NDT_Command  *);
            NDT_Command    Cmd                = (NDT_Command)va_arg( Args, NDT_Command);
            IMRT_Field    * Field_ptr         = (IMRT_Field  *)va_arg( Args, IMRT_Field  *);


            Command_Name = "NDD_CMD_INDEX_GET";

            switch(Cmd)
            {
                case IMRD_SDMD_CMD_INDEX0_PRINT:
                {
                    *Reply_Index_Id_Ptr = 0;
                    *Reply_Command_Ptr = NDD_CMD_VALUE_PRINT;

                    break;
                }

                case IMRD_SDMD_CMD_EXECUTE:
                {
                    *Reply_Index_Id_Ptr = 0;
                    *Reply_Command_Ptr = NDD_CMD_EXECUTE;
 				    break;
                }

				case IMRD_SDMD_CMD_API_SQL_DUMP:
				{
                    *Reply_Index_Id_Ptr = IMRD_SDMD_FIELD_IDX_ID;
                    *Reply_Command_Ptr = IMRD_SDMD_CMD_API_SQL_DUMP;
                    break;
				}

				case IMRD_SDMD_CMD_API_XML_DUMP:
				{
                    *Reply_Index_Id_Ptr = IMRD_SDMD_FIELD_IDX_ID;
                    *Reply_Command_Ptr = IMRD_SDMD_CMD_API_XML_DUMP;
                    break;
				}

                default:
                {
                    *Reply_Index_Id_Ptr = Index_Id;
                    *Reply_Command_Ptr = Cmd;

                    break;
                }
            }

            return( NDS_OK);
        }

        case NDD_CMD_VALUE_ALLOC:
        {
            /*
	      void       **Field_ptr_Ptr = (void **)va_arg( Args, void  **);
	      va_list      user_args     = (va_list)va_arg( Args, va_list);
	      user_type    user_data     = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
            */
            IMRT_Field  **Field_ptr_Ptr = va_arg( Args, IMRT_Field  **);


            Command_Name = "NDD_CMD_VALUE_ALLOC";

            if( ( *Field_ptr_Ptr = (IMRT_Field  *)malloc( sizeof(IMRT_Field))) == NULL)
            {
                return( NDS_ERRMEM);
            }
            else
            {
                SDM_InitField(*Field_ptr_Ptr);
                
                return( NDS_OK);
            }
        }

        case NDD_CMD_VALUE_FREE:
        {
            /*
	      void       *Field_ptr = (void *)va_arg( Args, void  *);
	      va_list     user_args = (va_list)va_arg( Args, va_list);
	      user_type   user_data = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
            */
            IMRT_Field  *Field_ptr = (IMRT_Field  *)va_arg( Args, IMRT_Field  *);


            Command_Name = "NDD_CMD_VALUE_FREE";

            free( Field_ptr);

            return( NDS_OK);
        }

        case NDD_CMD_VALUE_COMP:
        {
            /*
	      void       *Value1_Ptr = (void *)va_arg( Args, void  *);
	      void       *Value2_Ptr = (void *)va_arg( Args, void  *);
	      va_list     user_args  = (va_list)va_arg( Args, va_list);
	      user_type   user_data  = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
            */
            IMRT_Field    *Value1_Ptr = va_arg( Args, IMRT_Field  *);
            IMRT_Field    *Value2_Ptr = va_arg( Args, IMRT_Field  *);


            Command_Name = "NDD_CMD_VALUE_COMP";

            switch( Index_Id)
            {
                case IMRD_SDMD_FIELD_IDX_ID:
                {
                    if( Value1_Ptr->Id < Value2_Ptr->Id)
                    {
                        return( NDS_LOWER);
                    }
                    else
                    {
                        if( Value1_Ptr->Id > Value2_Ptr->Id)
                        {
                            return( NDS_GREATER);
                        }
                        else
                        {
                            return( NDS_EQUAL);
                        }
                    }
                }

                case IMRD_SDMD_FIELD_IDX_NAME:
                {
                    int     rc;


                    rc=strcmp( Value1_Ptr->Name, Value2_Ptr->Name);

                    if( rc <0)
                    {
                        return( NDS_LOWER);
                    }
                    else
                    {
                        if( rc > 0)
                        {
                            return( NDS_GREATER);
                        }
                        else
                        {
                            return( NDS_EQUAL);
                        }
                    }
                }

                default:
                {
                    printf( "Unknown COMP idx (%d) !\n", Index_Id);

                    return( NDS_KO);
                }
            }

            return( NDS_OK);
        }

        case NDD_CMD_VALUE_ADD:
        {
            /*
	      void       *Field_ptr = (void *)va_arg( Args, void  *);
	      va_list     user_args = (va_list)va_arg( Args, va_list);
	      user_type   user_data = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
            */

            Command_Name = "NDD_CMD_VALUE_ADD";

            return( NDS_OK);
        }

        case NDD_CMD_VALUE_REMOVE:
        {
            /*
	      void       *Field_ptr = (void *)va_arg( Args, void  *);
	      va_list     user_args = (va_list)va_arg( Args, va_list);
	      user_type   user_data = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
            */

            Command_Name = "NDD_CMD_VALUE_REMOVE";
            return( NDS_OK);
        }

        case NDD_CMD_VALUE_PRINT:
        {
            /*
	      NDT_Node             *Next_Node_Ptr   = (NDT_Node  *)va_arg( Args, NDT_Node  *);
	      va_list               lib_args        = (va_list)va_arg( Args, va_list);
	      FILE                 *Out             = (FILE  *)va_arg( lib_args, FILE  *);
	      NDT_Recursive_Mode    Recursive_Mode  = (NDT_Recursive_Mode)va_arg( lib_args, NDT_Recursive_Mode);
	      NDT_Recursive_Depth   Recursive_Depth = (NDT_Recursive_Depth)va_arg( lib_args, NDT_Recursive_Depth);
	      va_list               user_args       = (va_list)va_arg( lib_args, va_list);
	      user_type             user_data       = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
	      void                 *Field_ptr       = Node_Ptr->Value;
            */
            NDT_Node             *Next_Node_Ptr   =         (NDT_Node  *)va_arg( Args, NDT_Node  *);
            va_list               lib_args        =             (va_list)va_arg( Args, va_list);
            FILE                 *Out             =             (FILE  *)va_arg( lib_args, FILE  *);
            NDT_Recursive_Mode    Recursive_Mode  =  (NDT_Recursive_Mode)va_arg( lib_args, NDT_Recursive_Mode);
            NDT_Recursive_Depth   Recursive_Depth = (NDT_Recursive_Depth)va_arg( lib_args, NDT_Recursive_Depth);

            IMRT_Field           *Field_ptr       = Node_Ptr->Value;
			char                 Local_Exist;

            Command_Name = "NDD_CMD_VALUE_PRINT";

			if (Field_ptr->Exist == IMRD_TRUE)
			{
				Local_Exist = IMRD_YES;
			}
			else
			{
				Local_Exist = IMRD_NO;
			}

            fprintf( Out, "Name: (%s) Valueid: (%s) Exist: (%c)\n\tFixed - offset: (%d) size: (%d) Def_Val: (%s) Cnv_Name: (%s) Cnv_Attr: (%s)\n\tFML - tag: (%s) Def_Val: (%s) Cnv_Name: (%s) Cnv_Attr: (%s)\n\tXML - Tag: (%s) Def_Val: (%s) Cnv_Name: (%s) Cnv_Attr: (%s)\n\tTagged - Tag: (%s) Def_Val: (%s) Cnv_Name: (%s) Cnv_Attr: (%s)\n\tInternal - Tag: (%s) Def_Val: (%s) Cnv_Name: (%s) Cnv_Attr: (%s)\n\tDelimited - Id: (%d) Def_Val: (%s) Cnv_Name: (%s) Cnv_Attr: (%s)\n",
		     Field_ptr->Name, Field_ptr->Value_Id, Local_Exist,
		     Field_ptr->Fixed.Offset, Field_ptr->Fixed.Size, Field_ptr->Fixed.Def_Val, Field_ptr->Fixed.Cnv_Ptr->Name, Field_ptr->Fixed.Cnv_Attr,
		     Field_ptr->FML.Tag,      Field_ptr->FML.Def_Val,       Field_ptr->FML.Cnv_Ptr->Name,       Field_ptr->FML.Cnv_Attr,
		     Field_ptr->XML.Tag,      Field_ptr->XML.Def_Val,       Field_ptr->XML.Cnv_Ptr->Name,       Field_ptr->XML.Cnv_Attr,
		     Field_ptr->Tagged.Tag,   Field_ptr->Tagged.Def_Val,    Field_ptr->Tagged.Cnv_Ptr->Name,    Field_ptr->Tagged.Cnv_Attr, 
		     Field_ptr->Internal.Tag, Field_ptr->Internal.Def_Val,  Field_ptr->Internal.Cnv_Ptr->Name,  Field_ptr->Internal.Cnv_Attr, 
		     Field_ptr->Delimited.Id, Field_ptr->Delimited.Def_Val, Field_ptr->Delimited.Cnv_Ptr->Name, Field_ptr->Delimited.Cnv_Attr);

            return( NDS_OK);
        }

        case NDD_CMD_INFO_PRINT:
        {
            /*
	      NDT_Node             *Next_Node_Ptr   = (NDT_Node  *)va_arg( Args, NDT_Node  *);
	      va_list               lib_args        = (va_list)va_arg( Args, va_list);
	      FILE                 *Out             = (FILE  *)va_arg( lib_args, FILE  *);
	      NDT_Recursive_Mode    Recursive_Mode  = (NDT_Recursive_Mode)va_arg( lib_args, NDT_Recursive_Mode);
	      NDT_Recursive_Depth   Recursive_Depth = (NDT_Recursive_Depth)va_arg( lib_args, NDT_Recursive_Depth);
	      va_list               user_args       = (va_list)va_arg( lib_args, va_list);
	      user_type             user_data       = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
	      void                 *Field_ptr       = Node_Ptr->Value;
            */

            Command_Name = "NDD_CMD_INFO_PRINT";

            return( NDS_OK);
        }

        case NDD_CMD_EXECUTE:
        {
	    /*
	      NDT_Node             *Next_Node_Ptr   = (NDT_Node  *)va_arg( Args, NDT_Node  *);
	      va_list               lib_args        = (va_list)va_arg( Args, va_list);
	      FILE                 *Out             = (FILE  *)va_arg( lib_args, FILE  *);
	      NDT_Recursive_Mode    Recursive_Mode  = (NDT_Recursive_Mode)va_arg( lib_args, NDT_Recursive_Mode);
	      NDT_Recursive_Depth   Recursive_Depth = (NDT_Recursive_Depth)va_arg( lib_args, NDT_Recursive_Depth);
	      va_list               user_args       = (va_list)va_arg( lib_args, va_list);
	      user_type             user_data       = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
	      void                 *Field_ptr       = Node_Ptr->Value;
            */
            IMRT_Field *  Field_ptr            = (IMRT_Field *)Node_Ptr->Value;
            NDT_Node *Next_Node_Ptr            = (NDT_Node  *)va_arg( Args, NDT_Node  *);
            va_list lib_args                   = (va_list)va_arg( Args, va_list);
            FILE *Out                          = (FILE  *)va_arg( lib_args, FILE  *);
            NDT_Recursive_Mode Recursive_Mode  = (NDT_Recursive_Mode)va_arg( lib_args, NDT_Recursive_Mode);
            va_list user_args                  = (va_list)va_arg( Args, va_list);
            IMRT_SDM_IdentMessage * IdentStruct   = (IMRT_SDM_IdentMessage *)va_arg(lib_args,IMRT_SDM_IdentMessage *);
            
            if(Field_ptr->Exist == IMRD_TRUE)
            {
		status = IdentStruct-> Ident_Field_ExistFunc_Ptr(Field_ptr);
		
		
		/*if( ((Next_Node_Ptr == NULL) && (status == (NDT_Status)IMRS_NO_IDENT)) || (status == (NDT_Status)IMRS_KO))
		{
		    return( NDS_KO);
		}
                else if (status = (NDT_Status)IMRS_OK)
		{
		    return(NDS_IDENT_OK);
		    }*/
		
		if( status != IMRS_OK)
		{
		    return(status);
		}
            }

            if(strcmp(Field_ptr->Value_Id, IMRD_SDMD_EMPTY_STRING) != 0)
            {
		status = IdentStruct-> Ident_Field_ValueFunc_Ptr(Field_ptr, Field_ptr->Value_Id);
		
		
		/*if( ((Next_Node_Ptr == NULL) && (status == (NDT_Status)IMRS_NO_IDENT)) || (status == (NDT_Status)IMRS_KO) )
		{
		    return( NDS_KO);
		}
                else if (status == (NDT_Status)IMRS_OK)
                {
                    return(NDS_IDENT_OK);
		    }*/
                
		if( status != IMRS_OK)
		{
		    return(status);
		}

            }
	            
            return(NDS_OK);
        }

		case IMRD_SDMD_CMD_API_SQL_DUMP:
		{
		/*
			NDT_Node   *Next_Node_Ptr = (NDT_Node  *)va_arg( Args, NDT_Node  *);
			va_list     user_args     =     (va_list)va_arg( Args, va_list);
			user_type   user_data     =   (user_type)va_arg( user_args, user_type);
								  ... =         (...)va_arg( user_args, ...);

  			void       *Field_ptr     = Node_Ptr->Value;
		*/
			NDT_Node        *Next_Node_Ptr = (NDT_Node  *)va_arg( Args, NDT_Node  *);
			va_list          user_args     =     (va_list)va_arg( Args, va_list);
			FILE            *Out           =     (FILE  *)va_arg( user_args, FILE  *);
			IMRT_Field_Id    MsgId         =     (IMRT_Field_Id)va_arg( user_args, IMRT_Field_Id);

			IMRT_Field      *Field_Ptr     = Node_Ptr->Value;

			Command_Name = "IMRD_SDMD_CMD_API_SQL_DUMP";

			fprintf( Out, "INSERT INTO IMR_FIELD(FIELD_ID, FIELD_NAME, MSG_ID, FIELD_VALUEID, FIELD_EXIST) VALUES(%d, '%s', %d, '%s', ",
					Field_Ptr->Id, Field_Ptr->Name, MsgId, Field_Ptr->Value_Id);

			if(Field_Ptr->Exist == IMRD_TRUE)
			{
				fprintf( Out, "'%c');\n", IMRD_YES);
			}
			else
			{
				fprintf( Out, "'%c');\n", IMRD_NO);
			}



			if(Field_Ptr->Fixed.Size != -1)
			{
				fprintf( Out, "INSERT INTO IMR_FIXEDFIELD(FIELD_ID, FIXED_FIELD_OFFSET, FIXED_FIELD_SIZE, FIXED_FIELD_CNV_ID, FIXED_FIELD_CNV_ATTR, FIXED_FIELD_DEF_VAL) VALUES(%d, %d, %d, ",
					Field_Ptr->Id, Field_Ptr->Fixed.Offset, Field_Ptr->Fixed.Size);

				if(Field_Ptr->Fixed.Cnv_Ptr != NULL)
				{
					fprintf( Out, "%d, '%s', '%s');\n",
						Field_Ptr->Fixed.Cnv_Ptr->Id, Field_Ptr->Fixed.Cnv_Attr, Field_Ptr->Fixed.Def_Val);
				}
				else
				{
					fprintf( Out, "NULL, '%s', '%s');\n",
						Field_Ptr->Fixed.Cnv_Attr, Field_Ptr->Fixed.Def_Val);
				}
			}

			if(strcmp(Field_Ptr->FML.Tag, IMRD_SDMD_EMPTY_STRING) != 0)
			{
				fprintf( Out, "INSERT INTO IMR_FMLFIELD(FIELD_ID, FML_FIELD_TAG, FML_FIELD_CNV_ID, FML_FIELD_CNV_ATTR, FML_FIELD_DEF_VAL) VALUES(%d, '%s', ",
					Field_Ptr->Id, Field_Ptr->FML.Tag);

				if(Field_Ptr->FML.Cnv_Ptr != NULL)
				{
					fprintf( Out, "%d, '%s', '%s');\n",
						Field_Ptr->FML.Cnv_Ptr->Id, Field_Ptr->FML.Cnv_Attr, Field_Ptr->FML.Def_Val);
				}
				else
				{
					fprintf( Out, "NULL, '%s', '%s');\n",
						Field_Ptr->FML.Cnv_Attr, Field_Ptr->FML.Def_Val);
				}
			}

			if(strcmp(Field_Ptr->XML.Tag, IMRD_SDMD_EMPTY_STRING) != 0)
			{
				fprintf( Out, "INSERT INTO IMR_XMLFIELD(FIELD_ID, XML_FIELD_TAG, XML_FIELD_CNV_ID, XML_FIELD_CNV_ATTR, XML_FIELD_DEF_VAL) VALUES(%d, '%s', ",
					Field_Ptr->Id, Field_Ptr->XML.Tag);

				if(Field_Ptr->XML.Cnv_Ptr != NULL)
				{
					fprintf( Out, "%d, '%s', '%s');\n",
						Field_Ptr->XML.Cnv_Ptr->Id, Field_Ptr->XML.Cnv_Attr, Field_Ptr->XML.Def_Val);
				}
				else
				{
					fprintf( Out, "NULL, '%s', '%s');\n",
						Field_Ptr->XML.Cnv_Attr, Field_Ptr->XML.Def_Val);
				}
			}

			if(strcmp(Field_Ptr->Tagged.Tag, IMRD_SDMD_EMPTY_STRING) != 0)
			{
				fprintf( Out, "INSERT INTO IMR_TAGGEDFIELD(FIELD_ID, TAGGED_FIELD_TAG, TAGGED_FIELD_CNV_ID, TAGGED_FIELD_CNV_ATTR, TAGGED_FIELD_DEF_VAL) VALUES(%d, '%s', ",
					Field_Ptr->Id, Field_Ptr->Tagged.Tag);

				if(Field_Ptr->Tagged.Cnv_Ptr != NULL)
				{
					fprintf( Out, "%d, '%s', '%s');\n",
						Field_Ptr->Tagged.Cnv_Ptr->Id, Field_Ptr->Tagged.Cnv_Attr, Field_Ptr->Tagged.Def_Val);
				}
				else
				{
					fprintf( Out, "NULL, '%s', '%s');\n",
						Field_Ptr->Tagged.Cnv_Attr, Field_Ptr->Tagged.Def_Val);
				}
			}

			if(strcmp(Field_Ptr->Internal.Tag, IMRD_SDMD_EMPTY_STRING) != 0)
			{
				fprintf( Out, "INSERT INTO IMR_INTERNALFIELD(FIELD_ID, INTERNAL_FIELD_TAG, INTERNAL_FIELD_CNV_ID, INTERNAL_FIELD_CNV_ATTR, INTERNAL_FIELD_DEF_VAL) VALUES(%d, '%s', ",
					Field_Ptr->Id, Field_Ptr->Internal.Tag);

				if(Field_Ptr->Internal.Cnv_Ptr != NULL)
				{
					fprintf( Out, "%d, '%s', '%s');\n",
						Field_Ptr->Internal.Cnv_Ptr->Id, Field_Ptr->Internal.Cnv_Attr, Field_Ptr->Internal.Def_Val);
				}
				else
				{
					fprintf( Out, "NULL, '%s', '%s');\n",
						Field_Ptr->Internal.Cnv_Attr, Field_Ptr->Internal.Def_Val);
				}
			}

			if(Field_Ptr->Delimited.Id != IMRD_DELIMITED_ID_UNKNOWN)
			{
				fprintf( Out, "INSERT INTO IMR_DELIMITEDFIELD(FIELD_ID, DELIMITED_FIELD_ID, DELIMITED_FIELD_CNV_ID, DELIMITED_FIELD_CNV_ATTR, DELIMITED_FIELD_DEF_VAL) VALUES(%d, %d, ",
					Field_Ptr->Id, Field_Ptr->Delimited.Id);

				if(Field_Ptr->Delimited.Cnv_Ptr != NULL)
				{
					fprintf( Out, "%d, '%s', '%s');\n",
						Field_Ptr->Delimited.Cnv_Ptr->Id, Field_Ptr->Delimited.Cnv_Attr, Field_Ptr->Delimited.Def_Val);
				}
				else
				{
					fprintf( Out, "NULL, '%s', '%s');\n",
						Field_Ptr->Delimited.Cnv_Attr, Field_Ptr->Delimited.Def_Val);
				}
			}

			return( NDS_OK);
		}

		case IMRD_SDMD_CMD_API_XML_DUMP:
		{
		/*
			NDT_Node   *Next_Node_Ptr = (NDT_Node  *)va_arg( Args, NDT_Node  *);
			va_list     user_args     =     (va_list)va_arg( Args, va_list);
			user_type   user_data     =   (user_type)va_arg( user_args, user_type);
								  ... =         (...)va_arg( user_args, ...);

  			void       *Field_ptr     = Node_Ptr->Value;
		*/
			NDT_Node        *Next_Node_Ptr = (NDT_Node  *)va_arg( Args, NDT_Node  *);
			va_list          user_args     = (va_list)va_arg( Args, va_list);
			FILE            *Out           = (FILE  *)va_arg( user_args, FILE  *);

			IMRT_Field      *Field_Ptr     = Node_Ptr->Value;


			Command_Name = "IMRD_SDMD_CMD_API_XML_DUMP";

			fprintf( Out, "\t\t<%s>\n", IMRD_SDMD_TAG_FIELD);

			if(strcmp(Field_Ptr->Name, IMRD_SDMD_EMPTY_STRING) != 0)
			{
				fprintf( Out, "\t\t\t<%s>%s</%s>\n", IMRD_SDMD_TAG_NAME, Field_Ptr->Name, IMRD_SDMD_TAG_NAME);
			}

			if(strcmp(Field_Ptr->Value_Id, IMRD_SDMD_EMPTY_STRING) != 0)
			{
				fprintf( Out, "\t\t\t<%s>%s</%s>\n", IMRD_SDMD_TAG_VALUEID, Field_Ptr->Value_Id, IMRD_SDMD_TAG_VALUEID);
			}

			if(Field_Ptr->Exist == IMRD_TRUE)
			{
				fprintf( Out, "\t\t\t<%s>%c</%s>\n", IMRD_SDMD_TAG_EXIST, IMRD_YES, IMRD_SDMD_TAG_EXIST);
			}

			if((Field_Ptr->Fixed.Size != -1)||(Field_Ptr->Fixed.Cnv_Ptr != NULL)||(strcmp(Field_Ptr->Fixed.Def_Val, IMRD_SDMD_EMPTY_STRING) != 0))
			{
				fprintf( Out, "\t\t\t<%s>\n\t\t\t\t<%s>%d</%s>\n\t\t\t\t<%s>%d</%s>\n", IMRD_SDMD_TAG_FIXED, 
						IMRD_SDMD_TAG_OFFSET, Field_Ptr->Fixed.Offset, IMRD_SDMD_TAG_OFFSET, 
						IMRD_SDMD_TAG_SIZE, Field_Ptr->Fixed.Size, IMRD_SDMD_TAG_SIZE);

				if(Field_Ptr->Fixed.Cnv_Ptr != NULL)
				{
					fprintf( Out, "\t\t\t\t<%s>\n\t\t\t\t\t<%s>%s</%s>\n",
							IMRD_SDMD_TAG_CNV, 
							IMRD_SDMD_TAG_NAME, Field_Ptr->Fixed.Cnv_Ptr->Name, IMRD_SDMD_TAG_NAME);

					if(strcmp(Field_Ptr->Fixed.Cnv_Attr, IMRD_SDMD_EMPTY_STRING) != 0)
					{
						fprintf( Out, "\t\t\t\t\t<%s>%s</%s>\n", 
										IMRD_SDMD_TAG_ATTRIBUTE, Field_Ptr->Fixed.Cnv_Attr, IMRD_SDMD_TAG_ATTRIBUTE);
					}
					
					fprintf( Out, "\t\t\t\t</%s>\n", IMRD_SDMD_TAG_CNV);
				}

				if(strcmp(Field_Ptr->Fixed.Def_Val, IMRD_SDMD_EMPTY_STRING) != 0)
				{
					fprintf( Out, "\t\t\t\t<%s>%s</%s>\n",
					IMRD_SDMD_TAG_DEFVAL, Field_Ptr->Fixed.Def_Val, IMRD_SDMD_TAG_DEFVAL);
				}

				fprintf( Out, "\t\t\t</%s>\n", IMRD_SDMD_TAG_FIXED);
			}


			if((strcmp(Field_Ptr->Tagged.Tag, IMRD_SDMD_EMPTY_STRING) != 0)||
				(Field_Ptr->Tagged.Cnv_Ptr != NULL)||
				(strcmp(Field_Ptr->Tagged.Def_Val, IMRD_SDMD_EMPTY_STRING) != 0))
			{
				fprintf( Out, "\t\t\t<%s>\n\t\t\t\t<%s>%s</%s>\n", IMRD_SDMD_TAG_TAGGED, 
						IMRD_SDMD_TAG_NAME, Field_Ptr->Tagged.Tag, IMRD_SDMD_TAG_NAME);

				if(Field_Ptr->Tagged.Cnv_Ptr != NULL)
				{
					fprintf( Out, "\t\t\t\t<%s>\n\t\t\t\t\t<%s>%s</%s>\n",
							IMRD_SDMD_TAG_CNV, 
							IMRD_SDMD_TAG_NAME, Field_Ptr->Tagged.Cnv_Ptr->Name, IMRD_SDMD_TAG_NAME);

					if(strcmp(Field_Ptr->Tagged.Cnv_Attr, IMRD_SDMD_EMPTY_STRING) != 0)
					{
						fprintf( Out, "\t\t\t\t\t<%s>%s</%s>\n", 
										IMRD_SDMD_TAG_ATTRIBUTE, Field_Ptr->Tagged.Cnv_Attr, IMRD_SDMD_TAG_ATTRIBUTE);
					}
					
					fprintf( Out, "\t\t\t\t</%s>\n", IMRD_SDMD_TAG_CNV);
				}

				if(strcmp(Field_Ptr->Tagged.Def_Val, IMRD_SDMD_EMPTY_STRING) != 0)
				{
					fprintf( Out, "\t\t\t\t<%s>%s</%s>\n",
					IMRD_SDMD_TAG_DEFVAL, Field_Ptr->Tagged.Def_Val, IMRD_SDMD_TAG_DEFVAL);
				}

				fprintf( Out, "\t\t\t</%s>\n", IMRD_SDMD_TAG_TAGGED);

			}

			if((strcmp(Field_Ptr->Internal.Tag, IMRD_SDMD_EMPTY_STRING) != 0)||
				(Field_Ptr->Internal.Cnv_Ptr != NULL)||
				(strcmp(Field_Ptr->Internal.Def_Val, IMRD_SDMD_EMPTY_STRING) != 0))
			{
				fprintf( Out, "\t\t\t<%s>\n\t\t\t\t<%s>%s</%s>\n", IMRD_SDMD_TAG_INTERNE, 
						IMRD_SDMD_TAG_NAME, Field_Ptr->Internal.Tag, IMRD_SDMD_TAG_NAME);

				if(Field_Ptr->Internal.Cnv_Ptr != NULL)
				{
					fprintf( Out, "\t\t\t\t<%s>\n\t\t\t\t\t<%s>%s</%s>\n",
							IMRD_SDMD_TAG_CNV, 
							IMRD_SDMD_TAG_NAME, Field_Ptr->Internal.Cnv_Ptr->Name, IMRD_SDMD_TAG_NAME);

					if(strcmp(Field_Ptr->Internal.Cnv_Attr, IMRD_SDMD_EMPTY_STRING) != 0)
					{
						fprintf( Out, "\t\t\t\t\t<%s>%s</%s>\n", 
										IMRD_SDMD_TAG_ATTRIBUTE, Field_Ptr->Internal.Cnv_Attr, IMRD_SDMD_TAG_ATTRIBUTE);
					}
					
					fprintf( Out, "\t\t\t\t</%s>\n", IMRD_SDMD_TAG_CNV); 
				}

				if(strcmp(Field_Ptr->Internal.Def_Val, IMRD_SDMD_EMPTY_STRING) != 0)
				{
					fprintf( Out, "\t\t\t\t<%s>%s</%s>\n",
					IMRD_SDMD_TAG_DEFVAL, Field_Ptr->Internal.Def_Val, IMRD_SDMD_TAG_DEFVAL);
				}

				fprintf( Out, "\t\t\t</%s>\n", IMRD_SDMD_TAG_INTERNE);

			}


			if((strcmp(Field_Ptr->FML.Tag, IMRD_SDMD_EMPTY_STRING) != 0)||
				(Field_Ptr->FML.Cnv_Ptr != NULL)||
				(strcmp(Field_Ptr->FML.Def_Val, IMRD_SDMD_EMPTY_STRING) != 0))
			{
				fprintf( Out, "\t\t\t<%s>\n\t\t\t\t<%s>%s</%s>\n", IMRD_SDMD_TAG_FML, 
						IMRD_SDMD_TAG_NAME, Field_Ptr->FML.Tag, IMRD_SDMD_TAG_NAME);

				if(Field_Ptr->FML.Cnv_Ptr != NULL)
				{
					fprintf( Out, "\t\t\t\t<%s>\n\t\t\t\t\t<%s>%s</%s>\n",
							IMRD_SDMD_TAG_CNV, 
							IMRD_SDMD_TAG_NAME, Field_Ptr->FML.Cnv_Ptr->Name, IMRD_SDMD_TAG_NAME);

					if(strcmp(Field_Ptr->FML.Cnv_Attr, IMRD_SDMD_EMPTY_STRING) != 0)
					{
						fprintf( Out, "\t\t\t\t\t<%s>%s</%s>\n", 
										IMRD_SDMD_TAG_ATTRIBUTE, Field_Ptr->FML.Cnv_Attr, IMRD_SDMD_TAG_ATTRIBUTE);
					}
					
					fprintf( Out, "\t\t\t\t</%s>\n", IMRD_SDMD_TAG_CNV); 
				}

				if(strcmp(Field_Ptr->FML.Def_Val, IMRD_SDMD_EMPTY_STRING) != 0)
				{
					fprintf( Out, "\t\t\t\t<%s>%s</%s>\n",
					IMRD_SDMD_TAG_DEFVAL, Field_Ptr->FML.Def_Val, IMRD_SDMD_TAG_DEFVAL);
				}

				fprintf( Out, "\t\t\t</%s>\n", IMRD_SDMD_TAG_FML);

			}

			if((strcmp(Field_Ptr->XML.Tag, IMRD_SDMD_EMPTY_STRING) != 0)||
				(Field_Ptr->XML.Cnv_Ptr != NULL)||
				(strcmp(Field_Ptr->XML.Def_Val, IMRD_SDMD_EMPTY_STRING) != 0))
			{
				fprintf( Out, "\t\t\t<%s>\n\t\t\t\t<%s>%s</%s>\n", IMRD_SDMD_TAG_XML, 
						IMRD_SDMD_TAG_NAME, Field_Ptr->XML.Tag, IMRD_SDMD_TAG_NAME);

				if(Field_Ptr->XML.Cnv_Ptr != NULL)
				{
					fprintf( Out, "\t\t\t\t<%s>\n\t\t\t\t\t<%s>%s</%s>\n",
							IMRD_SDMD_TAG_CNV, 
							IMRD_SDMD_TAG_NAME, Field_Ptr->XML.Cnv_Ptr->Name, IMRD_SDMD_TAG_NAME);

					if(strcmp(Field_Ptr->XML.Cnv_Attr, IMRD_SDMD_EMPTY_STRING) != 0)
					{
						fprintf( Out, "\t\t\t\t\t<%s>%s</%s>\n", 
										IMRD_SDMD_TAG_ATTRIBUTE, Field_Ptr->XML.Cnv_Attr, IMRD_SDMD_TAG_ATTRIBUTE);
					}
					
					fprintf( Out, "\t\t\t\t</%s>\n", IMRD_SDMD_TAG_CNV); 
				}

				if(strcmp(Field_Ptr->XML.Def_Val, IMRD_SDMD_EMPTY_STRING) != 0)
				{
					fprintf( Out, "\t\t\t\t<%s>%s</%s>\n",
					IMRD_SDMD_TAG_DEFVAL, Field_Ptr->XML.Def_Val, IMRD_SDMD_TAG_DEFVAL);
				}

				fprintf( Out, "\t\t\t</%s>\n", IMRD_SDMD_TAG_XML);

			}


			if((Field_Ptr->Delimited.Id != IMRD_DELIMITED_ID_UNKNOWN)||
				(Field_Ptr->Delimited.Cnv_Ptr != NULL)||
				(strcmp(Field_Ptr->Delimited.Def_Val, IMRD_SDMD_EMPTY_STRING) != 0))
			{
				fprintf( Out, "\t\t\t<%s>\n\t\t\t\t<%s>%d</%s>\n", IMRD_SDMD_TAG_DELIMITED, 
						IMRD_SDMD_TAG_ID, Field_Ptr->Delimited.Id, IMRD_SDMD_TAG_ID);
				
				if(Field_Ptr->Delimited.Cnv_Ptr != NULL)
				{
					fprintf( Out, "\t\t\t\t<%s>\n\t\t\t\t\t<%s>%s</%s>\n",
							IMRD_SDMD_TAG_CNV, 
							IMRD_SDMD_TAG_NAME, Field_Ptr->Delimited.Cnv_Ptr->Name, IMRD_SDMD_TAG_NAME);

					if(strcmp(Field_Ptr->Delimited.Cnv_Attr, IMRD_SDMD_EMPTY_STRING) != 0)
					{
						fprintf( Out, "\t\t\t\t\t<%s>%s</%s>\n", 
										IMRD_SDMD_TAG_ATTRIBUTE, Field_Ptr->Delimited.Cnv_Attr, IMRD_SDMD_TAG_ATTRIBUTE);
					}
					
					fprintf( Out, "\t\t\t\t</%s>\n", IMRD_SDMD_TAG_CNV); 
				}

				if(strcmp(Field_Ptr->Delimited.Def_Val, IMRD_SDMD_EMPTY_STRING) != 0)
				{
					fprintf( Out, "\t\t\t\t<%s>%s</%s>\n",
					IMRD_SDMD_TAG_DEFVAL, Field_Ptr->Delimited.Def_Val, IMRD_SDMD_TAG_DEFVAL);
				}

				fprintf( Out, "\t\t\t</%s>\n", IMRD_SDMD_TAG_DELIMITED);
			}

			fprintf( Out, "\t\t</%s>\n", IMRD_SDMD_TAG_FIELD);

			return( NDS_OK);
		}


        default:
        {
            printf( "SDM_Field_Manager() called with an undefined command %d\n", Command);

            return( NDS_ERRAPI);
        }
    }

    printf( "SDM_Field_Manager() called with command %d (%s)\n", Command, Command_Name);

    return( NDS_OK);
}





/*----------------------------------------------------------------------------*/
/* Msg Manager                                                                */
/*----------------------------------------------------------------------------*/

NDT_Status  SDM_Msg_Manager( NDT_Root  *Root_Ptr, NDT_Index_Id  Index_Id, NDT_Node  *Node_Ptr, NDT_Command  Command, va_list  Args)
{
    NDT_Command_Name  Command_Name;
    NDT_Status         status;


    switch( Command)
    {
        case NDD_CMD_MANAGER_VERSION:
        {
            NDT_Version_Name  *Version_Name_Ptr = (NDT_Version_Name  *)va_arg( Args, NDT_Version_Name  *);


            Command_Name = "NDD_CMD_MANAGER_VERSION";

            *Version_Name_Ptr = "Revision: 2.1    Name: libnode-2_0_4-1    Date: 2002/02/28 22:00:35    Author: agibert ";

            return( NDS_OK);
        }

        case NDD_CMD_INDEX_GET:
        {
            /*
	      NDT_Index_Id  *Reply_Index_Id_Ptr = (NDT_Index_Id  *)va_arg( Args, NDT_Index_Id  *);
	      NDT_Command   *Reply_Command_Ptr  = (NDT_Command  *)va_arg( Args, NDT_Command  *);
	      NDT_Command    Cmd                = (NDT_Command)va_arg( Args, NDT_Command);
	      void          *Value_ptr          = (void  *)va_arg( Args, void  *);
            */
            NDT_Index_Id  *Reply_Index_Id_Ptr = (NDT_Index_Id  *)va_arg( Args, NDT_Index_Id  *);
            NDT_Command   *Reply_Command_Ptr  = (NDT_Command  *)va_arg( Args, NDT_Command  *);
            NDT_Command    Cmd                = (NDT_Command)va_arg( Args, NDT_Command);
            IMRT_Msg      *Msg_ptr            = (IMRT_Msg  *)va_arg( Args, IMRT_Msg  *);


            Command_Name = "NDD_CMD_INDEX_GET";

            switch(Cmd)
            {
                case IMRD_SDMD_CMD_INDEX0_PRINT:
                {
                    *Reply_Index_Id_Ptr = 0;
                    *Reply_Command_Ptr = NDD_CMD_VALUE_PRINT;

                    break;
                }

				case IMRD_SDMD_CMD_API_SQL_DUMP:
				{
                    *Reply_Index_Id_Ptr = IMRD_SDMD_MSG_IDX_ID;
                    *Reply_Command_Ptr = IMRD_SDMD_CMD_API_SQL_DUMP;
                    break;
				}

				case IMRD_SDMD_CMD_API_XML_DUMP:
				{
                    *Reply_Index_Id_Ptr = IMRD_SDMD_MSG_IDX_ID;
                    *Reply_Command_Ptr = IMRD_SDMD_CMD_API_XML_DUMP;
                    break;
				}

                default:
                {
                    *Reply_Index_Id_Ptr = Index_Id;
                    *Reply_Command_Ptr = Cmd;

                    break;
                }
            }

            return( NDS_OK);
        }

        case NDD_CMD_VALUE_ALLOC:
        {
            /*
	      void       **Msg_ptr_Ptr   = (void **)va_arg( Args, void  **);
	      va_list      user_args     = (va_list)va_arg( Args, va_list);
	      user_type    user_data     = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
            */
            IMRT_Msg  **Msg_ptr_Ptr = va_arg( Args, IMRT_Msg  **);
			NDT_Index_Type  *index_type_ptr;


            Command_Name = "NDD_CMD_VALUE_ALLOC";

            if( ( *Msg_ptr_Ptr = (IMRT_Msg  *)malloc( sizeof(IMRT_Msg))) == NULL)
            {
                return( NDS_ERRMEM);
            }
            else
            {

                SDM_InitMessage(*Msg_ptr_Ptr);
                
				index_type_ptr =  Msg_Struct_initial_idx_tab;

                if( ( status = ND_DataStruct_Open( &(*Msg_ptr_Ptr)->Field_Struct_Ptr, IMRD_SDMD_FIELD_STRUCT_IDX_NB, index_type_ptr, "SDM__Field_Manager", SDM_Field_Manager, NULL, NULL, NULL, NULL,0,  NULL)) != NDS_OK)
				{
				    printf( "ND_DataStruct_Open() Field failed (%d) !\n", status);
                    return(NDS_KO);
				}

                return( NDS_OK);
            }
        }

        case NDD_CMD_VALUE_FREE:
        {
            /*
	      void       *Msg_ptr   = (void *)va_arg( Args, void  *);
	      va_list     user_args = (va_list)va_arg( Args, va_list);
	      user_type   user_data = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
            */
            IMRT_Msg  *Msg_ptr  = (IMRT_Msg  *)va_arg( Args, IMRT_Msg  *);


            Command_Name = "NDD_CMD_VALUE_FREE";
            if(Msg_ptr->Field_Struct_Ptr != NULL)
            {  if( ( status = ND_DataStruct_Close(Msg_ptr->Field_Struct_Ptr)) != NDS_OK)
				{
				    printf( "ND_DataStruct_Close() failed (%d) !\n", status);
				    return(status);
				}
            }

            free( Msg_ptr);
            
            return( NDS_OK);
        }

        case NDD_CMD_VALUE_COMP:
        {
            /*
	      void       *Value1_Ptr = (void *)va_arg( Args, void  *);
	      void       *Value2_Ptr = (void *)va_arg( Args, void  *);
	      va_list     user_args  = (va_list)va_arg( Args, va_list);
	      user_type   user_data  = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
            */
            IMRT_Msg     *Value1_Ptr = va_arg( Args, IMRT_Msg  *);
            IMRT_Msg     *Value2_Ptr = va_arg( Args, IMRT_Msg  *);


            Command_Name = "NDD_CMD_VALUE_COMP";

            switch( Index_Id)
            {
                case IMRD_SDMD_MSG_IDX_ID:
                {
                    if( Value1_Ptr->Id < Value2_Ptr->Id)
                    {
                        return( NDS_LOWER);
                    }
                    else
                    {
                        if( Value1_Ptr->Id > Value2_Ptr->Id)
                        {
                            return( NDS_GREATER);
                        }
                        else
                        {
                            return( NDS_EQUAL);
                        }
                    }
                }

                case IMRD_SDMD_MSG_IDX_NAME:
                {
                    int     rc;
                    
                    rc = strcmp( Value1_Ptr->Name, Value2_Ptr->Name);

                    if( rc < 0)
                    {
                        return( NDS_LOWER);
                    }
                    else
                    {
                        if( rc > 0)
                        {
                            return( NDS_GREATER);
                        }
                        else
                        {
                            return( NDS_EQUAL);
                        }
                    }
                }

                 default:
                {
                    printf( "Unknown COMP idx (%d) !\n", Index_Id);
                    return( NDS_KO);
                }
            }

            return( NDS_OK);
        }

        case NDD_CMD_VALUE_ADD:
        {
            /*
	      void       *Msg_ptr   = (void *)va_arg( Args, void  *);
	      va_list     user_args = (va_list)va_arg( Args, va_list);
	      user_type   user_data = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
            */

            Command_Name = "NDD_CMD_VALUE_ADD";

            return( NDS_OK);
        }

        case NDD_CMD_VALUE_REMOVE:
        {
            /*
	      void       *Msg_ptr   = (void *)va_arg( Args, void  *);
	      va_list     user_args = (va_list)va_arg( Args, va_list);
	      user_type   user_data = (user_type)va_arg( user_args, user_type);
	      ... = (...)va_arg( user_args, ...);
            */

            Command_Name = "NDD_CMD_VALUE_REMOVE";

            return( NDS_OK);
        }

        case NDD_CMD_VALUE_PRINT:
        {
            /*
	      NDT_Node             *Next_Node_Ptr    = (NDT_Node  *)va_arg( Args, NDT_Node  *);
	      va_list               lib_args         = (va_list)va_arg( Args, va_list);
	      FILE                 *Out              = (FILE  *)va_arg( lib_args, FILE  *);
	      NDT_Recursive_Mode    Recursive_Mode   = (NDT_Recursive_Mode)va_arg( lib_args, NDT_Recursive_Mode);
	      NDT_Recursive_Depth   Recursive_Depth  = (NDT_Recursive_Depth)va_arg( lib_args, NDT_Recursive_Depth);
	      NDT_Recursive_Offset  Recursive_Offset = (NDT_Recursive_Offset)va_arg( lib_args, NDT_Recursive_Offset);
	      va_list               user_args        = (va_list)va_arg( lib_args, va_list);
	      user_type             user_data        = (user_type)va_arg( user_args, user_type);
	      ...  = (...)va_arg( user_args, ...);

	      void                 *Msg_ptr           = Node_Ptr->Value;
            */
            NDT_Node             *Next_Node_Ptr   =         (NDT_Node  *)va_arg( Args, NDT_Node  *);
            va_list               lib_args        =             (va_list)va_arg( Args, va_list);
            FILE                 *Out             =             (FILE  *)va_arg( lib_args, FILE  *);
            NDT_Recursive_Mode    Recursive_Mode  =  (NDT_Recursive_Mode)va_arg( lib_args, NDT_Recursive_Mode);
            NDT_Recursive_Depth   Recursive_Depth = (NDT_Recursive_Depth)va_arg( lib_args, NDT_Recursive_Depth);
            NDT_Recursive_Offset  Recursive_Offset = (NDT_Recursive_Offset)va_arg( lib_args, NDT_Recursive_Offset);

            IMRT_Msg             *Msg_ptr          = Node_Ptr->Value;


            Command_Name = "NDD_CMD_VALUE_PRINT";

            printf("Adresse avant traverse (%x)\n",Msg_ptr);

            fprintf( Out, "Type: (%s)   Size: (%d)   \n",
		     Msg_ptr->Name,  Msg_ptr->Size);

            return( ND_DataStruct_Value_Print( Out, Msg_ptr->Field_Struct_Ptr, NDD_RECURSIVE_MODE_PARENT_CHILD, --Recursive_Depth, ++Recursive_Offset));
            
        }

        case NDD_CMD_INFO_PRINT:
        {
            /*
	      NDT_Node             *Next_Node_Ptr    = (NDT_Node  *)va_arg( Args, NDT_Node  *);
	      va_list               lib_args         = (va_list)va_arg( Args, va_list);
	      FILE                 *Out              = (FILE  *)va_arg( lib_args, FILE  *);
	      NDT_Recursive_Mode    Recursive_Mode   = (NDT_Recursive_Mode)va_arg( lib_args, NDT_Recursive_Mode);
	      NDT_Recursive_Depth   Recursive_Depth  = (NDT_Recursive_Depth)va_arg( lib_args, NDT_Recursive_Depth);
	      NDT_Recursive_Offset  Recursive_Offset = (NDT_Recursive_Offset)va_arg( lib_args, NDT_Recursive_Offset);
	      va_list               user_args        = (va_list)va_arg( lib_args, va_list);
	      user_type             user_data        = (user_type)va_arg( user_args, user_type);
	      ...  = (...)va_arg( user_args, ...);

	      void                 *Msg_ptr          = Node_Ptr->Value;
            */

            NDT_Node             *Next_Node_Ptr    = (NDT_Node  *)va_arg( Args, NDT_Node  *);
	    va_list               lib_args         = (va_list)va_arg( Args, va_list);
	    FILE                 *Out              = (FILE  *)va_arg( lib_args, FILE  *);
	    NDT_Recursive_Mode    Recursive_Mode   = (NDT_Recursive_Mode)va_arg( lib_args, NDT_Recursive_Mode);
	    NDT_Recursive_Depth   Recursive_Depth  = (NDT_Recursive_Depth)va_arg( lib_args, NDT_Recursive_Depth);
	    NDT_Recursive_Offset  Recursive_Offset = (NDT_Recursive_Offset)va_arg( lib_args, NDT_Recursive_Offset);
			      
	    IMRT_Msg  *Msg_Ptr                     = Node_Ptr->Value;

        
	    Command_Name = "NDD_CMD_INFO_PRINT";


	    return( ND_DataStruct_Info_Print( Out, Msg_Ptr->Field_Struct_Ptr, NDD_RECURSIVE_MODE_PARENT_CHILD, --Recursive_Depth, ++Recursive_Offset));

	}

	case IMRD_SDMD_CMD_API_SQL_DUMP:
	{
	/*
		NDT_Node   *Next_Node_Ptr = (NDT_Node  *)va_arg( Args, NDT_Node  *);
		va_list     user_args     =     (va_list)va_arg( Args, va_list);
		user_type   user_data     =   (user_type)va_arg( user_args, user_type);
							  ... =         (...)va_arg( user_args, ...);

		void       *Msg_ptr       = Node_Ptr->Value;
	*/
		NDT_Node        *Next_Node_Ptr = (NDT_Node  *)va_arg( Args, NDT_Node  *);
		va_list          user_args     =     (va_list)va_arg( Args, va_list);
		FILE            *Out           =     (FILE  *)va_arg( user_args, FILE  *);

		IMRT_Msg        *Msg_Ptr       = Node_Ptr->Value;


		Command_Name = "IMRD_SDMD_CMD_API_SQL_DUMP";

		fprintf( Out, "INSERT INTO IMR_MSG(MSG_ID, MSG_NAME, MSG_SIZE) VALUES(%d, '%s', %d);\n",
				Msg_Ptr->Id, Msg_Ptr->Name, Msg_Ptr->Size);

	    if( ( status = ND_DataStruct_Traverse( Msg_Ptr->Field_Struct_Ptr, IMRD_SDMD_CMD_API_SQL_DUMP, Out, Msg_Ptr->Id)) != NDS_OK)
		{
			printf( "ND_DataStruct_Traverse failed (%d)...\n");
			return(NDS_KO);
		}

		return( NDS_OK);
	}

	case IMRD_SDMD_CMD_API_XML_DUMP:
	{
	/*
		NDT_Node   *Next_Node_Ptr = (NDT_Node  *)va_arg( Args, NDT_Node  *);
		va_list     user_args     =     (va_list)va_arg( Args, va_list);
		user_type   user_data     =   (user_type)va_arg( user_args, user_type);
							  ... =         (...)va_arg( user_args, ...);

		void       *Msg_ptr       = Node_Ptr->Value;
	*/
		NDT_Node        *Next_Node_Ptr = (NDT_Node  *)va_arg( Args, NDT_Node  *);
		va_list          user_args     =     (va_list)va_arg( Args, va_list);
		FILE            *Out           =     (FILE  *)va_arg( user_args, FILE  *);

		IMRT_Msg        *Msg_Ptr       = Node_Ptr->Value;


		Command_Name = "IMRD_SDMD_CMD_API_XML_DUMP";

		fprintf( Out, "\t<%s>\n\t\t<%s>%s</%s>\n", IMRD_SDMD_TAG_MESSAGE, 
					IMRD_SDMD_TAG_NAME, Msg_Ptr->Name, IMRD_SDMD_TAG_NAME);

		if(Msg_Ptr->Size != 0)
		{
			fprintf( Out, "\t\t<%s>%ld</%s>\n",
					IMRD_SDMD_TAG_SIZE, Msg_Ptr->Size, IMRD_SDMD_TAG_SIZE);
		}

	    if( ( status = ND_DataStruct_Traverse( Msg_Ptr->Field_Struct_Ptr, IMRD_SDMD_CMD_API_XML_DUMP, Out)) != NDS_OK)
		{
			printf( "ND_DataStruct_Traverse failed (%d)...\n");
			return(NDS_KO);
		}

		fprintf( Out, "\t</%s>\n", IMRD_SDMD_TAG_MESSAGE);

		return( NDS_OK);
	}

	default:
        {
            printf( "SDM_Msg_Manager() called with an undefined command %d\n", Command);

            return( NDS_ERRAPI);
        }
    }

    printf( "SDM_Msg_Manager() called with command %d (%s)\n", Command, Command_Name);

    return( NDS_OK);
}
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
IMRT_Status SDM_StructInfo_Init(IMRT_SDM_Config * Lib_Config_Ptr)
{
    IMRT_Status  status;

    if(Lib_Config_Ptr->Flag == IMRD_SDM_FLAG_XML)
    {
		status= SDM_XML_StructInfo_Init(Lib_Config_Ptr->Lib_Path);
    }
    else
    {
#ifdef ORACLE_SUPPORT
		status = SDM_SQL_StructInfo_Init();
#else
		status = IMRS_OK;
#endif /* ORACLE_SUPPORT */
    }

    return(status);
}
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
IMRT_Status SDM_Close_RootCnv(void)
{
	IMRT_Status  status;
	NDT_Status   nd_status;

	status = IMRS_OK;

	if(IMRG_SDM_Base.RootCnv != NULL)
	{
		if (IMRG_SDM_Base.RootCnv->User_Ptr != NULL)
		{
			free(IMRG_SDM_Base.RootCnv->User_Ptr);
		}

		if( ( nd_status = ND_DataStruct_Close(IMRG_SDM_Base.RootCnv)) != NDS_OK)
		{
			printf( "RootCnv ND_DataStruct_Close() failed (%d) !\n", nd_status);
			status = IMRS_KO;
		}
	}

	return(status);

}
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
IMRT_Status SDM_Close_RootMsg(void)
{
	IMRT_Status  status;
	NDT_Status   nd_status;

	status = IMRS_OK;

	if(IMRG_SDM_Base.RootMsg != NULL)
	{
		if (IMRG_SDM_Base.RootMsg->User_Ptr != NULL)
		{
			free(IMRG_SDM_Base.RootMsg->User_Ptr);
		}

		if( ( nd_status = ND_DataStruct_Close(IMRG_SDM_Base.RootMsg)) != NDS_OK)
		{
			printf( "RootMsg ND_DataStruct_Close() failed (%d) !\n", nd_status);
			status = IMRS_KO;
		}
	}

	return(status);

}
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
IMRT_Status SDM_StructInfo_DInit(void)
{
	IMRT_Status  status, StatusCnv, StatusMsg;

	status = IMRS_OK;

	StatusCnv = SDM_Close_RootCnv();
	StatusMsg = SDM_Close_RootMsg();

	if((StatusCnv == IMRS_KO)||(StatusMsg == IMRS_KO))
	{
		status = IMRS_KO;
	}

	return(status);
}
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
IMRT_Status  SDM_Cnv_Proceed( IMRT_SDMCB_Cnv_Add  *CCB_Cnv_Add_Ptr)
{

    IMRT_Status status;
    NDT_Status  nd_status;

	status = IMRS_OK;
    if( ( nd_status = ND_DataStruct_Traverse( IMRG_SDM_Base.RootCnv, IMRD_SDMD_CMD_EXECUTE, stdout, NDD_RECURSIVE_MODE_PARENT, CCB_Cnv_Add_Ptr)) != NDS_OK)
    {
	  printf( "ND_DataStruct_Traverse() failed (%d) !\n", nd_status);
	  status = IMRS_KO;
    }

    return(status);
}
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
void SDM_InitMessage(IMRT_Msg * mymessage)
{
    mymessage->Id = IMRD_MSG_ID_UNKNOWN;
    memset(mymessage->Name,0,sizeof(mymessage->Name));
    mymessage->Size      = 0;
    mymessage->Field_Struct_Ptr   = NULL;
}

void SDM_InitField(IMRT_Field * myfield)
{
    memset(myfield->Name,0,sizeof(myfield->Name));
    memset(myfield->Value_Id,0,sizeof(myfield->Value_Id));
	myfield->Exist = IMRD_FALSE;

    myfield->Fixed.Offset = 0;
    myfield->Fixed.Size = -1;
    memset(myfield->Fixed.Def_Val,'\0',sizeof(myfield->Fixed.Def_Val));
    myfield->Fixed.Cnv_Ptr = NULL;
    memset(myfield->Fixed.Cnv_Attr,0,sizeof(myfield->Fixed.Cnv_Attr));

    memset(myfield->XML.Tag,0,sizeof(myfield->XML.Tag));
    memset(myfield->XML.Def_Val,'\0',sizeof(myfield->XML.Def_Val));
    myfield->XML.Cnv_Ptr = NULL;
    memset(myfield->XML.Cnv_Attr,0,sizeof(myfield->XML.Cnv_Attr));

    memset(myfield->FML.Tag,0,sizeof(myfield->FML.Tag));
    memset(myfield->FML.Def_Val,'\0',sizeof(myfield->FML.Def_Val));
    myfield->FML.Cnv_Ptr = NULL;
    memset(myfield->FML.Cnv_Attr,0,sizeof(myfield->FML.Cnv_Attr));

    memset(myfield->Tagged.Tag,0,sizeof(myfield->Tagged.Tag));
    memset(myfield->Tagged.Def_Val,'\0',sizeof(myfield->Tagged.Def_Val));
    myfield->Tagged.Cnv_Ptr = NULL;
    memset(myfield->Tagged.Cnv_Attr,0,sizeof(myfield->Tagged.Cnv_Attr));

    memset(myfield->Internal.Tag,0,sizeof(myfield->Internal.Tag));
    memset(myfield->Internal.Def_Val,'\0',sizeof(myfield->Internal.Def_Val));
    myfield->Internal.Cnv_Ptr = NULL;
    memset(myfield->Internal.Cnv_Attr,0,sizeof(myfield->Internal.Cnv_Attr));

    myfield->Delimited.Id = IMRD_DELIMITED_ID_UNKNOWN;
    memset(myfield->Delimited.Def_Val,'\0',sizeof(myfield->Delimited.Def_Val));
    myfield->Delimited.Cnv_Ptr = NULL;
    memset(myfield->Delimited.Cnv_Attr,0,sizeof(myfield->Delimited.Cnv_Attr));
}

void SDM_InitCnv(IMRT_Cnv * Cnv_Ptr)
{
    Cnv_Ptr->Id = IMRD_CNV_ID_UNKNOWN;
    memset(Cnv_Ptr->Name,0,sizeof(Cnv_Ptr->Name));
}


