/*----------------------------------------------------------------------------*/
/* File: struct_info_xml.c   		         		  	      */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#define _LIBSTATIC_DATA_MNG_
#define _STRUCT_INFO_XML_C_

#include "struct_info_xml.h"


IMRT_Status SDM_XML_StructInfo_Init(IMRT_Path    Lib_Path)
{
    SDMT_StructInfo_Contexte StructInfo_Ctx;
    IMRT_Status  status, StatusTmp;
    char	     Xml_File_Name[255];

    StructInfo_Ctx.flagerror = 0;
    StructInfo_Ctx.CptCnv = IMRD_CNV_ID_UNKNOWN;



    IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Loading Cnv...");

    memset(Xml_File_Name,0,sizeof(Xml_File_Name));
	strcpy(Xml_File_Name, Lib_Path);
	strcat(Xml_File_Name, "/");
	strcat(Xml_File_Name, SDMD_CNV_FILE);
    status = SDM_XML_Load_Cnv((char *)Xml_File_Name, &(StructInfo_Ctx));
    if(status != IMRS_OK)
    {
	IMRD_LM_LOG_ERROR_0( "Can't load Cnv !");
    }
    else
    {
    	IMRG_SDM_Base.RootCnv = StructInfo_Ctx.RootCnv;
 	
 		IMRD_LM_LOG_INFO_1( "Cnv loaded: (%d)", IMRG_SDM_Base.RootCnv->Index_Tab[NDD_INDEX_PRIMARY].Node_Number);

	    StructInfo_Ctx.CptMsg = IMRD_MSG_ID_UNKNOWN;
	    StructInfo_Ctx.CptField = IMRD_FIELD_ID_UNKNOWN;
	    memset(StructInfo_Ctx.PathArray,0,sizeof(StructInfo_Ctx.PathArray));
	    StructInfo_Ctx.PresentLevelTag = IMRD_SDMD_MIN_LEVEL_TAG;

	    IMRD_LM_LOG_TRACE_0( IMRD_LOG_LEVEL_VERBOSE_0, "Loading Msg and Field...");

	    memset(Xml_File_Name,0,sizeof(Xml_File_Name));
		strcpy(Xml_File_Name, Lib_Path);
		strcat(Xml_File_Name, "/");
		strcat(Xml_File_Name, SDMD_MSGFIELD_FILE);

	    status =  SDM_XML_Load_MsgAndField((char *)Xml_File_Name, &(StructInfo_Ctx));

	    if(status != IMRS_OK)
	    {
		IMRD_LM_LOG_ERROR_0( "Can't load Msg and Field !");
			StatusTmp = SDM_Close_RootCnv();
	    }
	    else
	    {
	    	IMRG_SDM_Base.RootMsg = StructInfo_Ctx.RootMessage;

			IMRD_LM_LOG_INFO_1( "Msg and Field loaded: (%d)", IMRG_SDM_Base.RootMsg->Index_Tab[NDD_INDEX_PRIMARY].Node_Number);
	    }
	}

    return(status);

} 


static IMRT_Status error(SDMT_StructInfo_Contexte *ctx , const char *msg, ...)
{
    xmlParserCtxt * SaxCtx;
  
    SaxCtx = ctx->SaxCtxHdl;
  
    IMRD_LM_LOG_ERROR_1( "CallBack error: <%s> !",msg);

    SaxCtx->sax = &ErrorSAXHandlerStruct;
    ctx->flagerror = 1;
    ctx->FataError.errorCode = SDMD_XML_CODE_ERROR_1;
    strcpy(ctx->FataError.libError,msg) ;
}


IMRT_Status startElementNsCnv( SDMT_StructInfo_Contexte * ctx , 
			  const xmlChar *localname,
			  const xmlChar *prefix,
			  const xmlChar *URI,
			  int nb_namespaces,
			  const xmlChar **namespaces,
			  int nb_attributes,
			  int nb_defaulted,
			  const xmlChar **attributes)
{
    NDT_Status  status;
  
    if(strcmp((const char*)localname,IMRD_SDMD_TAG_CNV) == 0)
    {
		/*cration d'un nouvelle objet Cnv*/
		if( ( status = ND_Value_Alloc( ctx->RootCnv, (void  **)&(ctx->CurrentCnv)) != NDS_OK))
		{
		    IMRD_LM_LOG_ERROR_1( "ND_Value_Alloc() failed (%d) !", status);
		}
    }

}


IMRT_Status startElementNS( SDMT_StructInfo_Contexte * ctx , 
							       const xmlChar *localname,
							       const xmlChar *prefix,
							       const xmlChar *URI,
							       int nb_namespaces,
							       const xmlChar **namespaces,
							       int nb_attributes,
							       int nb_defaulted,
							       const xmlChar **attributes)
{
   NDT_Status         status;
   char DescErr[IMRD_SDMD_DESCERR_LEN];
   const char fonctionname [] = "startElementNS";
   
   
   if(strcmp((const char *)localname, IMRD_SDMD_TAG_MESSAGE) == 0)
   {
		/*cration d'un objet msg*/
		if( ( status = ND_Value_Alloc( ctx->RootMessage, (void  **)&(ctx->CurrentMessage))) != NDS_OK)
		{
		  snprintf(DescErr,sizeof(DescErr),SDMD_XML_LIB_ERROR_3,IMRD_SDMD_TAG_MESSAGE);
		  SDM_RaiseError(ctx->SaxCtxHdl,&(ctx->FataError),&(ctx->flagerror), fonctionname , DescErr,SDMD_XML_CODE_ERROR_3);
		}

		ctx->PresentLevelTag = ctx->PresentLevelTag + 1;
		strcpy(ctx->PathArray[ctx->PresentLevelTag], (char*)localname);
   }
   else if (strcmp((const char *)localname, IMRD_SDMD_TAG_FIELD) == 0)
   {
      if( ( status = ND_Value_Alloc( ctx->CurrentMessage->Field_Struct_Ptr, (void  **)&(ctx->CurrentField))) != NDS_OK)
      {
	    snprintf(DescErr,sizeof(DescErr),SDMD_XML_LIB_ERROR_3,IMRD_SDMD_TAG_MESSAGE);
	    SDM_RaiseError(ctx->SaxCtxHdl,&(ctx->FataError),&(ctx->flagerror), fonctionname , DescErr, SDMD_XML_CODE_ERROR_3 );
      }
      ctx->PresentLevelTag = ctx->PresentLevelTag + 1;
	  strcpy(ctx->PathArray[ctx->PresentLevelTag], (char*)localname);
   }
   else if ((strcmp((const char *)localname, IMRD_SDMD_TAG_FIXED)    == 0) ||
            (strcmp((const char *)localname, IMRD_SDMD_TAG_INTERNE)  == 0) ||
            (strcmp((const char *)localname, IMRD_SDMD_TAG_TAGGED)   == 0) ||
            (strcmp((const char *)localname, IMRD_SDMD_TAG_XML)      == 0) ||
            (strcmp((const char *)localname, IMRD_SDMD_TAG_DELIMITED)== 0) ||
            (strcmp((const char *)localname, IMRD_SDMD_TAG_FML)      == 0)
           )
   {
      ctx->PresentLevelTag = ctx->PresentLevelTag + 1;
	  strcpy(ctx->PathArray[ctx->PresentLevelTag], (char*)localname);
   }
   else if (strcmp((const char *)localname, IMRD_SDMD_TAG_CNV) == 0)
   {
      ctx->PresentLevelTag = ctx->PresentLevelTag + 1;
	  strcpy(ctx->PathArray[ctx->PresentLevelTag], (char*)localname);
   }

}


IMRT_Status endElementNsCnv( SDMT_StructInfo_Contexte * ctx, 
			const xmlChar *localname,
			const xmlChar *prefix,
			const xmlChar *URI)
{
    IMRT_Status       status_tmp;
    NDT_Status         status;
    const char        fonctionname [] ="endElementNsCnv";
    char              DescErr[IMRD_SDMD_DESCERR_LEN];
  
    if(strcmp((const char*)localname, IMRD_SDMD_TAG_CNV) == 0)
    {
	    ctx->CurrentCnv->Id =  ctx->CptCnv;
		if( ( status = ND_DataStruct_Value_Add( ctx->RootCnv, (void  *)(ctx->CurrentCnv))) != NDS_OK)
		{
		    IMRD_LM_LOG_ERROR_1( "ND_Value_Alloc() failed (%d) !", status);

		    snprintf(DescErr,sizeof(DescErr),SDMD_XML_LIB_ERROR_3,IMRD_SDMD_TAG_CNV);
		    SDM_RaiseError(ctx->SaxCtxHdl,&(ctx->FataError),&(ctx->flagerror), fonctionname , DescErr, SDMD_XML_CODE_ERROR_3 );
		}
	    ctx->CptCnv ++;
    }

    if(strcmp((const char*)localname, IMRD_SDMD_TAG_NAME) == 0)
    {
		strcpy(ctx->CurrentCnv->Name, ctx->value);
    }
    else if (strcmp((const char *)localname, IMRD_SDMD_TAG_VERSION) == 0)
    {
		IMRD_LM_LOG_INFO_1( "[Cnv]: [%s]", ctx->value);
		status_tmp = SDM_Set_Struct_Version((IMRT_Data_Version *)ctx->RootCnv->User_Ptr, (char *)ctx->value);
    }
}


IMRT_Status endElementNS( SDMT_StructInfo_Contexte * ctx, 
							     const xmlChar *localname,
							     const xmlChar *prefix,
							     const xmlChar *URI)
{
  IMRT_Status    status_tmp;
  NDT_Status     status;
  xmlParserCtxt * SaxCtx;
  char DescErr[IMRD_SDMD_DESCERR_LEN];
  const char fonctionname [] = "endElementNS";
  NDT_Node     *node_ptr;
  IMRT_Cnv      myCnv;
  NDT_Status    nd_status;
  NDT_Index_Type  *index_type_ptr;

  if (strcmp((const char *)localname, IMRD_SDMD_TAG_NAME) == 0)
  {
  	switch(ctx->PresentLevelTag)
  	{
  	  case IMRD_SDMD_LEVEL_TAG_0:
  	  {
  	  	strcpy(ctx->CurrentMessage->Name, ctx->value);
  		break;	
      }
  	  case IMRD_SDMD_LEVEL_TAG_1:
  	  {
  	  	strcpy(ctx->CurrentField->Name, ctx->value);
  		break;	
      }
  	  case IMRD_SDMD_LEVEL_TAG_2:
  	  {
  	  	if(strcmp(ctx->PathArray[ctx->PresentLevelTag], IMRD_SDMD_TAG_INTERNE) == 0)
  	  	{
  	  	    strcpy(ctx->CurrentField->Internal.Tag, ctx->value);
  	  	}
  	  	else if(strcmp(ctx->PathArray[ctx->PresentLevelTag], IMRD_SDMD_TAG_TAGGED) == 0)
  	  	{
  	  	    strcpy(ctx->CurrentField->Tagged.Tag, ctx->value);
        }
        else if(strcmp(ctx->PathArray[ctx->PresentLevelTag], IMRD_SDMD_TAG_XML) == 0)
  	  	{
  	  	    strcpy(ctx->CurrentField->XML.Tag, ctx->value);
  	  	}
        else if(strcmp(ctx->PathArray[ctx->PresentLevelTag], IMRD_SDMD_TAG_FML) == 0)
  	  	{
  	  	    strcpy(ctx->CurrentField->FML.Tag, ctx->value);
  	  	}
  	  	          
  		break;	
      }
  	  case IMRD_SDMD_LEVEL_TAG_3:
  	  {
  	  	if(strcmp(ctx->PathArray[ctx->PresentLevelTag-1], IMRD_SDMD_TAG_FIXED) == 0)
  	  	{
           strcpy(myCnv.Name, ctx->value);
	       if(((nd_status = ND_Index_Node_Find( &node_ptr, ctx->RootCnv,
			  	               IMRD_SDMD_CNV_IDX_NAME, &myCnv, NULL)) != NDS_OK)||
				(node_ptr == NULL))
	       {
	         snprintf(DescErr,sizeof(DescErr),SDMD_XML_LIB_ERROR_4,IMRD_SDMD_TAG_CNV, myCnv.Name );
	         SDM_RaiseError(ctx->SaxCtxHdl,&(ctx->FataError),&(ctx->flagerror), fonctionname , DescErr, SDMD_XML_CODE_ERROR_4 );
	       }
  	  	   else
  	  	   {
  	  		 ctx->CurrentField->Fixed.Cnv_Ptr = node_ptr->Value;
  	  	   }
  	  	}
  	  	else if(strcmp(ctx->PathArray[ctx->PresentLevelTag-1], IMRD_SDMD_TAG_INTERNE) == 0)
  	  	{
           strcpy(myCnv.Name, ctx->value);
	       if(((nd_status = ND_Index_Node_Find( &node_ptr, ctx->RootCnv,
			  	               IMRD_SDMD_CNV_IDX_NAME, &myCnv, NULL)) != NDS_OK)||
				(node_ptr == NULL))
	       {
	         snprintf(DescErr,sizeof(DescErr),SDMD_XML_LIB_ERROR_4,IMRD_SDMD_TAG_CNV, myCnv.Name );
	         SDM_RaiseError(ctx->SaxCtxHdl,&(ctx->FataError),&(ctx->flagerror), fonctionname , DescErr, SDMD_XML_CODE_ERROR_4 );
	       }
  	  	   else
  	  	   {
  	  		 ctx->CurrentField->Internal.Cnv_Ptr = node_ptr->Value;
  	  	   }
  	  	}
  	  	else if(strcmp(ctx->PathArray[ctx->PresentLevelTag-1], IMRD_SDMD_TAG_TAGGED) == 0)
  	  	{
           strcpy(myCnv.Name, ctx->value);
	       if(((nd_status = ND_Index_Node_Find( &node_ptr, ctx->RootCnv,
			  	               IMRD_SDMD_CNV_IDX_NAME, &myCnv, NULL)) != NDS_OK)||
				(node_ptr == NULL))
	       {
	         snprintf(DescErr,sizeof(DescErr),SDMD_XML_LIB_ERROR_4,IMRD_SDMD_TAG_CNV, myCnv.Name );
	         SDM_RaiseError(ctx->SaxCtxHdl,&(ctx->FataError),&(ctx->flagerror), fonctionname , DescErr, SDMD_XML_CODE_ERROR_4 );
	       }
  	  	   else
  	  	   {
  	  		 ctx->CurrentField->Tagged.Cnv_Ptr = node_ptr->Value;
  	  	   }
        }
        else if(strcmp(ctx->PathArray[ctx->PresentLevelTag-1], IMRD_SDMD_TAG_XML) == 0)
  	  	{
           strcpy(myCnv.Name, ctx->value);
	       if(((nd_status = ND_Index_Node_Find( &node_ptr, ctx->RootCnv,
			  	               IMRD_SDMD_CNV_IDX_NAME, &myCnv, NULL)) != NDS_OK)||
				(node_ptr == NULL))
	       {
	         snprintf(DescErr,sizeof(DescErr),SDMD_XML_LIB_ERROR_4,IMRD_SDMD_TAG_CNV, myCnv.Name );
	         SDM_RaiseError(ctx->SaxCtxHdl,&(ctx->FataError),&(ctx->flagerror), fonctionname , DescErr, SDMD_XML_CODE_ERROR_4 );
	       }
  	  	   else
  	  	   {
  	  		 ctx->CurrentField->XML.Cnv_Ptr = node_ptr->Value;
  	  	   }
  	  	}
        else if(strcmp(ctx->PathArray[ctx->PresentLevelTag-1], IMRD_SDMD_TAG_FML) == 0)
  	  	{
           strcpy(myCnv.Name, ctx->value);
	       if(((nd_status = ND_Index_Node_Find( &node_ptr, ctx->RootCnv,
			  	               IMRD_SDMD_CNV_IDX_NAME, &myCnv, NULL)) != NDS_OK)||
				(node_ptr == NULL))
	       {
	         snprintf(DescErr,sizeof(DescErr),SDMD_XML_LIB_ERROR_4,IMRD_SDMD_TAG_CNV, myCnv.Name );
	         SDM_RaiseError(ctx->SaxCtxHdl,&(ctx->FataError),&(ctx->flagerror), fonctionname , DescErr, SDMD_XML_CODE_ERROR_4 );
	       }
  	  	   else
  	  	   {
  	  		 ctx->CurrentField->FML.Cnv_Ptr = node_ptr->Value;
  	  	   }
  	  	}
        else if(strcmp(ctx->PathArray[ctx->PresentLevelTag-1], IMRD_SDMD_TAG_DELIMITED) == 0)
  	  	{
           strcpy(myCnv.Name, ctx->value);
	       if(((nd_status = ND_Index_Node_Find( &node_ptr, ctx->RootCnv,
			  	               IMRD_SDMD_CNV_IDX_NAME, &myCnv, NULL)) != NDS_OK)||
				(node_ptr == NULL))
	       {
	         snprintf(DescErr,sizeof(DescErr),SDMD_XML_LIB_ERROR_4,IMRD_SDMD_TAG_CNV, myCnv.Name );
	         SDM_RaiseError(ctx->SaxCtxHdl,&(ctx->FataError),&(ctx->flagerror), fonctionname , DescErr, SDMD_XML_CODE_ERROR_4 );
	       }
  	  	   else
  	  	   {
  	  		 ctx->CurrentField->Delimited.Cnv_Ptr = node_ptr->Value;
  	  	   }
  	  	}
  	  	          
  		break;	
      }
  	  default:
  	  {
        snprintf(DescErr, sizeof(DescErr), SDMD_XML_LIB_ERROR_10, SDMD_MSGFIELD_FILE);
        SDM_RaiseError(ctx->SaxCtxHdl,&(ctx->FataError),&(ctx->flagerror), fonctionname , DescErr, SDMD_XML_CODE_ERROR_10);
  	  }
    } /* end switch */
  }
  else if (strcmp((const char *)localname, IMRD_SDMD_TAG_SIZE) == 0)
  {
  	switch(ctx->PresentLevelTag)
  	{
  	  case IMRD_SDMD_LEVEL_TAG_0:
  	  {
  	  	ctx->CurrentMessage->Size = atoi(ctx->value);
  		break;	
      }
  	  case IMRD_SDMD_LEVEL_TAG_2:
  	  {
  	  	ctx->CurrentField->Fixed.Size = atoi(ctx->value);
  		break;	
      }
  	  default:
  	  {
        snprintf(DescErr, sizeof(DescErr), SDMD_XML_LIB_ERROR_10, SDMD_MSGFIELD_FILE);
        SDM_RaiseError(ctx->SaxCtxHdl,&(ctx->FataError),&(ctx->flagerror), fonctionname , DescErr, SDMD_XML_CODE_ERROR_10);
  	  }
    } /* end switch */
  }
  else if (strcmp((const char *)localname, IMRD_SDMD_TAG_ATTRIBUTE) == 0)
  {
  	switch(ctx->PresentLevelTag)
  	{
  	  case IMRD_SDMD_LEVEL_TAG_3:
  	  {
  	  	if(strcmp(ctx->PathArray[ctx->PresentLevelTag-1], IMRD_SDMD_TAG_FIXED) == 0)
  	  	{
  	  		strcpy(ctx->CurrentField->Fixed.Cnv_Attr, ctx->value);
  	  	}
  	  	else if(strcmp(ctx->PathArray[ctx->PresentLevelTag-1], IMRD_SDMD_TAG_INTERNE) == 0)
  	  	{
  	  	    strcpy(ctx->CurrentField->Internal.Cnv_Attr, ctx->value);
  	  	}
  	  	else if(strcmp(ctx->PathArray[ctx->PresentLevelTag-1], IMRD_SDMD_TAG_TAGGED) == 0)
  	  	{
  	  	    strcpy(ctx->CurrentField->Tagged.Cnv_Attr, ctx->value);
        }
        else if(strcmp(ctx->PathArray[ctx->PresentLevelTag-1], IMRD_SDMD_TAG_XML) == 0)
  	  	{
  	  	    strcpy(ctx->CurrentField->XML.Cnv_Attr, ctx->value);
  	  	}
        else if(strcmp(ctx->PathArray[ctx->PresentLevelTag-1], IMRD_SDMD_TAG_FML) == 0)
  	  	{
  	  	    strcpy(ctx->CurrentField->FML.Cnv_Attr, ctx->value);
  	  	}
        else if(strcmp(ctx->PathArray[ctx->PresentLevelTag-1], IMRD_SDMD_TAG_DELIMITED) == 0)
  	  	{
  	  	    strcpy(ctx->CurrentField->Delimited.Cnv_Attr, ctx->value);
  	  	}

  	  	break;	
      }
  	  default:
  	  {
        snprintf(DescErr, sizeof(DescErr), SDMD_XML_LIB_ERROR_10, SDMD_MSGFIELD_FILE);
        SDM_RaiseError(ctx->SaxCtxHdl,&(ctx->FataError),&(ctx->flagerror), fonctionname , DescErr, SDMD_XML_CODE_ERROR_10);
  	  }
    } /* end switch */
  }
  else if (strcmp((const char *)localname, IMRD_SDMD_TAG_DEFVAL) == 0)
  {
  	switch(ctx->PresentLevelTag)
  	{
  	  case IMRD_SDMD_LEVEL_TAG_2:
  	  {
  	  	if(strcmp(ctx->PathArray[ctx->PresentLevelTag], IMRD_SDMD_TAG_FIXED) == 0)
  	  	{
  	  		strcpy(ctx->CurrentField->Fixed.Def_Val, ctx->value);
  	  	}
  	  	else if(strcmp(ctx->PathArray[ctx->PresentLevelTag], IMRD_SDMD_TAG_INTERNE) == 0)
  	  	{
  	  	    strcpy(ctx->CurrentField->Internal.Def_Val, ctx->value);
  	  	}
  	  	else if(strcmp(ctx->PathArray[ctx->PresentLevelTag], IMRD_SDMD_TAG_TAGGED) == 0)
  	  	{
  	  	    strcpy(ctx->CurrentField->Tagged.Def_Val, ctx->value);
        }
        else if(strcmp(ctx->PathArray[ctx->PresentLevelTag], IMRD_SDMD_TAG_XML) == 0)
  	  	{
  	  	    strcpy(ctx->CurrentField->XML.Def_Val, ctx->value);
  	  	}
        else if(strcmp(ctx->PathArray[ctx->PresentLevelTag], IMRD_SDMD_TAG_FML) == 0)
  	  	{
  	  	    strcpy(ctx->CurrentField->FML.Def_Val, ctx->value);
  	  	}
        else if(strcmp(ctx->PathArray[ctx->PresentLevelTag], IMRD_SDMD_TAG_DELIMITED) == 0)
  	  	{
  	  	    strcpy(ctx->CurrentField->Delimited.Def_Val, ctx->value);
  	  	}

  	  	break;	
      }
  	  default:
  	  {
        snprintf(DescErr, sizeof(DescErr), SDMD_XML_LIB_ERROR_10, SDMD_MSGFIELD_FILE);
        SDM_RaiseError(ctx->SaxCtxHdl,&(ctx->FataError),&(ctx->flagerror), fonctionname , DescErr, SDMD_XML_CODE_ERROR_10);
  	  }
    } /* end switch */
  }
  else if (strcmp((const char *)localname, IMRD_SDMD_TAG_OFFSET) == 0)
  {
  	ctx->CurrentField->Fixed.Offset = atoi(ctx->value);
  }
  else if (strcmp((const char *)localname, IMRD_SDMD_TAG_ID) == 0)
  {
  	ctx->CurrentField->Delimited.Id = atoi(ctx->value);
  }
  else if (strcmp((const char *)localname, IMRD_SDMD_TAG_VALUEID) == 0)
  {
  	strcpy(ctx->CurrentField->Value_Id, ctx->value);
  }
  else if (strcmp((const char *)localname, IMRD_SDMD_TAG_EXIST) == 0)
  {
  	if(*(ctx->value) == IMRD_YES)
  	{
  		ctx->CurrentField->Exist = IMRD_TRUE;
  	}
  	else
  	{
  		ctx->CurrentField->Exist = IMRD_FALSE;
  	}
  }
  else if ( (strcmp((const char*)localname, IMRD_SDMD_TAG_MESSAGE)== 0) ) 
  {
 
    ctx->CurrentMessage->Id = ctx->CptMsg;

	index_type_ptr = Field_Struct_final_idx_tab;
	status = SDM_DataStruct_Finalize(ctx->CurrentMessage->Field_Struct_Ptr, index_type_ptr, IMRD_SDMD_FIELD_STRUCT_IDX_NB);
	if(status == IMRS_KO)
	{
		printf("Erreur lors de la finalisation de Field_Struct_Ptr \n");
	}

    if( ( nd_status = ND_DataStruct_Value_Add( ctx->RootMessage, (void  *)(ctx->CurrentMessage))) != NDS_OK)
    {
      snprintf(DescErr,sizeof(DescErr),SDMD_XML_LIB_ERROR_2,IMRD_SDMD_TAG_MESSAGE);
      SDM_RaiseError(ctx->SaxCtxHdl,&(ctx->FataError),&(ctx->flagerror), fonctionname , DescErr, SDMD_XML_CODE_ERROR_2 );
    }
    ctx->CptMsg++;
    memset(ctx->PathArray[ctx->PresentLevelTag],0,sizeof(IMRD_SDMD_SIZE_TAG_SIZE));
    ctx->PresentLevelTag = ctx->PresentLevelTag - 1;
  }
  else if( (strcmp((const char*)localname, IMRD_SDMD_TAG_FIELD)== 0) )
  {
    ctx->CptField++;
    ctx->CurrentField->Id = ctx->CptField;
    
    if( ( nd_status = ND_DataStruct_Value_Add( ctx->CurrentMessage->Field_Struct_Ptr, (void  *)(ctx->CurrentField))) != NDS_OK)
    {
      snprintf(DescErr,sizeof(DescErr),SDMD_XML_LIB_ERROR_2,IMRD_SDMD_TAG_FIELD);
      SDM_RaiseError(ctx->SaxCtxHdl,&(ctx->FataError),&(ctx->flagerror), fonctionname , DescErr, SDMD_XML_CODE_ERROR_2 );
    }
    memset(ctx->PathArray[ctx->PresentLevelTag],0,sizeof(IMRD_SDMD_SIZE_TAG_SIZE));
    ctx->PresentLevelTag = ctx->PresentLevelTag - 1;
  }
  else if ((strcmp((const char *)localname, IMRD_SDMD_TAG_FIXED)     == 0) ||
            (strcmp((const char *)localname, IMRD_SDMD_TAG_INTERNE)  == 0) ||
            (strcmp((const char *)localname, IMRD_SDMD_TAG_TAGGED)   == 0) ||
            (strcmp((const char *)localname, IMRD_SDMD_TAG_XML)      == 0) ||
            (strcmp((const char *)localname, IMRD_SDMD_TAG_DELIMITED)== 0) ||
            (strcmp((const char *)localname, IMRD_SDMD_TAG_CNV)      == 0) ||
            (strcmp((const char *)localname, IMRD_SDMD_TAG_FML)      == 0)
           )
  {
    memset(ctx->PathArray[ctx->PresentLevelTag],0,sizeof(IMRD_SDMD_SIZE_TAG_SIZE));
    ctx->PresentLevelTag = ctx->PresentLevelTag - 1;
  }
  else if (strcmp((const char *)localname, IMRD_SDMD_TAG_VERSION) == 0)
  {
	IMRD_LM_LOG_INFO_1( "[Msg-Field]: [%s]", ctx->value);
	status_tmp = SDM_Set_Struct_Version((IMRT_Data_Version *)ctx->RootMessage->User_Ptr, (char *)ctx->value);
  }
}

IMRT_Status charactersDebug(SDMT_StructInfo_Contexte * ctx , const xmlChar *ch, int len)
{
    char output[40];
    int i;
    
    memset(ctx->value,0, sizeof(ctx->value));
    
    for (i = 0;(i<len) && (i < IMRD_SDMD_VALUE_LEN);i++)
	    ctx->value[i] = ch[i];
      ctx->value[i] = 0;

}


IMRT_Status SDM_XML_Load_Cnv( char *demo_file, SDMT_StructInfo_Contexte *mycontexte)
{
    IMRT_Status  status;
    NDT_Status   nd_status;
    char * docname = NULL;
    NDT_Index_Type  *index_type_ptr;
    xmlDocPtr doc = NULL;
    xmlDtdPtr dtd;
    xmlParserCtxtPtr ctxt =NULL;
    xmlSAXHandlerPtr SAXHandler;
    xmlSAXHandlerPtr SAXHandler_Sav;
    const char functionname []="SDM_XML_Load_Cnv";
    char DescErr[IMRD_SDMD_DESCERR_LEN];
	IMRT_Data_Version   *Data_Version_Ptr;

    LIBXML_TEST_VERSION
	xmlKeepBlanksDefault(0);	
    xmlSAXDefaultVersion(2);
    xmlLineNumbersDefault(1);
	
    status = IMRS_OK;
    docname = demo_file;
    ctxt = xmlNewParserCtxt();
    /*initialisation des indexes*/
    index_type_ptr =  Appl_Struct_initial_idx_tab;
  

  if(ctxt != NULL)
  {
	mycontexte->flagerror  = IMRS_OK;
	SAXHandler            = &SaxCnvHandlerStruct;
	mycontexte->SaxCtxHdl = ctxt;
	ctxt->userData        = mycontexte;
	SAXHandler_Sav        = ctxt->sax;
	ctxt->sax             = SAXHandler;
	ctxt->sax2            = 1 ;
    
	Data_Version_Ptr = (IMRT_Data_Version *) malloc(sizeof(IMRT_Data_Version));
	Data_Version_Ptr->Label[0] = '\0';
	Data_Version_Ptr->Date[0] = '\0';
	if( ( nd_status = ND_DataStruct_Open(&(mycontexte->RootCnv),IMRD_SDMD_CNV_STRUCT_IDX_NB,index_type_ptr, "SDM_Cnv_Manager",SDM_Cnv_Manager, NULL, NULL, NULL, NULL,0, Data_Version_Ptr)) != NDS_OK)
	{
		status = IMRS_KO;
	    printf( "ND_DataStruct_Open() failed (%d) !\n", nd_status);
	    snprintf(DescErr,sizeof(DescErr),SDMD_XML_LIB_ERROR_5,IMRD_SDMD_TAG_CNV);
	    SDM_RaiseError(mycontexte->SaxCtxHdl,&(mycontexte->FataError),&(mycontexte->flagerror), functionname, DescErr, SDMD_XML_CODE_ERROR_5 );
	}
	else
	{
		/* DAHBI ND_DataStruct_Info_Print(stderr, mycontexte->RootCnv, NDD_RECURSIVE_MODE_PARENT_CHILD, 0, 0);*/
	    doc = xmlCtxtReadFile(ctxt, docname, NULL, XML_PARSE_DTDATTR+XML_PARSE_DTDVALID);
	    if (ctxt->lastError.code != XML_ERR_OK)
	    {
	    	status = IMRS_KO;
	    	printf("Erreur de parsing du fichier %s.\n%s\n", docname, ctxt->lastError.message);
	    }
		else if(mycontexte->flagerror != IMRS_OK)
		{
			status = IMRS_KO;
			printf("%s\n", mycontexte->FataError.libError);
		}
	    else
	    {
			index_type_ptr = Cnv_Struct_final_idx_tab;
			status = SDM_DataStruct_Finalize(mycontexte->RootCnv,index_type_ptr, IMRD_SDMD_CNV_STRUCT_IDX_NB);
			/*SDM_DataStruct_Dump(mycontexte->RootCnv);*/
	    }
	}

	if((status != IMRS_OK)&&(mycontexte->RootCnv != NULL))
	{
	    if(mycontexte->CurrentCnv != NULL)
            {
		free(mycontexte->CurrentCnv);
            }	
	    nd_status = ND_DataStruct_Close(mycontexte->RootCnv);
	}

	ctxt->sax = SAXHandler_Sav;
	xmlFreeParserCtxt(ctxt);
  } /* end if(ctxt != NULL) */
  else
  {
  	status = IMRS_KO;
  }

  return(status);

}


IMRT_Status  SDM_XML_Load_MsgAndField( char *demo_file, SDMT_StructInfo_Contexte * mycontexte)
{
  IMRT_Status     status;
  NDT_Status      nd_status;
  char * docname = NULL;
	NDT_Index_Type  *index_type_ptr;
	xmlDocPtr doc = NULL;
	xmlDtdPtr dtd;
	xmlParserCtxtPtr ctxt =NULL;
	xmlSAXHandlerPtr debugSAXHandler;
	xmlSAXHandlerPtr SAXHandler_Sav;
	const char functionname []="SDM_XML_Load_MsgAndField";
	char DescErr[IMRD_SDMD_DESCERR_LEN];
	IMRT_Data_Version   *Data_Version_Ptr;

  LIBXML_TEST_VERSION
	xmlKeepBlanksDefault(0);	
	xmlSAXDefaultVersion(2);
	xmlLineNumbersDefault(1);
	
  status = IMRS_OK;
  docname = demo_file;
  ctxt = xmlNewParserCtxt();
  /*initialisation des indexes*/
  index_type_ptr =  Msg_Struct_initial_idx_tab;

  if(ctxt != NULL)
  {
    mycontexte->flagerror  = IMRS_OK;
    debugSAXHandler        = &debugSAXHandlerStruct;
    mycontexte->SaxCtxHdl  = ctxt;
    ctxt->userData         = mycontexte;
    SAXHandler_Sav        = ctxt->sax;
    ctxt->sax              = debugSAXHandler;
    ctxt->sax2             = 1 ;
    
    /*xmlSetGenericErrorFunc(NULL,CatchError);*/
	Data_Version_Ptr = (IMRT_Data_Version *) malloc(sizeof(IMRT_Data_Version));
	Data_Version_Ptr->Label[0] = '\0';
	Data_Version_Ptr->Date[0] = '\0';
    if( ( nd_status = ND_DataStruct_Open(&(mycontexte->RootMessage),IMRD_SDMD_MSG_STRUCT_IDX_NB, index_type_ptr, "SDM__Msg_Manager",SDM_Msg_Manager, NULL, NULL, NULL, NULL,0, Data_Version_Ptr)) != NDS_OK)
    {
    	status = IMRS_KO;
        snprintf(DescErr,sizeof(DescErr),SDMD_XML_LIB_ERROR_5,IMRD_SDMD_TAG_MESSAGE);
        SDM_RaiseError(mycontexte->SaxCtxHdl,&(mycontexte->FataError),&(mycontexte->flagerror), functionname, DescErr, SDMD_XML_CODE_ERROR_5 );
    }
    else
    {
      doc = xmlCtxtReadFile(ctxt, docname, NULL, XML_PARSE_DTDATTR+XML_PARSE_DTDVALID);
      if (ctxt->lastError.code != XML_ERR_OK)
	  {
	    	status = IMRS_KO;
	    	printf("Erreur de parsing du fichier %s.\n%s\n", docname, ctxt->lastError.message);
	  }
	  else if(mycontexte->flagerror != IMRS_OK)
	  {
		status = IMRS_KO;
		printf("%s\n", mycontexte->FataError.libError);
	  }
      else
      {
			index_type_ptr = Msg_Struct_final_idx_tab;
			status = SDM_DataStruct_Finalize(mycontexte->RootMessage, index_type_ptr, IMRD_SDMD_MSG_STRUCT_IDX_NB);
			/*SDM_DataStruct_Dump( mycontexte->RootMessage);*/
	  }
    }

	if(status != IMRS_OK)
	{
	    if( mycontexte->CurrentMessage != NULL)
	    {
		if(mycontexte->CurrentMessage->Field_Struct_Ptr != NULL)
		{
                    nd_status = ND_DataStruct_Close(mycontexte->CurrentMessage->Field_Struct_Ptr);
                }
                if( nd_status != NDS_OK)
		{
		    free(mycontexte->CurrentMessage);
		}
            }
            if(mycontexte->CurrentField != NULL)
	    {
		free(mycontexte->CurrentField);
            }
	    if(mycontexte->RootMessage != NULL)
	    {
		nd_status = ND_DataStruct_Close(mycontexte->RootMessage);
	    }
	    /*if(mycontexte->RootCnv != NULL)
	    {
		nd_status = ND_DataStruct_Close(mycontexte->RootCnv);
		}*/
	}

	ctxt->sax = SAXHandler_Sav;
	xmlFreeParserCtxt(ctxt);
  }
  else
  {
  	status = IMRS_KO;
  }

  return(status);

}



/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
IMRT_Status  SDM_XML_Dump_Cnv_Header(FILE *dump_file)
{
	fprintf(dump_file, "<?xml version=\"1.0\" ?>\n");
	fprintf(dump_file, "<!-- Cnv Struct definitions -->\n");
	fprintf(dump_file, "<%s>\n", IMRD_SDMD_TAG_CNV_STRUCT);
	fprintf(dump_file, "<version>@(#) VERSIONIMR: $Label:  $   $Date: 2008/11/12 02:25:23 $   $Workfile:   struct_info_xml.c  $</version>\n", 
				((IMRT_Data_Version *)IMRG_SDM_Base.RootCnv->User_Ptr)->Label,
				((IMRT_Data_Version *)IMRG_SDM_Base.RootCnv->User_Ptr)->Date);
	return( IMRS_OK);

}
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
IMRT_Status  SDM_XML_Dump_Cnv_Body(FILE *dump_file)
{
	IMRT_Status  status;
	NDT_Status  nd_status;

	if( ( nd_status = ND_DataStruct_Traverse( IMRG_SDM_Base.RootCnv, IMRD_SDMD_CMD_API_XML_DUMP, dump_file)) != NDS_OK)
	{
		status = IMRS_KO;
		printf( "XML Dump RootCnv failed (%d)...\n");
	}
	else
	{
		status = IMRS_OK;
	}

	return(status);
}
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
IMRT_Status  SDM_XML_Dump_Cnv_Footer(FILE *dump_file)
{
	fprintf(dump_file, "<!-- Cnv Struct definitions footer -->\n");
	fprintf(dump_file, "</%s>\n", IMRD_SDMD_TAG_CNV_STRUCT);
	return( IMRS_OK);

}
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
IMRT_Status  SDM_XML_Dump_Cnv(void)
{
	IMRT_Status     status;
	char	    dump_filename[255];
	FILE	   *dump_file;

	strcpy(dump_filename, SDMD_CNV_FILE);

	if( ( dump_file = fopen( dump_filename, "w")) == NULL)
	{
		status = IMRS_KO;
		printf( "Can't open dump file (%s) !\n", dump_filename);
	}
	else
	{
		status = SDM_XML_Dump_Cnv_Header(dump_file);
		if(status == IMRS_OK)
		{
			status = SDM_XML_Dump_Cnv_Body(dump_file);
			if(status == IMRS_OK)
			{
				status = SDM_XML_Dump_Cnv_Footer(dump_file);
			}
		}

		fclose( dump_file);
	}

	return(status);

}
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
IMRT_Status  SDM_XML_Dump_Msg_Header(FILE *dump_file)
{
	fprintf(dump_file, "<?xml version=\"1.0\" ?>\n");
	fprintf(dump_file, "<!-- Msg Struct definitions -->\n");
	fprintf(dump_file, "<%s>\n", IMRD_SDMD_TAG_MSG_STRUCT);
	fprintf(dump_file, "<version>@(#) VERSIONIMR: $Label:  $   $Date: 2008/11/12 02:25:23 $   $Workfile:   struct_info_xml.c  $</version>\n", 
				((IMRT_Data_Version *)IMRG_SDM_Base.RootMsg->User_Ptr)->Label,
				((IMRT_Data_Version *)IMRG_SDM_Base.RootMsg->User_Ptr)->Date);
	return( IMRS_OK);
	
}
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
IMRT_Status  SDM_XML_Dump_Msg_Body(FILE *dump_file)
{
	IMRT_Status  status;
	NDT_Status   nd_status;

	if( ( nd_status = ND_DataStruct_Traverse( IMRG_SDM_Base.RootMsg, IMRD_SDMD_CMD_API_XML_DUMP, dump_file)) != NDS_OK)
	{
		status = IMRS_KO;
		printf( "XML Dump RootMsg failed (%d)...\n");
	}
	else
	{
		status = IMRS_OK;
	}

	return(status);
}
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
IMRT_Status  SDM_XML_Dump_Msg_Footer(FILE *dump_file)
{
	fprintf(dump_file, "<!-- Msg Struct definitions footer -->\n");
	fprintf(dump_file, "</%s>\n", IMRD_SDMD_TAG_MSG_STRUCT);
	return( IMRS_OK);

}
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
IMRT_Status  SDM_XML_Dump_Msg(void)
{
	IMRT_Status     status;
	char	    dump_filename[255];
	FILE	   *dump_file;

	strcpy(dump_filename, SDMD_MSGFIELD_FILE);

	if( ( dump_file = fopen( dump_filename, "w")) == NULL)
	{
		status = IMRS_KO;
		printf( "Can't open dump file (%s) !\n", dump_filename);
	}
	else
	{
		status = SDM_XML_Dump_Msg_Header(dump_file);
		if(status == IMRS_OK)
		{
			status = SDM_XML_Dump_Msg_Body(dump_file);
			if(status == IMRS_OK)
			{
				status = SDM_XML_Dump_Msg_Footer(dump_file);
			}
		}

		fclose( dump_file);
	}

	return(status);
}
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
IMRT_Status  SDM_XML_StructInfo_Dump(void)
{
	IMRT_Status     status;

	status = SDM_XML_Dump_Cnv();

	if(status == IMRS_OK)
	{
		status = SDM_XML_Dump_Msg();
	}

	return(status);
}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/
