/*----------------------------------------------------------------------------*/
/* File: struct_info_xml.h 				         	      */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* IMR - Interlinking Message Router                                          */
/* Copyright (C) 2005 Arnaud G. Gibert & Others (See ReadMe.txt)              */
/* mailto:arnaud@rx3.net                                                      */
/* http://www.rx3.org/dvp/?dvp=imr                                            */
/*----------------------------------------------------------------------------*/
/* This file is part of IMR                                                   */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.        */
/*----------------------------------------------------------------------------*/

#ifndef _STRUCT_INFO_XML_H_
#   define _STRUCT_INFO_XML_H_



/*----------------------------------------------------------------------------*/
/* Global Include                                                             */
/*----------------------------------------------------------------------------*/

#   include <libstatic_data_mng.h>

/*----------------------------------------------------------------------------*/
/* Local constantes                                                           */
/*----------------------------------------------------------------------------*/

/*definition des tags xml pour le chargement des structure Msg et Field*/
#define IMRD_SDMD_TAG_MESSAGE       "message"
#define IMRD_SDMD_TAG_FIELD         "field"
#define IMRD_SDMD_TAG_VALUEID       "valueid"
#define IMRD_SDMD_TAG_FIXED         "fixed"
#define IMRD_SDMD_TAG_OFFSET        "offset"
#define IMRD_SDMD_TAG_SIZE          "size"
#define IMRD_SDMD_TAG_TAGGED        "tagged"
#define IMRD_SDMD_TAG_TAG           "tag"
#define IMRD_SDMD_TAG_INTERNE       "interne"
#define IMRD_SDMD_TAG_INTERNAL_NAME "internal_name"
#define IMRD_SDMD_TAG_FORMAT        "format"
#define IMRD_SDMD_TAG_XML           "xml"
#define IMRD_SDMD_TAG_FML           "fml"
#define IMRD_SDMD_TAG_DELIMITED     "delimited"
#define IMRD_SDMD_TAG_ID            "id"
#define IMRD_SDMD_TAG_DEFVAL        "defval"
#define IMRD_SDMD_TAG_CNV           "cnv"
#define IMRD_SDMD_TAG_ATTRIBUTE     "attribute"
#define IMRD_SDMD_TAG_EXIST         "exist"


/*----------------------------------------------------------------------------*/
/* Local Definition                                                           */
/*----------------------------------------------------------------------------*/



typedef struct SDMT_StructInfo_Contexte
{
	char value[IMRD_SDMD_VALUE_LEN];
    IMRT_Array_Xml_Path PathArray;
    IMRT_Level_Tag PresentLevelTag;
    IMRT_Msg_Id     CptMsg;
    IMRT_Field_Id   CptField;
    IMRT_Cnv_Id     CptCnv;
	IMRT_Field    * CurrentField;
	IMRT_Msg      * CurrentMessage;
	IMRT_Cnv      * CurrentCnv;
	NDT_Root      * RootMessage;
	NDT_Root      * RootCnv;
	xmlParserCtxt * SaxCtxHdl; 
	IMRT_Error      FataError;
	IMRT_Status     flagerror;
}SDMT_StructInfo_Contexte;

/*----------------------------------------------------------------------------*/
/* Prototypes                                                                 */
/*----------------------------------------------------------------------------*/

#   ifdef _STRUCT_INFO_XML_C_

static IMRT_Status error(SDMT_StructInfo_Contexte *ctx , const char *msg, ...);

IMRT_Status startElementNsCnv( SDMT_StructInfo_Contexte * ctx , 
             const xmlChar *localname,
             const xmlChar *prefix,
             const xmlChar *URI,
             int nb_namespaces,
             const xmlChar **namespaces,
             int nb_attributes,
             int nb_defaulted,
             const xmlChar **attributes);

IMRT_Status startElementNS( SDMT_StructInfo_Contexte * ctx , 
		     const xmlChar *localname,
		     const xmlChar *prefix,
		     const xmlChar *URI,
		     int nb_namespaces,
		     const xmlChar **namespaces,
		     int nb_attributes,
		     int nb_defaulted,
		     const xmlChar **attributes);

IMRT_Status endElementNsCnv( SDMT_StructInfo_Contexte * ctx, 
		   const xmlChar *localname,
		   const xmlChar *prefix,
		   const xmlChar *URI);

IMRT_Status endElementNS( SDMT_StructInfo_Contexte * ctx, 
		   const xmlChar *localname,
		   const xmlChar *prefix,
		   const xmlChar *URI);

IMRT_Status charactersDebug(SDMT_StructInfo_Contexte * ctx , const xmlChar *ch, int len);

IMRT_Status  SDM_XML_Load_Cnv(   char *demo_file, SDMT_StructInfo_Contexte *mycontexte);
IMRT_Status  SDM_XML_Load_MsgAndField( char *demo_file, SDMT_StructInfo_Contexte * mycontexte);
IMRT_Status SDM_XML_StructInfo_Init(IMRT_Path    Lib_Path);
IMRT_Status  SDM_XML_Dump_Cnv_Header(FILE *dump_file);
IMRT_Status  SDM_XML_Dump_Cnv_Body(FILE *dump_file);
IMRT_Status  SDM_XML_Dump_Cnv_Footer(FILE *dump_file);
IMRT_Status  SDM_XML_Dump_Cnv(void);
IMRT_Status  SDM_XML_Dump_Msg_Header(FILE *dump_file);
IMRT_Status  SDM_XML_Dump_Msg_Body(FILE *dump_file);
IMRT_Status  SDM_XML_Dump_Msg_Footer(FILE *dump_file);
IMRT_Status  SDM_XML_Dump_Msg(void);
IMRT_Status  SDM_XML_StructInfo_Dump(void);

#   else

extern IMRT_Status  SDM_XML_Load_Cnv(   char *demo_file, SDMT_StructInfo_Contexte *mycontexte);
extern IMRT_Status  SDM_XML_Load_MsgAndField( char *demo_file, SDMT_StructInfo_Contexte * mycontexte);
extern IMRT_Status SDM_XML_StructInfo_Init(IMRT_Path    Lib_Path);
extern IMRT_Status  SDM_XML_Dump_Cnv_Header(FILE *dump_file);
extern IMRT_Status  SDM_XML_Dump_Cnv_Body(FILE *dump_file);
extern IMRT_Status  SDM_XML_Dump_Cnv_Footer(FILE *dump_file);
extern IMRT_Status  SDM_XML_Dump_Cnv(void);
extern IMRT_Status  SDM_XML_Dump_Msg_Header(FILE *dump_file);
extern IMRT_Status  SDM_XML_Dump_Msg_Body(FILE *dump_file);
extern IMRT_Status  SDM_XML_Dump_Msg_Footer(FILE *dump_file);
extern IMRT_Status  SDM_XML_Dump_Msg(void);
extern IMRT_Status  SDM_XML_StructInfo_Dump(void);

#   endif

/*----------------------------------------------------------------------------*/
/* Globale Variables                                                          */
/*----------------------------------------------------------------------------*/

#   ifdef _STRUCT_INFO_XML_C_

/*xmlSAXHandler ErrorSAXHandlerStruct = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    1,
    NULL,
    NULL,
    NULL,
    NULL
    };*/

xmlSAXHandler SaxCnvHandlerStruct = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    (charactersSAXFunc)(charactersDebug),
    NULL,
    NULL,
    NULL,
    NULL,
    (errorSAXFunc)error,
    NULL,
    NULL,
    NULL,
    NULL,
    1,
    NULL,
    (startElementNsSAX2Func)(startElementNsCnv),
    (endElementNsSAX2Func)(endElementNsCnv),
    NULL
};
xmlSAXHandler debugSAXHandlerStruct = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    (charactersSAXFunc)(charactersDebug),
    NULL,
    NULL,
    NULL,
    NULL,
    (errorSAXFunc)(error),
    NULL,
    NULL,
    NULL,
    NULL,
    1,
    NULL,
    (startElementNsSAX2Func)(startElementNS),
    (endElementNsSAX2Func)(endElementNS),
    NULL
};



#   endif

/*----------------------------------------------------------------------------*/

#endif
